--
-- PostgreSQL database dump
--

-- Dumped from database version 9.5.12
-- Dumped by pg_dump version 9.6.5

SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET row_security = off;

--
-- Name: pg_repack; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS pg_repack WITH SCHEMA public;


--
-- Name: EXTENSION pg_repack; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION pg_repack IS 'Reorganize tables in PostgreSQL databases with minimal locks';


--
-- Name: topology; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA topology;


--
-- Name: plpgsql; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;


--
-- Name: EXTENSION plpgsql; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION plpgsql IS 'PL/pgSQL procedural language';


--
-- Name: btree_gin; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS btree_gin WITH SCHEMA public;


--
-- Name: EXTENSION btree_gin; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION btree_gin IS 'support for indexing common datatypes in GIN';


--
-- Name: hstore; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS hstore WITH SCHEMA public;


--
-- Name: EXTENSION hstore; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION hstore IS 'data type for storing sets of (key, value) pairs';


--
-- Name: pg_trgm; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS pg_trgm WITH SCHEMA public;


--
-- Name: EXTENSION pg_trgm; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION pg_trgm IS 'text similarity measurement and index searching based on trigrams';


--
-- Name: pgstattuple; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS pgstattuple WITH SCHEMA public;


--
-- Name: EXTENSION pgstattuple; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION pgstattuple IS 'show tuple-level statistics';


--
-- Name: postgis; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS postgis WITH SCHEMA public;


--
-- Name: EXTENSION postgis; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION postgis IS 'PostGIS geometry, geography, and raster spatial types and functions';


--
-- Name: postgis_topology; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS postgis_topology WITH SCHEMA topology;


--
-- Name: EXTENSION postgis_topology; Type: COMMENT; Schema: -; Owner: -
--

COMMENT ON EXTENSION postgis_topology IS 'PostGIS topology spatial types and functions';


SET search_path = public, pg_catalog;

--
-- Name: billings_insert_trigger(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION billings_insert_trigger() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$
             DECLARE
               curY varchar(4);
               curM varchar(2);
               tbl varchar(30);
             BEGIN
                select cast(DATE_PART('year', new.date) as varchar) into curY;
                select lpad(cast(DATE_PART('month', new.date) as varchar), 2, '0') into curM;
                tbl := 'billings_y' || curY || 'm' || curM;
               EXECUTE format('INSERT into %I values ($1.*);', tbl) USING NEW;
               return NULL;
             END;
             $_$;


--
-- Name: clicks_delete_trigger(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION clicks_delete_trigger() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
       DECLARE
           r clicks%rowtype;
       BEGIN
           DELETE FROM ONLY clicks where id = new.id returning * into r;
           RETURN r;
       end;
       $$;


--
-- Name: clicks_insert_trigger(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION clicks_insert_trigger() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$
         DECLARE
           curY varchar(4);
           curM varchar(2);
           tbl varchar(30);
         BEGIN
            select cast(DATE_PART('year', new.created_at) as varchar) into curY;
            select lpad(cast(DATE_PART('month', new.created_at) as varchar), 2, '0') into curM;
            tbl := 'clicks_y' || curY || 'm' || curM;
            EXECUTE format('INSERT into %I values ($1.*);', tbl) USING NEW;
            return NEW;
         END;
         $_$;


--
-- Name: device_histories_insert_trigger(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION device_histories_insert_trigger() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$
         DECLARE
           curY varchar(4);
           curM varchar(2);
           tbl varchar(30);
         BEGIN
            select cast(DATE_PART('year', new.created_at) as varchar) into curY;
            select lpad(cast(DATE_PART('month', new.created_at) as varchar), 2, '0') into curM;
            tbl := 'device_histories_y' || curY || 'm' || curM;
            EXECUTE format('INSERT into %I values ($1.*);', tbl) USING NEW;
            return NULL;
         END;
         $_$;


--
-- Name: insight_stats_insert_trigger(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION insight_stats_insert_trigger() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$
         DECLARE
           curY varchar(4);
           curM varchar(2);
           tbl varchar(30);
         BEGIN
            select cast(DATE_PART('year', new.created_at) as varchar) into curY;
            select lpad(cast(DATE_PART('month', new.created_at) as varchar), 2, '0') into curM;
            tbl := 'insight_stats_y' || curY || 'm' || curM;
            EXECUTE format('INSERT into %I values ($1.*);', tbl) USING NEW;
            return NULL;
         END;
         $_$;


--
-- Name: interests_filter(json, json); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION interests_filter(interests_column json, user_interests json) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE
    AS $$
      DECLARE
          interest_rec json;
          len int;
          ret boolean;
      BEGIN
          -- If interests_column is not an array, return an empty array as the result
          BEGIN
              len := json_array_length(interests_column);
              IF len = 0 THEN
                RETURN FALSE;
              ELSIF len IS NULL THEN
                RETURN FALSE;
              END IF;
          EXCEPTION
              WHEN OTHERS THEN
                  RETURN FALSE;
          END;

          -- Launch loop for each of the campaign image interest
          ret := FALSE;
          FOR interest_rec IN SELECT json_array_elements(interests_column)
          LOOP
            --  If at least one of the interest required by the image is not present in users interests
            --  we should remove the image from result and stop the image verivication
            IF (SELECT ARRAY(
                   SELECT interest_rec::json->>'name'::text
                 ) <@ (
                   SELECT ARRAY(
                     SELECT json_object_keys(user_interests::json)
                   )
                 )
               ) = FALSE THEN

               RETURN FALSE;

            END IF;

            -- otherwise we are going to make needed comparations

            -- if at least one of the user interest doesn't fit the image requirements
            -- we should remove the image from result and stop the image verivication
            IF interest_rec::json->>'action' = 'gt' THEN
              ret := ((user_interests::json->>(interest_rec::json->>'name'))::int >= (interest_rec::json->>'value')::int);
            ELSE
              -- Possible needed SELECT FIRST?!
              ret := ((user_interests::json->>(interest_rec::json->>'name'))::int <= (interest_rec::json->>'value')::int);
            END IF;

            IF ret = FALSE THEN
              RETURN FALSE;
            END IF;
          END LOOP;

          -- If all is ok and all interests are right, return true
          RETURN TRUE;
      END $$;


--
-- Name: pgcompact_clean_pages_101744(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_101744(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_104606(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_104606(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_105118(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_105118(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_10597(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_10597(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_113684(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_113684(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_15096(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_15096(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_16381(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_16381(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_16473(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_16473(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_170654(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_170654(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_18327(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_18327(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_18717(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_18717(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_18802(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_18802(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_20171(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_20171(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_20246(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_20246(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_20534(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_20534(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_21488(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_21488(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_21952(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_21952(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_21996(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_21996(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_22280(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_22280(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_23771(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_23771(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_23796(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_23796(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_26412(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_26412(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_27636(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_27636(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_27887(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_27887(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_2843(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_2843(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_3679(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_3679(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_4293(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_4293(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_6081(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_6081(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_63219(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_63219(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_6441(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_6441(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_67436(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_67436(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_9460(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_9460(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_9584(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_9584(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: pgcompact_clean_pages_9785(text, text, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION pgcompact_clean_pages_9785(i_table_ident text, i_column_ident text, i_to_page integer, i_page_offset integer, i_max_tupples_per_page integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
DECLARE
    _from_page integer := i_to_page - i_page_offset + 1;
    _min_ctid tid;
    _max_ctid tid;
    _ctid_list tid[];
    _next_ctid_list tid[];
    _ctid tid;
    _loop integer;
    _result_page integer;
    _update_query text :=
        'UPDATE ONLY ' || i_table_ident ||
        ' SET ' || i_column_ident || ' = ' || i_column_ident ||
        ' WHERE ctid = ANY($1) RETURNING ctid';
BEGIN
    -- Check page argument values
    IF NOT (
        i_page_offset IS NOT NULL AND i_page_offset >= 1 AND
        i_to_page IS NOT NULL AND i_to_page >= 1 AND
        i_to_page >= i_page_offset)
    THEN
        RAISE EXCEPTION 'Wrong page arguments specified.';
    END IF;

    -- Check that session_replication_role is set to replica to
    -- prevent triggers firing
    IF NOT (
        SELECT setting = 'replica'
        FROM pg_catalog.pg_settings
        WHERE name = 'session_replication_role')
    THEN
        RAISE EXCEPTION 'The session_replication_role must be set to replica.';
    END IF;

    -- Define minimal and maximal ctid values of the range
    _min_ctid := (_from_page, 1)::text::tid;
    _max_ctid := (i_to_page, i_max_tupples_per_page)::text::tid;

    -- Build a list of possible ctid values of the range
    SELECT array_agg((pi, ti)::text::tid)
    INTO _ctid_list
    FROM generate_series(_from_page, i_to_page) AS pi
    CROSS JOIN generate_series(1, i_max_tupples_per_page) AS ti;

    <<_outer_loop>>
    FOR _loop IN 1..i_max_tupples_per_page LOOP
        _next_ctid_list := array[]::tid[];

        -- Update all the tuples in the range
        FOR _ctid IN EXECUTE _update_query USING _ctid_list
        LOOP
            IF _ctid > _max_ctid THEN
                _result_page := -1;
                EXIT _outer_loop;
            ELSIF _ctid >= _min_ctid THEN
                -- The tuple is still in the range, more updates are needed
                _next_ctid_list := _next_ctid_list || _ctid;
            END IF;
        END LOOP;

        _ctid_list := _next_ctid_list;

        -- Finish processing if there are no tupples in the range left
        IF coalesce(array_length(_ctid_list, 1), 0) = 0 THEN
            _result_page := _from_page - 1;
            EXIT _outer_loop;
        END IF;
    END LOOP;

    -- No result
    IF _loop = i_max_tupples_per_page AND _result_page IS NULL THEN
        RAISE EXCEPTION
            'Maximal loops count has been reached with no result.';
    END IF;

    RETURN _result_page;
END $_$;


--
-- Name: random_string(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION random_string(length integer) RETURNS text
    LANGUAGE plpgsql
    AS $$
declare
  chars text[] := '{0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z}';
  result text := '';
  i integer := 0;
begin
  if length < 0 then
    raise exception 'Given length cannot be less than 0';
  end if;
  for i in 1..length loop
    result := result || chars[1+random()*(array_length(chars, 1)-1)];
  end loop;
  return result;
end;
$$;


--
-- Name: set_fraud_score_in_ip_addresses_func(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION set_fraud_score_in_ip_addresses_func() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
             DECLARE
               spamhaus_int int;
               spamcop_int int;
               wpbl_int int;
               custom_int int;
               stopforumspam_int int;
             BEGIN
               spamhaus_int := (CASE NEW.spamhaus WHEN 't' THEN 1 ELSE 0 END);
               spamcop_int := (CASE NEW.spamcop WHEN 't' THEN 1 ELSE 0 END);
               wpbl_int := (CASE NEW.wpbl WHEN 't' THEN 1 ELSE 0 END);
               stopforumspam_int := (CASE NEW.stopforumspam WHEN 't' THEN 1 ELSE 0 END);
               custom_int := (CASE NEW.custom_block WHEN 't' THEN 10 ELSE 0 END);
               NEW.fraud_score := (spamhaus_int + spamcop_int + wpbl_int + stopforumspam_int + custom_int);
              
               return NEW;
             END;
             $$;


--
-- Name: vpn_ips_delete_trigger(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION vpn_ips_delete_trigger() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
       DECLARE
           r vpn_ips%rowtype;
       BEGIN
           DELETE FROM ONLY vpn_ips where id = new.id returning * into r;
           RETURN r;
       end;
       $$;


--
-- Name: vpn_ips_insert_trigger(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION vpn_ips_insert_trigger() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$
         DECLARE
           curY varchar(4);
           curM varchar(2);
           tbl varchar(30);
         BEGIN
            select cast(DATE_PART('year', new.created_at) as varchar) into curY;
            select lpad(cast(DATE_PART('month', new.created_at) as varchar), 2, '0') into curM;
            tbl := 'vpn_ips_y' || curY || 'm' || curM;
            EXECUTE format('INSERT into %I values ($1.*);', tbl) USING NEW;
            return NEW;
         END;
         $_$;


--
-- Name: bit_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY bit_ops USING gin;


--
-- Name: btree_geography_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY btree_geography_ops USING btree;


--
-- Name: btree_geometry_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY btree_geometry_ops USING btree;


--
-- Name: btree_hstore_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY btree_hstore_ops USING btree;


--
-- Name: bytea_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY bytea_ops USING gin;


--
-- Name: char_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY char_ops USING gin;


--
-- Name: cidr_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY cidr_ops USING gin;


--
-- Name: date_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY date_ops USING gin;


--
-- Name: float4_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY float4_ops USING gin;


--
-- Name: float8_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY float8_ops USING gin;


--
-- Name: gin_hstore_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY gin_hstore_ops USING gin;


--
-- Name: gist_geography_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY gist_geography_ops USING gist;


--
-- Name: gist_geometry_ops_2d; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY gist_geometry_ops_2d USING gist;


--
-- Name: gist_geometry_ops_nd; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY gist_geometry_ops_nd USING gist;


--
-- Name: gist_hstore_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY gist_hstore_ops USING gist;


--
-- Name: hash_hstore_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY hash_hstore_ops USING hash;


--
-- Name: hash_raster_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY hash_raster_ops USING hash;


--
-- Name: inet_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY inet_ops USING gin;


--
-- Name: int2_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY int2_ops USING gin;


--
-- Name: int4_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY int4_ops USING gin;


--
-- Name: int8_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY int8_ops USING gin;


--
-- Name: interval_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY interval_ops USING gin;


--
-- Name: macaddr_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY macaddr_ops USING gin;


--
-- Name: money_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY money_ops USING gin;


--
-- Name: numeric_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY numeric_ops USING gin;


--
-- Name: oid_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY oid_ops USING gin;


--
-- Name: text_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY text_ops USING gin;


--
-- Name: time_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY time_ops USING gin;


--
-- Name: timestamp_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY timestamp_ops USING gin;


--
-- Name: timestamptz_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY timestamptz_ops USING gin;


--
-- Name: timetz_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY timetz_ops USING gin;


--
-- Name: varbit_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY varbit_ops USING gin;


--
-- Name: varchar_ops; Type: OPERATOR FAMILY; Schema: public; Owner: -
--

CREATE OPERATOR FAMILY varchar_ops USING gin;


SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: account_employees; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE account_employees (
    id integer NOT NULL,
    name character varying(255),
    slack_name character varying(255),
    user_id integer,
    enabled boolean DEFAULT true,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    type character varying(255)
);


--
-- Name: account_employees_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE account_employees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: account_employees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE account_employees_id_seq OWNED BY account_employees.id;


--
-- Name: account_shares; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE account_shares (
    id integer NOT NULL,
    owner_id integer,
    developer_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    role integer DEFAULT 0,
    share_type integer DEFAULT 10,
    admin_only_visible boolean DEFAULT false,
    dev_share_type integer DEFAULT 20,
    campaigns_share_type integer DEFAULT 20,
    share_token_id integer
);


--
-- Name: account_shares_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE account_shares_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: account_shares_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE account_shares_id_seq OWNED BY account_shares.id;


--
-- Name: active_admin_comments; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE active_admin_comments (
    id integer NOT NULL,
    namespace character varying(255),
    body text,
    resource_id character varying(255) NOT NULL,
    resource_type character varying(255) NOT NULL,
    author_id integer,
    author_type character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: active_admin_comments_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE active_admin_comments_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: active_admin_comments_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE active_admin_comments_id_seq OWNED BY active_admin_comments.id;


--
-- Name: active_apps; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE active_apps (
    app_id integer,
    date date,
    id integer NOT NULL
);


--
-- Name: active_apps_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE active_apps_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: active_apps_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE active_apps_id_seq OWNED BY active_apps.id;


--
-- Name: active_users; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE active_users (
    id integer NOT NULL,
    user_id integer,
    date date,
    revenue numeric(8,2) DEFAULT 0 NOT NULL,
    fee numeric(8,2) DEFAULT 0 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: active_users_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE active_users_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: active_users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE active_users_id_seq OWNED BY active_users.id;


--
-- Name: ad_request_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE ad_request_settings (
    id integer NOT NULL,
    name character varying(255),
    float_value double precision,
    string_value character varying(255),
    boolean_value boolean,
    use_global_value boolean,
    value_type integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    text_value text,
    ignored_apps text
);


--
-- Name: ad_request_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE ad_request_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ad_request_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE ad_request_settings_id_seq OWNED BY ad_request_settings.id;


--
-- Name: ad_unit_templates; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE ad_unit_templates (
    id integer NOT NULL,
    bid_floor double precision NOT NULL,
    account_type character varying(255) NOT NULL,
    ad_type integer NOT NULL,
    format integer NOT NULL,
    width integer NOT NULL,
    height integer NOT NULL,
    is_landscape boolean DEFAULT false,
    bid_floor_local double precision,
    bid_floor_currency character varying(255) DEFAULT 'USD'::character varying NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    extra json DEFAULT '{}'::json NOT NULL,
    orientation integer DEFAULT 0 NOT NULL,
    device_type integer DEFAULT 2 NOT NULL,
    parallel_bidding boolean DEFAULT false NOT NULL,
    active boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    network_attributes json DEFAULT '{}'::json NOT NULL
);


--
-- Name: ad_unit_templates_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE ad_unit_templates_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ad_unit_templates_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE ad_unit_templates_id_seq OWNED BY ad_unit_templates.id;


--
-- Name: ad_unit_versions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE ad_unit_versions (
    id integer NOT NULL,
    item_type character varying(255) NOT NULL,
    item_id integer NOT NULL,
    event character varying(255) NOT NULL,
    whodunnit character varying(255),
    old_object text,
    created_at timestamp without time zone,
    object json,
    object_changes text
);


--
-- Name: ad_unit_versions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE ad_unit_versions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ad_unit_versions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE ad_unit_versions_id_seq OWNED BY ad_unit_versions.id;


--
-- Name: ad_units_backup; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE ad_units_backup (
    id integer NOT NULL,
    code character varying(255) NOT NULL,
    bid_floor double precision NOT NULL,
    app_id integer NOT NULL,
    account_id integer NOT NULL,
    account_type character varying(255) NOT NULL,
    ad_type integer NOT NULL,
    format integer NOT NULL,
    width integer NOT NULL,
    height integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    is_landscape boolean DEFAULT false,
    bid_floor_local double precision,
    bid_floor_currency character varying(255) DEFAULT 'USD'::character varying NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    extra json DEFAULT '{}'::json NOT NULL,
    orientation integer DEFAULT 0 NOT NULL,
    device_type integer DEFAULT 0 NOT NULL
);


--
-- Name: ad_units_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE ad_units_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ad_units_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE ad_units_id_seq OWNED BY ad_units_backup.id;


--
-- Name: ad_units; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE ad_units (
    id integer DEFAULT nextval('ad_units_id_seq'::regclass) NOT NULL,
    code character varying(255) NOT NULL,
    bid_floor double precision NOT NULL,
    app_id integer NOT NULL,
    account_id integer NOT NULL,
    account_type character varying(255) NOT NULL,
    ad_type integer NOT NULL,
    format integer NOT NULL,
    width integer NOT NULL,
    height integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    is_landscape boolean DEFAULT false,
    bid_floor_local double precision,
    bid_floor_currency character varying(255) DEFAULT 'USD'::character varying NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    extra json DEFAULT '{}'::json NOT NULL,
    orientation integer DEFAULT 0 NOT NULL,
    device_type integer DEFAULT 0 NOT NULL,
    package_name character varying,
    campaign_id integer,
    parallel_bidding boolean,
    ad_unit_template_id integer,
    country_ids_array integer[],
    dcc boolean DEFAULT false NOT NULL,
    manual_ecpm boolean DEFAULT false NOT NULL,
    campaign_frozen boolean DEFAULT false NOT NULL,
    enabled boolean DEFAULT true NOT NULL,
    label character varying
);


--
-- Name: adcolony_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE adcolony_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    api_key character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: adcolony_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE adcolony_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: adcolony_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE adcolony_accounts_id_seq OWNED BY adcolony_accounts.id;


--
-- Name: additional_user_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE additional_user_logs (
    id integer NOT NULL,
    user_id integer,
    fields json,
    active boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    delay_interval integer DEFAULT 7 NOT NULL,
    for_appsflyer boolean DEFAULT false NOT NULL
);


--
-- Name: additional_user_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE additional_user_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: additional_user_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE additional_user_logs_id_seq OWNED BY additional_user_logs.id;


--
-- Name: admob_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE admob_accounts (
    id integer NOT NULL,
    email character varying(255),
    client_id character varying(255),
    client_secret character varying(255),
    access_token character varying(255),
    refresh_token character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    account_id character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    is_default_white boolean DEFAULT false NOT NULL,
    password character varying(255),
    is_approve_default boolean DEFAULT false,
    owner_id integer DEFAULT 0 NOT NULL,
    deleted_at timestamp without time zone,
    invalid_credentials_count integer DEFAULT 0,
    invalid_credentials_last_date date,
    skip_reports boolean DEFAULT false
);


--
-- Name: admob_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE admob_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: admob_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE admob_accounts_id_seq OWNED BY admob_accounts.id;


--
-- Name: admob_ad_unit_versions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE admob_ad_unit_versions (
    id integer NOT NULL,
    codes character varying[] DEFAULT '{}'::character varying[],
    ad_unit_id integer NOT NULL,
    app_id integer NOT NULL,
    account_id integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: admob_ad_unit_versions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE admob_ad_unit_versions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: admob_ad_unit_versions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE admob_ad_unit_versions_id_seq OWNED BY admob_ad_unit_versions.id;


--
-- Name: admob_app_bid_floors; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE admob_app_bid_floors (
    id integer NOT NULL,
    bid_floors character varying,
    ad_type integer,
    format integer,
    device_type integer,
    app_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: admob_app_bid_floors_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE admob_app_bid_floors_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: admob_app_bid_floors_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE admob_app_bid_floors_id_seq OWNED BY admob_app_bid_floors.id;


--
-- Name: admob_user_bid_floors; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE admob_user_bid_floors (
    id integer NOT NULL,
    bid_floors character varying(255),
    ad_type integer,
    format integer,
    device_type integer,
    user_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: admob_user_bid_floors_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE admob_user_bid_floors_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: admob_user_bid_floors_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE admob_user_bid_floors_id_seq OWNED BY admob_user_bid_floors.id;


--
-- Name: adms; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE adms (
    id integer NOT NULL,
    complain_id integer,
    content text,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: adms_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE adms_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: adms_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE adms_id_seq OWNED BY adms.id;


--
-- Name: advertisement_partners; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE advertisement_partners (
    id integer NOT NULL,
    name character varying(255),
    share_multiplier double precision,
    syscampaign_prefix character varying(255),
    campaigns_type integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    user_id integer,
    default_fee double precision DEFAULT 0 NOT NULL
);


--
-- Name: advertisement_partners_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE advertisement_partners_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: advertisement_partners_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE advertisement_partners_id_seq OWNED BY advertisement_partners.id;


--
-- Name: advertisement_partners_users; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE advertisement_partners_users (
    user_id integer NOT NULL,
    advertisement_partner_id integer NOT NULL
);


--
-- Name: ae_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE ae_settings (
    id integer NOT NULL,
    name character varying NOT NULL,
    ad_type integer,
    cpm_type integer,
    app_ids_array integer[] DEFAULT '{}'::integer[] NOT NULL,
    app_ids_include boolean,
    enabled boolean,
    probability integer,
    delta double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    pricefloors double precision[]
);


--
-- Name: ae_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE ae_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ae_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE ae_settings_id_seq OWNED BY ae_settings.id;


--
-- Name: amazon_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE amazon_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    owner_id integer DEFAULT 0 NOT NULL,
    is_for_kids boolean
);


--
-- Name: amazon_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE amazon_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: amazon_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE amazon_accounts_id_seq OWNED BY amazon_accounts.id;


--
-- Name: api_queues; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE api_queues (
    id integer NOT NULL,
    user_id integer,
    params text,
    filename character varying(255),
    endpoint character varying(255),
    api_version character varying(255),
    status character varying(2),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: api_queues_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE api_queues_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: api_queues_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE api_queues_id_seq OWNED BY api_queues.id;


--
-- Name: app_fraud_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE app_fraud_logs (
    id integer NOT NULL,
    app_id integer,
    message text DEFAULT ''::text,
    fraud_type character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: app_fraud_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE app_fraud_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: app_fraud_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE app_fraud_logs_id_seq OWNED BY app_fraud_logs.id;


--
-- Name: app_frauds; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE app_frauds (
    id integer NOT NULL,
    ip_fraud integer DEFAULT 0,
    ecpm_fraud integer DEFAULT 0,
    ctr_fraud integer DEFAULT 0,
    app_package_fraud integer DEFAULT 0,
    googleplay_fraud integer DEFAULT 0,
    notification_sent integer DEFAULT 0,
    requests_count integer DEFAULT 0,
    app_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    log_message text DEFAULT ''::character varying
);


--
-- Name: app_frauds_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE app_frauds_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: app_frauds_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE app_frauds_id_seq OWNED BY app_frauds.id;


--
-- Name: app_installer_data; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE app_installer_data (
    id integer NOT NULL,
    app_id integer,
    total_impressions_count integer,
    impressions_count integer,
    installer_type character varying,
    report_date date,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: app_installer_data_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE app_installer_data_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: app_installer_data_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE app_installer_data_id_seq OWNED BY app_installer_data.id;


--
-- Name: app_network_profiles; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE app_network_profiles (
    id integer NOT NULL,
    network integer NOT NULL,
    app_id integer NOT NULL,
    data jsonb DEFAULT '{}'::jsonb,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL,
    account_id integer
);


--
-- Name: app_network_profiles_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE app_network_profiles_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: app_network_profiles_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE app_network_profiles_id_seq OWNED BY app_network_profiles.id;


--
-- Name: app_revenue_shares; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE app_revenue_shares (
    id integer NOT NULL,
    master_app_id integer,
    slave_app_id integer,
    share double precision DEFAULT 0.0,
    start timestamp without time zone,
    stop timestamp without time zone,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    slave_owner_id integer
);


--
-- Name: app_revenue_shares_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE app_revenue_shares_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: app_revenue_shares_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE app_revenue_shares_id_seq OWNED BY app_revenue_shares.id;


--
-- Name: appflood_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE appflood_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    is_default boolean DEFAULT false NOT NULL
);


--
-- Name: appflood_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE appflood_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: appflood_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE appflood_accounts_id_seq OWNED BY appflood_accounts.id;


--
-- Name: applovin_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE applovin_accounts (
    id integer NOT NULL,
    email character varying(255),
    api_key character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    is_default boolean DEFAULT false NOT NULL,
    password character varying(255),
    is_approve_default boolean DEFAULT false,
    owner_id integer DEFAULT 0 NOT NULL,
    deleted_at timestamp without time zone,
    sdk_key character varying(255),
    ad_type integer DEFAULT 0,
    default_app_id character varying(255),
    frequency_capping integer
);


--
-- Name: applovin_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE applovin_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: applovin_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE applovin_accounts_id_seq OWNED BY applovin_accounts.id;


--
-- Name: applovin_zones; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE applovin_zones (
    id integer NOT NULL,
    platform_id integer NOT NULL,
    bid_floor double precision DEFAULT 0.0 NOT NULL,
    account_id integer NOT NULL,
    ad_type integer NOT NULL,
    zone_id character varying DEFAULT ''::character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    parallel_bidding boolean DEFAULT false NOT NULL
);


--
-- Name: applovin_zones_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE applovin_zones_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: applovin_zones_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE applovin_zones_id_seq OWNED BY applovin_zones.id;


--
-- Name: appnexus_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE appnexus_accounts (
    id integer NOT NULL,
    username character varying(255),
    password character varying(255),
    token character varying(32),
    token_updated_at timestamp without time zone,
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    publisher_id character varying(255)
);


--
-- Name: appnexus_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE appnexus_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: appnexus_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE appnexus_accounts_id_seq OWNED BY appnexus_accounts.id;


--
-- Name: apps; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE apps (
    id integer NOT NULL,
    user_id integer,
    platform_id integer,
    name character varying(255),
    package_name character varying(255),
    app_key character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    chartboost_id character varying(255),
    chartboost_signature character varying(255),
    applovin_enabled boolean,
    openx_uid character varying(255),
    orientation character varying(15) DEFAULT 'both'::character varying NOT NULL,
    is_deleted boolean DEFAULT false,
    mopub_app_key character varying(255),
    admob_image character varying(255),
    admob_text character varying(255),
    admob_app_id character varying(255),
    sdk_version character varying(255),
    is_game boolean,
    applovin_app_id character varying(255),
    is_approve boolean DEFAULT false,
    amazon_key character varying(255),
    ip_fraud integer DEFAULT 0,
    app_package_fraud integer DEFAULT 0,
    googleplay_fraud integer DEFAULT 0,
    ecpm_fraud integer DEFAULT 0,
    ctr_fraud integer DEFAULT 0,
    first_1000_requests_reached_at date,
    fraud_approved boolean DEFAULT false,
    packagemistake_message_sent boolean DEFAULT false,
    adcolony_app_id character varying(50),
    first_package_fraud_detected_on timestamp without time zone,
    applovin_sdk_key character varying(255),
    unity_ads_id character varying(50),
    vungle_app_id character varying(50),
    facebook_key character varying(50),
    views_reached integer DEFAULT 0,
    facebook_app_id bigint,
    state character varying NOT NULL,
    is_for_kids boolean DEFAULT false,
    banned_in_admob boolean DEFAULT false,
    inmobi_id character varying(255),
    applovin_video_sdk_key character varying(255),
    mopub_video_app_key character varying(255),
    used_types integer DEFAULT 0,
    framework character varying(255),
    facebook_app_secret character varying(255),
    target_pad_group_id integer,
    video_wo_banners boolean DEFAULT false,
    video_skippable boolean DEFAULT false,
    interstitials_disabled boolean DEFAULT false,
    video_disabled boolean DEFAULT false,
    banners_disabled boolean DEFAULT false,
    active_types integer DEFAULT 0,
    admob_plugin_disabled boolean DEFAULT false,
    yandex_metrica_id integer,
    adcolony_id character varying(10),
    startapp_app_id character varying(255),
    test_mode boolean DEFAULT false,
    yandex_new_metrica_id character varying(255),
    reward_amount integer,
    reward_name character varying(255),
    inmobi_key character varying(255),
    banner_refresh_period integer DEFAULT 15,
    flurry_key character varying(50),
    flurry_space character varying(50),
    pubnative_key character varying(100),
    avocarrot_app_key character varying(100),
    video_views_reached integer DEFAULT 0,
    inmobi_old boolean DEFAULT false,
    video_fraud_detected boolean,
    disable_rtb boolean DEFAULT false NOT NULL,
    pubnative_id character varying(255),
    flurry_app_id integer,
    traffic_limit boolean,
    traffic_limit_notified boolean DEFAULT false,
    mopub_rewarded_app_key character varying(50),
    flurry_space_id integer,
    applovin_video_five_capped_sdk_key character varying(255),
    applovin_video_uncapped_sdk_key character varying(255),
    applovin_uncapped_app_id character varying(255),
    applovin_five_capped_app_id character varying(255),
    yandex_app_id character varying(255),
    yandex_partner_id integer,
    banned_in_store boolean DEFAULT false NOT NULL,
    test_mode_type integer DEFAULT 0 NOT NULL,
    reward_url character varying(255),
    reward_key character varying(255),
    yandex_enabled boolean DEFAULT false,
    afd boolean DEFAULT false NOT NULL,
    max_duration integer DEFAULT 0 NOT NULL,
    disable_long_video boolean DEFAULT false NOT NULL,
    video_native_autostart boolean DEFAULT true NOT NULL,
    yandex_created_at timestamp without time zone,
    traffic_limit_count integer DEFAULT 2000,
    ads_settings json DEFAULT '{}'::json NOT NULL,
    device_type character varying(255) DEFAULT '1,2'::character varying NOT NULL,
    yandex_app_model_id character varying(255),
    statuses_updated_at timestamp without time zone,
    statuses_updated_jid character varying(255),
    rtb_black_list text,
    filter_mature_content boolean,
    coppa boolean,
    tapjoy_data character varying(500),
    unity_ads_project_id character varying(255),
    unity_ads_platform_id character varying(255),
    probably_banned_in_admob boolean DEFAULT false NOT NULL,
    native_disabled boolean DEFAULT false,
    mrec_disabled boolean DEFAULT false,
    main_category_id integer,
    iron_source_data character varying(255),
    domain character varying(255),
    keywords character varying(255),
    package_version character varying(255),
    ext json,
    privacy_policy boolean,
    fyber_app_id character varying(255),
    deleted boolean DEFAULT false,
    badv character varying,
    interstitial_type integer DEFAULT 0,
    lostlastdate integer DEFAULT 0,
    rewarded_video_disabled boolean DEFAULT false,
    analyze_ad_units boolean DEFAULT true NOT NULL,
    store_reviews_control boolean DEFAULT false,
    google_dfp_key character varying(255),
    applovin_one_capped_app_id character varying(255),
    share_user_only boolean,
    ogury_app_key character varying(255),
    smaato_app_id character varying(255),
    with_big_revenue boolean DEFAULT false NOT NULL,
    is_corona boolean DEFAULT false NOT NULL,
    mobvista_app_id character varying(255),
    app_settings jsonb DEFAULT '{}'::jsonb,
    is_game_by_category boolean,
    aol_app_id character varying,
    appodealx_sync boolean DEFAULT true,
    has_enough_views boolean,
    coppa_exclusions integer[] DEFAULT '{}'::integer[],
    store_state character varying,
    displayio_app_id integer,
    fmc_exclusions integer[] DEFAULT '{}'::integer[],
    state_by_traffic character varying,
    inner_active_app_id character varying
);


--
-- Name: apps_campaign_images; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE apps_campaign_images (
    id integer NOT NULL,
    campaign_image_id integer,
    app_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
)
WITH (autovacuum_enabled='true', autovacuum_vacuum_scale_factor='0.01', autovacuum_analyze_scale_factor='0.005');


--
-- Name: apps_campaign_images_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE apps_campaign_images_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: apps_campaign_images_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE apps_campaign_images_id_seq OWNED BY apps_campaign_images.id;


--
-- Name: apps_categories; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE apps_categories (
    id integer NOT NULL,
    app_id integer,
    category_id integer
);


--
-- Name: apps_categories_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE apps_categories_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: apps_categories_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE apps_categories_id_seq OWNED BY apps_categories.id;


--
-- Name: apps_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE apps_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: apps_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE apps_id_seq OWNED BY apps.id;


--
-- Name: apps_network_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE apps_network_accounts (
    id integer NOT NULL,
    app_id integer,
    network_account_id integer,
    method_name character varying,
    network integer,
    ad_type integer
);


--
-- Name: apps_network_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE apps_network_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: apps_network_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE apps_network_accounts_id_seq OWNED BY apps_network_accounts.id;


--
-- Name: apps_website_links; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE apps_website_links (
    id integer NOT NULL,
    initial_file character varying,
    parsed_file character varying,
    platform character varying,
    updated_at timestamp without time zone,
    created_at timestamp without time zone
);


--
-- Name: apps_website_links_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE apps_website_links_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: apps_website_links_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE apps_website_links_id_seq OWNED BY apps_website_links.id;


--
-- Name: avocarrot_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE avocarrot_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    api_key character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    publisher_id character varying(255),
    client_id character varying(255),
    client_secret character varying(255)
);


--
-- Name: avocarrot_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE avocarrot_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: avocarrot_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE avocarrot_accounts_id_seq OWNED BY avocarrot_accounts.id;


--
-- Name: bad_devices; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bad_devices (
    id integer NOT NULL,
    android_id character varying(255),
    fraud_score integer,
    banned_at timestamp without time zone,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    blocked boolean DEFAULT false,
    blocked_at timestamp without time zone,
    ban_reason character varying(255),
    last_ip character varying(255),
    last_ua character varying(255),
    banned boolean DEFAULT false
);


--
-- Name: bad_devices_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE bad_devices_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: bad_devices_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE bad_devices_id_seq OWNED BY bad_devices.id;


--
-- Name: bid_floor_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bid_floor_settings (
    id integer NOT NULL,
    account_type character varying(60) NOT NULL,
    bid_floors character varying(255) NOT NULL,
    ad_type integer NOT NULL,
    format integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    device_type integer DEFAULT 0 NOT NULL,
    pb_bid_floors double precision[] DEFAULT '{}'::double precision[],
    campaign_type_id integer
);


--
-- Name: bid_floor_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE bid_floor_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: bid_floor_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE bid_floor_settings_id_seq OWNED BY bid_floor_settings.id;


--
-- Name: bidder_bidder_formats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bidder_bidder_formats (
    id integer NOT NULL,
    bidder_id integer,
    bidder_format_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: bidder_bidder_formats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE bidder_bidder_formats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: bidder_bidder_formats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE bidder_bidder_formats_id_seq OWNED BY bidder_bidder_formats.id;


--
-- Name: bidder_campaigns; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bidder_campaigns (
    id integer NOT NULL,
    bidder_id integer,
    campaign_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: bidder_campaigns_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE bidder_campaigns_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: bidder_campaigns_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE bidder_campaigns_id_seq OWNED BY bidder_campaigns.id;


--
-- Name: bidder_countries; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bidder_countries (
    id integer NOT NULL,
    bidder_id integer,
    country_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: bidder_countries_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE bidder_countries_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: bidder_countries_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE bidder_countries_id_seq OWNED BY bidder_countries.id;


--
-- Name: bidder_formats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bidder_formats (
    id integer NOT NULL,
    title character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: bidder_formats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE bidder_formats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: bidder_formats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE bidder_formats_id_seq OWNED BY bidder_formats.id;


--
-- Name: bidder_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bidder_settings (
    id integer NOT NULL,
    bidder_id integer,
    setting_name character varying(255),
    setting_value character varying(255)
);


--
-- Name: bidder_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE bidder_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: bidder_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE bidder_settings_id_seq OWNED BY bidder_settings.id;


--
-- Name: bids; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE bids (
    date date NOT NULL,
    bidder_id integer NOT NULL,
    bid_count integer DEFAULT 0,
    country_id integer NOT NULL,
    win_count integer DEFAULT 0,
    ad_type integer NOT NULL,
    platform_id integer NOT NULL,
    click_count integer DEFAULT 0,
    app_id integer NOT NULL
);


--
-- Name: billings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings (
    id integer NOT NULL,
    user_id integer,
    amount double precision DEFAULT 0.0,
    network_type integer,
    record_type integer,
    comment text,
    refs text,
    source integer,
    date date,
    balance double precision DEFAULT 0.0,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    fee numeric DEFAULT 0.0
);


--
-- Name: billings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE billings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: billings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE billings_id_seq OWNED BY billings.id;


--
-- Name: billings_y2016m10; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2016m10 (
    CONSTRAINT billings_y2016m10_date_check CHECK (((date >= '2016-10-01'::date) AND (date < '2016-11-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2016m11; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2016m11 (
    CONSTRAINT billings_y2016m11_date_check CHECK (((date >= '2016-11-01'::date) AND (date < '2016-12-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2016m12; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2016m12 (
    CONSTRAINT billings_y2016m12_date_check CHECK (((date >= '2016-12-01'::date) AND (date < '2017-01-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m01; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m01 (
    CONSTRAINT billings_y2017m01_date_check CHECK (((date >= '2017-01-01'::date) AND (date < '2017-02-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m02; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m02 (
    CONSTRAINT billings_y2017m02_date_check CHECK (((date >= '2017-02-01'::date) AND (date < '2017-03-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m03; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m03 (
    CONSTRAINT billings_y2017m03_date_check CHECK (((date >= '2017-03-01'::date) AND (date < '2017-04-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m04; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m04 (
    CONSTRAINT billings_y2017m04_date_check CHECK (((date >= '2017-04-01'::date) AND (date < '2017-05-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m05; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m05 (
    CONSTRAINT billings_y2017m05_date_check CHECK (((date >= '2017-05-01'::date) AND (date < '2017-06-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m06; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m06 (
    CONSTRAINT billings_y2017m06_date_check CHECK (((date >= '2017-06-01'::date) AND (date < '2017-07-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m07; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m07 (
    CONSTRAINT billings_y2017m07_date_check CHECK (((date >= '2017-07-01'::date) AND (date < '2017-08-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m08; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m08 (
    CONSTRAINT billings_y2017m08_date_check CHECK (((date >= '2017-08-01'::date) AND (date < '2017-09-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m09; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m09 (
    CONSTRAINT billings_y2017m09_date_check CHECK (((date >= '2017-09-01'::date) AND (date < '2017-10-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m10; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m10 (
    CONSTRAINT billings_y2017m10_date_check CHECK (((date >= '2017-10-01'::date) AND (date < '2017-11-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m11; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m11 (
    CONSTRAINT billings_y2017m11_date_check CHECK (((date >= '2017-11-01'::date) AND (date < '2017-12-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2017m12; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2017m12 (
    CONSTRAINT billings_y2017m12_date_check CHECK (((date >= '2017-12-01'::date) AND (date < '2018-01-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m01; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m01 (
    CONSTRAINT billings_y2018m01_date_check CHECK (((date >= '2018-01-01'::date) AND (date < '2018-02-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m02; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m02 (
    CONSTRAINT billings_y2018m02_date_check CHECK (((date >= '2018-02-01'::date) AND (date < '2018-03-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m03; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m03 (
    CONSTRAINT billings_y2018m03_date_check CHECK (((date >= '2018-03-01'::date) AND (date < '2018-04-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m04; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m04 (
    CONSTRAINT billings_y2018m04_date_check CHECK (((date >= '2018-04-01'::date) AND (date < '2018-05-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m05; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m05 (
    CONSTRAINT billings_y2018m05_date_check CHECK (((date >= '2018-05-01'::date) AND (date < '2018-06-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m06; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m06 (
    CONSTRAINT billings_y2018m06_date_check CHECK (((date >= '2018-06-01'::date) AND (date < '2018-07-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m07; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m07 (
    CONSTRAINT billings_y2018m07_date_check CHECK (((date >= '2018-07-01'::date) AND (date < '2018-08-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m08; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m08 (
    CONSTRAINT billings_y2018m08_date_check CHECK (((date >= '2018-08-01'::date) AND (date < '2018-09-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m09; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m09 (
    CONSTRAINT billings_y2018m09_date_check CHECK (((date >= '2018-09-01'::date) AND (date < '2018-10-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m10; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m10 (
    CONSTRAINT billings_y2018m10_date_check CHECK (((date >= '2018-10-01'::date) AND (date < '2018-11-01'::date)))
)
INHERITS (billings);


--
-- Name: billings_y2018m11; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE billings_y2018m11 (
    CONSTRAINT billings_y2018m11_date_check CHECK (((date >= '2018-11-01'::date) AND (date < '2018-12-01'::date)))
)
INHERITS (billings);


--
-- Name: block_devices; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE block_devices (
    id integer NOT NULL,
    device_id character varying NOT NULL,
    ip character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: block_devices_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE block_devices_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: block_devices_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE block_devices_id_seq OWNED BY block_devices.id;


--
-- Name: book_keeping_reports; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE book_keeping_reports (
    id integer NOT NULL,
    date date,
    user_sum double precision,
    appodeal_sum double precision,
    instant_payments_sum double precision,
    instant_payments_fee double precision,
    regular_payments_sum double precision,
    invalid_activity_sum double precision,
    blocked_accounts_sum double precision,
    bonuses_sum double precision,
    affiliate_commission double precision,
    sales_commission double precision,
    user_payable double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: book_keeping_reports_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE book_keeping_reports_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: book_keeping_reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE book_keeping_reports_id_seq OWNED BY book_keeping_reports.id;


--
-- Name: cached_files; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE cached_files (
    id integer NOT NULL,
    file character varying(255),
    marker character varying(255),
    name character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    type character varying(255),
    date date,
    account_id integer,
    account_type character varying(255),
    finalized boolean DEFAULT true NOT NULL,
    revenue double precision DEFAULT 0.0 NOT NULL
);


--
-- Name: cached_files_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE cached_files_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: cached_files_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE cached_files_id_seq OWNED BY cached_files.id;


--
-- Name: campaign_aggregated_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_aggregated_stats (
    id integer NOT NULL,
    revenue double precision DEFAULT 0.0 NOT NULL,
    impressions integer DEFAULT 0 NOT NULL,
    campaign_id integer NOT NULL,
    country_id integer NOT NULL,
    app_id integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: campaign_aggregated_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_aggregated_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_aggregated_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_aggregated_stats_id_seq OWNED BY campaign_aggregated_stats.id;


--
-- Name: campaign_commissions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_commissions (
    id integer NOT NULL,
    campaign_global_id integer,
    commission double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    ad_type integer,
    active boolean DEFAULT false NOT NULL,
    user_id integer,
    app_id integer,
    user_only integer,
    started_at date DEFAULT '2014-01-01'::date
);


--
-- Name: campaign_commissions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_commissions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_commissions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_commissions_id_seq OWNED BY campaign_commissions.id;


--
-- Name: campaign_coordinates; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_coordinates (
    id integer NOT NULL,
    latitude double precision,
    longitude double precision,
    address character varying(255),
    radius integer,
    country_id integer,
    spot geography(Polygon,4326),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: campaign_coordinates_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_coordinates_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_coordinates_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_coordinates_id_seq OWNED BY campaign_coordinates.id;


--
-- Name: campaign_image_coordinates; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_image_coordinates (
    id integer NOT NULL,
    campaign_image_id integer,
    latitude double precision,
    longitude double precision,
    address character varying(255),
    radius integer,
    country_id integer
);


--
-- Name: campaign_image_coordinates_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_image_coordinates_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_image_coordinates_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_image_coordinates_id_seq OWNED BY campaign_image_coordinates.id;


--
-- Name: campaign_image_coordinates_images; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_image_coordinates_images (
    id integer NOT NULL,
    campaign_image_id integer,
    campaign_image_coordinate_id integer
);


--
-- Name: campaign_image_coordinates_images_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_image_coordinates_images_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_image_coordinates_images_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_image_coordinates_images_id_seq OWNED BY campaign_image_coordinates_images.id;


--
-- Name: campaign_image_devices; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_image_devices (
    id integer NOT NULL,
    campaign_image_id integer,
    device_id character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: campaign_image_devices_csvs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_image_devices_csvs (
    id integer NOT NULL,
    campaign_image_id integer,
    file character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    lines_amount integer,
    no_parse boolean DEFAULT false
);


--
-- Name: campaign_image_devices_csvs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_image_devices_csvs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_image_devices_csvs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_image_devices_csvs_id_seq OWNED BY campaign_image_devices_csvs.id;


--
-- Name: campaign_image_devices_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_image_devices_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_image_devices_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_image_devices_id_seq OWNED BY campaign_image_devices.id;


--
-- Name: campaign_images; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_images (
    id integer NOT NULL,
    campaign_id integer,
    type integer,
    image_file_name character varying(255),
    image_content_type character varying(255),
    image_file_size integer,
    image_updated_at timestamp without time zone,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    platform_id integer,
    width integer,
    height integer,
    html text,
    impression_pixel text,
    bid_floor double precision,
    is_backfill boolean DEFAULT false NOT NULL,
    interests_targeting text,
    has_apps_targeting boolean DEFAULT false,
    min_android_version integer DEFAULT 0,
    max_android_version integer DEFAULT 999,
    min_ios_version integer DEFAULT 0,
    max_ios_version integer DEFAULT 999,
    schedule bit(168) DEFAULT B'111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'::bit(168),
    has_device_targeting boolean DEFAULT false,
    has_models_targeting boolean DEFAULT false,
    has_models_exclusion_targeting boolean DEFAULT false,
    has_mobile_network_targeting boolean DEFAULT false,
    deleted boolean DEFAULT false,
    frequency_capping_value integer DEFAULT 0,
    frequency_capping_blocked boolean DEFAULT false,
    connection_type integer DEFAULT 0,
    has_android_sdk_targeting boolean DEFAULT false,
    has_ios_sdk_targeting boolean DEFAULT false,
    android_sdk_version_min bigint,
    android_sdk_version_max bigint,
    ios_sdk_version_min bigint,
    ios_sdk_version_max bigint,
    bid_floor_local double precision,
    bid_floor_currency character varying(255) DEFAULT 'USD'::character varying NOT NULL,
    gender integer DEFAULT 0,
    html_as_json boolean,
    html_macroses_enabled boolean DEFAULT false,
    is_rewarded_video_only boolean DEFAULT false,
    device_type integer DEFAULT 0,
    has_negative_keywords_targeting boolean DEFAULT false,
    negative_keywords character varying(255)[] DEFAULT '{}'::character varying[],
    has_auto_ecpm boolean DEFAULT false,
    priority_finish_at timestamp without time zone,
    priority_app_ids integer[] DEFAULT '{}'::integer[],
    system_locale_targeting integer,
    orientation integer,
    title character varying(255),
    device_model_ids integer[] DEFAULT '{}'::integer[] NOT NULL,
    min_age integer DEFAULT 0 NOT NULL,
    max_age integer DEFAULT 120 NOT NULL,
    tag boolean DEFAULT false NOT NULL,
    base_url character varying(255),
    creative_id integer,
    creative_type character varying
)
WITH (autovacuum_enabled='true', autovacuum_vacuum_scale_factor='0.001', autovacuum_analyze_scale_factor='0.0005', autovacuum_vacuum_cost_delay='5', autovacuum_vacuum_cost_limit='20', autovacuum_analyze_threshold='5', autovacuum_vacuum_threshold='10');


--
-- Name: campaign_images_categories; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_images_categories (
    id integer NOT NULL,
    campaign_image_id integer,
    category_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: campaign_images_categories_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_images_categories_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_images_categories_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_images_categories_id_seq OWNED BY campaign_images_categories.id;


--
-- Name: campaign_images_countries; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_images_countries (
    id integer NOT NULL,
    campaign_image_id integer,
    country_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: campaign_images_countries_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_images_countries_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_images_countries_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_images_countries_id_seq OWNED BY campaign_images_countries.id;


--
-- Name: campaign_images_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_images_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_images_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_images_id_seq OWNED BY campaign_images.id;


--
-- Name: campaign_images_max_mind_cities; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_images_max_mind_cities (
    id integer NOT NULL,
    campaign_image_id integer,
    max_mind_city_id integer
);


--
-- Name: campaign_images_max_mind_cities_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_images_max_mind_cities_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_images_max_mind_cities_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_images_max_mind_cities_id_seq OWNED BY campaign_images_max_mind_cities.id;


--
-- Name: campaign_images_mobile_networks; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_images_mobile_networks (
    campaign_image_id integer NOT NULL,
    mobile_network_id integer NOT NULL
);


--
-- Name: campaign_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_settings (
    id integer NOT NULL,
    campaign_id integer,
    app_inclusion boolean DEFAULT true NOT NULL,
    activated_at timestamp without time zone,
    country_inclusion boolean DEFAULT true
);


--
-- Name: campaign_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_settings_id_seq OWNED BY campaign_settings.id;


--
-- Name: campaign_types; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_types (
    id integer NOT NULL,
    human_name character varying(255),
    sys_package_name character varying(255),
    api_name character varying(255),
    network_model integer,
    is_fee_campaign boolean DEFAULT false,
    show_at_frontend boolean,
    partner_type integer DEFAULT 0,
    partner_id integer DEFAULT 0,
    system_campaign_id integer DEFAULT 0,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    deprecated boolean DEFAULT false NOT NULL,
    ad_source_type integer DEFAULT 0,
    bidding_mode bit(2) DEFAULT B'10'::"bit" NOT NULL
);


--
-- Name: campaign_types_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_types_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_types_id_seq OWNED BY campaign_types.id;


--
-- Name: campaign_versions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaign_versions (
    id integer NOT NULL,
    item_type character varying NOT NULL,
    item_id integer NOT NULL,
    event character varying NOT NULL,
    whodunnit character varying,
    object jsonb,
    object_changes jsonb,
    targeting_changes jsonb,
    created_at timestamp without time zone,
    form_changes jsonb DEFAULT '{}'::jsonb
);


--
-- Name: campaign_versions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaign_versions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaign_versions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaign_versions_id_seq OWNED BY campaign_versions.id;


--
-- Name: campaigns; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaigns (
    id integer NOT NULL,
    name character varying(255),
    package_name character varying(255),
    title character varying(255),
    description character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    active boolean DEFAULT true,
    is_network boolean DEFAULT false NOT NULL,
    cpm double precision,
    cpa double precision,
    global_type integer,
    is_bidder boolean DEFAULT false,
    cpc double precision,
    user_id integer,
    payout numeric,
    deleted boolean DEFAULT false,
    capped_at timestamp without time zone,
    frequency_capping_value integer DEFAULT 0,
    frequency_capping_type integer DEFAULT 0,
    frequency_capping_period integer DEFAULT 1,
    network_model integer,
    loss_limit double precision,
    appodeal_is_paid_for integer DEFAULT 0,
    is_partner_campaign boolean DEFAULT false,
    appodeal_share double precision DEFAULT 1,
    start_date timestamp without time zone,
    end_date timestamp without time zone,
    max_priority boolean DEFAULT false,
    frequency_capping_per_user_value integer DEFAULT 1,
    frequency_capping_per_user_interval integer DEFAULT 1,
    frequency_capping_per_user_period integer DEFAULT 2,
    frequency_capping_per_user_type integer DEFAULT 2,
    frequency_capping_time_span integer DEFAULT 0,
    frequency_capping_per_session_value integer DEFAULT 0,
    partner_id integer DEFAULT 0,
    coppa_complain boolean DEFAULT false,
    bundle_id character varying(255),
    second_price_disabled boolean DEFAULT false,
    top boolean DEFAULT false,
    filter_mature_content_complain boolean,
    prediction_factor double precision,
    predictor_type character varying(255),
    trained_at timestamp without time zone,
    prediction boolean,
    form_data json,
    payment_model integer,
    ecpm_flag character varying(255) DEFAULT 'manual_ecpm_tab'::character varying NOT NULL,
    loss_limit_achieved boolean DEFAULT false NOT NULL,
    stop_after_install boolean DEFAULT false,
    stop_after_click boolean DEFAULT false,
    is_old_cross_promo boolean DEFAULT false NOT NULL,
    click_url text,
    app_url character varying(255),
    native_waterfall_type integer DEFAULT 1,
    target_locales integer[],
    app_ids_array integer[] DEFAULT '{}'::integer[] NOT NULL,
    app_ids_include boolean,
    country_ids_array integer[],
    max_mind_city_ids_array integer[],
    coordinate_ids_array integer[],
    frequency_capping_blocked boolean DEFAULT false NOT NULL,
    frequency_capping_total_max integer,
    test_ad_ids uuid[],
    dsp boolean DEFAULT false,
    parallel_bidding boolean DEFAULT false,
    target_placements character varying[],
    only_in_placements boolean,
    bidding_mode bit(2) DEFAULT B'10'::"bit" NOT NULL,
    impression_url text,
    target_placement_ids integer[]
)
WITH (autovacuum_enabled='true', autovacuum_vacuum_scale_factor='0.01', autovacuum_analyze_scale_factor='0.005');


--
-- Name: campaigns_creative_sets; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE campaigns_creative_sets (
    id integer NOT NULL,
    campaign_id integer NOT NULL,
    creative_set_id integer NOT NULL,
    campaign_image_ids integer[] DEFAULT '{}'::integer[] NOT NULL
);


--
-- Name: campaigns_creative_sets_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaigns_creative_sets_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaigns_creative_sets_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaigns_creative_sets_id_seq OWNED BY campaigns_creative_sets.id;


--
-- Name: campaigns_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE campaigns_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: campaigns_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE campaigns_id_seq OWNED BY campaigns.id;


--
-- Name: categories; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE categories (
    id integer NOT NULL,
    human_name character varying(255),
    store_name character varying(255) NOT NULL,
    platform_id integer DEFAULT 1 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    iab character varying(255),
    iab_name character varying(255)
);


--
-- Name: categories_contents; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE categories_contents (
    id integer NOT NULL,
    category_id integer,
    content_id integer
);


--
-- Name: categories_contents_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE categories_contents_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: categories_contents_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE categories_contents_id_seq OWNED BY categories_contents.id;


--
-- Name: categories_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE categories_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: categories_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE categories_id_seq OWNED BY categories.id;


--
-- Name: categories_users; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE categories_users (
    id integer NOT NULL,
    category_id integer,
    user_id integer
);


--
-- Name: categories_users_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE categories_users_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: categories_users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE categories_users_id_seq OWNED BY categories_users.id;


--
-- Name: changelogs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE changelogs (
    id integer NOT NULL,
    revision character varying NOT NULL,
    published_at date NOT NULL,
    revision_mark character varying,
    summary text,
    type character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    summary_ru text
);


--
-- Name: changelogs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE changelogs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: changelogs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE changelogs_id_seq OWNED BY changelogs.id;


--
-- Name: chartboost_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE chartboost_accounts (
    id integer NOT NULL,
    email character varying(255),
    user_id character varying(255),
    user_signature character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    chartboost_password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    is_default_white boolean DEFAULT false NOT NULL,
    is_approve_default boolean DEFAULT false,
    last_report_date date,
    owner_id integer DEFAULT 0 NOT NULL,
    deleted_at timestamp without time zone,
    is_syncing boolean DEFAULT false
);


--
-- Name: chartboost_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE chartboost_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: chartboost_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE chartboost_accounts_id_seq OWNED BY chartboost_accounts.id;


--
-- Name: chartboost_campaigns; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE chartboost_campaigns (
    id integer NOT NULL,
    app_id integer,
    chartboost_account_id integer,
    remote_id character varying(255),
    priority character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    campaign_id integer,
    name character varying(255),
    real_priority character varying(255)
);


--
-- Name: chartboost_campaigns_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE chartboost_campaigns_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: chartboost_campaigns_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE chartboost_campaigns_id_seq OWNED BY chartboost_campaigns.id;


--
-- Name: cities; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE cities (
    id integer NOT NULL,
    country_id integer,
    subdivision1_id integer,
    subdivision2_id integer,
    geoname_id integer,
    name character varying(255),
    name_ru character varying(255),
    lat character varying(255),
    lon character varying(255),
    zip character varying(255),
    radius integer
);


--
-- Name: cities_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE cities_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: cities_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE cities_id_seq OWNED BY cities.id;


--
-- Name: clicks; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE clicks (
    id bigint NOT NULL,
    type integer,
    campaign_url_id integer,
    app_id integer,
    device_id integer,
    android_id character varying(255),
    aarki_id character varying(255),
    converted_at timestamp without time zone,
    payout numeric,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    campaign_image_id integer,
    country_id integer,
    white_sdk boolean DEFAULT false,
    ip character varying(255),
    x_forwarded character varying(255),
    ua character varying(255),
    user_only boolean,
    segment_id integer,
    placement_id integer,
    campaign_id integer,
    account_id integer,
    date date,
    ad_id uuid,
    uuid uuid
);


--
-- Name: clicks_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE clicks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: clicks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE clicks_id_seq OWNED BY clicks.id;


--
-- Name: clicks_master; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE clicks_master (
    id integer DEFAULT nextval('clicks_id_seq'::regclass) NOT NULL,
    type integer,
    campaign_url_id integer,
    app_id integer,
    device_id integer,
    android_id character varying(255),
    aarki_id character varying(255),
    converted_at timestamp without time zone,
    payout numeric,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    campaign_image_id integer,
    country_id integer,
    white_sdk boolean DEFAULT false,
    ip character varying(255),
    x_forwarded character varying(255),
    ua character varying(255),
    user_only boolean,
    segment_id integer,
    placement_id integer
);


--
-- Name: clicks_y2018m08; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE clicks_y2018m08 (
    CONSTRAINT clicks_y2018m08_created_at_check CHECK (((created_at >= '2018-08-01'::date) AND (created_at < '2018-09-01'::date)))
)
INHERITS (clicks);


--
-- Name: clicks_y2018m09; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE clicks_y2018m09 (
    CONSTRAINT clicks_y2018m09_created_at_check CHECK (((created_at >= '2018-09-01'::date) AND (created_at < '2018-10-01'::date)))
)
INHERITS (clicks);


--
-- Name: clicks_y2018m10; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE clicks_y2018m10 (
    CONSTRAINT clicks_y2018m10_created_at_check CHECK (((created_at >= '2018-10-01'::date) AND (created_at < '2018-11-01'::date)))
)
INHERITS (clicks);


--
-- Name: clicks_y2018m11; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE clicks_y2018m11 (
    CONSTRAINT clicks_y2018m11_created_at_check CHECK (((created_at >= '2018-11-01'::date) AND (created_at < '2018-12-01'::date)))
)
INHERITS (clicks);


--
-- Name: client_histories; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE client_histories (
    id integer NOT NULL,
    accountable_id integer,
    accountable_type character varying(255),
    app_id integer,
    ad_units_counter integer DEFAULT 0,
    aasm_state character varying(255),
    error text,
    error_backtrace text,
    date date,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: client_histories_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE client_histories_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: client_histories_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE client_histories_id_seq OWNED BY client_histories.id;


--
-- Name: client_history_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE client_history_stats (
    id integer NOT NULL,
    accountable_id integer,
    accountable_type character varying(255),
    aasm_state character varying(255),
    date date,
    ad_units_counter integer DEFAULT 0,
    success_apps integer DEFAULT 0,
    failed_apps integer DEFAULT 0,
    cancelled_apps integer DEFAULT 0,
    deleted_apps integer DEFAULT 0,
    success_ad_units integer DEFAULT 0,
    failed_ad_units integer DEFAULT 0
);


--
-- Name: client_history_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE client_history_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: client_history_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE client_history_stats_id_seq OWNED BY client_history_stats.id;


--
-- Name: compensations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE compensations (
    id integer NOT NULL,
    app_id integer NOT NULL,
    percents double precision DEFAULT 0.0 NOT NULL,
    compensation_type integer DEFAULT 0 NOT NULL,
    date_start date NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: compensations_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE compensations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: compensations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE compensations_id_seq OWNED BY compensations.id;


--
-- Name: complains; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE complains (
    id integer NOT NULL,
    campaign_image_id integer,
    "time" integer,
    os character varying(255),
    bundle character varying(255),
    sdk character varying(255),
    reason character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: complains_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE complains_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: complains_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE complains_id_seq OWNED BY complains.id;


--
-- Name: contents; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE contents (
    id integer NOT NULL,
    episode integer,
    title character varying(255),
    series character varying(255),
    season character varying(255),
    url character varying(255),
    videoquality integer,
    context integer,
    content_rating character varying(255),
    user_rating character varying(255),
    qag_media_rating integer,
    keywords character varying(255),
    livestream integer,
    source_relationship integer,
    len integer,
    language character varying(255),
    embeddable integer,
    ext json,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    user_id integer,
    app_id integer
);


--
-- Name: contents_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE contents_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: contents_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE contents_id_seq OWNED BY contents.id;


--
-- Name: countries; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE countries (
    id integer NOT NULL,
    code character varying(255),
    name character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    name_ru character varying(255) DEFAULT ''::character varying NOT NULL,
    continent_code character varying(2) DEFAULT ''::character varying NOT NULL,
    continent_name_en character varying(255),
    continent_name_ru character varying(255),
    latitude double precision,
    longitude double precision,
    alpha_3_code character varying,
    alternatives character varying
);


--
-- Name: countries_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE countries_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: countries_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE countries_id_seq OWNED BY countries.id;


--
-- Name: creative_files; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE creative_files (
    id integer NOT NULL,
    creative_file_name character varying(255),
    creative_content_type character varying(255),
    creative_file_size character varying(255),
    width integer,
    height integer,
    creative_set_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    file_type integer,
    duration integer,
    bitrate integer,
    codec_name integer
);


--
-- Name: creative_files_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE creative_files_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: creative_files_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE creative_files_id_seq OWNED BY creative_files.id;


--
-- Name: creative_items; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE creative_items (
    id integer NOT NULL,
    creative_set_id integer,
    content jsonb DEFAULT '{}'::jsonb,
    type character varying,
    creative_file character varying,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: creative_items_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE creative_items_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: creative_items_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE creative_items_id_seq OWNED BY creative_items.id;


--
-- Name: creative_sets; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE creative_sets (
    id integer NOT NULL,
    title character varying(255),
    ad_type integer,
    icon_file_name character varying(255),
    icon_content_type character varying(255),
    icon_file_size character varying(255),
    description text,
    user_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    html text,
    html_only boolean DEFAULT false NOT NULL,
    native_title character varying(255),
    native_description character varying(255),
    native_cta character varying(255),
    can_edit boolean DEFAULT true NOT NULL,
    width integer,
    height integer,
    start_event_url character varying(255),
    first_quartile_event_url character varying(255),
    midpoint_event_url character varying(255),
    third_quartile_event_url character varying(255),
    complete_event_url character varying(255),
    creative_view_event_url character varying(255),
    mute_event_url character varying(255),
    unmute_event_url character varying(255),
    pause_event_url character varying(255),
    resume_event_url character varying(255),
    fullscreen_event_url character varying(255),
    html_macroses_enabled boolean DEFAULT false,
    tag boolean DEFAULT false NOT NULL,
    base_url character varying(255),
    mode integer,
    ad_content text
);


--
-- Name: creative_sets_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE creative_sets_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: creative_sets_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE creative_sets_id_seq OWNED BY creative_sets.id;


--
-- Name: custom_app_bid_floors; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE custom_app_bid_floors (
    id integer NOT NULL,
    account_type character varying,
    bid_floors character varying,
    ad_type integer,
    format integer,
    device_type integer,
    app_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    campaign_type_id integer
);


--
-- Name: custom_app_bid_floors_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE custom_app_bid_floors_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: custom_app_bid_floors_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE custom_app_bid_floors_id_seq OWNED BY custom_app_bid_floors.id;


--
-- Name: custom_user_commissions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE custom_user_commissions (
    id integer NOT NULL,
    user_id integer,
    commission double precision,
    start date,
    type character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: custom_user_commissions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE custom_user_commissions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: custom_user_commissions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE custom_user_commissions_id_seq OWNED BY custom_user_commissions.id;


--
-- Name: dash_views_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE dash_views_logs (
    id integer NOT NULL,
    user_id integer,
    fullpath text,
    group_params character varying(255),
    filter_params character varying(255),
    views_amount integer DEFAULT 0,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    log_date date,
    date_to date,
    date_from date
);


--
-- Name: dash_views_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE dash_views_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: dash_views_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE dash_views_logs_id_seq OWNED BY dash_views_logs.id;


--
-- Name: dashboard_predefined_filters; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE dashboard_predefined_filters (
    id integer NOT NULL,
    user_id integer,
    name character varying(255),
    params text,
    status character varying(2),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: dashboard_predefined_filters_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE dashboard_predefined_filters_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: dashboard_predefined_filters_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE dashboard_predefined_filters_id_seq OWNED BY dashboard_predefined_filters.id;


--
-- Name: data_migrations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE data_migrations (
    version character varying(255) NOT NULL
);


--
-- Name: deals; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE deals (
    id integer NOT NULL,
    user_id integer,
    deal_start integer,
    deal_end integer,
    network_names character varying[],
    app_keys character varying[],
    countries character varying[],
    ad_types character varying[]
);


--
-- Name: deals_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE deals_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: deals_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE deals_id_seq OWNED BY deals.id;


--
-- Name: device_manufacturers; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE device_manufacturers (
    id integer NOT NULL,
    name character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: device_manufacturers_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE device_manufacturers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: device_manufacturers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE device_manufacturers_id_seq OWNED BY device_manufacturers.id;


--
-- Name: device_models; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE device_models (
    id integer NOT NULL,
    device_manufacturer_id integer,
    marketing_name character varying(255),
    device_name character varying(255),
    model character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    device_type integer
);


--
-- Name: device_models_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE device_models_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: device_models_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE device_models_id_seq OWNED BY device_models.id;


--
-- Name: devices; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE devices (
    id integer NOT NULL,
    model character varying(255),
    android character varying(255),
    sdk character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: devices_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE devices_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: devices_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE devices_id_seq OWNED BY devices.id;


--
-- Name: disabled_networks; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE disabled_networks (
    id integer NOT NULL,
    title character varying(255) NOT NULL,
    name character varying(255) NOT NULL,
    ios_name character varying(255) NOT NULL,
    default_class character varying(255),
    active boolean DEFAULT true NOT NULL,
    supported_types json,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    icon character varying
);


--
-- Name: disabled_networks_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE disabled_networks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: disabled_networks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE disabled_networks_id_seq OWNED BY disabled_networks.id;


--
-- Name: docs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE docs (
    id integer NOT NULL,
    name character varying(255),
    platform integer,
    framework integer,
    doc_type integer DEFAULT 0 NOT NULL,
    status integer DEFAULT 0 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    disabled_networks json DEFAULT '[]'::json NOT NULL,
    ad_types json DEFAULT '[]'::json NOT NULL,
    banner_positions json DEFAULT '[]'::json NOT NULL,
    integration_sample character varying(255),
    version integer DEFAULT 1 NOT NULL,
    alias_url character varying
);


--
-- Name: docs_histories; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE docs_histories (
    id integer NOT NULL,
    doc_id integer NOT NULL,
    user_id integer NOT NULL,
    labels character varying(255)[],
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: docs_histories_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE docs_histories_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: docs_histories_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE docs_histories_id_seq OWNED BY docs_histories.id;


--
-- Name: docs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE docs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: docs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE docs_id_seq OWNED BY docs.id;


--
-- Name: docs_sections; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE docs_sections (
    id integer NOT NULL,
    name character varying(255),
    value_en text,
    value_ru text,
    section_type integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: docs_sections_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE docs_sections_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: docs_sections_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE docs_sections_id_seq OWNED BY docs_sections.id;


--
-- Name: docs_sets; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE docs_sets (
    id integer NOT NULL,
    doc_id integer,
    docs_section_id integer,
    "full" boolean DEFAULT true NOT NULL,
    proguard boolean DEFAULT false NOT NULL,
    mediation_admob boolean DEFAULT false NOT NULL,
    mediation_mopub boolean DEFAULT false NOT NULL,
    fs_spec boolean DEFAULT false NOT NULL,
    fs_load boolean DEFAULT false NOT NULL,
    earn boolean DEFAULT false NOT NULL,
    manual_integration boolean DEFAULT false NOT NULL,
    cocoapods_integration boolean DEFAULT false NOT NULL,
    ad_type_interstitial boolean DEFAULT false NOT NULL,
    ad_type_skippable_video boolean DEFAULT false NOT NULL,
    ad_type_nonskippable_video boolean DEFAULT false NOT NULL,
    ad_type_banner boolean DEFAULT false NOT NULL,
    ad_type_native boolean DEFAULT false NOT NULL,
    ad_type_mrec boolean DEFAULT false NOT NULL,
    banner_position_top boolean DEFAULT false NOT NULL,
    banner_position_bottom boolean DEFAULT false NOT NULL,
    banner_position_center boolean DEFAULT false NOT NULL,
    banner_position_layout boolean DEFAULT false NOT NULL,
    "order" integer,
    history_id integer
);


--
-- Name: docs_sets_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE docs_sets_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: docs_sets_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE docs_sets_id_seq OWNED BY docs_sets.id;


--
-- Name: domain_font_files; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE domain_font_files (
    id integer NOT NULL,
    domain_font_id integer,
    font_file_file_name character varying(255),
    font_file_content_type character varying(255),
    font_file_file_size integer,
    font_file_updated_at timestamp without time zone,
    ext_url character varying(255),
    format integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: domain_font_files_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE domain_font_files_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: domain_font_files_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE domain_font_files_id_seq OWNED BY domain_font_files.id;


--
-- Name: domain_fonts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE domain_fonts (
    id integer NOT NULL,
    domain_id integer,
    font_family character varying(255) DEFAULT ''::character varying,
    font_weight character varying(255) DEFAULT 'normal'::character varying,
    font_style character varying(255) DEFAULT 'normal'::character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    "default" boolean
);


--
-- Name: domain_fonts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE domain_fonts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: domain_fonts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE domain_fonts_id_seq OWNED BY domain_fonts.id;


--
-- Name: domains; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE domains (
    id integer NOT NULL,
    advertisement_partners_id integer DEFAULT 0,
    title character varying(255) NOT NULL,
    host character varying(255) NOT NULL,
    styles json NOT NULL,
    logo_file_name character varying(255),
    logo_content_type character varying(255),
    logo_file_size integer,
    logo_updated_at timestamp without time zone,
    logo_bottom_file_name character varying(255),
    logo_bottom_content_type character varying(255),
    logo_bottom_file_size integer,
    logo_bottom_updated_at timestamp without time zone,
    is_default boolean DEFAULT false NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    locale character varying(2) DEFAULT 'en'::character varying NOT NULL,
    status character varying(255) DEFAULT 'new'::character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    favicon_file_name character varying(255),
    favicon_content_type character varying(255),
    favicon_file_size integer,
    favicon_updated_at timestamp without time zone,
    admob_plugin_labels json DEFAULT '{}'::json NOT NULL,
    allow_user_activate boolean DEFAULT false
);


--
-- Name: domains_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE domains_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: domains_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE domains_id_seq OWNED BY domains.id;


--
-- Name: email_smtp_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE email_smtp_settings (
    id integer NOT NULL,
    domain_id integer,
    address character varying NOT NULL,
    port character varying NOT NULL,
    host character varying NOT NULL,
    user_name character varying NOT NULL,
    password character varying NOT NULL,
    authentication character varying DEFAULT 'login'::character varying,
    enable_starttls_auto boolean DEFAULT false,
    status boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    tls boolean DEFAULT false
);


--
-- Name: email_smtp_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE email_smtp_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: email_smtp_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE email_smtp_settings_id_seq OWNED BY email_smtp_settings.id;


--
-- Name: email_templates; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE email_templates (
    id integer NOT NULL,
    domain_id integer,
    name character varying,
    params text,
    status boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: email_templates_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE email_templates_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: email_templates_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE email_templates_id_seq OWNED BY email_templates.id;


--
-- Name: exceptional_targetings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE exceptional_targetings (
    id integer NOT NULL,
    campaign_id integer,
    app_id integer,
    country_id integer,
    expires_in timestamp without time zone
);


--
-- Name: exceptional_targetings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE exceptional_targetings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: exceptional_targetings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE exceptional_targetings_id_seq OWNED BY exceptional_targetings.id;


--
-- Name: exchange_coefficients; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE exchange_coefficients (
    id integer NOT NULL,
    percent double precision,
    effective_date date,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: exchange_coefficients_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE exchange_coefficients_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: exchange_coefficients_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE exchange_coefficients_id_seq OWNED BY exchange_coefficients.id;


--
-- Name: exchange_rates; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE exchange_rates (
    id integer NOT NULL,
    "from" character varying(255),
    "to" character varying(255),
    rate double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    date date
);


--
-- Name: exchange_rates_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE exchange_rates_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: exchange_rates_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE exchange_rates_id_seq OWNED BY exchange_rates.id;


--
-- Name: extension_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE extension_logs (
    id integer NOT NULL,
    user_id integer,
    output_at timestamp without time zone,
    content text,
    part integer,
    mode integer,
    version character varying(255),
    note character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: extension_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE extension_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: extension_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE extension_logs_id_seq OWNED BY extension_logs.id;


--
-- Name: facebook_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE facebook_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: facebook_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE facebook_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: facebook_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE facebook_accounts_id_seq OWNED BY facebook_accounts.id;


--
-- Name: fast_affiliate_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_affiliate_stats (
    id integer NOT NULL,
    date date,
    user_id integer,
    clicks_count integer DEFAULT 0,
    signups_count integer DEFAULT 0,
    apps_count integer DEFAULT 0
);


--
-- Name: fast_affiliate_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fast_affiliate_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fast_affiliate_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fast_affiliate_stats_id_seq OWNED BY fast_affiliate_stats.id;


--
-- Name: fast_ecpm_capping_options; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_capping_options (
    id integer NOT NULL,
    app_id integer NOT NULL,
    country_ids integer[],
    network_ids integer[],
    ad_types integer[],
    enabled boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: fast_ecpm_capping_options_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fast_ecpm_capping_options_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fast_ecpm_capping_options_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fast_ecpm_capping_options_id_seq OWNED BY fast_ecpm_capping_options.id;


--
-- Name: fast_ecpm_comparison_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_comparison_stats (
    id integer NOT NULL,
    campaign_image_id integer,
    old_ecpm double precision,
    new_ecpm double precision,
    old_stage integer,
    new_stage integer,
    fast_ecpm_comparison_stats_logger_id integer,
    historical_ecpm double precision
);


--
-- Name: fast_ecpm_comparison_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fast_ecpm_comparison_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fast_ecpm_comparison_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fast_ecpm_comparison_stats_id_seq OWNED BY fast_ecpm_comparison_stats.id;


--
-- Name: fast_ecpm_comparison_stats_loggers; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_comparison_stats_loggers (
    id integer NOT NULL,
    "timestamp" timestamp without time zone,
    fast_ecpm_new_option_id integer
);


--
-- Name: fast_ecpm_comparison_stats_loggers_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fast_ecpm_comparison_stats_loggers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fast_ecpm_comparison_stats_loggers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fast_ecpm_comparison_stats_loggers_id_seq OWNED BY fast_ecpm_comparison_stats_loggers.id;


--
-- Name: fast_ecpm_new_options; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_new_options (
    id integer NOT NULL,
    app_id integer NOT NULL,
    country_id integer,
    enabled boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: fast_ecpm_new_options_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fast_ecpm_new_options_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fast_ecpm_new_options_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fast_ecpm_new_options_id_seq OWNED BY fast_ecpm_new_options.id;


--
-- Name: fast_ecpm_stats_old; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_stats_old (
    id integer NOT NULL,
    campaign_image_id integer NOT NULL,
    country_id integer NOT NULL,
    app_id integer NOT NULL,
    ctr double precision,
    ecpm double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    impression_count integer DEFAULT 0 NOT NULL,
    stage integer DEFAULT 1 NOT NULL,
    impression_sum integer
);


--
-- Name: fast_ecpm_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fast_ecpm_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fast_ecpm_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fast_ecpm_stats_id_seq OWNED BY fast_ecpm_stats_old.id;


--
-- Name: fast_ecpm_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_stats (
    id integer DEFAULT nextval('fast_ecpm_stats_id_seq'::regclass) NOT NULL,
    campaign_image_id integer NOT NULL,
    country_id integer NOT NULL,
    app_id integer NOT NULL,
    ctr double precision,
    ecpm double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    impression_count integer DEFAULT 0 NOT NULL,
    stage integer DEFAULT 1 NOT NULL,
    impression_sum integer,
    ecpm_v2 double precision
);


--
-- Name: fast_ecpm_stats_exclusions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_stats_exclusions (
    id integer NOT NULL,
    app_id integer NOT NULL,
    country_id integer,
    campaign_image_id integer,
    ecpm integer DEFAULT 0.0,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    set json,
    for_reports boolean DEFAULT false NOT NULL,
    start timestamp without time zone,
    stop timestamp without time zone,
    deal_id integer
);


--
-- Name: fast_ecpm_stats_exclusions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fast_ecpm_stats_exclusions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fast_ecpm_stats_exclusions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fast_ecpm_stats_exclusions_id_seq OWNED BY fast_ecpm_stats_exclusions.id;


--
-- Name: fast_ecpm_stats_tmp; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fast_ecpm_stats_tmp (
    id integer DEFAULT nextval('fast_ecpm_stats_id_seq'::regclass) NOT NULL,
    campaign_image_id integer NOT NULL,
    country_id integer NOT NULL,
    app_id integer NOT NULL,
    ctr double precision,
    ecpm double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    impression_count integer DEFAULT 0 NOT NULL,
    stage integer DEFAULT 1 NOT NULL,
    impression_sum integer
);


--
-- Name: finalized_time_for_reports; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE finalized_time_for_reports (
    id integer NOT NULL,
    date date NOT NULL,
    account_type character varying NOT NULL,
    account_id integer,
    revenue integer NOT NULL,
    report_date timestamp without time zone NOT NULL
);


--
-- Name: finalized_time_for_reports_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE finalized_time_for_reports_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: finalized_time_for_reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE finalized_time_for_reports_id_seq OWNED BY finalized_time_for_reports.id;


--
-- Name: fixed_admob_campaigns_sets; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fixed_admob_campaigns_sets (
    id integer NOT NULL,
    app_id integer,
    type_id integer,
    campaigns_hash json
);


--
-- Name: fixed_admob_campaigns_sets_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fixed_admob_campaigns_sets_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fixed_admob_campaigns_sets_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fixed_admob_campaigns_sets_id_seq OWNED BY fixed_admob_campaigns_sets.id;


--
-- Name: flurry_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE flurry_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    api_key text,
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    company_id character varying(255),
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: flurry_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE flurry_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: flurry_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE flurry_accounts_id_seq OWNED BY flurry_accounts.id;


--
-- Name: fraud_click_series; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fraud_click_series (
    id integer NOT NULL,
    app_id integer,
    clicks integer,
    fraud_clicks integer,
    series_interval integer,
    ftr numeric(5,2),
    created_at timestamp without time zone,
    series_hours integer,
    revenue numeric(7,2),
    fraud_revenue numeric(7,2)
);


--
-- Name: fraud_click_series_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fraud_click_series_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fraud_click_series_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fraud_click_series_id_seq OWNED BY fraud_click_series.id;


--
-- Name: fraud_detections; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fraud_detections (
    id integer NOT NULL,
    app_id integer,
    time_interval integer,
    begin_time timestamp without time zone,
    end_time timestamp without time zone,
    type character varying(255),
    results_per_interval text[] DEFAULT '{}'::text[],
    state character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: fraud_detections_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fraud_detections_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fraud_detections_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fraud_detections_id_seq OWNED BY fraud_detections.id;


--
-- Name: fraud_user_emails; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fraud_user_emails (
    id integer NOT NULL,
    value character varying(255),
    user_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: fraud_user_emails_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fraud_user_emails_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fraud_user_emails_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fraud_user_emails_id_seq OWNED BY fraud_user_emails.id;


--
-- Name: fraud_user_keys; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE fraud_user_keys (
    id integer NOT NULL,
    value character varying(255),
    owner integer DEFAULT 0 NOT NULL,
    user_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: fraud_user_keys_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE fraud_user_keys_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: fraud_user_keys_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE fraud_user_keys_id_seq OWNED BY fraud_user_keys.id;


--
-- Name: frequency_capping_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE frequency_capping_stats (
    id integer NOT NULL,
    campaign_id integer NOT NULL,
    amount integer DEFAULT 0,
    "limit" integer DEFAULT 0,
    blocked boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: frequency_capping_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE frequency_capping_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: frequency_capping_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE frequency_capping_stats_id_seq OWNED BY frequency_capping_stats.id;


--
-- Name: global_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE global_settings (
    id integer NOT NULL,
    setting_name character varying(255),
    setting_value character varying(255),
    setting_text text,
    description text,
    global_settings_section_id integer
);


--
-- Name: global_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE global_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: global_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE global_settings_id_seq OWNED BY global_settings.id;


--
-- Name: global_settings_sections; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE global_settings_sections (
    id integer NOT NULL,
    name character varying(255),
    description text,
    parent_id integer,
    lft integer,
    rgt integer,
    depth integer
);


--
-- Name: global_settings_sections_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE global_settings_sections_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: global_settings_sections_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE global_settings_sections_id_seq OWNED BY global_settings_sections.id;


--
-- Name: globals; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE globals (
    id integer NOT NULL,
    store text
);


--
-- Name: globals_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE globals_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: globals_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE globals_id_seq OWNED BY globals.id;


--
-- Name: graph_ql_mutations_tracking_data; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE graph_ql_mutations_tracking_data (
    id integer NOT NULL,
    mutation_name character varying,
    scope character varying,
    arguments json,
    user_id integer,
    success boolean,
    details json,
    requested_at timestamp without time zone,
    responded_at timestamp without time zone,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: graph_ql_mutations_tracking_data_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE graph_ql_mutations_tracking_data_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: graph_ql_mutations_tracking_data_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE graph_ql_mutations_tracking_data_id_seq OWNED BY graph_ql_mutations_tracking_data.id;


--
-- Name: heyzap_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE heyzap_accounts (
    id integer NOT NULL,
    login character varying(255),
    publisher_id character varying(255),
    password character varying(255),
    api_key character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: heyzap_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE heyzap_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: heyzap_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE heyzap_accounts_id_seq OWNED BY heyzap_accounts.id;


--
-- Name: inmobi_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE inmobi_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    api_key character varying(255),
    session_id character varying(255),
    session_updated_at timestamp without time zone,
    account_id character varying(255),
    video_id character varying(50),
    rewarded_id character varying(50),
    video_key character varying(30),
    rewarded_key character varying(30),
    native_id character varying(50),
    native_key character varying(30),
    has_defaults boolean DEFAULT false NOT NULL,
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: inmobi_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE inmobi_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: inmobi_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE inmobi_accounts_id_seq OWNED BY inmobi_accounts.id;


--
-- Name: inmobi_ad_units; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE inmobi_ad_units (
    id integer NOT NULL,
    property character varying(255),
    placement character varying(255),
    platform_id integer,
    inmobi_account_id integer,
    is_for_kids boolean DEFAULT false NOT NULL,
    is_rewarded boolean DEFAULT false NOT NULL,
    ad_type integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: inmobi_ad_units_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE inmobi_ad_units_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: inmobi_ad_units_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE inmobi_ad_units_id_seq OWNED BY inmobi_ad_units.id;


--
-- Name: inner_active_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE inner_active_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    publisher_id character varying(255),
    secret_key character varying(255),
    consumer_key character varying(255),
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: inner_active_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE inner_active_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: inner_active_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE inner_active_accounts_id_seq OWNED BY inner_active_accounts.id;


--
-- Name: insight_stat_histories; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE insight_stat_histories (
    id integer NOT NULL,
    message character varying(255),
    event_date date,
    event_type character varying(255),
    status character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: insight_stat_histories_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE insight_stat_histories_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: insight_stat_histories_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE insight_stat_histories_id_seq OWNED BY insight_stat_histories.id;


--
-- Name: insight_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE insight_stats (
    id integer NOT NULL,
    app_id integer,
    value character varying(255),
    group_name character varying(255),
    impressions integer,
    clicks integer,
    report_date date,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    ecpm double precision DEFAULT 0,
    ctr double precision DEFAULT 0,
    revenue double precision DEFAULT 0,
    ad_type integer
);


--
-- Name: insight_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE insight_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: insight_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE insight_stats_id_seq OWNED BY insight_stats.id;


--
-- Name: insight_stats_y2018m03; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE insight_stats_y2018m03 (
    CONSTRAINT insight_stats_y2018m03_created_at_check CHECK (((created_at >= '2018-03-01'::date) AND (created_at < '2018-04-01'::date)))
)
INHERITS (insight_stats);


--
-- Name: insight_stats_y2018m04; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE insight_stats_y2018m04 (
    CONSTRAINT insight_stats_y2018m04_created_at_check CHECK (((created_at >= '2018-04-01'::date) AND (created_at < '2018-05-01'::date)))
)
INHERITS (insight_stats);


--
-- Name: insight_stats_y2018m05; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE insight_stats_y2018m05 (
    CONSTRAINT insight_stats_y2018m05_created_at_check CHECK (((created_at >= '2018-05-01'::date) AND (created_at < '2018-06-01'::date)))
)
INHERITS (insight_stats);


--
-- Name: integration_frameworks; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE integration_frameworks (
    id integer NOT NULL,
    title character varying(255) NOT NULL,
    name character varying(255) NOT NULL,
    native boolean DEFAULT false NOT NULL,
    default_class character varying(255),
    default_parent_class character varying(255),
    platform integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    icon character varying
);


--
-- Name: integration_frameworks_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE integration_frameworks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: integration_frameworks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE integration_frameworks_id_seq OWNED BY integration_frameworks.id;


--
-- Name: intercom_conversation_parts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE intercom_conversation_parts (
    id integer NOT NULL,
    part_id bigint,
    conversation_id bigint,
    created_at_intercom timestamp without time zone,
    body text,
    part_type character varying(255)
);


--
-- Name: intercom_conversation_parts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE intercom_conversation_parts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: intercom_conversation_parts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE intercom_conversation_parts_id_seq OWNED BY intercom_conversation_parts.id;


--
-- Name: intercom_conversations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE intercom_conversations (
    id integer NOT NULL,
    conversation_id bigint,
    created_at_intercom timestamp without time zone,
    user_email character varying(255),
    user_id integer,
    subject text,
    body text,
    conversation_web character varying(255)
);


--
-- Name: intercom_conversations_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE intercom_conversations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: intercom_conversations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE intercom_conversations_id_seq OWNED BY intercom_conversations.id;


--
-- Name: intercom_network_recommendeds; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE intercom_network_recommendeds (
    id integer NOT NULL,
    app_id integer NOT NULL,
    network integer NOT NULL,
    monitor_decision boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: intercom_network_recommendeds_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE intercom_network_recommendeds_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: intercom_network_recommendeds_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE intercom_network_recommendeds_id_seq OWNED BY intercom_network_recommendeds.id;


--
-- Name: invoice_requests; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE invoice_requests (
    id integer NOT NULL,
    user_id integer,
    amount numeric(8,2) DEFAULT 0,
    fee numeric(8,2) DEFAULT 0,
    regular boolean DEFAULT false,
    status integer DEFAULT 0 NOT NULL,
    rejected_by integer,
    completed_at timestamp without time zone,
    scheduled_at date,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: invoice_requests_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE invoice_requests_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: invoice_requests_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE invoice_requests_id_seq OWNED BY invoice_requests.id;


--
-- Name: ip_addresses; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE ip_addresses (
    id integer NOT NULL,
    address character varying(255),
    spamhaus boolean DEFAULT false NOT NULL,
    spamhaus_checked_at timestamp without time zone,
    spamcop boolean DEFAULT false NOT NULL,
    spamcop_checked_at timestamp without time zone,
    wpbl boolean DEFAULT false NOT NULL,
    wpbl_checked_at timestamp without time zone,
    stopforumspam boolean DEFAULT false NOT NULL,
    stopforumspam_checked_at timestamp without time zone,
    fraud_score integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    custom_block boolean,
    custom_blocked_at timestamp without time zone,
    vpn boolean
);


--
-- Name: ip_addresses_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE ip_addresses_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ip_addresses_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE ip_addresses_id_seq OWNED BY ip_addresses.id;


--
-- Name: liverail_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE liverail_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: liverail_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE liverail_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: liverail_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE liverail_accounts_id_seq OWNED BY liverail_accounts.id;


--
-- Name: liverail_ad_units; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE liverail_ad_units (
    id integer NOT NULL,
    code character varying(255),
    ad_type integer,
    width character varying(255),
    height character varying(255),
    bid_floor double precision,
    is_rewarded boolean DEFAULT false NOT NULL,
    liverail_account_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: liverail_ad_units_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE liverail_ad_units_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: liverail_ad_units_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE liverail_ad_units_id_seq OWNED BY liverail_ad_units.id;


--
-- Name: long_term_account_revenues; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE long_term_account_revenues (
    id integer NOT NULL,
    account_class character varying(255),
    account_id integer,
    date date,
    revenue double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    history text DEFAULT '{}'::text
);


--
-- Name: long_term_account_revenues_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE long_term_account_revenues_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: long_term_account_revenues_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE long_term_account_revenues_id_seq OWNED BY long_term_account_revenues.id;


--
-- Name: magic_monies; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE magic_monies (
    id integer NOT NULL,
    amount numeric(8,2) DEFAULT 0.0,
    user_id integer NOT NULL,
    type_id integer DEFAULT 0 NOT NULL,
    description character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: magic_monies_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE magic_monies_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: magic_monies_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE magic_monies_id_seq OWNED BY magic_monies.id;


--
-- Name: managers; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE managers (
    id integer NOT NULL,
    full_name_ru character varying(255),
    full_name_en character varying(255),
    short_name_ru character varying(255),
    short_name_en character varying(255),
    email character varying(255),
    lang character varying(255),
    phone character varying(255),
    skype character varying(255),
    male boolean DEFAULT true,
    share integer,
    users_count integer,
    invitation_link character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    slack_nickname character varying(255),
    lead_capture_enabled boolean,
    user_id integer,
    new_user_enabled boolean DEFAULT false,
    team_id integer,
    sf_id character varying(255) DEFAULT ''::character varying,
    region character varying(255)
);


--
-- Name: managers_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE managers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: managers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE managers_id_seq OWNED BY managers.id;


--
-- Name: max_mind_cities; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE max_mind_cities (
    id integer NOT NULL,
    geoname_id integer,
    city_name character varying(255),
    country_iso_code character varying(255),
    country_name character varying(255),
    subdivision_1_iso_code character varying(255),
    subdivision_1_name character varying(255),
    subdivision_2_iso_code character varying(255),
    subdivision_2_name character varying(255),
    time_zone character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    country_id integer,
    google_string_address character varying(255),
    latitude double precision,
    longitude double precision,
    zip character varying,
    alternatives character varying,
    continent_code character varying,
    continent_name character varying
);


--
-- Name: max_mind_cities_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE max_mind_cities_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: max_mind_cities_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE max_mind_cities_id_seq OWNED BY max_mind_cities.id;


--
-- Name: mmedia_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE mmedia_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean,
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: mmedia_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE mmedia_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: mmedia_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE mmedia_accounts_id_seq OWNED BY mmedia_accounts.id;


--
-- Name: mobile_networks; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE mobile_networks (
    id integer NOT NULL,
    mcc integer,
    mnc integer,
    iso character varying(255),
    country_name character varying(255),
    country_id integer,
    country_code integer,
    network character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: mobile_networks_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE mobile_networks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: mobile_networks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE mobile_networks_id_seq OWNED BY mobile_networks.id;


--
-- Name: mobilecore_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE mobilecore_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    uid character varying(255),
    export_data_key character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: mobilecore_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE mobilecore_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: mobilecore_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE mobilecore_accounts_id_seq OWNED BY mobilecore_accounts.id;


--
-- Name: monitor_events; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE monitor_events (
    id integer NOT NULL,
    event_type character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: monitor_events_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE monitor_events_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: monitor_events_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE monitor_events_id_seq OWNED BY monitor_events.id;


--
-- Name: monthly_book_keeping_reports; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE monthly_book_keeping_reports (
    id integer NOT NULL,
    year integer,
    month integer,
    total_sum double precision,
    user_sum double precision,
    appodeal_sum double precision,
    shared_sum double precision,
    instant_payments_sum double precision,
    instant_payments_fee double precision,
    regular_payments_sum double precision,
    invalid_activity_sum double precision,
    blocked_accounts_sum double precision,
    bonuses_sum double precision,
    corona_commission double precision,
    affiliate_commission double precision,
    sales_commission double precision,
    user_payable double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    integration_bonuses_sum numeric DEFAULT 0.0 NOT NULL
);


--
-- Name: monthly_book_keeping_reports_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE monthly_book_keeping_reports_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: monthly_book_keeping_reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE monthly_book_keeping_reports_id_seq OWNED BY monthly_book_keeping_reports.id;


--
-- Name: mopub_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE mopub_accounts (
    id integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    email character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    password character varying(255),
    report_id character varying(255),
    is_approve_default boolean DEFAULT false,
    owner_id integer DEFAULT 0 NOT NULL,
    deleted_at timestamp without time zone,
    is_video boolean DEFAULT false NOT NULL,
    is_default_video boolean DEFAULT false NOT NULL,
    is_rewarded boolean DEFAULT false NOT NULL,
    is_limit_reached boolean DEFAULT false NOT NULL,
    is_for_kids boolean,
    extra json DEFAULT '{}'::json NOT NULL,
    skip_reports boolean DEFAULT false NOT NULL
);


--
-- Name: mopub_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE mopub_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: mopub_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE mopub_accounts_id_seq OWNED BY mopub_accounts.id;


--
-- Name: network_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_accounts (
    id integer NOT NULL,
    type character varying(255),
    login character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    is_limit_reached boolean DEFAULT false NOT NULL,
    owner_id integer DEFAULT 0 NOT NULL,
    extra jsonb DEFAULT '{}'::jsonb NOT NULL,
    ad_type integer,
    deleted_at timestamp without time zone,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    deleted boolean DEFAULT false NOT NULL,
    skip_reports boolean DEFAULT false NOT NULL,
    priority integer DEFAULT 0
);


--
-- Name: network_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_accounts_id_seq OWNED BY network_accounts.id;


--
-- Name: network_active_types; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_active_types (
    id integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    network integer,
    processed_by_monitor boolean DEFAULT false NOT NULL,
    ad_type integer,
    status boolean DEFAULT false NOT NULL,
    sdk_name character varying
);


--
-- Name: network_active_types_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_active_types_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_active_types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_active_types_id_seq OWNED BY network_active_types.id;


--
-- Name: network_connection_restrictions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_connection_restrictions (
    id integer NOT NULL,
    network_name character varying(255),
    sentence text,
    connection_type integer,
    ad_type integer NOT NULL,
    repeat boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    network integer,
    phase_number integer DEFAULT 0 NOT NULL,
    final_phase boolean DEFAULT true NOT NULL,
    instructions character varying(255) DEFAULT ''::character varying
);


--
-- Name: network_connection_restrictions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_connection_restrictions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_connection_restrictions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_connection_restrictions_id_seq OWNED BY network_connection_restrictions.id;


--
-- Name: network_connection_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_connection_settings (
    id integer NOT NULL,
    ad_type integer,
    active boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    auto_create boolean DEFAULT false,
    network integer
);


--
-- Name: network_connection_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_connection_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_connection_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_connection_settings_id_seq OWNED BY network_connection_settings.id;


--
-- Name: network_connection_statuses; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_connection_statuses (
    id integer NOT NULL,
    app_id integer,
    network_connection_restriction_id integer,
    ad_type integer,
    status boolean NOT NULL,
    message text,
    repeat boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    monitor_decision boolean,
    stage character varying(255),
    network integer
);


--
-- Name: network_connection_statuses_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_connection_statuses_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_connection_statuses_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_connection_statuses_id_seq OWNED BY network_connection_statuses.id;


--
-- Name: network_content_filters; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_content_filters (
    id integer NOT NULL,
    network_name character varying(255),
    description text,
    excluded_categories_for_coppa text,
    all_categories text
);


--
-- Name: network_content_filters_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_content_filters_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_content_filters_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_content_filters_id_seq OWNED BY network_content_filters.id;


--
-- Name: network_daemon_jobs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_daemon_jobs (
    id integer NOT NULL,
    app_id integer NOT NULL,
    network integer NOT NULL,
    instructions character varying NOT NULL,
    ad_type integer NOT NULL,
    phase_number integer DEFAULT 0 NOT NULL,
    error_message text,
    error_backtrace text,
    account_method character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
)
WITH (autovacuum_vacuum_threshold='10', autovacuum_analyze_threshold='10');


--
-- Name: network_daemon_jobs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_daemon_jobs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_daemon_jobs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_daemon_jobs_id_seq OWNED BY network_daemon_jobs.id;


--
-- Name: network_mailing_app_statuses; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_mailing_app_statuses (
    id integer NOT NULL,
    app_id integer,
    success_date date,
    network integer,
    status integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    detailed_data text,
    revenue double precision,
    network_mailing_setting_id integer
);


--
-- Name: network_mailing_app_statuses_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_mailing_app_statuses_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_mailing_app_statuses_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_mailing_app_statuses_id_seq OWNED BY network_mailing_app_statuses.id;


--
-- Name: network_mailing_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_mailing_settings (
    id integer NOT NULL,
    network integer,
    active boolean DEFAULT true NOT NULL,
    revenue_border double precision DEFAULT 0.0 NOT NULL,
    countries_count integer DEFAULT 0 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    emails character varying(255),
    app_limit integer DEFAULT 10,
    user_id integer,
    types character varying(255),
    owner integer DEFAULT 0 NOT NULL
);


--
-- Name: network_mailing_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_mailing_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_mailing_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_mailing_settings_id_seq OWNED BY network_mailing_settings.id;


--
-- Name: network_migrations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_migrations (
    id integer NOT NULL,
    account_type character varying(255),
    id_orig integer,
    id_new integer,
    data_orig hstore,
    data_new hstore,
    status boolean DEFAULT false
);


--
-- Name: network_migrations_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_migrations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_migrations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_migrations_id_seq OWNED BY network_migrations.id;


--
-- Name: network_reporting_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_reporting_settings (
    id integer NOT NULL,
    network_name character varying(255),
    period integer,
    readers_list character varying(255),
    editors_list character varying(255),
    active boolean DEFAULT true NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    testers_list character varying(255),
    test_spreadsheet_id character varying(255)
);


--
-- Name: network_reporting_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_reporting_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_reporting_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_reporting_settings_id_seq OWNED BY network_reporting_settings.id;


--
-- Name: network_request_restrictions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE network_request_restrictions (
    id integer NOT NULL,
    network_id integer NOT NULL,
    max_requests integer NOT NULL,
    max_requests_pf integer NOT NULL,
    max_nofills integer NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    app_ids_array integer[] DEFAULT '{}'::integer[] NOT NULL,
    sdk_min integer DEFAULT 2000000 NOT NULL,
    sdk_max integer DEFAULT 999999999 NOT NULL,
    app_ids_include boolean
);


--
-- Name: network_request_restrictions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE network_request_restrictions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: network_request_restrictions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE network_request_restrictions_id_seq OWNED BY network_request_restrictions.id;


--
-- Name: new_fast_ecpm_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE new_fast_ecpm_stats (
    id integer DEFAULT nextval('fast_ecpm_stats_id_seq'::regclass) NOT NULL,
    campaign_image_id integer NOT NULL,
    country_id integer NOT NULL,
    app_id integer NOT NULL,
    ctr double precision,
    ecpm double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    impression_count integer DEFAULT 0 NOT NULL,
    stage integer DEFAULT 1 NOT NULL,
    impression_sum integer
);


--
-- Name: new_fast_ecpm_stats_old; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE new_fast_ecpm_stats_old (
    id integer DEFAULT nextval('fast_ecpm_stats_id_seq'::regclass) NOT NULL,
    campaign_image_id integer NOT NULL,
    country_id integer NOT NULL,
    app_id integer NOT NULL,
    ctr double precision,
    ecpm double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    impression_count integer DEFAULT 0 NOT NULL,
    stage integer DEFAULT 1 NOT NULL,
    impression_sum integer
);


--
-- Name: nexage_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE nexage_accounts (
    id integer NOT NULL,
    email character varying(255),
    company_id character varying(255),
    company_access_key character varying(255),
    secret_key character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    dcn character varying(32)
);


--
-- Name: nexage_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE nexage_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: nexage_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE nexage_accounts_id_seq OWNED BY nexage_accounts.id;


--
-- Name: openx_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE openx_accounts (
    id integer NOT NULL,
    email character varying(255) NOT NULL,
    password character varying(255) NOT NULL,
    site_url character varying(255) NOT NULL,
    consumer_key character varying(255) NOT NULL,
    consumer_secret character varying(255) NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    is_default boolean DEFAULT false NOT NULL,
    openx_uid character varying(255),
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: openx_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE openx_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: openx_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE openx_accounts_id_seq OWNED BY openx_accounts.id;


--
-- Name: organization_commission_presets; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE organization_commission_presets (
    id integer NOT NULL,
    organization_id integer NOT NULL,
    calculation_period integer DEFAULT 7 NOT NULL,
    calculation_offset integer DEFAULT 2 NOT NULL,
    commission_mapping jsonb DEFAULT '{}'::jsonb NOT NULL,
    fixed_commission integer DEFAULT 0 NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: organization_commission_presets_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE organization_commission_presets_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: organization_commission_presets_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE organization_commission_presets_id_seq OWNED BY organization_commission_presets.id;


--
-- Name: organization_roles; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE organization_roles (
    id integer NOT NULL,
    user_id integer NOT NULL,
    organization_id integer NOT NULL,
    role integer DEFAULT 0 NOT NULL,
    with_commission boolean DEFAULT true NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL,
    calculate_commission boolean DEFAULT true NOT NULL
);


--
-- Name: organization_roles_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE organization_roles_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: organization_roles_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE organization_roles_id_seq OWNED BY organization_roles.id;


--
-- Name: organizations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE organizations (
    id integer NOT NULL,
    name character varying DEFAULT ''::character varying NOT NULL,
    commission_mode integer DEFAULT 0 NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: organizations_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE organizations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: organizations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE organizations_id_seq OWNED BY organizations.id;


--
-- Name: page_categories_apps; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE page_categories_apps (
    id integer NOT NULL,
    page_category_id integer,
    app_id integer
);


--
-- Name: page_categories_apps_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE page_categories_apps_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: page_categories_apps_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE page_categories_apps_id_seq OWNED BY page_categories_apps.id;


--
-- Name: partner_app_commissions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE partner_app_commissions (
    id integer NOT NULL,
    app_id integer,
    commission double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: partner_app_commissions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE partner_app_commissions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: partner_app_commissions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE partner_app_commissions_id_seq OWNED BY partner_app_commissions.id;


--
-- Name: payment_requests; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE payment_requests (
    id integer NOT NULL,
    user_id integer,
    amount numeric(8,2),
    status integer DEFAULT 0,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    admin_notified_about_creation boolean DEFAULT false,
    user_notified_about_creation boolean DEFAULT false,
    user_notified_about_processing boolean DEFAULT false,
    regular boolean DEFAULT true,
    user_requisites text NOT NULL,
    fee numeric(8,2) DEFAULT 0,
    scheduled_at date,
    completed_at timestamp without time zone,
    payment_system character varying(255),
    rejected_by integer
);


--
-- Name: payment_requests_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE payment_requests_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: payment_requests_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE payment_requests_id_seq OWNED BY payment_requests.id;


--
-- Name: payouts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE payouts (
    id integer NOT NULL,
    campaign_image_id integer,
    country_id integer,
    app_id integer,
    value numeric,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: payouts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE payouts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: payouts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE payouts_id_seq OWNED BY payouts.id;


--
-- Name: platforms; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE platforms (
    id integer NOT NULL,
    name character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: platforms_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE platforms_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: platforms_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE platforms_id_seq OWNED BY platforms.id;


--
-- Name: premium_user_commissions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE premium_user_commissions (
    id integer NOT NULL,
    user_id integer,
    commission double precision,
    start date,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: premium_user_commissions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE premium_user_commissions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: premium_user_commissions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE premium_user_commissions_id_seq OWNED BY premium_user_commissions.id;


--
-- Name: price_floors; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE price_floors (
    id integer NOT NULL,
    app_id integer,
    country_id integer,
    value double precision,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: price_floors_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE price_floors_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: price_floors_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE price_floors_id_seq OWNED BY price_floors.id;


--
-- Name: publisher_monitor_messages; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE publisher_monitor_messages (
    id integer NOT NULL,
    network integer NOT NULL,
    ad_type integer NOT NULL,
    show_switcher boolean DEFAULT false NOT NULL,
    en_message character varying,
    ru_message character varying,
    en_popup character varying,
    ru_popup character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: publisher_monitor_messages_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE publisher_monitor_messages_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: publisher_monitor_messages_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE publisher_monitor_messages_id_seq OWNED BY publisher_monitor_messages.id;


--
-- Name: pubnative_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE pubnative_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    token character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    report_token character varying(255),
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: pubnative_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE pubnative_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: pubnative_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE pubnative_accounts_id_seq OWNED BY pubnative_accounts.id;


--
-- Name: qubole_sdk_data; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE qubole_sdk_data (
    id integer NOT NULL,
    app_id integer,
    sdk_version character varying(255),
    impressions_amount integer,
    total integer,
    percent numeric(5,2),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    report_date date,
    ad_type integer,
    group_name character varying,
    framework character varying
);


--
-- Name: qubole_sdk_data_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE qubole_sdk_data_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: qubole_sdk_data_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE qubole_sdk_data_id_seq OWNED BY qubole_sdk_data.id;


--
-- Name: report_processing_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE report_processing_logs (
    id integer NOT NULL,
    cached_file_id integer NOT NULL,
    account_id integer NOT NULL,
    account_type character varying NOT NULL,
    report_date date NOT NULL,
    "time" double precision DEFAULT 0.0,
    event integer NOT NULL,
    runner integer NOT NULL,
    process_id integer,
    revenue double precision DEFAULT 0.0,
    note character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: report_processing_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE report_processing_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: report_processing_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE report_processing_logs_id_seq OWNED BY report_processing_logs.id;


--
-- Name: reports; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE reports (
    id integer NOT NULL,
    fillrate numeric,
    ctr numeric,
    ecpm numeric,
    platform integer,
    week_date date,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    country_id integer,
    report_type integer DEFAULT 0,
    network_type integer,
    ad_type integer
);


--
-- Name: reports_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE reports_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE reports_id_seq OWNED BY reports.id;


--
-- Name: request_blacklists; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE request_blacklists (
    id integer NOT NULL,
    regex character varying(255),
    bl_type integer,
    active boolean DEFAULT true,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    ends_on timestamp without time zone
);


--
-- Name: request_blacklists_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE request_blacklists_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: request_blacklists_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE request_blacklists_id_seq OWNED BY request_blacklists.id;


--
-- Name: revenue_analysis_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE revenue_analysis_settings (
    id integer NOT NULL,
    account_class character varying(255),
    account_id integer,
    info boolean,
    warn boolean,
    error boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    compare_stats boolean DEFAULT true NOT NULL,
    revenue_difference_border double precision DEFAULT 1.0 NOT NULL,
    retry_limit integer DEFAULT 1 NOT NULL,
    skip_comparison character varying(255),
    positive_comparison character varying(255)
);


--
-- Name: revenue_analysis_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE revenue_analysis_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: revenue_analysis_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE revenue_analysis_settings_id_seq OWNED BY revenue_analysis_settings.id;


--
-- Name: revenue_change_limit_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE revenue_change_limit_settings (
    id integer NOT NULL,
    network integer,
    "limit" double precision,
    active boolean DEFAULT true NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: revenue_change_limit_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE revenue_change_limit_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: revenue_change_limit_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE revenue_change_limit_settings_id_seq OWNED BY revenue_change_limit_settings.id;


--
-- Name: revenue_comparison_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE revenue_comparison_stats (
    id integer NOT NULL,
    f_revenue double precision,
    d_revenue double precision,
    d_difference double precision,
    date date,
    account_class character varying(255),
    account_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    processing boolean DEFAULT false NOT NULL,
    processing_log text DEFAULT ''::text,
    file_saved boolean DEFAULT false NOT NULL,
    zero_expected boolean DEFAULT false NOT NULL,
    decision boolean DEFAULT false NOT NULL,
    network_revenue double precision,
    n_difference double precision,
    decision_description character varying(255),
    collector_log character varying(255) DEFAULT '{}'::character varying,
    stage integer DEFAULT 0 NOT NULL,
    retry_count integer DEFAULT 0 NOT NULL,
    v_revenue double precision DEFAULT 0,
    v_difference double precision DEFAULT 0
);


--
-- Name: revenue_comparison_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE revenue_comparison_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: revenue_comparison_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE revenue_comparison_stats_id_seq OWNED BY revenue_comparison_stats.id;


--
-- Name: roles; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE roles (
    id integer NOT NULL,
    name character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: roles_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE roles_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: roles_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE roles_id_seq OWNED BY roles.id;


--
-- Name: rubicon_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE rubicon_accounts (
    id integer NOT NULL,
    email character varying(255),
    login character varying(255),
    password character varying(255),
    server character varying(255),
    publisher_id character varying(255),
    is_default boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: rubicon_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE rubicon_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: rubicon_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE rubicon_accounts_id_seq OWNED BY rubicon_accounts.id;


--
-- Name: schema_migrations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE schema_migrations (
    version character varying(255) NOT NULL
);


--
-- Name: sdk_mapping_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE sdk_mapping_settings (
    id integer NOT NULL,
    platform_id integer,
    type_id integer,
    network character varying(255),
    sdk_version character varying(255),
    result boolean DEFAULT true NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: sdk_mapping_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE sdk_mapping_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: sdk_mapping_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE sdk_mapping_settings_id_seq OWNED BY sdk_mapping_settings.id;


--
-- Name: sdk_releases; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE sdk_releases (
    id integer NOT NULL,
    platform_id integer,
    framework character varying(255),
    sdk_version character varying(255),
    file character varying(255),
    external_url character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    release_type integer DEFAULT 0 NOT NULL,
    release_version character varying(255),
    kind integer DEFAULT 0 NOT NULL
);


--
-- Name: sdk_releases_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE sdk_releases_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: sdk_releases_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE sdk_releases_id_seq OWNED BY sdk_releases.id;


--
-- Name: section_categories_apps; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE section_categories_apps (
    id integer NOT NULL,
    section_category_id integer,
    app_id integer
);


--
-- Name: section_categories_apps_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE section_categories_apps_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: section_categories_apps_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE section_categories_apps_id_seq OWNED BY section_categories_apps.id;


--
-- Name: segment_placements; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE segment_placements (
    id integer NOT NULL,
    name character varying(255),
    segment_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    "default" boolean DEFAULT false,
    template boolean DEFAULT false NOT NULL,
    deleted boolean DEFAULT false NOT NULL
);


--
-- Name: segment_placements_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE segment_placements_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: segment_placements_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE segment_placements_id_seq OWNED BY segment_placements.id;


--
-- Name: segment_rule_countries; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE segment_rule_countries (
    id integer NOT NULL,
    segment_rule_id integer,
    country_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: segment_rule_countries_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE segment_rule_countries_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: segment_rule_countries_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE segment_rule_countries_id_seq OWNED BY segment_rule_countries.id;


--
-- Name: segment_rules; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE segment_rules (
    id integer NOT NULL,
    name character varying(255),
    value text,
    operator character varying(255),
    segment_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    field_type character varying(255),
    additional_type character varying(255),
    additional_value character varying(255)
);


--
-- Name: segment_rules_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE segment_rules_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: segment_rules_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE segment_rules_id_seq OWNED BY segment_rules.id;


--
-- Name: segment_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE segment_settings (
    id integer NOT NULL,
    name character varying(255),
    value text,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    segment_placement_id integer
);


--
-- Name: segment_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE segment_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: segment_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE segment_settings_id_seq OWNED BY segment_settings.id;


--
-- Name: segments; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE segments (
    id integer NOT NULL,
    name character varying(255),
    match_rule character varying(255),
    priority integer DEFAULT 0,
    user_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    setting character varying(35),
    settings text,
    platform character varying(255),
    deleted boolean DEFAULT false,
    active boolean DEFAULT false,
    state character varying(255)
);


--
-- Name: segments_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE segments_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: segments_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE segments_id_seq OWNED BY segments.id;


--
-- Name: share_tokens; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE share_tokens (
    id integer NOT NULL,
    email character varying,
    value character varying
);


--
-- Name: share_tokens_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE share_tokens_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: share_tokens_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE share_tokens_id_seq OWNED BY share_tokens.id;


--
-- Name: simple_dashboard; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE simple_dashboard (
    date date,
    revenue numeric,
    revenuex numeric,
    fee numeric,
    feex numeric,
    expenses numeric,
    payments numeric,
    tickets integer,
    issues integer,
    lost_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    lost_clients numeric,
    lost_churn numeric,
    revenue_churn numeric,
    active_apps integer,
    active_users integer,
    dau integer,
    impressions integer,
    visitors integer,
    registrations integer,
    id integer NOT NULL,
    total_users integer,
    revenuem numeric,
    active_users_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_data json DEFAULT '[]'::json NOT NULL,
    lost_fee numeric(12,2) DEFAULT 0 NOT NULL,
    active_users_fee_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_fee_data json DEFAULT '[]'::json NOT NULL,
    growing_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    growing_users_data json DEFAULT '[]'::json NOT NULL,
    active_users_fee integer DEFAULT 0 NOT NULL,
    expenses_data json DEFAULT '{}'::json NOT NULL,
    platforms json DEFAULT '[]'::json NOT NULL
);


--
-- Name: simple_dashboard_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE simple_dashboard_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: simple_dashboard_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE simple_dashboard_id_seq OWNED BY simple_dashboard.id;


--
-- Name: simple_dashboard_monthly; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE simple_dashboard_monthly (
    year integer,
    month integer,
    lost_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    lost_clients numeric,
    lost_churn numeric,
    revenue_churn numeric,
    active_apps integer,
    active_users integer,
    mau integer,
    visitors integer,
    id integer NOT NULL,
    total_users integer,
    active_users_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_data json DEFAULT '[]'::json NOT NULL,
    lost_fee numeric(12,2) DEFAULT 0 NOT NULL,
    active_users_fee_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_fee_data json DEFAULT '[]'::json NOT NULL,
    growing_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    growing_users_data json DEFAULT '[]'::json NOT NULL,
    active_users_fee integer DEFAULT 0 NOT NULL,
    expenses_data json DEFAULT '{}'::json NOT NULL,
    platforms json DEFAULT '[]'::json NOT NULL
);


--
-- Name: simple_dashboard_monthly_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE simple_dashboard_monthly_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: simple_dashboard_monthly_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE simple_dashboard_monthly_id_seq OWNED BY simple_dashboard_monthly.id;


--
-- Name: simple_dashboard_quarterly; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE simple_dashboard_quarterly (
    year integer,
    quarter integer,
    lost_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    lost_clients numeric,
    lost_churn numeric,
    revenue_churn numeric,
    active_apps integer,
    active_users integer,
    visitors integer,
    id integer NOT NULL,
    total_users integer,
    active_users_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_data json DEFAULT '[]'::json NOT NULL,
    lost_fee numeric(12,2) DEFAULT 0 NOT NULL,
    active_users_fee_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_fee_data json DEFAULT '[]'::json NOT NULL,
    growing_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    growing_users_data json DEFAULT '[]'::json NOT NULL,
    active_users_fee integer DEFAULT 0 NOT NULL,
    expenses_data json DEFAULT '{}'::json NOT NULL,
    platforms json DEFAULT '[]'::json NOT NULL
);


--
-- Name: simple_dashboard_quarterly_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE simple_dashboard_quarterly_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: simple_dashboard_quarterly_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE simple_dashboard_quarterly_id_seq OWNED BY simple_dashboard_quarterly.id;


--
-- Name: simple_dashboard_weekly; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE simple_dashboard_weekly (
    year integer,
    week integer,
    lost_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    lost_clients numeric,
    lost_churn numeric,
    revenue_churn numeric,
    active_apps integer,
    active_users integer,
    visitors integer,
    id integer NOT NULL,
    total_users integer,
    active_users_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_data json DEFAULT '[]'::json NOT NULL,
    lost_fee numeric(12,2) DEFAULT 0 NOT NULL,
    active_users_fee_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_fee_data json DEFAULT '[]'::json NOT NULL,
    growing_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    growing_users_data json DEFAULT '[]'::json NOT NULL,
    active_users_fee integer DEFAULT 0 NOT NULL,
    expenses_data json DEFAULT '{}'::json NOT NULL,
    platforms json DEFAULT '[]'::json NOT NULL
);


--
-- Name: simple_dashboard_weekly_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE simple_dashboard_weekly_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: simple_dashboard_weekly_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE simple_dashboard_weekly_id_seq OWNED BY simple_dashboard_weekly.id;


--
-- Name: simple_dashboard_yearly; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE simple_dashboard_yearly (
    year integer,
    lost_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    lost_clients numeric,
    lost_churn numeric,
    revenue_churn numeric,
    active_apps integer,
    active_users integer,
    visitors integer,
    id integer NOT NULL,
    total_users integer,
    active_users_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_data json DEFAULT '[]'::json NOT NULL,
    lost_fee numeric(12,2) DEFAULT 0 NOT NULL,
    active_users_fee_data json DEFAULT '[]'::json NOT NULL,
    lost_clients_fee_data json DEFAULT '[]'::json NOT NULL,
    growing_revenue numeric(12,2) DEFAULT 0 NOT NULL,
    growing_users_data json DEFAULT '[]'::json NOT NULL,
    active_users_fee integer DEFAULT 0 NOT NULL,
    expenses_data json DEFAULT '{}'::json NOT NULL,
    platforms json DEFAULT '[]'::json NOT NULL
);


--
-- Name: simple_dashboard_yearly_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE simple_dashboard_yearly_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: simple_dashboard_yearly_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE simple_dashboard_yearly_id_seq OWNED BY simple_dashboard_yearly.id;


--
-- Name: smaato_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE smaato_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    publisher_id bigint,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    is_default boolean DEFAULT false NOT NULL,
    client_key character varying(255),
    client_secret character varying(255),
    owner_id integer DEFAULT 0 NOT NULL
);


--
-- Name: smaato_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE smaato_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: smaato_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE smaato_accounts_id_seq OWNED BY smaato_accounts.id;


--
-- Name: spotx_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE spotx_accounts (
    id integer NOT NULL,
    username character varying(255) NOT NULL,
    password character varying(255) NOT NULL,
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: spotx_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE spotx_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: spotx_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE spotx_accounts_id_seq OWNED BY spotx_accounts.id;


--
-- Name: startapp_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE startapp_accounts (
    id integer NOT NULL,
    email character varying(255) NOT NULL,
    password character varying(255) NOT NULL,
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    dev_id character varying(255),
    owner_id integer DEFAULT 0 NOT NULL,
    partner_id character varying(255),
    token character varying(255),
    coppa boolean DEFAULT false
);


--
-- Name: startapp_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE startapp_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: startapp_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE startapp_accounts_id_seq OWNED BY startapp_accounts.id;


--
-- Name: statistic_reports; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE statistic_reports (
    id integer NOT NULL,
    name character varying(255),
    file character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    start date,
    stop date
);


--
-- Name: statistic_reports_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE statistic_reports_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: statistic_reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE statistic_reports_id_seq OWNED BY statistic_reports.id;


--
-- Name: statistics; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE statistics (
    id integer NOT NULL,
    type integer,
    campaign_url_id integer,
    device_id integer,
    date date,
    request_count integer DEFAULT 0,
    impression_count integer DEFAULT 0,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    app_id integer,
    campaign_image_id integer,
    country_id integer,
    android_id character varying(255),
    white_sdk boolean DEFAULT false,
    fill_count bigint,
    ip character varying(255),
    x_forwarded character varying(255),
    ua character varying(255),
    user_only boolean
);


--
-- Name: statistics_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE statistics_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: statistics_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE statistics_id_seq OWNED BY statistics.id;


--
-- Name: store_infos; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE store_infos (
    id integer NOT NULL,
    store_name character varying,
    bundle character varying,
    store_id character varying,
    developer character varying,
    developer_id character varying,
    icon character varying,
    icon_small character varying,
    release_date timestamp without time zone,
    advisory character varying,
    categories text,
    app_id integer,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL,
    url character varying,
    name character varying,
    price character varying,
    privacy_policy_url character varying
);


--
-- Name: store_infos_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE store_infos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: store_infos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE store_infos_id_seq OWNED BY store_infos.id;


--
-- Name: subdivision_ones; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE subdivision_ones (
    id integer NOT NULL,
    country_id integer,
    iso_code character varying(255),
    name character varying(255),
    name_ru character varying(255)
);


--
-- Name: subdivision_ones_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE subdivision_ones_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: subdivision_ones_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE subdivision_ones_id_seq OWNED BY subdivision_ones.id;


--
-- Name: subdivision_twos; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE subdivision_twos (
    id integer NOT NULL,
    country_id integer,
    subdivision1_id integer,
    iso_code character varying(255),
    name character varying(255),
    name_ru character varying(255)
);


--
-- Name: subdivision_twos_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE subdivision_twos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: subdivision_twos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE subdivision_twos_id_seq OWNED BY subdivision_twos.id;


--
-- Name: supersonic_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE supersonic_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    access_key character varying(255),
    secret_key character varying(255),
    is_default boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: supersonic_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE supersonic_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: supersonic_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE supersonic_accounts_id_seq OWNED BY supersonic_accounts.id;


--
-- Name: support_work_metriks; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE support_work_metriks (
    id integer NOT NULL,
    name character varying NOT NULL,
    description character varying,
    support_work_report_id integer NOT NULL,
    visible boolean DEFAULT true NOT NULL,
    post_to_slack boolean DEFAULT false NOT NULL,
    show_on_int_ash boolean DEFAULT false NOT NULL,
    value_type integer DEFAULT 0 NOT NULL,
    display_order integer DEFAULT 0 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: support_work_metriks_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE support_work_metriks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: support_work_metriks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE support_work_metriks_id_seq OWNED BY support_work_metriks.id;


--
-- Name: support_work_reports; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE support_work_reports (
    id integer NOT NULL,
    name character varying NOT NULL,
    description character varying,
    slack_messenger_hook_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    channel character varying
);


--
-- Name: support_work_reports_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE support_work_reports_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: support_work_reports_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE support_work_reports_id_seq OWNED BY support_work_reports.id;


--
-- Name: support_work_values; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE support_work_values (
    date date DEFAULT now() NOT NULL,
    support_work_metrik_id integer NOT NULL,
    value character varying,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: survey_responses; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE survey_responses (
    id integer NOT NULL,
    user_id integer,
    survey_id integer,
    comment text,
    points integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: survey_responses_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE survey_responses_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: survey_responses_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE survey_responses_id_seq OWNED BY survey_responses.id;


--
-- Name: surveys; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE surveys (
    id integer NOT NULL,
    question_ru character varying(255),
    question_en character varying(255),
    active boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: surveys_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE surveys_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: surveys_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE surveys_id_seq OWNED BY surveys.id;


--
-- Name: tapsense_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE tapsense_accounts (
    id integer NOT NULL,
    email character varying(255) NOT NULL,
    password_digest character varying(64) NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    is_default boolean DEFAULT false NOT NULL,
    source character varying(50),
    company_name character varying(255)
);


--
-- Name: tapsense_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE tapsense_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: tapsense_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE tapsense_accounts_id_seq OWNED BY tapsense_accounts.id;


--
-- Name: target_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE target_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    owner_id integer DEFAULT 0 NOT NULL,
    client_id character varying(255),
    client_secret character varying(255),
    access_token character varying(255),
    token_expires_in timestamp without time zone,
    refresh_token character varying(255),
    api_access_token character varying(255)
);


--
-- Name: target_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE target_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: target_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE target_accounts_id_seq OWNED BY target_accounts.id;


--
-- Name: teams; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE teams (
    id integer NOT NULL,
    email character varying(255) NOT NULL,
    name_en character varying(255) NOT NULL,
    name_ru character varying(255) NOT NULL,
    position_en character varying(255) NOT NULL,
    position_ru character varying(255) NOT NULL,
    company_en character varying(255) NOT NULL,
    company_ru character varying(255) NOT NULL,
    description_en text NOT NULL,
    description_ru text NOT NULL,
    "order" integer DEFAULT 0 NOT NULL,
    image_file_name character varying(255),
    image_content_type character varying(255),
    image_file_size integer,
    image_updated_at timestamp without time zone,
    visible boolean DEFAULT true NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: teams_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE teams_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: teams_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE teams_id_seq OWNED BY teams.id;


--
-- Name: temp_apps_network_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE temp_apps_network_accounts (
    id integer NOT NULL,
    app_id integer,
    network_account_id integer,
    expires_at date
);


--
-- Name: temp_apps_network_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE temp_apps_network_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: temp_apps_network_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE temp_apps_network_accounts_id_seq OWNED BY temp_apps_network_accounts.id;


--
-- Name: test_device_ids; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE test_device_ids (
    id integer NOT NULL,
    app_id integer,
    device_id character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: test_device_ids_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE test_device_ids_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: test_device_ids_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE test_device_ids_id_seq OWNED BY test_device_ids.id;


--
-- Name: traffic_limit_blocks; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE traffic_limit_blocks (
    id integer NOT NULL,
    app_id integer NOT NULL,
    campaign_image_id integer NOT NULL,
    country_id integer NOT NULL,
    date date NOT NULL,
    redis_impressions_count integer,
    dash_stats_impressions_count integer,
    fecpm_impression_sum integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: traffic_limit_blocks_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE traffic_limit_blocks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: traffic_limit_blocks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE traffic_limit_blocks_id_seq OWNED BY traffic_limit_blocks.id;


--
-- Name: traffic_limit_exclusions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE traffic_limit_exclusions (
    id integer NOT NULL,
    name character varying NOT NULL,
    enabled boolean,
    app_id integer,
    network_ids integer[],
    campaign_ids integer[],
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    user_id integer
);


--
-- Name: traffic_limit_exclusions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE traffic_limit_exclusions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: traffic_limit_exclusions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE traffic_limit_exclusions_id_seq OWNED BY traffic_limit_exclusions.id;


--
-- Name: unity_ads_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE unity_ads_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    api_key character varying(64),
    is_new_dash boolean DEFAULT false NOT NULL,
    udn boolean DEFAULT false NOT NULL,
    owner_id integer DEFAULT 0 NOT NULL,
    is_deleted boolean DEFAULT false NOT NULL,
    organization character varying(255)
);


--
-- Name: unity_ads_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE unity_ads_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: unity_ads_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE unity_ads_accounts_id_seq OWNED BY unity_ads_accounts.id;


--
-- Name: user_ach_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_ach_requisites (
    id integer NOT NULL,
    user_id integer,
    beneficiary_name character varying(255),
    bank_name character varying(255),
    bank_address character varying(255),
    account_number character varying(255),
    ach_routing_number character varying(255),
    active boolean DEFAULT false,
    country_id integer,
    street_address_1 character varying(255),
    street_address_2 character varying(255),
    city character varying(255),
    zip_code character varying(255),
    state character varying(255),
    type_of_account character varying(10),
    last_update timestamp without time zone
);


--
-- Name: user_ach_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_ach_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_ach_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_ach_requisites_id_seq OWNED BY user_ach_requisites.id;


--
-- Name: user_apps_commissions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_apps_commissions (
    id integer NOT NULL,
    user_id integer,
    app_ids text[] DEFAULT '{}'::text[],
    campaign_global_types text[] DEFAULT '{}'::text[],
    country_ids text[] DEFAULT '{}'::text[],
    commission numeric(3,2)
);


--
-- Name: user_apps_commissions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_apps_commissions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_apps_commissions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_apps_commissions_id_seq OWNED BY user_apps_commissions.id;


--
-- Name: user_bitcoin_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_bitcoin_requisites (
    id integer NOT NULL,
    user_id integer,
    account_number character varying(255),
    active boolean DEFAULT false,
    coinbase_email character varying
);


--
-- Name: user_bitcoin_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_bitcoin_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_bitcoin_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_bitcoin_requisites_id_seq OWNED BY user_bitcoin_requisites.id;


--
-- Name: user_confirmations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_confirmations (
    id integer NOT NULL,
    user_id integer NOT NULL,
    kind integer NOT NULL,
    extra json,
    ch_hash character varying(255),
    state integer DEFAULT 0 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: user_confirmations_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_confirmations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_confirmations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_confirmations_id_seq OWNED BY user_confirmations.id;


--
-- Name: user_daily_stats; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_daily_stats (
    id integer NOT NULL,
    date date,
    user_id integer,
    requests integer,
    fills integer,
    impressions integer,
    fillrate double precision,
    clicks integer,
    ctr double precision,
    revenue double precision,
    appodeal_revenue_cents double precision,
    affiliate_revenue_cents double precision,
    sales_revenue_cents double precision,
    ecpm double precision
);


--
-- Name: user_daily_stats_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_daily_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_daily_stats_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_daily_stats_id_seq OWNED BY user_daily_stats.id;


--
-- Name: user_ethereum_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_ethereum_requisites (
    id integer NOT NULL,
    user_id integer,
    account_number character varying(255),
    active boolean DEFAULT false,
    coinbase_email character varying
);


--
-- Name: user_ethereum_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_ethereum_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_ethereum_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_ethereum_requisites_id_seq OWNED BY user_ethereum_requisites.id;


--
-- Name: user_fraud_warning_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_fraud_warning_logs (
    id integer NOT NULL,
    user_id integer,
    user_email character varying(255),
    user_name character varying(255),
    account_manager_email character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: user_fraud_warning_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_fraud_warning_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_fraud_warning_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_fraud_warning_logs_id_seq OWNED BY user_fraud_warning_logs.id;


--
-- Name: user_google_wallet_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_google_wallet_requisites (
    id integer NOT NULL,
    user_id integer,
    account_number character varying(255),
    active boolean DEFAULT false
);


--
-- Name: user_google_wallet_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_google_wallet_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_google_wallet_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_google_wallet_requisites_id_seq OWNED BY user_google_wallet_requisites.id;


--
-- Name: user_intercom_events; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_intercom_events (
    id integer NOT NULL,
    user_id integer,
    event_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    app_id integer
);


--
-- Name: user_intercom_events_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_intercom_events_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_intercom_events_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_intercom_events_id_seq OWNED BY user_intercom_events.id;


--
-- Name: user_login_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_login_logs (
    id integer NOT NULL,
    user_id integer,
    ip character varying(255),
    ua character varying(255),
    cookies character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    developer_id integer
);


--
-- Name: user_login_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_login_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_login_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_login_logs_id_seq OWNED BY user_login_logs.id;


--
-- Name: user_notifications; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_notifications (
    id integer NOT NULL,
    class_name character varying(255) NOT NULL,
    method_name character varying(255) NOT NULL,
    user_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: user_notifications_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_notifications_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_notifications_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_notifications_id_seq OWNED BY user_notifications.id;


--
-- Name: user_payment_details_change_confirms; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_payment_details_change_confirms (
    id integer NOT NULL,
    user_id integer NOT NULL,
    payment_details json,
    ch_hash character varying(255),
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    form_w9 character varying(255)
);


--
-- Name: user_payment_details_change_confirms_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_payment_details_change_confirms_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_payment_details_change_confirms_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_payment_details_change_confirms_id_seq OWNED BY user_payment_details_change_confirms.id;


--
-- Name: user_payoneer_bank_transfer_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_payoneer_bank_transfer_requisites (
    id integer NOT NULL,
    user_id integer,
    payee_id character varying,
    client_session_id character varying,
    registration boolean DEFAULT false,
    approved boolean DEFAULT false,
    active boolean DEFAULT false
);


--
-- Name: user_payoneer_bank_transfer_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_payoneer_bank_transfer_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_payoneer_bank_transfer_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_payoneer_bank_transfer_requisites_id_seq OWNED BY user_payoneer_bank_transfer_requisites.id;


--
-- Name: user_payoneer_paypal_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_payoneer_paypal_requisites (
    id integer NOT NULL,
    user_id integer,
    payee_id character varying,
    client_session_id character varying,
    registration boolean DEFAULT false,
    approved boolean DEFAULT false,
    active boolean DEFAULT false
);


--
-- Name: user_payoneer_paypal_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_payoneer_paypal_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_payoneer_paypal_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_payoneer_paypal_requisites_id_seq OWNED BY user_payoneer_paypal_requisites.id;


--
-- Name: user_payoneer_prepaid_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_payoneer_prepaid_requisites (
    id integer NOT NULL,
    user_id integer,
    payee_id character varying,
    client_session_id character varying,
    registration boolean DEFAULT false,
    approved boolean DEFAULT false,
    active boolean DEFAULT false
);


--
-- Name: user_payoneer_prepaid_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_payoneer_prepaid_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_payoneer_prepaid_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_payoneer_prepaid_requisites_id_seq OWNED BY user_payoneer_prepaid_requisites.id;


--
-- Name: user_paypal_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_paypal_requisites (
    id integer NOT NULL,
    user_id integer,
    email character varying(255),
    active boolean DEFAULT false
);


--
-- Name: user_paypal_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_paypal_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_paypal_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_paypal_requisites_id_seq OWNED BY user_paypal_requisites.id;


--
-- Name: user_webmoney_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_webmoney_requisites (
    id integer NOT NULL,
    user_id integer,
    account_number character varying(255),
    active boolean DEFAULT false
);


--
-- Name: user_webmoney_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_webmoney_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_webmoney_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_webmoney_requisites_id_seq OWNED BY user_webmoney_requisites.id;


--
-- Name: user_wire_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_wire_requisites (
    id integer NOT NULL,
    user_id integer NOT NULL,
    account_name character varying(255),
    account_number character varying(255),
    swift character varying(255),
    bank_name character varying(255),
    bank_address_1 character varying(255),
    bank_address_2 character varying(255),
    bank_city character varying(255),
    bank_state character varying(255),
    bank_zip_code character varying(255),
    country_id integer,
    intermediary_bank_swift character varying(255),
    intermediary_bank_name character varying(255),
    intermediary_account_number character varying(255),
    active boolean DEFAULT false,
    bank_country_id integer,
    street_address_1 character varying(255),
    street_address_2 character varying(255),
    city character varying(255),
    zip_code character varying(255),
    state character varying(255),
    euros boolean DEFAULT false,
    last_update timestamp without time zone
);


--
-- Name: user_wire_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_wire_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_wire_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_wire_requisites_id_seq OWNED BY user_wire_requisites.id;


--
-- Name: user_yandex_money_requisites; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE user_yandex_money_requisites (
    id integer NOT NULL,
    user_id integer,
    account_number character varying(255),
    active boolean DEFAULT false
);


--
-- Name: user_yandex_money_requisites_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE user_yandex_money_requisites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: user_yandex_money_requisites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE user_yandex_money_requisites_id_seq OWNED BY user_yandex_money_requisites.id;


--
-- Name: users; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users (
    id integer NOT NULL,
    email character varying(255),
    is_admin boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    balance numeric(8,2) DEFAULT 0,
    balance_date date,
    change_pwd_hash character varying(255),
    remember_digest character varying(255),
    allow_own_accounts boolean DEFAULT false NOT NULL,
    settings hstore,
    name character varying(255),
    phone character varying(255),
    postal_address character varying(255),
    comment text,
    contacts text,
    api_key character varying(255),
    aff_code character varying(255) DEFAULT ''::character varying,
    affiliated_by integer DEFAULT 0,
    remote_ip character varying(255),
    country_code character varying(255) DEFAULT 'US'::character varying,
    aff_balance_date date,
    revenue double precision DEFAULT 0.0,
    revenue_appodeal double precision DEFAULT 0.0,
    revenue_date timestamp without time zone,
    affiliated_revenue double precision DEFAULT 0.0,
    lead_owner_id integer,
    role integer,
    no_natalie_emails boolean DEFAULT false,
    lead_state character varying(255) DEFAULT 'Account'::character varying,
    monthly_payout numeric(8,2) DEFAULT 0,
    banned_at timestamp without time zone,
    payout_threshold integer DEFAULT 20,
    fraud_balance numeric(7,2) DEFAULT 0,
    ftr numeric(5,2) DEFAULT 0,
    terms boolean DEFAULT false NOT NULL,
    skype character varying(50),
    is_special boolean DEFAULT false,
    auto_payout_threshold integer,
    segment_enable boolean DEFAULT false,
    default_filter_id integer,
    sf_id character varying(255) DEFAULT ''::character varying,
    auto_payout_disable boolean,
    extension_error_at date,
    min_payout integer,
    affiliate_program boolean DEFAULT false NOT NULL,
    closeio_id character varying(255) DEFAULT ''::character varying,
    utm_source character varying(255),
    fb_alert boolean DEFAULT false,
    new_design boolean DEFAULT false NOT NULL,
    account_manager_id integer,
    encrypted_password character varying(255),
    reset_password_token character varying(255),
    reset_password_sent_at timestamp without time zone,
    failed_attempts integer DEFAULT 0,
    unlock_token character varying(255),
    locked_at timestamp without time zone,
    insights_enable boolean DEFAULT false,
    confirmation_token character varying(255),
    password_new character varying(255),
    form_w9 character varying(255),
    payouts_disable boolean DEFAULT false,
    domain character varying(255),
    ext json,
    status integer DEFAULT 1 NOT NULL,
    account_type integer DEFAULT 0 NOT NULL,
    mediation_fee double precision DEFAULT 10 NOT NULL,
    lead_amount character varying(255) DEFAULT ''::character varying NOT NULL,
    regular_payment_limit double precision DEFAULT 0.0,
    remember_created_at timestamp without time zone,
    tz character varying DEFAULT 'UTC'::character varying NOT NULL,
    v2_segments_enabled boolean DEFAULT false NOT NULL,
    wl_domain_id integer,
    state character varying,
    regular_payout_threshold integer,
    wl_data json DEFAULT '{}'::json NOT NULL,
    subscribed boolean DEFAULT false NOT NULL,
    unsubscribed_list_ids text[],
    confirmed_at timestamp without time zone,
    confirmation_sent_at timestamp without time zone,
    unconfirmed_email character varying
);


--
-- Name: users_abilities; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users_abilities (
    id integer NOT NULL,
    user_role integer,
    member_action character varying(255)[] NOT NULL,
    resources_name character varying(255) NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    resource_category character varying(255) DEFAULT 'active_admin_resources'::character varying NOT NULL,
    role_id integer
);


--
-- Name: users_abilities_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE users_abilities_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: users_abilities_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE users_abilities_id_seq OWNED BY users_abilities.id;


--
-- Name: users_admin_notifications; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users_admin_notifications (
    id integer NOT NULL,
    user_id integer NOT NULL,
    type_status integer DEFAULT 1,
    title character varying(255),
    body text,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    global_message boolean DEFAULT false
);


--
-- Name: users_admin_notifications_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE users_admin_notifications_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: users_admin_notifications_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE users_admin_notifications_id_seq OWNED BY users_admin_notifications.id;


--
-- Name: users_dev_apps; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users_dev_apps (
    id integer NOT NULL,
    user_id integer,
    app_id integer,
    account_share_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: users_dev_apps_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE users_dev_apps_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: users_dev_apps_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE users_dev_apps_id_seq OWNED BY users_dev_apps.id;


--
-- Name: users_fake_apps; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users_fake_apps (
    id integer NOT NULL,
    user_id integer,
    app_id integer
);


--
-- Name: users_fake_apps_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE users_fake_apps_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: users_fake_apps_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE users_fake_apps_id_seq OWNED BY users_fake_apps.id;


--
-- Name: users_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE users_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE users_id_seq OWNED BY users.id;


--
-- Name: users_managed_campaigns; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users_managed_campaigns (
    id integer NOT NULL,
    user_id integer,
    campaign_id integer,
    account_share_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: users_managed_campaigns_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE users_managed_campaigns_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: users_managed_campaigns_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE users_managed_campaigns_id_seq OWNED BY users_managed_campaigns.id;


--
-- Name: users_roles; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users_roles (
    user_id integer,
    role_id integer
);


--
-- Name: users_shared_apps; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE users_shared_apps (
    id integer NOT NULL,
    user_id integer,
    app_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    account_share_id integer
);


--
-- Name: users_shared_apps_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE users_shared_apps_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: users_shared_apps_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE users_shared_apps_id_seq OWNED BY users_shared_apps.id;


--
-- Name: v2_placements; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE v2_placements (
    id integer NOT NULL,
    app_id integer,
    name character varying DEFAULT ''::character varying NOT NULL,
    description text DEFAULT ''::text NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    settings text DEFAULT '[]'::text,
    sdk_settings text DEFAULT ''::text
);


--
-- Name: v2_placements_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE v2_placements_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: v2_placements_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE v2_placements_id_seq OWNED BY v2_placements.id;


--
-- Name: v2_placements_versions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE v2_placements_versions (
    id integer NOT NULL,
    item_type character varying NOT NULL,
    item_id integer NOT NULL,
    event character varying NOT NULL,
    whodunnit character varying,
    object text,
    object_changes text,
    created_at timestamp without time zone
);


--
-- Name: v2_placements_versions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE v2_placements_versions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: v2_placements_versions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE v2_placements_versions_id_seq OWNED BY v2_placements_versions.id;


--
-- Name: v2_segments; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE v2_segments (
    id integer NOT NULL,
    user_id integer,
    name character varying DEFAULT ''::character varying NOT NULL,
    preset boolean DEFAULT false NOT NULL,
    default_preset boolean DEFAULT false NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    settings text DEFAULT '[]'::text,
    priority integer DEFAULT 0 NOT NULL,
    restrictions text DEFAULT '[]'::text,
    placements text DEFAULT '[]'::text,
    match_rule integer,
    sdk_restrictions text DEFAULT ''::text,
    sdk_placements text DEFAULT ''::text,
    sdk_settings text DEFAULT ''::text,
    enabled boolean DEFAULT false
);


--
-- Name: v2_segments_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE v2_segments_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: v2_segments_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE v2_segments_id_seq OWNED BY v2_segments.id;


--
-- Name: v2_segments_versions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE v2_segments_versions (
    id integer NOT NULL,
    item_type character varying NOT NULL,
    item_id integer NOT NULL,
    event character varying NOT NULL,
    whodunnit character varying,
    object text,
    object_changes text,
    created_at timestamp without time zone
);


--
-- Name: v2_segments_versions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE v2_segments_versions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: v2_segments_versions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE v2_segments_versions_id_seq OWNED BY v2_segments_versions.id;


--
-- Name: vacuum_stats; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW vacuum_stats AS
 SELECT psut.relname,
    to_char(psut.last_vacuum, 'YYYY-MM-DD HH24:MI'::text) AS last_vacuum,
    to_char(psut.last_autovacuum, 'YYYY-MM-DD HH24:MI'::text) AS last_autovacuum,
    to_char(pg_class.reltuples, '9G999G999G999'::text) AS n_tup,
    to_char(psut.n_dead_tup, '9G999G999G999'::text) AS dead_tup,
    to_char((((current_setting('autovacuum_vacuum_threshold'::text))::bigint)::double precision + (((current_setting('autovacuum_vacuum_scale_factor'::text))::numeric)::double precision * pg_class.reltuples)), '9G999G999G999'::text) AS av_threshold,
        CASE
            WHEN ((((current_setting('autovacuum_vacuum_threshold'::text))::bigint)::double precision + (((current_setting('autovacuum_vacuum_scale_factor'::text))::numeric)::double precision * pg_class.reltuples)) < (psut.n_dead_tup)::double precision) THEN '*'::text
            ELSE ''::text
        END AS expect_av
   FROM (pg_stat_user_tables psut
     JOIN pg_class ON ((psut.relid = pg_class.oid)))
  ORDER BY psut.relname;


--
-- Name: versions; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE versions (
    id integer NOT NULL,
    item_type character varying(255) NOT NULL,
    item_id integer NOT NULL,
    event character varying(255) NOT NULL,
    whodunnit character varying(255),
    object text,
    created_at timestamp without time zone,
    object_changes text,
    comment character varying(255),
    aerospike_changes text
);


--
-- Name: versions_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE versions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: versions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE versions_id_seq OWNED BY versions.id;


--
-- Name: vpn_ips; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vpn_ips (
    id integer NOT NULL,
    address character varying(255),
    is_vpn boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: vpn_ips_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE vpn_ips_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: vpn_ips_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE vpn_ips_id_seq OWNED BY vpn_ips.id;


--
-- Name: vpn_ips_y2018m08; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vpn_ips_y2018m08 (
    CONSTRAINT vpn_ips_y2018m08_created_at_check CHECK (((created_at >= '2018-08-01'::date) AND (created_at < '2018-09-01'::date)))
)
INHERITS (vpn_ips);


--
-- Name: vpn_ips_y2018m09; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vpn_ips_y2018m09 (
    CONSTRAINT vpn_ips_y2018m09_created_at_check CHECK (((created_at >= '2018-09-01'::date) AND (created_at < '2018-10-01'::date)))
)
INHERITS (vpn_ips);


--
-- Name: vpn_ips_y2018m10; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vpn_ips_y2018m10 (
    CONSTRAINT vpn_ips_y2018m10_created_at_check CHECK (((created_at >= '2018-10-01'::date) AND (created_at < '2018-11-01'::date)))
)
INHERITS (vpn_ips);


--
-- Name: vpn_ips_y2018m11; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vpn_ips_y2018m11 (
    CONSTRAINT vpn_ips_y2018m11_created_at_check CHECK (((created_at >= '2018-11-01'::date) AND (created_at < '2018-12-01'::date)))
)
INHERITS (vpn_ips);


--
-- Name: vungle_acceptations; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vungle_acceptations (
    id integer NOT NULL,
    app_id integer NOT NULL,
    status integer DEFAULT 0 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: vungle_acceptations_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE vungle_acceptations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: vungle_acceptations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE vungle_acceptations_id_seq OWNED BY vungle_acceptations.id;


--
-- Name: vungle_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vungle_accounts (
    id integer NOT NULL,
    email character varying(255),
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    api_key character varying(32)
);


--
-- Name: vungle_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE vungle_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: vungle_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE vungle_accounts_id_seq OWNED BY vungle_accounts.id;


--
-- Name: vungle_agreements; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE vungle_agreements (
    id integer NOT NULL,
    user_id integer NOT NULL,
    status integer DEFAULT 0 NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: vungle_agreements_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE vungle_agreements_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: vungle_agreements_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE vungle_agreements_id_seq OWNED BY vungle_agreements.id;


--
-- Name: waterfall_logs; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_logs (
    id integer NOT NULL,
    waterfall_monitor_id integer,
    country_id integer,
    ad_type integer,
    warning boolean DEFAULT false,
    status character varying,
    response jsonb DEFAULT '{}'::jsonb,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: waterfall_logs_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_logs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_logs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_logs_id_seq OWNED BY waterfall_logs.id;


--
-- Name: waterfall_monitors; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_monitors (
    id integer NOT NULL,
    app_id integer,
    country_ids integer[] DEFAULT '{}'::integer[],
    ad_types integer[] DEFAULT '{}'::integer[],
    active boolean DEFAULT true,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone NOT NULL
);


--
-- Name: waterfall_monitors_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_monitors_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_monitors_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_monitors_id_seq OWNED BY waterfall_monitors.id;


--
-- Name: waterfall_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings (
    id integer NOT NULL,
    ad_watch_interstitial boolean,
    ad_watch_video boolean,
    ad_watch_banner boolean,
    ad_watch_native boolean,
    ad_watch_mrec boolean,
    ad_watch_rewarded_video boolean,
    complains boolean,
    app_id integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    network_id integer
);


--
-- Name: waterfall_settings_categories; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings_categories (
    id integer NOT NULL,
    name character varying,
    tags character varying[] DEFAULT '{}'::character varying[]
);


--
-- Name: waterfall_settings_categories_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_categories_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_categories_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_categories_id_seq OWNED BY waterfall_settings_categories.id;


--
-- Name: waterfall_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_id_seq OWNED BY waterfall_settings.id;


--
-- Name: waterfall_settings_root_flag_types; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings_root_flag_types (
    id integer NOT NULL,
    name character varying NOT NULL,
    kind character varying NOT NULL,
    skipped boolean DEFAULT false NOT NULL,
    "default" bytea,
    description text
);


--
-- Name: waterfall_settings_root_flag_types_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_root_flag_types_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_root_flag_types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_root_flag_types_id_seq OWNED BY waterfall_settings_root_flag_types.id;


--
-- Name: waterfall_settings_root_flags; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings_root_flags (
    id integer NOT NULL,
    name character varying NOT NULL,
    kind character varying NOT NULL,
    flag_type_id integer NOT NULL,
    setting_id integer NOT NULL,
    skipped boolean DEFAULT false NOT NULL,
    checksum character varying NOT NULL,
    value bytea
);


--
-- Name: waterfall_settings_root_flags_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_root_flags_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_root_flags_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_root_flags_id_seq OWNED BY waterfall_settings_root_flags.id;


--
-- Name: waterfall_settings_root_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings_root_settings (
    id integer NOT NULL,
    app_id integer,
    network_id integer,
    real_sdk_version_min integer DEFAULT 0,
    real_sdk_version_max integer DEFAULT 999999999,
    real_sdk_min integer DEFAULT 0,
    real_sdk_max integer DEFAULT 999999999,
    real_android_min integer DEFAULT 0,
    real_android_max integer DEFAULT 999999999,
    real_ios_min integer DEFAULT 0,
    real_ios_max integer DEFAULT 999999999,
    ad_type_interstitial boolean DEFAULT false NOT NULL,
    ad_type_video boolean DEFAULT false NOT NULL,
    ad_type_banner boolean DEFAULT false NOT NULL,
    ad_type_native boolean DEFAULT false NOT NULL,
    ad_type_mrec boolean DEFAULT false NOT NULL,
    ad_type_rewarded_video boolean DEFAULT false NOT NULL,
    stored_flags json,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    checksum character varying NOT NULL,
    score_by_count double precision NOT NULL,
    score_by_accuracy double precision NOT NULL,
    score_summary double precision NOT NULL,
    ad_ids uuid[],
    tags character varying[] DEFAULT '{}'::character varying[],
    action character varying DEFAULT 'get'::character varying NOT NULL
);


--
-- Name: waterfall_settings_root_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_root_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_root_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_root_settings_id_seq OWNED BY waterfall_settings_root_settings.id;


--
-- Name: waterfall_settings_unit_flag_types; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings_unit_flag_types (
    id integer NOT NULL,
    name character varying NOT NULL,
    kind character varying NOT NULL,
    skipped boolean DEFAULT false NOT NULL,
    "default" bytea,
    description text
);


--
-- Name: waterfall_settings_unit_flag_types_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_unit_flag_types_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_unit_flag_types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_unit_flag_types_id_seq OWNED BY waterfall_settings_unit_flag_types.id;


--
-- Name: waterfall_settings_unit_flags; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings_unit_flags (
    id integer NOT NULL,
    name character varying NOT NULL,
    kind character varying NOT NULL,
    flag_type_id integer NOT NULL,
    setting_id integer NOT NULL,
    skipped boolean DEFAULT false NOT NULL,
    checksum character varying NOT NULL,
    value bytea
);


--
-- Name: waterfall_settings_unit_flags_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_unit_flags_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_unit_flags_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_unit_flags_id_seq OWNED BY waterfall_settings_unit_flags.id;


--
-- Name: waterfall_settings_unit_settings; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE waterfall_settings_unit_settings (
    id integer NOT NULL,
    app_id integer,
    network_id integer,
    real_sdk_version_min integer DEFAULT 0,
    real_sdk_version_max integer DEFAULT 999999999,
    real_sdk_min integer DEFAULT 0,
    real_sdk_max integer DEFAULT 999999999,
    real_android_min integer DEFAULT 0,
    real_android_max integer DEFAULT 999999999,
    real_ios_min integer DEFAULT 0,
    real_ios_max integer DEFAULT 999999999,
    ad_type_interstitial boolean DEFAULT false NOT NULL,
    ad_type_video boolean DEFAULT false NOT NULL,
    ad_type_banner boolean DEFAULT false NOT NULL,
    ad_type_native boolean DEFAULT false NOT NULL,
    ad_type_mrec boolean DEFAULT false NOT NULL,
    ad_type_rewarded_video boolean DEFAULT false NOT NULL,
    stored_flags json,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    checksum character varying NOT NULL,
    score_by_count double precision NOT NULL,
    score_by_accuracy double precision NOT NULL,
    score_summary double precision NOT NULL,
    ad_ids uuid[],
    tags character varying[] DEFAULT '{}'::character varying[],
    action character varying DEFAULT 'get'::character varying NOT NULL
);


--
-- Name: waterfall_settings_unit_settings_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE waterfall_settings_unit_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: waterfall_settings_unit_settings_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE waterfall_settings_unit_settings_id_seq OWNED BY waterfall_settings_unit_settings.id;


--
-- Name: wl_domain_fees; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE wl_domain_fees (
    id integer NOT NULL,
    wl_domain_id integer,
    fee_type integer,
    user_id integer,
    fee double precision DEFAULT 0.0 NOT NULL,
    start date,
    stop date,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: wl_domain_fees_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE wl_domain_fees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wl_domain_fees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE wl_domain_fees_id_seq OWNED BY wl_domain_fees.id;


--
-- Name: wl_domain_font_files; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE wl_domain_font_files (
    id integer NOT NULL,
    wl_domain_font_id integer,
    ext_url character varying,
    format integer,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    font_file character varying
);


--
-- Name: wl_domain_font_files_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE wl_domain_font_files_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wl_domain_font_files_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE wl_domain_font_files_id_seq OWNED BY wl_domain_font_files.id;


--
-- Name: wl_domain_fonts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE wl_domain_fonts (
    id integer NOT NULL,
    wl_domain_id integer,
    font_family character varying DEFAULT ''::character varying,
    font_weight character varying DEFAULT 'normal'::character varying,
    font_style character varying DEFAULT 'normal'::character varying,
    "default" boolean,
    created_at timestamp without time zone,
    updated_at timestamp without time zone
);


--
-- Name: wl_domain_fonts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE wl_domain_fonts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wl_domain_fonts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE wl_domain_fonts_id_seq OWNED BY wl_domain_fonts.id;


--
-- Name: wl_domains; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE wl_domains (
    id integer NOT NULL,
    title character varying NOT NULL,
    host character varying NOT NULL,
    locale character varying(2) DEFAULT 'en'::character varying NOT NULL,
    status character varying DEFAULT 'new'::character varying,
    faq_url text,
    styles json NOT NULL,
    is_default boolean DEFAULT false NOT NULL,
    deleted boolean DEFAULT false NOT NULL,
    admob_plugin_labels json DEFAULT '{}'::json NOT NULL,
    allow_user_activate boolean DEFAULT false,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    not_paid boolean DEFAULT false,
    favicon character varying,
    logo character varying,
    logo_bottom character varying
);


--
-- Name: wl_domains_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE wl_domains_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wl_domains_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE wl_domains_id_seq OWNED BY wl_domains.id;


--
-- Name: yandex_accounts; Type: TABLE; Schema: public; Owner: -
--

CREATE TABLE yandex_accounts (
    id integer NOT NULL,
    login character varying(255) NOT NULL,
    password character varying(255),
    is_default boolean DEFAULT false NOT NULL,
    created_at timestamp without time zone,
    updated_at timestamp without time zone,
    oauth_token character varying(255),
    client_id character varying(255)
);


--
-- Name: yandex_accounts_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE yandex_accounts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: yandex_accounts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE yandex_accounts_id_seq OWNED BY yandex_accounts.id;


--
-- Name: account_employees id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY account_employees ALTER COLUMN id SET DEFAULT nextval('account_employees_id_seq'::regclass);


--
-- Name: account_shares id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY account_shares ALTER COLUMN id SET DEFAULT nextval('account_shares_id_seq'::regclass);


--
-- Name: active_admin_comments id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY active_admin_comments ALTER COLUMN id SET DEFAULT nextval('active_admin_comments_id_seq'::regclass);


--
-- Name: active_apps id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY active_apps ALTER COLUMN id SET DEFAULT nextval('active_apps_id_seq'::regclass);


--
-- Name: active_users id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY active_users ALTER COLUMN id SET DEFAULT nextval('active_users_id_seq'::regclass);


--
-- Name: ad_request_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_request_settings ALTER COLUMN id SET DEFAULT nextval('ad_request_settings_id_seq'::regclass);


--
-- Name: ad_unit_templates id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_unit_templates ALTER COLUMN id SET DEFAULT nextval('ad_unit_templates_id_seq'::regclass);


--
-- Name: ad_unit_versions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_unit_versions ALTER COLUMN id SET DEFAULT nextval('ad_unit_versions_id_seq'::regclass);


--
-- Name: ad_units_backup id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_units_backup ALTER COLUMN id SET DEFAULT nextval('ad_units_id_seq'::regclass);


--
-- Name: adcolony_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY adcolony_accounts ALTER COLUMN id SET DEFAULT nextval('adcolony_accounts_id_seq'::regclass);


--
-- Name: additional_user_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY additional_user_logs ALTER COLUMN id SET DEFAULT nextval('additional_user_logs_id_seq'::regclass);


--
-- Name: admob_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_accounts ALTER COLUMN id SET DEFAULT nextval('admob_accounts_id_seq'::regclass);


--
-- Name: admob_ad_unit_versions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_ad_unit_versions ALTER COLUMN id SET DEFAULT nextval('admob_ad_unit_versions_id_seq'::regclass);


--
-- Name: admob_app_bid_floors id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_app_bid_floors ALTER COLUMN id SET DEFAULT nextval('admob_app_bid_floors_id_seq'::regclass);


--
-- Name: admob_user_bid_floors id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_user_bid_floors ALTER COLUMN id SET DEFAULT nextval('admob_user_bid_floors_id_seq'::regclass);


--
-- Name: adms id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY adms ALTER COLUMN id SET DEFAULT nextval('adms_id_seq'::regclass);


--
-- Name: advertisement_partners id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY advertisement_partners ALTER COLUMN id SET DEFAULT nextval('advertisement_partners_id_seq'::regclass);


--
-- Name: ae_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY ae_settings ALTER COLUMN id SET DEFAULT nextval('ae_settings_id_seq'::regclass);


--
-- Name: amazon_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY amazon_accounts ALTER COLUMN id SET DEFAULT nextval('amazon_accounts_id_seq'::regclass);


--
-- Name: api_queues id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY api_queues ALTER COLUMN id SET DEFAULT nextval('api_queues_id_seq'::regclass);


--
-- Name: app_fraud_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_fraud_logs ALTER COLUMN id SET DEFAULT nextval('app_fraud_logs_id_seq'::regclass);


--
-- Name: app_frauds id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_frauds ALTER COLUMN id SET DEFAULT nextval('app_frauds_id_seq'::regclass);


--
-- Name: app_installer_data id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_installer_data ALTER COLUMN id SET DEFAULT nextval('app_installer_data_id_seq'::regclass);


--
-- Name: app_network_profiles id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_network_profiles ALTER COLUMN id SET DEFAULT nextval('app_network_profiles_id_seq'::regclass);


--
-- Name: app_revenue_shares id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_revenue_shares ALTER COLUMN id SET DEFAULT nextval('app_revenue_shares_id_seq'::regclass);


--
-- Name: appflood_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY appflood_accounts ALTER COLUMN id SET DEFAULT nextval('appflood_accounts_id_seq'::regclass);


--
-- Name: applovin_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY applovin_accounts ALTER COLUMN id SET DEFAULT nextval('applovin_accounts_id_seq'::regclass);


--
-- Name: applovin_zones id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY applovin_zones ALTER COLUMN id SET DEFAULT nextval('applovin_zones_id_seq'::regclass);


--
-- Name: appnexus_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY appnexus_accounts ALTER COLUMN id SET DEFAULT nextval('appnexus_accounts_id_seq'::regclass);


--
-- Name: apps id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps ALTER COLUMN id SET DEFAULT nextval('apps_id_seq'::regclass);


--
-- Name: apps_campaign_images id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_campaign_images ALTER COLUMN id SET DEFAULT nextval('apps_campaign_images_id_seq'::regclass);


--
-- Name: apps_categories id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_categories ALTER COLUMN id SET DEFAULT nextval('apps_categories_id_seq'::regclass);


--
-- Name: apps_network_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_network_accounts ALTER COLUMN id SET DEFAULT nextval('apps_network_accounts_id_seq'::regclass);


--
-- Name: apps_website_links id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_website_links ALTER COLUMN id SET DEFAULT nextval('apps_website_links_id_seq'::regclass);


--
-- Name: avocarrot_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY avocarrot_accounts ALTER COLUMN id SET DEFAULT nextval('avocarrot_accounts_id_seq'::regclass);


--
-- Name: bad_devices id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY bad_devices ALTER COLUMN id SET DEFAULT nextval('bad_devices_id_seq'::regclass);


--
-- Name: bid_floor_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY bid_floor_settings ALTER COLUMN id SET DEFAULT nextval('bid_floor_settings_id_seq'::regclass);


--
-- Name: bidder_bidder_formats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_bidder_formats ALTER COLUMN id SET DEFAULT nextval('bidder_bidder_formats_id_seq'::regclass);


--
-- Name: bidder_campaigns id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_campaigns ALTER COLUMN id SET DEFAULT nextval('bidder_campaigns_id_seq'::regclass);


--
-- Name: bidder_countries id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_countries ALTER COLUMN id SET DEFAULT nextval('bidder_countries_id_seq'::regclass);


--
-- Name: bidder_formats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_formats ALTER COLUMN id SET DEFAULT nextval('bidder_formats_id_seq'::regclass);


--
-- Name: bidder_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_settings ALTER COLUMN id SET DEFAULT nextval('bidder_settings_id_seq'::regclass);


--
-- Name: billings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2016m10 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m10 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2016m10 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m10 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2016m10 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m10 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2016m10 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m10 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2016m11 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m11 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2016m11 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m11 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2016m11 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m11 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2016m11 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m11 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2016m12 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m12 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2016m12 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m12 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2016m12 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m12 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2016m12 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m12 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m01 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m01 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m01 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m01 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m01 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m01 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m01 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m01 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m02 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m02 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m02 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m02 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m02 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m02 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m02 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m02 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m03 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m03 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m03 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m03 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m03 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m03 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m03 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m03 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m04 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m04 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m04 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m04 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m04 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m04 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m04 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m04 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m05 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m05 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m05 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m05 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m05 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m05 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m05 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m05 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m06 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m06 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m06 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m06 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m06 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m06 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m06 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m06 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m07 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m07 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m07 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m07 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m07 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m07 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m07 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m07 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m08 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m08 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m08 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m08 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m08 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m08 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m08 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m08 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m09 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m09 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m09 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m09 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m09 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m09 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m09 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m09 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m10 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m10 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m10 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m10 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m10 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m10 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m10 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m10 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m11 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m11 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m11 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m11 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m11 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m11 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m11 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m11 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2017m12 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m12 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2017m12 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m12 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2017m12 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m12 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2017m12 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m12 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m01 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m01 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m01 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m01 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m01 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m01 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m01 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m01 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m02 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m02 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m02 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m02 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m02 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m02 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m02 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m02 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m03 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m03 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m03 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m03 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m03 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m03 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m03 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m03 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m04 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m04 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m04 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m04 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m04 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m04 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m04 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m04 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m05 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m05 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m05 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m05 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m05 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m05 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m05 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m05 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m06 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m06 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m06 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m06 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m06 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m06 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m06 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m06 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m07 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m07 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m07 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m07 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m07 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m07 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m07 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m07 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m08 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m08 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m08 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m08 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m08 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m08 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m08 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m08 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m09 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m09 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m09 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m09 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m09 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m09 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m09 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m09 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m10 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m10 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m10 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m10 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m10 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m10 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m10 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m10 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: billings_y2018m11 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m11 ALTER COLUMN id SET DEFAULT nextval('billings_id_seq'::regclass);


--
-- Name: billings_y2018m11 amount; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m11 ALTER COLUMN amount SET DEFAULT 0.0;


--
-- Name: billings_y2018m11 balance; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m11 ALTER COLUMN balance SET DEFAULT 0.0;


--
-- Name: billings_y2018m11 fee; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m11 ALTER COLUMN fee SET DEFAULT 0.0;


--
-- Name: block_devices id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY block_devices ALTER COLUMN id SET DEFAULT nextval('block_devices_id_seq'::regclass);


--
-- Name: book_keeping_reports id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY book_keeping_reports ALTER COLUMN id SET DEFAULT nextval('book_keeping_reports_id_seq'::regclass);


--
-- Name: cached_files id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY cached_files ALTER COLUMN id SET DEFAULT nextval('cached_files_id_seq'::regclass);


--
-- Name: campaign_aggregated_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_aggregated_stats ALTER COLUMN id SET DEFAULT nextval('campaign_aggregated_stats_id_seq'::regclass);


--
-- Name: campaign_commissions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_commissions ALTER COLUMN id SET DEFAULT nextval('campaign_commissions_id_seq'::regclass);


--
-- Name: campaign_coordinates id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_coordinates ALTER COLUMN id SET DEFAULT nextval('campaign_coordinates_id_seq'::regclass);


--
-- Name: campaign_image_coordinates id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_coordinates ALTER COLUMN id SET DEFAULT nextval('campaign_image_coordinates_id_seq'::regclass);


--
-- Name: campaign_image_coordinates_images id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_coordinates_images ALTER COLUMN id SET DEFAULT nextval('campaign_image_coordinates_images_id_seq'::regclass);


--
-- Name: campaign_image_devices id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_devices ALTER COLUMN id SET DEFAULT nextval('campaign_image_devices_id_seq'::regclass);


--
-- Name: campaign_image_devices_csvs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_devices_csvs ALTER COLUMN id SET DEFAULT nextval('campaign_image_devices_csvs_id_seq'::regclass);


--
-- Name: campaign_images id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images ALTER COLUMN id SET DEFAULT nextval('campaign_images_id_seq'::regclass);


--
-- Name: campaign_images_categories id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images_categories ALTER COLUMN id SET DEFAULT nextval('campaign_images_categories_id_seq'::regclass);


--
-- Name: campaign_images_countries id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images_countries ALTER COLUMN id SET DEFAULT nextval('campaign_images_countries_id_seq'::regclass);


--
-- Name: campaign_images_max_mind_cities id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images_max_mind_cities ALTER COLUMN id SET DEFAULT nextval('campaign_images_max_mind_cities_id_seq'::regclass);


--
-- Name: campaign_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_settings ALTER COLUMN id SET DEFAULT nextval('campaign_settings_id_seq'::regclass);


--
-- Name: campaign_types id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_types ALTER COLUMN id SET DEFAULT nextval('campaign_types_id_seq'::regclass);


--
-- Name: campaign_versions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_versions ALTER COLUMN id SET DEFAULT nextval('campaign_versions_id_seq'::regclass);


--
-- Name: campaigns id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaigns ALTER COLUMN id SET DEFAULT nextval('campaigns_id_seq'::regclass);


--
-- Name: campaigns_creative_sets id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaigns_creative_sets ALTER COLUMN id SET DEFAULT nextval('campaigns_creative_sets_id_seq'::regclass);


--
-- Name: categories id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY categories ALTER COLUMN id SET DEFAULT nextval('categories_id_seq'::regclass);


--
-- Name: categories_contents id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY categories_contents ALTER COLUMN id SET DEFAULT nextval('categories_contents_id_seq'::regclass);


--
-- Name: categories_users id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY categories_users ALTER COLUMN id SET DEFAULT nextval('categories_users_id_seq'::regclass);


--
-- Name: changelogs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY changelogs ALTER COLUMN id SET DEFAULT nextval('changelogs_id_seq'::regclass);


--
-- Name: chartboost_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY chartboost_accounts ALTER COLUMN id SET DEFAULT nextval('chartboost_accounts_id_seq'::regclass);


--
-- Name: chartboost_campaigns id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY chartboost_campaigns ALTER COLUMN id SET DEFAULT nextval('chartboost_campaigns_id_seq'::regclass);


--
-- Name: cities id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY cities ALTER COLUMN id SET DEFAULT nextval('cities_id_seq'::regclass);


--
-- Name: clicks id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks ALTER COLUMN id SET DEFAULT nextval('clicks_id_seq'::regclass);


--
-- Name: clicks_y2018m08 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m08 ALTER COLUMN id SET DEFAULT nextval('clicks_id_seq'::regclass);


--
-- Name: clicks_y2018m08 white_sdk; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m08 ALTER COLUMN white_sdk SET DEFAULT false;


--
-- Name: clicks_y2018m09 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m09 ALTER COLUMN id SET DEFAULT nextval('clicks_id_seq'::regclass);


--
-- Name: clicks_y2018m09 white_sdk; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m09 ALTER COLUMN white_sdk SET DEFAULT false;


--
-- Name: clicks_y2018m10 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m10 ALTER COLUMN id SET DEFAULT nextval('clicks_id_seq'::regclass);


--
-- Name: clicks_y2018m10 white_sdk; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m10 ALTER COLUMN white_sdk SET DEFAULT false;


--
-- Name: clicks_y2018m11 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m11 ALTER COLUMN id SET DEFAULT nextval('clicks_id_seq'::regclass);


--
-- Name: clicks_y2018m11 white_sdk; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m11 ALTER COLUMN white_sdk SET DEFAULT false;


--
-- Name: client_histories id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY client_histories ALTER COLUMN id SET DEFAULT nextval('client_histories_id_seq'::regclass);


--
-- Name: client_history_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY client_history_stats ALTER COLUMN id SET DEFAULT nextval('client_history_stats_id_seq'::regclass);


--
-- Name: compensations id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY compensations ALTER COLUMN id SET DEFAULT nextval('compensations_id_seq'::regclass);


--
-- Name: complains id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY complains ALTER COLUMN id SET DEFAULT nextval('complains_id_seq'::regclass);


--
-- Name: contents id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY contents ALTER COLUMN id SET DEFAULT nextval('contents_id_seq'::regclass);


--
-- Name: countries id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY countries ALTER COLUMN id SET DEFAULT nextval('countries_id_seq'::regclass);


--
-- Name: creative_files id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY creative_files ALTER COLUMN id SET DEFAULT nextval('creative_files_id_seq'::regclass);


--
-- Name: creative_items id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY creative_items ALTER COLUMN id SET DEFAULT nextval('creative_items_id_seq'::regclass);


--
-- Name: creative_sets id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY creative_sets ALTER COLUMN id SET DEFAULT nextval('creative_sets_id_seq'::regclass);


--
-- Name: custom_app_bid_floors id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY custom_app_bid_floors ALTER COLUMN id SET DEFAULT nextval('custom_app_bid_floors_id_seq'::regclass);


--
-- Name: custom_user_commissions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY custom_user_commissions ALTER COLUMN id SET DEFAULT nextval('custom_user_commissions_id_seq'::regclass);


--
-- Name: dash_views_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY dash_views_logs ALTER COLUMN id SET DEFAULT nextval('dash_views_logs_id_seq'::regclass);


--
-- Name: dashboard_predefined_filters id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY dashboard_predefined_filters ALTER COLUMN id SET DEFAULT nextval('dashboard_predefined_filters_id_seq'::regclass);


--
-- Name: deals id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY deals ALTER COLUMN id SET DEFAULT nextval('deals_id_seq'::regclass);


--
-- Name: device_manufacturers id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY device_manufacturers ALTER COLUMN id SET DEFAULT nextval('device_manufacturers_id_seq'::regclass);


--
-- Name: device_models id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY device_models ALTER COLUMN id SET DEFAULT nextval('device_models_id_seq'::regclass);


--
-- Name: devices id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY devices ALTER COLUMN id SET DEFAULT nextval('devices_id_seq'::regclass);


--
-- Name: disabled_networks id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY disabled_networks ALTER COLUMN id SET DEFAULT nextval('disabled_networks_id_seq'::regclass);


--
-- Name: docs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs ALTER COLUMN id SET DEFAULT nextval('docs_id_seq'::regclass);


--
-- Name: docs_histories id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs_histories ALTER COLUMN id SET DEFAULT nextval('docs_histories_id_seq'::regclass);


--
-- Name: docs_sections id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs_sections ALTER COLUMN id SET DEFAULT nextval('docs_sections_id_seq'::regclass);


--
-- Name: docs_sets id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs_sets ALTER COLUMN id SET DEFAULT nextval('docs_sets_id_seq'::regclass);


--
-- Name: domain_font_files id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY domain_font_files ALTER COLUMN id SET DEFAULT nextval('domain_font_files_id_seq'::regclass);


--
-- Name: domain_fonts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY domain_fonts ALTER COLUMN id SET DEFAULT nextval('domain_fonts_id_seq'::regclass);


--
-- Name: domains id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY domains ALTER COLUMN id SET DEFAULT nextval('domains_id_seq'::regclass);


--
-- Name: email_smtp_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY email_smtp_settings ALTER COLUMN id SET DEFAULT nextval('email_smtp_settings_id_seq'::regclass);


--
-- Name: email_templates id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY email_templates ALTER COLUMN id SET DEFAULT nextval('email_templates_id_seq'::regclass);


--
-- Name: exceptional_targetings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY exceptional_targetings ALTER COLUMN id SET DEFAULT nextval('exceptional_targetings_id_seq'::regclass);


--
-- Name: exchange_coefficients id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY exchange_coefficients ALTER COLUMN id SET DEFAULT nextval('exchange_coefficients_id_seq'::regclass);


--
-- Name: exchange_rates id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY exchange_rates ALTER COLUMN id SET DEFAULT nextval('exchange_rates_id_seq'::regclass);


--
-- Name: extension_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY extension_logs ALTER COLUMN id SET DEFAULT nextval('extension_logs_id_seq'::regclass);


--
-- Name: facebook_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY facebook_accounts ALTER COLUMN id SET DEFAULT nextval('facebook_accounts_id_seq'::regclass);


--
-- Name: fast_affiliate_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_affiliate_stats ALTER COLUMN id SET DEFAULT nextval('fast_affiliate_stats_id_seq'::regclass);


--
-- Name: fast_ecpm_capping_options id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_capping_options ALTER COLUMN id SET DEFAULT nextval('fast_ecpm_capping_options_id_seq'::regclass);


--
-- Name: fast_ecpm_comparison_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_comparison_stats ALTER COLUMN id SET DEFAULT nextval('fast_ecpm_comparison_stats_id_seq'::regclass);


--
-- Name: fast_ecpm_comparison_stats_loggers id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_comparison_stats_loggers ALTER COLUMN id SET DEFAULT nextval('fast_ecpm_comparison_stats_loggers_id_seq'::regclass);


--
-- Name: fast_ecpm_new_options id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_new_options ALTER COLUMN id SET DEFAULT nextval('fast_ecpm_new_options_id_seq'::regclass);


--
-- Name: fast_ecpm_stats_exclusions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_stats_exclusions ALTER COLUMN id SET DEFAULT nextval('fast_ecpm_stats_exclusions_id_seq'::regclass);


--
-- Name: fast_ecpm_stats_old id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_stats_old ALTER COLUMN id SET DEFAULT nextval('fast_ecpm_stats_id_seq'::regclass);


--
-- Name: finalized_time_for_reports id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY finalized_time_for_reports ALTER COLUMN id SET DEFAULT nextval('finalized_time_for_reports_id_seq'::regclass);


--
-- Name: fixed_admob_campaigns_sets id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fixed_admob_campaigns_sets ALTER COLUMN id SET DEFAULT nextval('fixed_admob_campaigns_sets_id_seq'::regclass);


--
-- Name: flurry_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY flurry_accounts ALTER COLUMN id SET DEFAULT nextval('flurry_accounts_id_seq'::regclass);


--
-- Name: fraud_click_series id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_click_series ALTER COLUMN id SET DEFAULT nextval('fraud_click_series_id_seq'::regclass);


--
-- Name: fraud_detections id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_detections ALTER COLUMN id SET DEFAULT nextval('fraud_detections_id_seq'::regclass);


--
-- Name: fraud_user_emails id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_user_emails ALTER COLUMN id SET DEFAULT nextval('fraud_user_emails_id_seq'::regclass);


--
-- Name: fraud_user_keys id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_user_keys ALTER COLUMN id SET DEFAULT nextval('fraud_user_keys_id_seq'::regclass);


--
-- Name: frequency_capping_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY frequency_capping_stats ALTER COLUMN id SET DEFAULT nextval('frequency_capping_stats_id_seq'::regclass);


--
-- Name: global_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY global_settings ALTER COLUMN id SET DEFAULT nextval('global_settings_id_seq'::regclass);


--
-- Name: global_settings_sections id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY global_settings_sections ALTER COLUMN id SET DEFAULT nextval('global_settings_sections_id_seq'::regclass);


--
-- Name: globals id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY globals ALTER COLUMN id SET DEFAULT nextval('globals_id_seq'::regclass);


--
-- Name: graph_ql_mutations_tracking_data id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY graph_ql_mutations_tracking_data ALTER COLUMN id SET DEFAULT nextval('graph_ql_mutations_tracking_data_id_seq'::regclass);


--
-- Name: heyzap_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY heyzap_accounts ALTER COLUMN id SET DEFAULT nextval('heyzap_accounts_id_seq'::regclass);


--
-- Name: inmobi_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY inmobi_accounts ALTER COLUMN id SET DEFAULT nextval('inmobi_accounts_id_seq'::regclass);


--
-- Name: inmobi_ad_units id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY inmobi_ad_units ALTER COLUMN id SET DEFAULT nextval('inmobi_ad_units_id_seq'::regclass);


--
-- Name: inner_active_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY inner_active_accounts ALTER COLUMN id SET DEFAULT nextval('inner_active_accounts_id_seq'::regclass);


--
-- Name: insight_stat_histories id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stat_histories ALTER COLUMN id SET DEFAULT nextval('insight_stat_histories_id_seq'::regclass);


--
-- Name: insight_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats ALTER COLUMN id SET DEFAULT nextval('insight_stats_id_seq'::regclass);


--
-- Name: insight_stats_y2018m03 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m03 ALTER COLUMN id SET DEFAULT nextval('insight_stats_id_seq'::regclass);


--
-- Name: insight_stats_y2018m03 ecpm; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m03 ALTER COLUMN ecpm SET DEFAULT 0;


--
-- Name: insight_stats_y2018m03 ctr; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m03 ALTER COLUMN ctr SET DEFAULT 0;


--
-- Name: insight_stats_y2018m03 revenue; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m03 ALTER COLUMN revenue SET DEFAULT 0;


--
-- Name: insight_stats_y2018m04 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m04 ALTER COLUMN id SET DEFAULT nextval('insight_stats_id_seq'::regclass);


--
-- Name: insight_stats_y2018m04 ecpm; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m04 ALTER COLUMN ecpm SET DEFAULT 0;


--
-- Name: insight_stats_y2018m04 ctr; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m04 ALTER COLUMN ctr SET DEFAULT 0;


--
-- Name: insight_stats_y2018m04 revenue; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m04 ALTER COLUMN revenue SET DEFAULT 0;


--
-- Name: insight_stats_y2018m05 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m05 ALTER COLUMN id SET DEFAULT nextval('insight_stats_id_seq'::regclass);


--
-- Name: insight_stats_y2018m05 ecpm; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m05 ALTER COLUMN ecpm SET DEFAULT 0;


--
-- Name: insight_stats_y2018m05 ctr; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m05 ALTER COLUMN ctr SET DEFAULT 0;


--
-- Name: insight_stats_y2018m05 revenue; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m05 ALTER COLUMN revenue SET DEFAULT 0;


--
-- Name: integration_frameworks id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY integration_frameworks ALTER COLUMN id SET DEFAULT nextval('integration_frameworks_id_seq'::regclass);


--
-- Name: intercom_conversation_parts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY intercom_conversation_parts ALTER COLUMN id SET DEFAULT nextval('intercom_conversation_parts_id_seq'::regclass);


--
-- Name: intercom_conversations id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY intercom_conversations ALTER COLUMN id SET DEFAULT nextval('intercom_conversations_id_seq'::regclass);


--
-- Name: intercom_network_recommendeds id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY intercom_network_recommendeds ALTER COLUMN id SET DEFAULT nextval('intercom_network_recommendeds_id_seq'::regclass);


--
-- Name: invoice_requests id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY invoice_requests ALTER COLUMN id SET DEFAULT nextval('invoice_requests_id_seq'::regclass);


--
-- Name: ip_addresses id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY ip_addresses ALTER COLUMN id SET DEFAULT nextval('ip_addresses_id_seq'::regclass);


--
-- Name: liverail_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY liverail_accounts ALTER COLUMN id SET DEFAULT nextval('liverail_accounts_id_seq'::regclass);


--
-- Name: liverail_ad_units id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY liverail_ad_units ALTER COLUMN id SET DEFAULT nextval('liverail_ad_units_id_seq'::regclass);


--
-- Name: long_term_account_revenues id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY long_term_account_revenues ALTER COLUMN id SET DEFAULT nextval('long_term_account_revenues_id_seq'::regclass);


--
-- Name: magic_monies id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY magic_monies ALTER COLUMN id SET DEFAULT nextval('magic_monies_id_seq'::regclass);


--
-- Name: managers id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY managers ALTER COLUMN id SET DEFAULT nextval('managers_id_seq'::regclass);


--
-- Name: max_mind_cities id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY max_mind_cities ALTER COLUMN id SET DEFAULT nextval('max_mind_cities_id_seq'::regclass);


--
-- Name: mmedia_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY mmedia_accounts ALTER COLUMN id SET DEFAULT nextval('mmedia_accounts_id_seq'::regclass);


--
-- Name: mobile_networks id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY mobile_networks ALTER COLUMN id SET DEFAULT nextval('mobile_networks_id_seq'::regclass);


--
-- Name: mobilecore_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY mobilecore_accounts ALTER COLUMN id SET DEFAULT nextval('mobilecore_accounts_id_seq'::regclass);


--
-- Name: monitor_events id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY monitor_events ALTER COLUMN id SET DEFAULT nextval('monitor_events_id_seq'::regclass);


--
-- Name: monthly_book_keeping_reports id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY monthly_book_keeping_reports ALTER COLUMN id SET DEFAULT nextval('monthly_book_keeping_reports_id_seq'::regclass);


--
-- Name: mopub_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY mopub_accounts ALTER COLUMN id SET DEFAULT nextval('mopub_accounts_id_seq'::regclass);


--
-- Name: network_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_accounts ALTER COLUMN id SET DEFAULT nextval('network_accounts_id_seq'::regclass);


--
-- Name: network_active_types id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_active_types ALTER COLUMN id SET DEFAULT nextval('network_active_types_id_seq'::regclass);


--
-- Name: network_connection_restrictions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_connection_restrictions ALTER COLUMN id SET DEFAULT nextval('network_connection_restrictions_id_seq'::regclass);


--
-- Name: network_connection_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_connection_settings ALTER COLUMN id SET DEFAULT nextval('network_connection_settings_id_seq'::regclass);


--
-- Name: network_connection_statuses id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_connection_statuses ALTER COLUMN id SET DEFAULT nextval('network_connection_statuses_id_seq'::regclass);


--
-- Name: network_content_filters id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_content_filters ALTER COLUMN id SET DEFAULT nextval('network_content_filters_id_seq'::regclass);


--
-- Name: network_daemon_jobs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_daemon_jobs ALTER COLUMN id SET DEFAULT nextval('network_daemon_jobs_id_seq'::regclass);


--
-- Name: network_mailing_app_statuses id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_mailing_app_statuses ALTER COLUMN id SET DEFAULT nextval('network_mailing_app_statuses_id_seq'::regclass);


--
-- Name: network_mailing_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_mailing_settings ALTER COLUMN id SET DEFAULT nextval('network_mailing_settings_id_seq'::regclass);


--
-- Name: network_migrations id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_migrations ALTER COLUMN id SET DEFAULT nextval('network_migrations_id_seq'::regclass);


--
-- Name: network_reporting_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_reporting_settings ALTER COLUMN id SET DEFAULT nextval('network_reporting_settings_id_seq'::regclass);


--
-- Name: network_request_restrictions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_request_restrictions ALTER COLUMN id SET DEFAULT nextval('network_request_restrictions_id_seq'::regclass);


--
-- Name: nexage_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY nexage_accounts ALTER COLUMN id SET DEFAULT nextval('nexage_accounts_id_seq'::regclass);


--
-- Name: openx_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY openx_accounts ALTER COLUMN id SET DEFAULT nextval('openx_accounts_id_seq'::regclass);


--
-- Name: organization_commission_presets id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization_commission_presets ALTER COLUMN id SET DEFAULT nextval('organization_commission_presets_id_seq'::regclass);


--
-- Name: organization_roles id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization_roles ALTER COLUMN id SET DEFAULT nextval('organization_roles_id_seq'::regclass);


--
-- Name: organizations id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY organizations ALTER COLUMN id SET DEFAULT nextval('organizations_id_seq'::regclass);


--
-- Name: page_categories_apps id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY page_categories_apps ALTER COLUMN id SET DEFAULT nextval('page_categories_apps_id_seq'::regclass);


--
-- Name: partner_app_commissions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY partner_app_commissions ALTER COLUMN id SET DEFAULT nextval('partner_app_commissions_id_seq'::regclass);


--
-- Name: payment_requests id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY payment_requests ALTER COLUMN id SET DEFAULT nextval('payment_requests_id_seq'::regclass);


--
-- Name: payouts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY payouts ALTER COLUMN id SET DEFAULT nextval('payouts_id_seq'::regclass);


--
-- Name: platforms id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY platforms ALTER COLUMN id SET DEFAULT nextval('platforms_id_seq'::regclass);


--
-- Name: premium_user_commissions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY premium_user_commissions ALTER COLUMN id SET DEFAULT nextval('premium_user_commissions_id_seq'::regclass);


--
-- Name: price_floors id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY price_floors ALTER COLUMN id SET DEFAULT nextval('price_floors_id_seq'::regclass);


--
-- Name: publisher_monitor_messages id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY publisher_monitor_messages ALTER COLUMN id SET DEFAULT nextval('publisher_monitor_messages_id_seq'::regclass);


--
-- Name: pubnative_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY pubnative_accounts ALTER COLUMN id SET DEFAULT nextval('pubnative_accounts_id_seq'::regclass);


--
-- Name: qubole_sdk_data id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY qubole_sdk_data ALTER COLUMN id SET DEFAULT nextval('qubole_sdk_data_id_seq'::regclass);


--
-- Name: report_processing_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY report_processing_logs ALTER COLUMN id SET DEFAULT nextval('report_processing_logs_id_seq'::regclass);


--
-- Name: reports id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY reports ALTER COLUMN id SET DEFAULT nextval('reports_id_seq'::regclass);


--
-- Name: request_blacklists id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY request_blacklists ALTER COLUMN id SET DEFAULT nextval('request_blacklists_id_seq'::regclass);


--
-- Name: revenue_analysis_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY revenue_analysis_settings ALTER COLUMN id SET DEFAULT nextval('revenue_analysis_settings_id_seq'::regclass);


--
-- Name: revenue_change_limit_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY revenue_change_limit_settings ALTER COLUMN id SET DEFAULT nextval('revenue_change_limit_settings_id_seq'::regclass);


--
-- Name: revenue_comparison_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY revenue_comparison_stats ALTER COLUMN id SET DEFAULT nextval('revenue_comparison_stats_id_seq'::regclass);


--
-- Name: roles id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY roles ALTER COLUMN id SET DEFAULT nextval('roles_id_seq'::regclass);


--
-- Name: rubicon_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY rubicon_accounts ALTER COLUMN id SET DEFAULT nextval('rubicon_accounts_id_seq'::regclass);


--
-- Name: sdk_mapping_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY sdk_mapping_settings ALTER COLUMN id SET DEFAULT nextval('sdk_mapping_settings_id_seq'::regclass);


--
-- Name: sdk_releases id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY sdk_releases ALTER COLUMN id SET DEFAULT nextval('sdk_releases_id_seq'::regclass);


--
-- Name: section_categories_apps id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY section_categories_apps ALTER COLUMN id SET DEFAULT nextval('section_categories_apps_id_seq'::regclass);


--
-- Name: segment_placements id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_placements ALTER COLUMN id SET DEFAULT nextval('segment_placements_id_seq'::regclass);


--
-- Name: segment_rule_countries id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_rule_countries ALTER COLUMN id SET DEFAULT nextval('segment_rule_countries_id_seq'::regclass);


--
-- Name: segment_rules id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_rules ALTER COLUMN id SET DEFAULT nextval('segment_rules_id_seq'::regclass);


--
-- Name: segment_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_settings ALTER COLUMN id SET DEFAULT nextval('segment_settings_id_seq'::regclass);


--
-- Name: segments id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY segments ALTER COLUMN id SET DEFAULT nextval('segments_id_seq'::regclass);


--
-- Name: share_tokens id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY share_tokens ALTER COLUMN id SET DEFAULT nextval('share_tokens_id_seq'::regclass);


--
-- Name: simple_dashboard id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard ALTER COLUMN id SET DEFAULT nextval('simple_dashboard_id_seq'::regclass);


--
-- Name: simple_dashboard_monthly id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_monthly ALTER COLUMN id SET DEFAULT nextval('simple_dashboard_monthly_id_seq'::regclass);


--
-- Name: simple_dashboard_quarterly id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_quarterly ALTER COLUMN id SET DEFAULT nextval('simple_dashboard_quarterly_id_seq'::regclass);


--
-- Name: simple_dashboard_weekly id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_weekly ALTER COLUMN id SET DEFAULT nextval('simple_dashboard_weekly_id_seq'::regclass);


--
-- Name: simple_dashboard_yearly id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_yearly ALTER COLUMN id SET DEFAULT nextval('simple_dashboard_yearly_id_seq'::regclass);


--
-- Name: smaato_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY smaato_accounts ALTER COLUMN id SET DEFAULT nextval('smaato_accounts_id_seq'::regclass);


--
-- Name: spotx_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY spotx_accounts ALTER COLUMN id SET DEFAULT nextval('spotx_accounts_id_seq'::regclass);


--
-- Name: startapp_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY startapp_accounts ALTER COLUMN id SET DEFAULT nextval('startapp_accounts_id_seq'::regclass);


--
-- Name: statistic_reports id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY statistic_reports ALTER COLUMN id SET DEFAULT nextval('statistic_reports_id_seq'::regclass);


--
-- Name: statistics id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY statistics ALTER COLUMN id SET DEFAULT nextval('statistics_id_seq'::regclass);


--
-- Name: store_infos id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY store_infos ALTER COLUMN id SET DEFAULT nextval('store_infos_id_seq'::regclass);


--
-- Name: subdivision_ones id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY subdivision_ones ALTER COLUMN id SET DEFAULT nextval('subdivision_ones_id_seq'::regclass);


--
-- Name: subdivision_twos id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY subdivision_twos ALTER COLUMN id SET DEFAULT nextval('subdivision_twos_id_seq'::regclass);


--
-- Name: supersonic_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY supersonic_accounts ALTER COLUMN id SET DEFAULT nextval('supersonic_accounts_id_seq'::regclass);


--
-- Name: support_work_metriks id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY support_work_metriks ALTER COLUMN id SET DEFAULT nextval('support_work_metriks_id_seq'::regclass);


--
-- Name: support_work_reports id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY support_work_reports ALTER COLUMN id SET DEFAULT nextval('support_work_reports_id_seq'::regclass);


--
-- Name: survey_responses id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY survey_responses ALTER COLUMN id SET DEFAULT nextval('survey_responses_id_seq'::regclass);


--
-- Name: surveys id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY surveys ALTER COLUMN id SET DEFAULT nextval('surveys_id_seq'::regclass);


--
-- Name: tapsense_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY tapsense_accounts ALTER COLUMN id SET DEFAULT nextval('tapsense_accounts_id_seq'::regclass);


--
-- Name: target_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY target_accounts ALTER COLUMN id SET DEFAULT nextval('target_accounts_id_seq'::regclass);


--
-- Name: teams id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY teams ALTER COLUMN id SET DEFAULT nextval('teams_id_seq'::regclass);


--
-- Name: temp_apps_network_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY temp_apps_network_accounts ALTER COLUMN id SET DEFAULT nextval('temp_apps_network_accounts_id_seq'::regclass);


--
-- Name: test_device_ids id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY test_device_ids ALTER COLUMN id SET DEFAULT nextval('test_device_ids_id_seq'::regclass);


--
-- Name: traffic_limit_blocks id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY traffic_limit_blocks ALTER COLUMN id SET DEFAULT nextval('traffic_limit_blocks_id_seq'::regclass);


--
-- Name: traffic_limit_exclusions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY traffic_limit_exclusions ALTER COLUMN id SET DEFAULT nextval('traffic_limit_exclusions_id_seq'::regclass);


--
-- Name: unity_ads_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY unity_ads_accounts ALTER COLUMN id SET DEFAULT nextval('unity_ads_accounts_id_seq'::regclass);


--
-- Name: user_ach_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_ach_requisites ALTER COLUMN id SET DEFAULT nextval('user_ach_requisites_id_seq'::regclass);


--
-- Name: user_apps_commissions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_apps_commissions ALTER COLUMN id SET DEFAULT nextval('user_apps_commissions_id_seq'::regclass);


--
-- Name: user_bitcoin_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_bitcoin_requisites ALTER COLUMN id SET DEFAULT nextval('user_bitcoin_requisites_id_seq'::regclass);


--
-- Name: user_confirmations id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_confirmations ALTER COLUMN id SET DEFAULT nextval('user_confirmations_id_seq'::regclass);


--
-- Name: user_daily_stats id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_daily_stats ALTER COLUMN id SET DEFAULT nextval('user_daily_stats_id_seq'::regclass);


--
-- Name: user_ethereum_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_ethereum_requisites ALTER COLUMN id SET DEFAULT nextval('user_ethereum_requisites_id_seq'::regclass);


--
-- Name: user_fraud_warning_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_fraud_warning_logs ALTER COLUMN id SET DEFAULT nextval('user_fraud_warning_logs_id_seq'::regclass);


--
-- Name: user_google_wallet_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_google_wallet_requisites ALTER COLUMN id SET DEFAULT nextval('user_google_wallet_requisites_id_seq'::regclass);


--
-- Name: user_intercom_events id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_intercom_events ALTER COLUMN id SET DEFAULT nextval('user_intercom_events_id_seq'::regclass);


--
-- Name: user_login_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_login_logs ALTER COLUMN id SET DEFAULT nextval('user_login_logs_id_seq'::regclass);


--
-- Name: user_notifications id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_notifications ALTER COLUMN id SET DEFAULT nextval('user_notifications_id_seq'::regclass);


--
-- Name: user_payment_details_change_confirms id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payment_details_change_confirms ALTER COLUMN id SET DEFAULT nextval('user_payment_details_change_confirms_id_seq'::regclass);


--
-- Name: user_payoneer_bank_transfer_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payoneer_bank_transfer_requisites ALTER COLUMN id SET DEFAULT nextval('user_payoneer_bank_transfer_requisites_id_seq'::regclass);


--
-- Name: user_payoneer_paypal_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payoneer_paypal_requisites ALTER COLUMN id SET DEFAULT nextval('user_payoneer_paypal_requisites_id_seq'::regclass);


--
-- Name: user_payoneer_prepaid_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payoneer_prepaid_requisites ALTER COLUMN id SET DEFAULT nextval('user_payoneer_prepaid_requisites_id_seq'::regclass);


--
-- Name: user_paypal_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_paypal_requisites ALTER COLUMN id SET DEFAULT nextval('user_paypal_requisites_id_seq'::regclass);


--
-- Name: user_webmoney_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_webmoney_requisites ALTER COLUMN id SET DEFAULT nextval('user_webmoney_requisites_id_seq'::regclass);


--
-- Name: user_wire_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_wire_requisites ALTER COLUMN id SET DEFAULT nextval('user_wire_requisites_id_seq'::regclass);


--
-- Name: user_yandex_money_requisites id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_yandex_money_requisites ALTER COLUMN id SET DEFAULT nextval('user_yandex_money_requisites_id_seq'::regclass);


--
-- Name: users id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY users ALTER COLUMN id SET DEFAULT nextval('users_id_seq'::regclass);


--
-- Name: users_abilities id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_abilities ALTER COLUMN id SET DEFAULT nextval('users_abilities_id_seq'::regclass);


--
-- Name: users_admin_notifications id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_admin_notifications ALTER COLUMN id SET DEFAULT nextval('users_admin_notifications_id_seq'::regclass);


--
-- Name: users_dev_apps id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_dev_apps ALTER COLUMN id SET DEFAULT nextval('users_dev_apps_id_seq'::regclass);


--
-- Name: users_fake_apps id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_fake_apps ALTER COLUMN id SET DEFAULT nextval('users_fake_apps_id_seq'::regclass);


--
-- Name: users_managed_campaigns id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_managed_campaigns ALTER COLUMN id SET DEFAULT nextval('users_managed_campaigns_id_seq'::regclass);


--
-- Name: users_shared_apps id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_shared_apps ALTER COLUMN id SET DEFAULT nextval('users_shared_apps_id_seq'::regclass);


--
-- Name: v2_placements id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_placements ALTER COLUMN id SET DEFAULT nextval('v2_placements_id_seq'::regclass);


--
-- Name: v2_placements_versions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_placements_versions ALTER COLUMN id SET DEFAULT nextval('v2_placements_versions_id_seq'::regclass);


--
-- Name: v2_segments id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_segments ALTER COLUMN id SET DEFAULT nextval('v2_segments_id_seq'::regclass);


--
-- Name: v2_segments_versions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_segments_versions ALTER COLUMN id SET DEFAULT nextval('v2_segments_versions_id_seq'::regclass);


--
-- Name: versions id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY versions ALTER COLUMN id SET DEFAULT nextval('versions_id_seq'::regclass);


--
-- Name: vpn_ips id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips ALTER COLUMN id SET DEFAULT nextval('vpn_ips_id_seq'::regclass);


--
-- Name: vpn_ips_y2018m08 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m08 ALTER COLUMN id SET DEFAULT nextval('vpn_ips_id_seq'::regclass);


--
-- Name: vpn_ips_y2018m08 is_vpn; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m08 ALTER COLUMN is_vpn SET DEFAULT false;


--
-- Name: vpn_ips_y2018m09 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m09 ALTER COLUMN id SET DEFAULT nextval('vpn_ips_id_seq'::regclass);


--
-- Name: vpn_ips_y2018m09 is_vpn; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m09 ALTER COLUMN is_vpn SET DEFAULT false;


--
-- Name: vpn_ips_y2018m10 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m10 ALTER COLUMN id SET DEFAULT nextval('vpn_ips_id_seq'::regclass);


--
-- Name: vpn_ips_y2018m10 is_vpn; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m10 ALTER COLUMN is_vpn SET DEFAULT false;


--
-- Name: vpn_ips_y2018m11 id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m11 ALTER COLUMN id SET DEFAULT nextval('vpn_ips_id_seq'::regclass);


--
-- Name: vpn_ips_y2018m11 is_vpn; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m11 ALTER COLUMN is_vpn SET DEFAULT false;


--
-- Name: vungle_acceptations id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vungle_acceptations ALTER COLUMN id SET DEFAULT nextval('vungle_acceptations_id_seq'::regclass);


--
-- Name: vungle_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vungle_accounts ALTER COLUMN id SET DEFAULT nextval('vungle_accounts_id_seq'::regclass);


--
-- Name: vungle_agreements id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY vungle_agreements ALTER COLUMN id SET DEFAULT nextval('vungle_agreements_id_seq'::regclass);


--
-- Name: waterfall_logs id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_logs ALTER COLUMN id SET DEFAULT nextval('waterfall_logs_id_seq'::regclass);


--
-- Name: waterfall_monitors id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_monitors ALTER COLUMN id SET DEFAULT nextval('waterfall_monitors_id_seq'::regclass);


--
-- Name: waterfall_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_id_seq'::regclass);


--
-- Name: waterfall_settings_categories id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_categories ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_categories_id_seq'::regclass);


--
-- Name: waterfall_settings_root_flag_types id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_flag_types ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_root_flag_types_id_seq'::regclass);


--
-- Name: waterfall_settings_root_flags id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_flags ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_root_flags_id_seq'::regclass);


--
-- Name: waterfall_settings_root_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_settings ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_root_settings_id_seq'::regclass);


--
-- Name: waterfall_settings_unit_flag_types id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_flag_types ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_unit_flag_types_id_seq'::regclass);


--
-- Name: waterfall_settings_unit_flags id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_flags ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_unit_flags_id_seq'::regclass);


--
-- Name: waterfall_settings_unit_settings id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_settings ALTER COLUMN id SET DEFAULT nextval('waterfall_settings_unit_settings_id_seq'::regclass);


--
-- Name: wl_domain_fees id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domain_fees ALTER COLUMN id SET DEFAULT nextval('wl_domain_fees_id_seq'::regclass);


--
-- Name: wl_domain_font_files id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domain_font_files ALTER COLUMN id SET DEFAULT nextval('wl_domain_font_files_id_seq'::regclass);


--
-- Name: wl_domain_fonts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domain_fonts ALTER COLUMN id SET DEFAULT nextval('wl_domain_fonts_id_seq'::regclass);


--
-- Name: wl_domains id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domains ALTER COLUMN id SET DEFAULT nextval('wl_domains_id_seq'::regclass);


--
-- Name: yandex_accounts id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY yandex_accounts ALTER COLUMN id SET DEFAULT nextval('yandex_accounts_id_seq'::regclass);


--
-- Name: account_employees account_analysts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY account_employees
    ADD CONSTRAINT account_analysts_pkey PRIMARY KEY (id);


--
-- Name: account_shares account_shares_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY account_shares
    ADD CONSTRAINT account_shares_pkey PRIMARY KEY (id);


--
-- Name: active_admin_comments active_admin_comments_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY active_admin_comments
    ADD CONSTRAINT active_admin_comments_pkey PRIMARY KEY (id);


--
-- Name: active_apps active_apps_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY active_apps
    ADD CONSTRAINT active_apps_pkey PRIMARY KEY (id);


--
-- Name: active_users active_users_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY active_users
    ADD CONSTRAINT active_users_pkey PRIMARY KEY (id);


--
-- Name: ad_request_settings ad_request_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_request_settings
    ADD CONSTRAINT ad_request_settings_pkey PRIMARY KEY (id);


--
-- Name: ad_unit_templates ad_unit_templates_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_unit_templates
    ADD CONSTRAINT ad_unit_templates_pkey PRIMARY KEY (id);


--
-- Name: ad_unit_versions ad_unit_versions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_unit_versions
    ADD CONSTRAINT ad_unit_versions_pkey PRIMARY KEY (id);


--
-- Name: ad_units_backup ad_units_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_units_backup
    ADD CONSTRAINT ad_units_pkey PRIMARY KEY (id);


--
-- Name: ad_units ad_units_pkey1; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY ad_units
    ADD CONSTRAINT ad_units_pkey1 PRIMARY KEY (id);


--
-- Name: adcolony_accounts adcolony_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY adcolony_accounts
    ADD CONSTRAINT adcolony_accounts_pkey PRIMARY KEY (id);


--
-- Name: additional_user_logs additional_user_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY additional_user_logs
    ADD CONSTRAINT additional_user_logs_pkey PRIMARY KEY (id);


--
-- Name: admob_accounts admob_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_accounts
    ADD CONSTRAINT admob_accounts_pkey PRIMARY KEY (id);


--
-- Name: admob_ad_unit_versions admob_ad_unit_versions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_ad_unit_versions
    ADD CONSTRAINT admob_ad_unit_versions_pkey PRIMARY KEY (id);


--
-- Name: admob_app_bid_floors admob_app_bid_floors_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_app_bid_floors
    ADD CONSTRAINT admob_app_bid_floors_pkey PRIMARY KEY (id);


--
-- Name: admob_user_bid_floors admob_user_bid_floors_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY admob_user_bid_floors
    ADD CONSTRAINT admob_user_bid_floors_pkey PRIMARY KEY (id);


--
-- Name: adms adms_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY adms
    ADD CONSTRAINT adms_pkey PRIMARY KEY (id);


--
-- Name: advertisement_partners advertisement_partners_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY advertisement_partners
    ADD CONSTRAINT advertisement_partners_pkey PRIMARY KEY (id);


--
-- Name: advertisement_partners_users advertisement_partners_users_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY advertisement_partners_users
    ADD CONSTRAINT advertisement_partners_users_pkey PRIMARY KEY (user_id, advertisement_partner_id);


--
-- Name: ae_settings ae_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY ae_settings
    ADD CONSTRAINT ae_settings_pkey PRIMARY KEY (id);


--
-- Name: amazon_accounts amazon_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY amazon_accounts
    ADD CONSTRAINT amazon_accounts_pkey PRIMARY KEY (id);


--
-- Name: api_queues api_queues_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY api_queues
    ADD CONSTRAINT api_queues_pkey PRIMARY KEY (id);


--
-- Name: app_fraud_logs app_fraud_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_fraud_logs
    ADD CONSTRAINT app_fraud_logs_pkey PRIMARY KEY (id);


--
-- Name: app_frauds app_frauds_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_frauds
    ADD CONSTRAINT app_frauds_pkey PRIMARY KEY (id);


--
-- Name: app_installer_data app_installer_data_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_installer_data
    ADD CONSTRAINT app_installer_data_pkey PRIMARY KEY (id);


--
-- Name: app_network_profiles app_network_profiles_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_network_profiles
    ADD CONSTRAINT app_network_profiles_pkey PRIMARY KEY (id);


--
-- Name: app_revenue_shares app_revenue_shares_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY app_revenue_shares
    ADD CONSTRAINT app_revenue_shares_pkey PRIMARY KEY (id);


--
-- Name: appflood_accounts appflood_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY appflood_accounts
    ADD CONSTRAINT appflood_accounts_pkey PRIMARY KEY (id);


--
-- Name: applovin_accounts applovin_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY applovin_accounts
    ADD CONSTRAINT applovin_accounts_pkey PRIMARY KEY (id);


--
-- Name: applovin_zones applovin_zones_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY applovin_zones
    ADD CONSTRAINT applovin_zones_pkey PRIMARY KEY (id);


--
-- Name: appnexus_accounts appnexus_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY appnexus_accounts
    ADD CONSTRAINT appnexus_accounts_pkey PRIMARY KEY (id);


--
-- Name: apps_campaign_images apps_campaign_images_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_campaign_images
    ADD CONSTRAINT apps_campaign_images_pkey PRIMARY KEY (id);


--
-- Name: apps_categories apps_categories_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_categories
    ADD CONSTRAINT apps_categories_pkey PRIMARY KEY (id);


--
-- Name: apps_network_accounts apps_network_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_network_accounts
    ADD CONSTRAINT apps_network_accounts_pkey PRIMARY KEY (id);


--
-- Name: apps apps_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps
    ADD CONSTRAINT apps_pkey PRIMARY KEY (id);


--
-- Name: apps_website_links apps_website_links_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY apps_website_links
    ADD CONSTRAINT apps_website_links_pkey PRIMARY KEY (id);


--
-- Name: avocarrot_accounts avocarrot_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY avocarrot_accounts
    ADD CONSTRAINT avocarrot_accounts_pkey PRIMARY KEY (id);


--
-- Name: bad_devices bad_devices_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bad_devices
    ADD CONSTRAINT bad_devices_pkey PRIMARY KEY (id);


--
-- Name: bid_floor_settings bid_floor_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bid_floor_settings
    ADD CONSTRAINT bid_floor_settings_pkey PRIMARY KEY (id);


--
-- Name: bidder_bidder_formats bidder_bidder_formats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_bidder_formats
    ADD CONSTRAINT bidder_bidder_formats_pkey PRIMARY KEY (id);


--
-- Name: bidder_campaigns bidder_campaigns_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_campaigns
    ADD CONSTRAINT bidder_campaigns_pkey PRIMARY KEY (id);


--
-- Name: bidder_countries bidder_countries_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_countries
    ADD CONSTRAINT bidder_countries_pkey PRIMARY KEY (id);


--
-- Name: bidder_formats bidder_formats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_formats
    ADD CONSTRAINT bidder_formats_pkey PRIMARY KEY (id);


--
-- Name: bidder_settings bidder_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bidder_settings
    ADD CONSTRAINT bidder_settings_pkey PRIMARY KEY (id);


--
-- Name: bids bids_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bids
    ADD CONSTRAINT bids_pkey PRIMARY KEY (date, bidder_id, ad_type, app_id, country_id, platform_id);


--
-- Name: billings billings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings
    ADD CONSTRAINT billings_pkey PRIMARY KEY (id);


--
-- Name: billings_y2016m10 billings_y2016m10_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m10
    ADD CONSTRAINT billings_y2016m10_pkey PRIMARY KEY (id);


--
-- Name: billings_y2016m11 billings_y2016m11_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m11
    ADD CONSTRAINT billings_y2016m11_pkey PRIMARY KEY (id);


--
-- Name: billings_y2016m12 billings_y2016m12_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2016m12
    ADD CONSTRAINT billings_y2016m12_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m01 billings_y2017m01_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m01
    ADD CONSTRAINT billings_y2017m01_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m02 billings_y2017m02_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m02
    ADD CONSTRAINT billings_y2017m02_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m03 billings_y2017m03_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m03
    ADD CONSTRAINT billings_y2017m03_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m04 billings_y2017m04_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m04
    ADD CONSTRAINT billings_y2017m04_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m05 billings_y2017m05_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m05
    ADD CONSTRAINT billings_y2017m05_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m06 billings_y2017m06_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m06
    ADD CONSTRAINT billings_y2017m06_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m07 billings_y2017m07_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m07
    ADD CONSTRAINT billings_y2017m07_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m08 billings_y2017m08_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m08
    ADD CONSTRAINT billings_y2017m08_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m09 billings_y2017m09_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m09
    ADD CONSTRAINT billings_y2017m09_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m10 billings_y2017m10_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m10
    ADD CONSTRAINT billings_y2017m10_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m11 billings_y2017m11_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m11
    ADD CONSTRAINT billings_y2017m11_pkey PRIMARY KEY (id);


--
-- Name: billings_y2017m12 billings_y2017m12_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2017m12
    ADD CONSTRAINT billings_y2017m12_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m01 billings_y2018m01_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m01
    ADD CONSTRAINT billings_y2018m01_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m02 billings_y2018m02_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m02
    ADD CONSTRAINT billings_y2018m02_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m03 billings_y2018m03_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m03
    ADD CONSTRAINT billings_y2018m03_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m04 billings_y2018m04_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m04
    ADD CONSTRAINT billings_y2018m04_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m05 billings_y2018m05_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m05
    ADD CONSTRAINT billings_y2018m05_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m06 billings_y2018m06_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m06
    ADD CONSTRAINT billings_y2018m06_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m07 billings_y2018m07_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m07
    ADD CONSTRAINT billings_y2018m07_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m08 billings_y2018m08_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m08
    ADD CONSTRAINT billings_y2018m08_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m09 billings_y2018m09_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m09
    ADD CONSTRAINT billings_y2018m09_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m10 billings_y2018m10_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m10
    ADD CONSTRAINT billings_y2018m10_pkey PRIMARY KEY (id);


--
-- Name: billings_y2018m11 billings_y2018m11_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY billings_y2018m11
    ADD CONSTRAINT billings_y2018m11_pkey PRIMARY KEY (id);


--
-- Name: block_devices block_devices_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY block_devices
    ADD CONSTRAINT block_devices_pkey PRIMARY KEY (id);


--
-- Name: book_keeping_reports book_keeping_reports_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY book_keeping_reports
    ADD CONSTRAINT book_keeping_reports_pkey PRIMARY KEY (id);


--
-- Name: cached_files cached_files_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY cached_files
    ADD CONSTRAINT cached_files_pkey PRIMARY KEY (id);


--
-- Name: campaign_aggregated_stats campaign_aggregated_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_aggregated_stats
    ADD CONSTRAINT campaign_aggregated_stats_pkey PRIMARY KEY (id);


--
-- Name: campaign_commissions campaign_commissions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_commissions
    ADD CONSTRAINT campaign_commissions_pkey PRIMARY KEY (id);


--
-- Name: campaign_coordinates campaign_coordinates_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_coordinates
    ADD CONSTRAINT campaign_coordinates_pkey PRIMARY KEY (id);


--
-- Name: campaign_image_coordinates_images campaign_image_coordinates_images_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_coordinates_images
    ADD CONSTRAINT campaign_image_coordinates_images_pkey PRIMARY KEY (id);


--
-- Name: campaign_image_coordinates campaign_image_coordinates_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_coordinates
    ADD CONSTRAINT campaign_image_coordinates_pkey PRIMARY KEY (id);


--
-- Name: campaign_image_devices_csvs campaign_image_devices_csvs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_devices_csvs
    ADD CONSTRAINT campaign_image_devices_csvs_pkey PRIMARY KEY (id);


--
-- Name: campaign_image_devices campaign_image_devices_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_image_devices
    ADD CONSTRAINT campaign_image_devices_pkey PRIMARY KEY (id);


--
-- Name: campaign_images_categories campaign_images_categories_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images_categories
    ADD CONSTRAINT campaign_images_categories_pkey PRIMARY KEY (id);


--
-- Name: campaign_images_countries campaign_images_countries_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images_countries
    ADD CONSTRAINT campaign_images_countries_pkey PRIMARY KEY (id);


--
-- Name: campaign_images_max_mind_cities campaign_images_max_mind_cities_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images_max_mind_cities
    ADD CONSTRAINT campaign_images_max_mind_cities_pkey PRIMARY KEY (id);


--
-- Name: campaign_images_mobile_networks campaign_images_mobile_networks_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images_mobile_networks
    ADD CONSTRAINT campaign_images_mobile_networks_pkey PRIMARY KEY (campaign_image_id, mobile_network_id);


--
-- Name: campaign_images campaign_images_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_images
    ADD CONSTRAINT campaign_images_pkey PRIMARY KEY (id);


--
-- Name: campaign_settings campaign_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_settings
    ADD CONSTRAINT campaign_settings_pkey PRIMARY KEY (id);


--
-- Name: statistics campaign_statistics_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY statistics
    ADD CONSTRAINT campaign_statistics_pkey PRIMARY KEY (id);


--
-- Name: campaign_types campaign_types_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_types
    ADD CONSTRAINT campaign_types_pkey PRIMARY KEY (id);


--
-- Name: campaign_versions campaign_versions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaign_versions
    ADD CONSTRAINT campaign_versions_pkey PRIMARY KEY (id);


--
-- Name: campaigns_creative_sets campaigns_creative_sets_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaigns_creative_sets
    ADD CONSTRAINT campaigns_creative_sets_pkey PRIMARY KEY (id);


--
-- Name: campaigns campaigns_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY campaigns
    ADD CONSTRAINT campaigns_pkey PRIMARY KEY (id);


--
-- Name: categories_contents categories_contents_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY categories_contents
    ADD CONSTRAINT categories_contents_pkey PRIMARY KEY (id);


--
-- Name: categories categories_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY categories
    ADD CONSTRAINT categories_pkey PRIMARY KEY (id);


--
-- Name: categories_users categories_users_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY categories_users
    ADD CONSTRAINT categories_users_pkey PRIMARY KEY (id);


--
-- Name: changelogs changelogs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY changelogs
    ADD CONSTRAINT changelogs_pkey PRIMARY KEY (id);


--
-- Name: chartboost_accounts chartboost_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY chartboost_accounts
    ADD CONSTRAINT chartboost_accounts_pkey PRIMARY KEY (id);


--
-- Name: chartboost_campaigns chartboost_campaigns_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY chartboost_campaigns
    ADD CONSTRAINT chartboost_campaigns_pkey PRIMARY KEY (id);


--
-- Name: cities cities_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY cities
    ADD CONSTRAINT cities_pkey PRIMARY KEY (id);


--
-- Name: clicks_master clicks_master_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_master
    ADD CONSTRAINT clicks_master_pkey PRIMARY KEY (id);


--
-- Name: clicks clicks_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks
    ADD CONSTRAINT clicks_pkey PRIMARY KEY (id);


--
-- Name: clicks_y2018m08 clicks_y2018m08_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m08
    ADD CONSTRAINT clicks_y2018m08_pkey PRIMARY KEY (id);


--
-- Name: clicks_y2018m09 clicks_y2018m09_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m09
    ADD CONSTRAINT clicks_y2018m09_pkey PRIMARY KEY (id);


--
-- Name: clicks_y2018m10 clicks_y2018m10_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m10
    ADD CONSTRAINT clicks_y2018m10_pkey PRIMARY KEY (id);


--
-- Name: clicks_y2018m11 clicks_y2018m11_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY clicks_y2018m11
    ADD CONSTRAINT clicks_y2018m11_pkey PRIMARY KEY (id);


--
-- Name: client_histories client_histories_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY client_histories
    ADD CONSTRAINT client_histories_pkey PRIMARY KEY (id);


--
-- Name: client_history_stats client_history_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY client_history_stats
    ADD CONSTRAINT client_history_stats_pkey PRIMARY KEY (id);


--
-- Name: compensations compensations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY compensations
    ADD CONSTRAINT compensations_pkey PRIMARY KEY (id);


--
-- Name: complains complains_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY complains
    ADD CONSTRAINT complains_pkey PRIMARY KEY (id);


--
-- Name: contents contents_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY contents
    ADD CONSTRAINT contents_pkey PRIMARY KEY (id);


--
-- Name: countries countries_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY countries
    ADD CONSTRAINT countries_pkey PRIMARY KEY (id);


--
-- Name: creative_files creative_files_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY creative_files
    ADD CONSTRAINT creative_files_pkey PRIMARY KEY (id);


--
-- Name: creative_items creative_items_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY creative_items
    ADD CONSTRAINT creative_items_pkey PRIMARY KEY (id);


--
-- Name: creative_sets creative_sets_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY creative_sets
    ADD CONSTRAINT creative_sets_pkey PRIMARY KEY (id);


--
-- Name: custom_app_bid_floors custom_app_bid_floors_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY custom_app_bid_floors
    ADD CONSTRAINT custom_app_bid_floors_pkey PRIMARY KEY (id);


--
-- Name: custom_user_commissions custom_user_commissions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY custom_user_commissions
    ADD CONSTRAINT custom_user_commissions_pkey PRIMARY KEY (id);


--
-- Name: dash_views_logs dash_views_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY dash_views_logs
    ADD CONSTRAINT dash_views_logs_pkey PRIMARY KEY (id);


--
-- Name: dashboard_predefined_filters dashboard_predefined_filters_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY dashboard_predefined_filters
    ADD CONSTRAINT dashboard_predefined_filters_pkey PRIMARY KEY (id);


--
-- Name: data_migrations data_migrations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY data_migrations
    ADD CONSTRAINT data_migrations_pkey PRIMARY KEY (version);


--
-- Name: deals deals_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY deals
    ADD CONSTRAINT deals_pkey PRIMARY KEY (id);


--
-- Name: device_manufacturers device_manufacturers_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY device_manufacturers
    ADD CONSTRAINT device_manufacturers_pkey PRIMARY KEY (id);


--
-- Name: device_models device_models_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY device_models
    ADD CONSTRAINT device_models_pkey PRIMARY KEY (id);


--
-- Name: devices devices_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY devices
    ADD CONSTRAINT devices_pkey PRIMARY KEY (id);


--
-- Name: disabled_networks disabled_networks_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY disabled_networks
    ADD CONSTRAINT disabled_networks_pkey PRIMARY KEY (id);


--
-- Name: docs_histories docs_histories_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs_histories
    ADD CONSTRAINT docs_histories_pkey PRIMARY KEY (id);


--
-- Name: docs docs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs
    ADD CONSTRAINT docs_pkey PRIMARY KEY (id);


--
-- Name: docs_sections docs_sections_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs_sections
    ADD CONSTRAINT docs_sections_pkey PRIMARY KEY (id);


--
-- Name: docs_sets docs_sets_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY docs_sets
    ADD CONSTRAINT docs_sets_pkey PRIMARY KEY (id);


--
-- Name: domain_font_files domain_font_files_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY domain_font_files
    ADD CONSTRAINT domain_font_files_pkey PRIMARY KEY (id);


--
-- Name: domain_fonts domain_fonts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY domain_fonts
    ADD CONSTRAINT domain_fonts_pkey PRIMARY KEY (id);


--
-- Name: domains domains_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY domains
    ADD CONSTRAINT domains_pkey PRIMARY KEY (id);


--
-- Name: email_smtp_settings email_smtp_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY email_smtp_settings
    ADD CONSTRAINT email_smtp_settings_pkey PRIMARY KEY (id);


--
-- Name: email_templates email_templates_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY email_templates
    ADD CONSTRAINT email_templates_pkey PRIMARY KEY (id);


--
-- Name: exceptional_targetings exceptional_targetings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY exceptional_targetings
    ADD CONSTRAINT exceptional_targetings_pkey PRIMARY KEY (id);


--
-- Name: exchange_coefficients exchange_coefficients_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY exchange_coefficients
    ADD CONSTRAINT exchange_coefficients_pkey PRIMARY KEY (id);


--
-- Name: exchange_rates exchange_rates_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY exchange_rates
    ADD CONSTRAINT exchange_rates_pkey PRIMARY KEY (id);


--
-- Name: extension_logs extension_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY extension_logs
    ADD CONSTRAINT extension_logs_pkey PRIMARY KEY (id);


--
-- Name: facebook_accounts facebook_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY facebook_accounts
    ADD CONSTRAINT facebook_accounts_pkey PRIMARY KEY (id);


--
-- Name: fast_affiliate_stats fast_affiliate_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_affiliate_stats
    ADD CONSTRAINT fast_affiliate_stats_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_capping_options fast_ecpm_capping_options_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_capping_options
    ADD CONSTRAINT fast_ecpm_capping_options_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_comparison_stats_loggers fast_ecpm_comparison_stats_loggers_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_comparison_stats_loggers
    ADD CONSTRAINT fast_ecpm_comparison_stats_loggers_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_comparison_stats fast_ecpm_comparison_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_comparison_stats
    ADD CONSTRAINT fast_ecpm_comparison_stats_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_new_options fast_ecpm_new_options_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_new_options
    ADD CONSTRAINT fast_ecpm_new_options_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_stats fast_ecpm_stats_cleaned_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_stats
    ADD CONSTRAINT fast_ecpm_stats_cleaned_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_stats_exclusions fast_ecpm_stats_exclusions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_stats_exclusions
    ADD CONSTRAINT fast_ecpm_stats_exclusions_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_stats_old fast_ecpm_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_stats_old
    ADD CONSTRAINT fast_ecpm_stats_pkey PRIMARY KEY (id);


--
-- Name: fast_ecpm_stats_tmp fast_ecpm_stats_tmp_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fast_ecpm_stats_tmp
    ADD CONSTRAINT fast_ecpm_stats_tmp_pkey PRIMARY KEY (id);


--
-- Name: finalized_time_for_reports finalized_time_for_reports_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY finalized_time_for_reports
    ADD CONSTRAINT finalized_time_for_reports_pkey PRIMARY KEY (id);


--
-- Name: fixed_admob_campaigns_sets fixed_admob_campaigns_sets_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fixed_admob_campaigns_sets
    ADD CONSTRAINT fixed_admob_campaigns_sets_pkey PRIMARY KEY (id);


--
-- Name: flurry_accounts flurry_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY flurry_accounts
    ADD CONSTRAINT flurry_accounts_pkey PRIMARY KEY (id);


--
-- Name: fraud_click_series fraud_click_series_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_click_series
    ADD CONSTRAINT fraud_click_series_pkey PRIMARY KEY (id);


--
-- Name: fraud_detections fraud_detections_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_detections
    ADD CONSTRAINT fraud_detections_pkey PRIMARY KEY (id);


--
-- Name: fraud_user_emails fraud_user_emails_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_user_emails
    ADD CONSTRAINT fraud_user_emails_pkey PRIMARY KEY (id);


--
-- Name: fraud_user_keys fraud_user_keys_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY fraud_user_keys
    ADD CONSTRAINT fraud_user_keys_pkey PRIMARY KEY (id);


--
-- Name: frequency_capping_stats frequency_capping_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY frequency_capping_stats
    ADD CONSTRAINT frequency_capping_stats_pkey PRIMARY KEY (id);


--
-- Name: global_settings global_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY global_settings
    ADD CONSTRAINT global_settings_pkey PRIMARY KEY (id);


--
-- Name: global_settings_sections global_settings_sections_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY global_settings_sections
    ADD CONSTRAINT global_settings_sections_pkey PRIMARY KEY (id);


--
-- Name: globals globals_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY globals
    ADD CONSTRAINT globals_pkey PRIMARY KEY (id);


--
-- Name: graph_ql_mutations_tracking_data graph_ql_mutations_tracking_data_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY graph_ql_mutations_tracking_data
    ADD CONSTRAINT graph_ql_mutations_tracking_data_pkey PRIMARY KEY (id);


--
-- Name: heyzap_accounts heyzap_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY heyzap_accounts
    ADD CONSTRAINT heyzap_accounts_pkey PRIMARY KEY (id);


--
-- Name: inmobi_accounts inmobi_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY inmobi_accounts
    ADD CONSTRAINT inmobi_accounts_pkey PRIMARY KEY (id);


--
-- Name: inmobi_ad_units inmobi_ad_units_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY inmobi_ad_units
    ADD CONSTRAINT inmobi_ad_units_pkey PRIMARY KEY (id);


--
-- Name: inner_active_accounts inner_active_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY inner_active_accounts
    ADD CONSTRAINT inner_active_accounts_pkey PRIMARY KEY (id);


--
-- Name: insight_stat_histories insight_stat_histories_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stat_histories
    ADD CONSTRAINT insight_stat_histories_pkey PRIMARY KEY (id);


--
-- Name: insight_stats insight_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats
    ADD CONSTRAINT insight_stats_pkey PRIMARY KEY (id);


--
-- Name: insight_stats_y2018m03 insight_stats_y2018m03_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m03
    ADD CONSTRAINT insight_stats_y2018m03_pkey PRIMARY KEY (id);


--
-- Name: insight_stats_y2018m04 insight_stats_y2018m04_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m04
    ADD CONSTRAINT insight_stats_y2018m04_pkey PRIMARY KEY (id);


--
-- Name: insight_stats_y2018m05 insight_stats_y2018m05_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY insight_stats_y2018m05
    ADD CONSTRAINT insight_stats_y2018m05_pkey PRIMARY KEY (id);


--
-- Name: integration_frameworks integration_frameworks_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY integration_frameworks
    ADD CONSTRAINT integration_frameworks_pkey PRIMARY KEY (id);


--
-- Name: intercom_conversation_parts intercom_conversation_parts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY intercom_conversation_parts
    ADD CONSTRAINT intercom_conversation_parts_pkey PRIMARY KEY (id);


--
-- Name: intercom_conversations intercom_conversations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY intercom_conversations
    ADD CONSTRAINT intercom_conversations_pkey PRIMARY KEY (id);


--
-- Name: intercom_network_recommendeds intercom_network_recommendeds_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY intercom_network_recommendeds
    ADD CONSTRAINT intercom_network_recommendeds_pkey PRIMARY KEY (id);


--
-- Name: invoice_requests invoice_requests_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY invoice_requests
    ADD CONSTRAINT invoice_requests_pkey PRIMARY KEY (id);


--
-- Name: ip_addresses ip_addresses_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY ip_addresses
    ADD CONSTRAINT ip_addresses_pkey PRIMARY KEY (id);


--
-- Name: liverail_accounts liverail_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY liverail_accounts
    ADD CONSTRAINT liverail_accounts_pkey PRIMARY KEY (id);


--
-- Name: liverail_ad_units liverail_ad_units_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY liverail_ad_units
    ADD CONSTRAINT liverail_ad_units_pkey PRIMARY KEY (id);


--
-- Name: long_term_account_revenues long_term_account_revenues_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY long_term_account_revenues
    ADD CONSTRAINT long_term_account_revenues_pkey PRIMARY KEY (id);


--
-- Name: magic_monies magic_monies_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY magic_monies
    ADD CONSTRAINT magic_monies_pkey PRIMARY KEY (id);


--
-- Name: managers managers_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY managers
    ADD CONSTRAINT managers_pkey PRIMARY KEY (id);


--
-- Name: max_mind_cities max_mind_cities_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY max_mind_cities
    ADD CONSTRAINT max_mind_cities_pkey PRIMARY KEY (id);


--
-- Name: mmedia_accounts mmedia_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY mmedia_accounts
    ADD CONSTRAINT mmedia_accounts_pkey PRIMARY KEY (id);


--
-- Name: mobile_networks mobile_networks_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY mobile_networks
    ADD CONSTRAINT mobile_networks_pkey PRIMARY KEY (id);


--
-- Name: mobilecore_accounts mobilecore_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY mobilecore_accounts
    ADD CONSTRAINT mobilecore_accounts_pkey PRIMARY KEY (id);


--
-- Name: monitor_events monitor_events_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY monitor_events
    ADD CONSTRAINT monitor_events_pkey PRIMARY KEY (id);


--
-- Name: monthly_book_keeping_reports monthly_book_keeping_reports_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY monthly_book_keeping_reports
    ADD CONSTRAINT monthly_book_keeping_reports_pkey PRIMARY KEY (id);


--
-- Name: mopub_accounts mopub_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY mopub_accounts
    ADD CONSTRAINT mopub_accounts_pkey PRIMARY KEY (id);


--
-- Name: network_accounts network_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_accounts
    ADD CONSTRAINT network_accounts_pkey PRIMARY KEY (id);


--
-- Name: network_active_types network_active_types_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_active_types
    ADD CONSTRAINT network_active_types_pkey PRIMARY KEY (id);


--
-- Name: network_connection_restrictions network_connection_restrictions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_connection_restrictions
    ADD CONSTRAINT network_connection_restrictions_pkey PRIMARY KEY (id);


--
-- Name: network_connection_settings network_connection_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_connection_settings
    ADD CONSTRAINT network_connection_settings_pkey PRIMARY KEY (id);


--
-- Name: network_connection_statuses network_connection_statuses_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_connection_statuses
    ADD CONSTRAINT network_connection_statuses_pkey PRIMARY KEY (id);


--
-- Name: network_content_filters network_content_filters_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_content_filters
    ADD CONSTRAINT network_content_filters_pkey PRIMARY KEY (id);


--
-- Name: network_daemon_jobs network_daemon_jobs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_daemon_jobs
    ADD CONSTRAINT network_daemon_jobs_pkey PRIMARY KEY (id);


--
-- Name: network_mailing_app_statuses network_mailing_app_statuses_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_mailing_app_statuses
    ADD CONSTRAINT network_mailing_app_statuses_pkey PRIMARY KEY (id);


--
-- Name: network_mailing_settings network_mailing_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_mailing_settings
    ADD CONSTRAINT network_mailing_settings_pkey PRIMARY KEY (id);


--
-- Name: network_migrations network_migrations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_migrations
    ADD CONSTRAINT network_migrations_pkey PRIMARY KEY (id);


--
-- Name: network_reporting_settings network_reporting_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_reporting_settings
    ADD CONSTRAINT network_reporting_settings_pkey PRIMARY KEY (id);


--
-- Name: network_request_restrictions network_request_restrictions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY network_request_restrictions
    ADD CONSTRAINT network_request_restrictions_pkey PRIMARY KEY (id);


--
-- Name: new_fast_ecpm_stats new_fast_ecpm_stats_cleaned_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY new_fast_ecpm_stats
    ADD CONSTRAINT new_fast_ecpm_stats_cleaned_pkey PRIMARY KEY (id);


--
-- Name: new_fast_ecpm_stats_old new_fast_ecpm_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY new_fast_ecpm_stats_old
    ADD CONSTRAINT new_fast_ecpm_stats_pkey PRIMARY KEY (id);


--
-- Name: nexage_accounts nexage_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY nexage_accounts
    ADD CONSTRAINT nexage_accounts_pkey PRIMARY KEY (id);


--
-- Name: openx_accounts openx_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY openx_accounts
    ADD CONSTRAINT openx_accounts_pkey PRIMARY KEY (id);


--
-- Name: organization_commission_presets organization_commission_presets_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization_commission_presets
    ADD CONSTRAINT organization_commission_presets_pkey PRIMARY KEY (id);


--
-- Name: organization_roles organization_roles_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization_roles
    ADD CONSTRAINT organization_roles_pkey PRIMARY KEY (id);


--
-- Name: organizations organizations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY organizations
    ADD CONSTRAINT organizations_pkey PRIMARY KEY (id);


--
-- Name: page_categories_apps page_categories_apps_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY page_categories_apps
    ADD CONSTRAINT page_categories_apps_pkey PRIMARY KEY (id);


--
-- Name: partner_app_commissions partner_app_commissions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY partner_app_commissions
    ADD CONSTRAINT partner_app_commissions_pkey PRIMARY KEY (id);


--
-- Name: payment_requests payment_requests_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY payment_requests
    ADD CONSTRAINT payment_requests_pkey PRIMARY KEY (id);


--
-- Name: payouts payouts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY payouts
    ADD CONSTRAINT payouts_pkey PRIMARY KEY (id);


--
-- Name: platforms platforms_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY platforms
    ADD CONSTRAINT platforms_pkey PRIMARY KEY (id);


--
-- Name: premium_user_commissions premium_user_commissions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY premium_user_commissions
    ADD CONSTRAINT premium_user_commissions_pkey PRIMARY KEY (id);


--
-- Name: price_floors price_floors_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY price_floors
    ADD CONSTRAINT price_floors_pkey PRIMARY KEY (id);


--
-- Name: publisher_monitor_messages publisher_monitor_messages_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY publisher_monitor_messages
    ADD CONSTRAINT publisher_monitor_messages_pkey PRIMARY KEY (id);


--
-- Name: pubnative_accounts pubnative_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY pubnative_accounts
    ADD CONSTRAINT pubnative_accounts_pkey PRIMARY KEY (id);


--
-- Name: qubole_sdk_data qubole_sdk_data_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY qubole_sdk_data
    ADD CONSTRAINT qubole_sdk_data_pkey PRIMARY KEY (id);


--
-- Name: report_processing_logs report_processing_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY report_processing_logs
    ADD CONSTRAINT report_processing_logs_pkey PRIMARY KEY (id);


--
-- Name: reports reports_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY reports
    ADD CONSTRAINT reports_pkey PRIMARY KEY (id);


--
-- Name: request_blacklists request_blacklists_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY request_blacklists
    ADD CONSTRAINT request_blacklists_pkey PRIMARY KEY (id);


--
-- Name: revenue_change_limit_settings revenue_change_limit_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY revenue_change_limit_settings
    ADD CONSTRAINT revenue_change_limit_settings_pkey PRIMARY KEY (id);


--
-- Name: revenue_comparison_stats revenue_comparison_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY revenue_comparison_stats
    ADD CONSTRAINT revenue_comparison_stats_pkey PRIMARY KEY (id);


--
-- Name: revenue_analysis_settings revenue_logger_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY revenue_analysis_settings
    ADD CONSTRAINT revenue_logger_settings_pkey PRIMARY KEY (id);


--
-- Name: roles roles_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY roles
    ADD CONSTRAINT roles_pkey PRIMARY KEY (id);


--
-- Name: rubicon_accounts rubicon_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY rubicon_accounts
    ADD CONSTRAINT rubicon_accounts_pkey PRIMARY KEY (id);


--
-- Name: schema_migrations schema_migrations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY schema_migrations
    ADD CONSTRAINT schema_migrations_pkey PRIMARY KEY (version);


--
-- Name: sdk_mapping_settings sdk_mapping_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY sdk_mapping_settings
    ADD CONSTRAINT sdk_mapping_settings_pkey PRIMARY KEY (id);


--
-- Name: sdk_releases sdk_releases_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY sdk_releases
    ADD CONSTRAINT sdk_releases_pkey PRIMARY KEY (id);


--
-- Name: section_categories_apps section_categories_apps_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY section_categories_apps
    ADD CONSTRAINT section_categories_apps_pkey PRIMARY KEY (id);


--
-- Name: segment_placements segment_placements_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_placements
    ADD CONSTRAINT segment_placements_pkey PRIMARY KEY (id);


--
-- Name: segment_rule_countries segment_rule_countries_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_rule_countries
    ADD CONSTRAINT segment_rule_countries_pkey PRIMARY KEY (id);


--
-- Name: segment_rules segment_rules_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_rules
    ADD CONSTRAINT segment_rules_pkey PRIMARY KEY (id);


--
-- Name: segment_settings segment_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY segment_settings
    ADD CONSTRAINT segment_settings_pkey PRIMARY KEY (id);


--
-- Name: segments segments_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY segments
    ADD CONSTRAINT segments_pkey PRIMARY KEY (id);


--
-- Name: share_tokens share_tokens_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY share_tokens
    ADD CONSTRAINT share_tokens_pkey PRIMARY KEY (id);


--
-- Name: simple_dashboard_monthly simple_dashboard_monthly_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_monthly
    ADD CONSTRAINT simple_dashboard_monthly_pkey PRIMARY KEY (id);


--
-- Name: simple_dashboard simple_dashboard_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard
    ADD CONSTRAINT simple_dashboard_pkey PRIMARY KEY (id);


--
-- Name: simple_dashboard_quarterly simple_dashboard_quarterly_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_quarterly
    ADD CONSTRAINT simple_dashboard_quarterly_pkey PRIMARY KEY (id);


--
-- Name: simple_dashboard_weekly simple_dashboard_weekly_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_weekly
    ADD CONSTRAINT simple_dashboard_weekly_pkey PRIMARY KEY (id);


--
-- Name: simple_dashboard_yearly simple_dashboard_yearly_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY simple_dashboard_yearly
    ADD CONSTRAINT simple_dashboard_yearly_pkey PRIMARY KEY (id);


--
-- Name: smaato_accounts smaato_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY smaato_accounts
    ADD CONSTRAINT smaato_accounts_pkey PRIMARY KEY (id);


--
-- Name: spotx_accounts spotx_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY spotx_accounts
    ADD CONSTRAINT spotx_accounts_pkey PRIMARY KEY (id);


--
-- Name: startapp_accounts startapp_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY startapp_accounts
    ADD CONSTRAINT startapp_accounts_pkey PRIMARY KEY (id);


--
-- Name: statistic_reports statistic_reports_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY statistic_reports
    ADD CONSTRAINT statistic_reports_pkey PRIMARY KEY (id);


--
-- Name: store_infos store_infos_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY store_infos
    ADD CONSTRAINT store_infos_pkey PRIMARY KEY (id);


--
-- Name: subdivision_ones subdivision_ones_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY subdivision_ones
    ADD CONSTRAINT subdivision_ones_pkey PRIMARY KEY (id);


--
-- Name: subdivision_twos subdivision_twos_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY subdivision_twos
    ADD CONSTRAINT subdivision_twos_pkey PRIMARY KEY (id);


--
-- Name: supersonic_accounts supersonic_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY supersonic_accounts
    ADD CONSTRAINT supersonic_accounts_pkey PRIMARY KEY (id);


--
-- Name: support_work_metriks support_work_metriks_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY support_work_metriks
    ADD CONSTRAINT support_work_metriks_pkey PRIMARY KEY (id);


--
-- Name: support_work_reports support_work_reports_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY support_work_reports
    ADD CONSTRAINT support_work_reports_pkey PRIMARY KEY (id);


--
-- Name: survey_responses survey_responses_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY survey_responses
    ADD CONSTRAINT survey_responses_pkey PRIMARY KEY (id);


--
-- Name: surveys surveys_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY surveys
    ADD CONSTRAINT surveys_pkey PRIMARY KEY (id);


--
-- Name: tapsense_accounts tapsense_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY tapsense_accounts
    ADD CONSTRAINT tapsense_accounts_pkey PRIMARY KEY (id);


--
-- Name: target_accounts target_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY target_accounts
    ADD CONSTRAINT target_accounts_pkey PRIMARY KEY (id);


--
-- Name: teams teams_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY teams
    ADD CONSTRAINT teams_pkey PRIMARY KEY (id);


--
-- Name: temp_apps_network_accounts temp_apps_network_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY temp_apps_network_accounts
    ADD CONSTRAINT temp_apps_network_accounts_pkey PRIMARY KEY (id);


--
-- Name: test_device_ids test_device_ids_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY test_device_ids
    ADD CONSTRAINT test_device_ids_pkey PRIMARY KEY (id);


--
-- Name: traffic_limit_blocks traffic_limit_blocks_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY traffic_limit_blocks
    ADD CONSTRAINT traffic_limit_blocks_pkey PRIMARY KEY (id);


--
-- Name: traffic_limit_exclusions traffic_limit_exclusions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY traffic_limit_exclusions
    ADD CONSTRAINT traffic_limit_exclusions_pkey PRIMARY KEY (id);


--
-- Name: unity_ads_accounts unity_ads_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY unity_ads_accounts
    ADD CONSTRAINT unity_ads_accounts_pkey PRIMARY KEY (id);


--
-- Name: user_ach_requisites user_ach_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_ach_requisites
    ADD CONSTRAINT user_ach_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_apps_commissions user_apps_commissions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_apps_commissions
    ADD CONSTRAINT user_apps_commissions_pkey PRIMARY KEY (id);


--
-- Name: user_bitcoin_requisites user_bitcoin_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_bitcoin_requisites
    ADD CONSTRAINT user_bitcoin_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_confirmations user_confirmations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_confirmations
    ADD CONSTRAINT user_confirmations_pkey PRIMARY KEY (id);


--
-- Name: user_daily_stats user_daily_stats_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_daily_stats
    ADD CONSTRAINT user_daily_stats_pkey PRIMARY KEY (id);


--
-- Name: user_ethereum_requisites user_ethereum_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_ethereum_requisites
    ADD CONSTRAINT user_ethereum_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_fraud_warning_logs user_fraud_warning_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_fraud_warning_logs
    ADD CONSTRAINT user_fraud_warning_logs_pkey PRIMARY KEY (id);


--
-- Name: user_google_wallet_requisites user_google_wallet_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_google_wallet_requisites
    ADD CONSTRAINT user_google_wallet_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_intercom_events user_intercom_events_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_intercom_events
    ADD CONSTRAINT user_intercom_events_pkey PRIMARY KEY (id);


--
-- Name: user_login_logs user_login_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_login_logs
    ADD CONSTRAINT user_login_logs_pkey PRIMARY KEY (id);


--
-- Name: user_notifications user_notifications_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_notifications
    ADD CONSTRAINT user_notifications_pkey PRIMARY KEY (id);


--
-- Name: user_payment_details_change_confirms user_payment_details_change_confirms_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payment_details_change_confirms
    ADD CONSTRAINT user_payment_details_change_confirms_pkey PRIMARY KEY (id);


--
-- Name: user_payoneer_bank_transfer_requisites user_payoneer_bank_transfer_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payoneer_bank_transfer_requisites
    ADD CONSTRAINT user_payoneer_bank_transfer_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_payoneer_paypal_requisites user_payoneer_paypal_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payoneer_paypal_requisites
    ADD CONSTRAINT user_payoneer_paypal_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_payoneer_prepaid_requisites user_payoneer_prepaid_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_payoneer_prepaid_requisites
    ADD CONSTRAINT user_payoneer_prepaid_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_paypal_requisites user_paypal_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_paypal_requisites
    ADD CONSTRAINT user_paypal_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_webmoney_requisites user_webmoney_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_webmoney_requisites
    ADD CONSTRAINT user_webmoney_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_wire_requisites user_wire_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_wire_requisites
    ADD CONSTRAINT user_wire_requisites_pkey PRIMARY KEY (id);


--
-- Name: user_yandex_money_requisites user_yandex_money_requisites_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY user_yandex_money_requisites
    ADD CONSTRAINT user_yandex_money_requisites_pkey PRIMARY KEY (id);


--
-- Name: users_abilities users_abilities_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_abilities
    ADD CONSTRAINT users_abilities_pkey PRIMARY KEY (id);


--
-- Name: users_admin_notifications users_admin_notifications_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_admin_notifications
    ADD CONSTRAINT users_admin_notifications_pkey PRIMARY KEY (id);


--
-- Name: users_dev_apps users_dev_apps_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_dev_apps
    ADD CONSTRAINT users_dev_apps_pkey PRIMARY KEY (id);


--
-- Name: users_fake_apps users_fake_apps_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_fake_apps
    ADD CONSTRAINT users_fake_apps_pkey PRIMARY KEY (id);


--
-- Name: users_managed_campaigns users_managed_campaigns_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_managed_campaigns
    ADD CONSTRAINT users_managed_campaigns_pkey PRIMARY KEY (id);


--
-- Name: users users_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_pkey PRIMARY KEY (id);


--
-- Name: users_shared_apps users_shared_apps_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_shared_apps
    ADD CONSTRAINT users_shared_apps_pkey PRIMARY KEY (id);


--
-- Name: v2_placements v2_placements_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_placements
    ADD CONSTRAINT v2_placements_pkey PRIMARY KEY (id);


--
-- Name: v2_placements_versions v2_placements_versions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_placements_versions
    ADD CONSTRAINT v2_placements_versions_pkey PRIMARY KEY (id);


--
-- Name: v2_segments v2_segments_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_segments
    ADD CONSTRAINT v2_segments_pkey PRIMARY KEY (id);


--
-- Name: v2_segments_versions v2_segments_versions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY v2_segments_versions
    ADD CONSTRAINT v2_segments_versions_pkey PRIMARY KEY (id);


--
-- Name: versions versions_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY versions
    ADD CONSTRAINT versions_pkey PRIMARY KEY (id);


--
-- Name: vpn_ips vpn_ips_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips
    ADD CONSTRAINT vpn_ips_pkey PRIMARY KEY (id);


--
-- Name: vpn_ips_y2018m08 vpn_ips_y2018m08_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m08
    ADD CONSTRAINT vpn_ips_y2018m08_pkey PRIMARY KEY (id);


--
-- Name: vpn_ips_y2018m09 vpn_ips_y2018m09_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m09
    ADD CONSTRAINT vpn_ips_y2018m09_pkey PRIMARY KEY (id);


--
-- Name: vpn_ips_y2018m10 vpn_ips_y2018m10_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m10
    ADD CONSTRAINT vpn_ips_y2018m10_pkey PRIMARY KEY (id);


--
-- Name: vpn_ips_y2018m11 vpn_ips_y2018m11_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vpn_ips_y2018m11
    ADD CONSTRAINT vpn_ips_y2018m11_pkey PRIMARY KEY (id);


--
-- Name: vungle_acceptations vungle_acceptations_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vungle_acceptations
    ADD CONSTRAINT vungle_acceptations_pkey PRIMARY KEY (id);


--
-- Name: vungle_accounts vungle_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vungle_accounts
    ADD CONSTRAINT vungle_accounts_pkey PRIMARY KEY (id);


--
-- Name: vungle_agreements vungle_agreements_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vungle_agreements
    ADD CONSTRAINT vungle_agreements_pkey PRIMARY KEY (id);


--
-- Name: waterfall_logs waterfall_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_logs
    ADD CONSTRAINT waterfall_logs_pkey PRIMARY KEY (id);


--
-- Name: waterfall_monitors waterfall_monitors_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_monitors
    ADD CONSTRAINT waterfall_monitors_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings_categories waterfall_settings_categories_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_categories
    ADD CONSTRAINT waterfall_settings_categories_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings waterfall_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings
    ADD CONSTRAINT waterfall_settings_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings_root_flag_types waterfall_settings_root_flag_types_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_flag_types
    ADD CONSTRAINT waterfall_settings_root_flag_types_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings_root_flags waterfall_settings_root_flags_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_flags
    ADD CONSTRAINT waterfall_settings_root_flags_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings_root_settings waterfall_settings_root_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_settings
    ADD CONSTRAINT waterfall_settings_root_settings_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings_unit_flag_types waterfall_settings_unit_flag_types_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_flag_types
    ADD CONSTRAINT waterfall_settings_unit_flag_types_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings_unit_flags waterfall_settings_unit_flags_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_flags
    ADD CONSTRAINT waterfall_settings_unit_flags_pkey PRIMARY KEY (id);


--
-- Name: waterfall_settings_unit_settings waterfall_settings_unit_settings_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_settings
    ADD CONSTRAINT waterfall_settings_unit_settings_pkey PRIMARY KEY (id);


--
-- Name: wl_domain_fees wl_domain_fees_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domain_fees
    ADD CONSTRAINT wl_domain_fees_pkey PRIMARY KEY (id);


--
-- Name: wl_domain_font_files wl_domain_font_files_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domain_font_files
    ADD CONSTRAINT wl_domain_font_files_pkey PRIMARY KEY (id);


--
-- Name: wl_domain_fonts wl_domain_fonts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domain_fonts
    ADD CONSTRAINT wl_domain_fonts_pkey PRIMARY KEY (id);


--
-- Name: wl_domains wl_domains_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY wl_domains
    ADD CONSTRAINT wl_domains_pkey PRIMARY KEY (id);


--
-- Name: yandex_accounts yandex_accounts_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY yandex_accounts
    ADD CONSTRAINT yandex_accounts_pkey PRIMARY KEY (id);


--
-- Name: account_class_date_account_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX account_class_date_account_id ON public.revenue_comparison_stats USING btree (account_class, date, account_id);


--
-- Name: active_app_typed_compensation_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX active_app_typed_compensation_index ON public.compensations USING btree (app_id, compensation_type, date_start);


--
-- Name: ad_units_account_id_account_type_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX ad_units_account_id_account_type_idx ON public.ad_units USING btree (account_id, account_type);


--
-- Name: ad_units_app_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX ad_units_app_id_idx ON public.ad_units USING btree (app_id);


--
-- Name: ad_units_big_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX ad_units_big_idx ON public.ad_units USING btree (app_id, account_type, account_id, ad_type, device_type, format, width, height);


--
-- Name: ad_units_uniq_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX ad_units_uniq_idx ON public.ad_units USING btree (app_id, account_id, campaign_id);


--
-- Name: android_id_created_at_clicks; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX android_id_created_at_clicks ON public.clicks USING btree (android_id, created_at);


--
-- Name: android_sdk_mixed1; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX android_sdk_mixed1 ON public.campaign_images USING btree (android_sdk_version_min, android_sdk_version_max);


--
-- Name: android_sdk_mixed2; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX android_sdk_mixed2 ON public.campaign_images USING btree (has_android_sdk_targeting, android_sdk_version_min, android_sdk_version_max);


--
-- Name: app_id_created_at_clicks; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX app_id_created_at_clicks ON public.clicks USING btree (app_id, created_at);


--
-- Name: app_name_trgm; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX app_name_trgm ON public.apps USING gin (name gin_trgm_ops);


--
-- Name: billings_y2016m10_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m10_date_key ON public.billings_y2016m10 USING btree (date);


--
-- Name: billings_y2016m10_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m10_network_type_key ON public.billings_y2016m10 USING btree (network_type);


--
-- Name: billings_y2016m10_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m10_record_type_key ON public.billings_y2016m10 USING btree (record_type);


--
-- Name: billings_y2016m10_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m10_user_id_key ON public.billings_y2016m10 USING btree (user_id);


--
-- Name: billings_y2016m11_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m11_date_key ON public.billings_y2016m11 USING btree (date);


--
-- Name: billings_y2016m11_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m11_network_type_key ON public.billings_y2016m11 USING btree (network_type);


--
-- Name: billings_y2016m11_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m11_record_type_key ON public.billings_y2016m11 USING btree (record_type);


--
-- Name: billings_y2016m11_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m11_user_id_key ON public.billings_y2016m11 USING btree (user_id);


--
-- Name: billings_y2016m12_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m12_date_key ON public.billings_y2016m12 USING btree (date);


--
-- Name: billings_y2016m12_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m12_network_type_key ON public.billings_y2016m12 USING btree (network_type);


--
-- Name: billings_y2016m12_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m12_record_type_key ON public.billings_y2016m12 USING btree (record_type);


--
-- Name: billings_y2016m12_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2016m12_user_id_key ON public.billings_y2016m12 USING btree (user_id);


--
-- Name: billings_y2017m01_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m01_date_key ON public.billings_y2017m01 USING btree (date);


--
-- Name: billings_y2017m01_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m01_network_type_key ON public.billings_y2017m01 USING btree (network_type);


--
-- Name: billings_y2017m01_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m01_record_type_key ON public.billings_y2017m01 USING btree (record_type);


--
-- Name: billings_y2017m01_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m01_user_id_key ON public.billings_y2017m01 USING btree (user_id);


--
-- Name: billings_y2017m02_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m02_date_key ON public.billings_y2017m02 USING btree (date);


--
-- Name: billings_y2017m02_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m02_network_type_key ON public.billings_y2017m02 USING btree (network_type);


--
-- Name: billings_y2017m02_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m02_record_type_key ON public.billings_y2017m02 USING btree (record_type);


--
-- Name: billings_y2017m02_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m02_user_id_key ON public.billings_y2017m02 USING btree (user_id);


--
-- Name: billings_y2017m03_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m03_date_key ON public.billings_y2017m03 USING btree (date);


--
-- Name: billings_y2017m03_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m03_network_type_key ON public.billings_y2017m03 USING btree (network_type);


--
-- Name: billings_y2017m03_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m03_record_type_key ON public.billings_y2017m03 USING btree (record_type);


--
-- Name: billings_y2017m03_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m03_user_id_key ON public.billings_y2017m03 USING btree (user_id);


--
-- Name: billings_y2017m04_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m04_date_key ON public.billings_y2017m04 USING btree (date);


--
-- Name: billings_y2017m04_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m04_network_type_key ON public.billings_y2017m04 USING btree (network_type);


--
-- Name: billings_y2017m04_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m04_record_type_key ON public.billings_y2017m04 USING btree (record_type);


--
-- Name: billings_y2017m04_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m04_user_id_key ON public.billings_y2017m04 USING btree (user_id);


--
-- Name: billings_y2017m05_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m05_date_key ON public.billings_y2017m05 USING btree (date);


--
-- Name: billings_y2017m05_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m05_network_type_key ON public.billings_y2017m05 USING btree (network_type);


--
-- Name: billings_y2017m05_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m05_record_type_key ON public.billings_y2017m05 USING btree (record_type);


--
-- Name: billings_y2017m05_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m05_user_id_key ON public.billings_y2017m05 USING btree (user_id);


--
-- Name: billings_y2017m06_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m06_date_key ON public.billings_y2017m06 USING btree (date);


--
-- Name: billings_y2017m06_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m06_network_type_key ON public.billings_y2017m06 USING btree (network_type);


--
-- Name: billings_y2017m06_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m06_record_type_key ON public.billings_y2017m06 USING btree (record_type);


--
-- Name: billings_y2017m06_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m06_user_id_key ON public.billings_y2017m06 USING btree (user_id);


--
-- Name: billings_y2017m07_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m07_date_key ON public.billings_y2017m07 USING btree (date);


--
-- Name: billings_y2017m07_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m07_network_type_key ON public.billings_y2017m07 USING btree (network_type);


--
-- Name: billings_y2017m07_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m07_record_type_key ON public.billings_y2017m07 USING btree (record_type);


--
-- Name: billings_y2017m07_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m07_user_id_key ON public.billings_y2017m07 USING btree (user_id);


--
-- Name: billings_y2017m08_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m08_date_key ON public.billings_y2017m08 USING btree (date);


--
-- Name: billings_y2017m08_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m08_network_type_key ON public.billings_y2017m08 USING btree (network_type);


--
-- Name: billings_y2017m08_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m08_record_type_key ON public.billings_y2017m08 USING btree (record_type);


--
-- Name: billings_y2017m08_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m08_user_id_key ON public.billings_y2017m08 USING btree (user_id);


--
-- Name: billings_y2017m09_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m09_date_key ON public.billings_y2017m09 USING btree (date);


--
-- Name: billings_y2017m09_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m09_network_type_key ON public.billings_y2017m09 USING btree (network_type);


--
-- Name: billings_y2017m09_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m09_record_type_key ON public.billings_y2017m09 USING btree (record_type);


--
-- Name: billings_y2017m09_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m09_user_id_key ON public.billings_y2017m09 USING btree (user_id);


--
-- Name: billings_y2017m10_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m10_date_key ON public.billings_y2017m10 USING btree (date);


--
-- Name: billings_y2017m10_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m10_network_type_key ON public.billings_y2017m10 USING btree (network_type);


--
-- Name: billings_y2017m10_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m10_record_type_key ON public.billings_y2017m10 USING btree (record_type);


--
-- Name: billings_y2017m10_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m10_user_id_key ON public.billings_y2017m10 USING btree (user_id);


--
-- Name: billings_y2017m11_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m11_date_key ON public.billings_y2017m11 USING btree (date);


--
-- Name: billings_y2017m11_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m11_network_type_key ON public.billings_y2017m11 USING btree (network_type);


--
-- Name: billings_y2017m11_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m11_record_type_key ON public.billings_y2017m11 USING btree (record_type);


--
-- Name: billings_y2017m11_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m11_user_id_key ON public.billings_y2017m11 USING btree (user_id);


--
-- Name: billings_y2017m12_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m12_date_key ON public.billings_y2017m12 USING btree (date);


--
-- Name: billings_y2017m12_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m12_network_type_key ON public.billings_y2017m12 USING btree (network_type);


--
-- Name: billings_y2017m12_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m12_record_type_key ON public.billings_y2017m12 USING btree (record_type);


--
-- Name: billings_y2017m12_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2017m12_user_id_key ON public.billings_y2017m12 USING btree (user_id);


--
-- Name: billings_y2018m01_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m01_date_key ON public.billings_y2018m01 USING btree (date);


--
-- Name: billings_y2018m01_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m01_network_type_key ON public.billings_y2018m01 USING btree (network_type);


--
-- Name: billings_y2018m01_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m01_record_type_key ON public.billings_y2018m01 USING btree (record_type);


--
-- Name: billings_y2018m01_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m01_user_id_key ON public.billings_y2018m01 USING btree (user_id);


--
-- Name: billings_y2018m02_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m02_date_key ON public.billings_y2018m02 USING btree (date);


--
-- Name: billings_y2018m02_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m02_network_type_key ON public.billings_y2018m02 USING btree (network_type);


--
-- Name: billings_y2018m02_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m02_record_type_key ON public.billings_y2018m02 USING btree (record_type);


--
-- Name: billings_y2018m02_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m02_user_id_key ON public.billings_y2018m02 USING btree (user_id);


--
-- Name: billings_y2018m03_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m03_date_key ON public.billings_y2018m03 USING btree (date);


--
-- Name: billings_y2018m03_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m03_network_type_key ON public.billings_y2018m03 USING btree (network_type);


--
-- Name: billings_y2018m03_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m03_record_type_key ON public.billings_y2018m03 USING btree (record_type);


--
-- Name: billings_y2018m03_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m03_user_id_key ON public.billings_y2018m03 USING btree (user_id);


--
-- Name: billings_y2018m04_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m04_date_key ON public.billings_y2018m04 USING btree (date);


--
-- Name: billings_y2018m04_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m04_network_type_key ON public.billings_y2018m04 USING btree (network_type);


--
-- Name: billings_y2018m04_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m04_record_type_key ON public.billings_y2018m04 USING btree (record_type);


--
-- Name: billings_y2018m04_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m04_user_id_key ON public.billings_y2018m04 USING btree (user_id);


--
-- Name: billings_y2018m05_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m05_date_key ON public.billings_y2018m05 USING btree (date);


--
-- Name: billings_y2018m05_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m05_network_type_key ON public.billings_y2018m05 USING btree (network_type);


--
-- Name: billings_y2018m05_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m05_record_type_key ON public.billings_y2018m05 USING btree (record_type);


--
-- Name: billings_y2018m05_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m05_user_id_key ON public.billings_y2018m05 USING btree (user_id);


--
-- Name: billings_y2018m06_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m06_date_key ON public.billings_y2018m06 USING btree (date);


--
-- Name: billings_y2018m06_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m06_network_type_key ON public.billings_y2018m06 USING btree (network_type);


--
-- Name: billings_y2018m06_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m06_record_type_key ON public.billings_y2018m06 USING btree (record_type);


--
-- Name: billings_y2018m06_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m06_user_id_key ON public.billings_y2018m06 USING btree (user_id);


--
-- Name: billings_y2018m07_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m07_date_key ON public.billings_y2018m07 USING btree (date);


--
-- Name: billings_y2018m07_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m07_network_type_key ON public.billings_y2018m07 USING btree (network_type);


--
-- Name: billings_y2018m07_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m07_record_type_key ON public.billings_y2018m07 USING btree (record_type);


--
-- Name: billings_y2018m07_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m07_user_id_key ON public.billings_y2018m07 USING btree (user_id);


--
-- Name: billings_y2018m08_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m08_date_key ON public.billings_y2018m08 USING btree (date);


--
-- Name: billings_y2018m08_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m08_network_type_key ON public.billings_y2018m08 USING btree (network_type);


--
-- Name: billings_y2018m08_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m08_record_type_key ON public.billings_y2018m08 USING btree (record_type);


--
-- Name: billings_y2018m08_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m08_user_id_key ON public.billings_y2018m08 USING btree (user_id);


--
-- Name: billings_y2018m09_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m09_date_key ON public.billings_y2018m09 USING btree (date);


--
-- Name: billings_y2018m09_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m09_network_type_key ON public.billings_y2018m09 USING btree (network_type);


--
-- Name: billings_y2018m09_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m09_record_type_key ON public.billings_y2018m09 USING btree (record_type);


--
-- Name: billings_y2018m09_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m09_user_id_key ON public.billings_y2018m09 USING btree (user_id);


--
-- Name: billings_y2018m10_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m10_date_key ON public.billings_y2018m10 USING btree (date);


--
-- Name: billings_y2018m10_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m10_network_type_key ON public.billings_y2018m10 USING btree (network_type);


--
-- Name: billings_y2018m10_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m10_record_type_key ON public.billings_y2018m10 USING btree (record_type);


--
-- Name: billings_y2018m10_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m10_user_id_key ON public.billings_y2018m10 USING btree (user_id);


--
-- Name: billings_y2018m11_date_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m11_date_key ON public.billings_y2018m11 USING btree (date);


--
-- Name: billings_y2018m11_network_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m11_network_type_key ON public.billings_y2018m11 USING btree (network_type);


--
-- Name: billings_y2018m11_record_type_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m11_record_type_key ON public.billings_y2018m11 USING btree (record_type);


--
-- Name: billings_y2018m11_user_id_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX billings_y2018m11_user_id_key ON public.billings_y2018m11 USING btree (user_id);


--
-- Name: campaign_aggregated_stats_campaign_country_app; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX campaign_aggregated_stats_campaign_country_app ON public.campaign_aggregated_stats USING btree (campaign_id, country_id, app_id);


--
-- Name: campaign_commission_unique_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX campaign_commission_unique_index ON public.campaign_commissions USING btree (ad_type, campaign_global_id, app_id, user_id, user_only);


--
-- Name: campaign_images_deleted_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX campaign_images_deleted_idx ON public.campaign_images USING btree (deleted);


--
-- Name: campaign_images_platform_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX campaign_images_platform_id_idx ON public.campaign_images USING btree (platform_id);


--
-- Name: campaign_images_type_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX campaign_images_type_idx ON public.campaign_images USING btree (type);


--
-- Name: campaigns_deleted_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX campaigns_deleted_idx ON public.campaigns USING btree (deleted);


--
-- Name: campaigns_is_network_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX campaigns_is_network_idx ON public.campaigns USING btree (is_network);


--
-- Name: cities_uniq_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX cities_uniq_index ON public.cities USING btree (country_id, subdivision1_id, subdivision1_id, geoname_id);


--
-- Name: clicks_master_android_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_android_id_idx ON public.clicks_master USING btree (android_id);


--
-- Name: clicks_master_app_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_app_id_idx ON public.clicks_master USING btree (app_id);


--
-- Name: clicks_master_campaign_image_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_campaign_image_id_idx ON public.clicks_master USING btree (campaign_image_id);


--
-- Name: clicks_master_country_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_country_id_idx ON public.clicks_master USING btree (country_id);


--
-- Name: clicks_master_created_at_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_created_at_idx ON public.clicks_master USING btree (created_at);


--
-- Name: clicks_master_device_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_device_id_idx ON public.clicks_master USING btree (device_id);


--
-- Name: clicks_master_ip_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_ip_idx ON public.clicks_master USING btree (ip);


--
-- Name: clicks_master_ua_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_ua_idx ON public.clicks_master USING btree (ua);


--
-- Name: clicks_master_user_only_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_user_only_idx ON public.clicks_master USING btree (user_only);


--
-- Name: clicks_master_white_sdk_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_white_sdk_idx ON public.clicks_master USING btree (white_sdk);


--
-- Name: clicks_master_x_forwarded_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX clicks_master_x_forwarded_idx ON public.clicks_master USING btree (x_forwarded);


--
-- Name: client_history_stats_first_unique_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX client_history_stats_first_unique_index ON public.client_history_stats USING btree (accountable_id, accountable_type, date, aasm_state);


--
-- Name: conv_parts_conversations; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX conv_parts_conversations ON public.intercom_conversation_parts USING btree (conversation_id, part_id);


--
-- Name: dash_views_logs_multi1; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX dash_views_logs_multi1 ON public.dash_views_logs USING btree (user_id, log_date, group_params, filter_params);


--
-- Name: del_apps; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX del_apps ON public.apps USING btree (deleted);


--
-- Name: device_models_multi; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX device_models_multi ON public.device_models USING btree (device_manufacturer_id, marketing_name, device_name, model);


--
-- Name: fast_ecpm_cap_option_c_index; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX fast_ecpm_cap_option_c_index ON public.fast_ecpm_capping_options USING gin (app_id, country_ids, network_ids, ad_types);


--
-- Name: fast_ecpm_option_c_index; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX fast_ecpm_option_c_index ON public.fast_ecpm_new_options USING btree (app_id, country_id, enabled);


--
-- Name: fast_ecpm_stats_cleaned_uniq; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX fast_ecpm_stats_cleaned_uniq ON public.fast_ecpm_stats USING btree (app_id, country_id, campaign_image_id);


--
-- Name: fast_ecpm_stats_tmp_campaign_image_id_country_id_app_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX fast_ecpm_stats_tmp_campaign_image_id_country_id_app_id_idx ON public.fast_ecpm_stats_tmp USING btree (campaign_image_id, country_id, app_id);


--
-- Name: fast_ecpm_stats_uniq; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX fast_ecpm_stats_uniq ON public.fast_ecpm_stats_old USING btree (campaign_image_id, country_id, app_id);


--
-- Name: fraud_users_emails_index; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX fraud_users_emails_index ON public.fraud_user_emails USING gin (value gin_trgm_ops);


--
-- Name: full_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX full_index ON public.fast_ecpm_stats_exclusions USING btree (app_id, country_id, campaign_image_id, ecpm);


--
-- Name: geo_country_latitude_longitude_index; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX geo_country_latitude_longitude_index ON public.campaign_image_coordinates USING btree (country_id, latitude, longitude);


--
-- Name: idx_root_setting_condition; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX idx_root_setting_condition ON public.waterfall_settings_root_settings USING btree (app_id, network_id, real_sdk_version_min, real_sdk_version_max, real_sdk_min, real_sdk_max, real_android_min, real_android_max, real_ios_min, real_ios_max, ad_type_interstitial, ad_type_video, ad_type_banner, ad_type_native, ad_type_mrec, ad_type_rewarded_video);


--
-- Name: idx_unit_setting_condition; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX idx_unit_setting_condition ON public.waterfall_settings_unit_settings USING btree (app_id, network_id, real_sdk_version_min, real_sdk_version_max, real_sdk_min, real_sdk_max, real_android_min, real_android_max, real_ios_min, real_ios_max, ad_type_interstitial, ad_type_video, ad_type_banner, ad_type_native, ad_type_mrec, ad_type_rewarded_video);


--
-- Name: idx_waterfall_settings_root_settings_condition3; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX idx_waterfall_settings_root_settings_condition3 ON public.waterfall_settings_root_settings USING btree (app_id, ad_ids, network_id, real_sdk_version_min, real_sdk_version_max, real_sdk_min, real_sdk_max, real_android_min, real_android_max, real_ios_min, real_ios_max, ad_type_interstitial, ad_type_video, ad_type_banner, ad_type_native, ad_type_mrec, ad_type_rewarded_video);


--
-- Name: idx_waterfall_settings_unit_settings_condition3; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX idx_waterfall_settings_unit_settings_condition3 ON public.waterfall_settings_unit_settings USING btree (app_id, ad_ids, network_id, real_sdk_version_min, real_sdk_version_max, real_sdk_min, real_sdk_max, real_android_min, real_android_max, real_ios_min, real_ios_max, ad_type_interstitial, ad_type_video, ad_type_banner, ad_type_native, ad_type_mrec, ad_type_rewarded_video);


--
-- Name: index_account_employees_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_account_employees_on_user_id ON public.account_employees USING btree (user_id);


--
-- Name: index_account_shares_on_owner_id_and_developer_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_account_shares_on_owner_id_and_developer_id ON public.account_shares USING btree (owner_id, developer_id);


--
-- Name: index_account_shares_on_role; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_account_shares_on_role ON public.account_shares USING btree (role);


--
-- Name: index_account_shares_on_share_token_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_account_shares_on_share_token_id ON public.account_shares USING btree (share_token_id);


--
-- Name: index_active_admin_comments_on_author_type_and_author_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_active_admin_comments_on_author_type_and_author_id ON public.active_admin_comments USING btree (author_type, author_id);


--
-- Name: index_active_admin_comments_on_namespace; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_active_admin_comments_on_namespace ON public.active_admin_comments USING btree (namespace);


--
-- Name: index_active_admin_comments_on_resource_type_and_resource_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_active_admin_comments_on_resource_type_and_resource_id ON public.active_admin_comments USING btree (resource_type, resource_id);


--
-- Name: index_active_apps_on_app_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_active_apps_on_app_id_and_date ON public.active_apps USING btree (app_id, date);


--
-- Name: index_active_users_on_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_active_users_on_date ON public.active_users USING btree (date);


--
-- Name: index_active_users_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_active_users_on_user_id ON public.active_users USING btree (user_id);


--
-- Name: index_ad_request_settings_on_name; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_ad_request_settings_on_name ON public.ad_request_settings USING btree (name);


--
-- Name: index_ad_unit_versions_on_item_type_and_item_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ad_unit_versions_on_item_type_and_item_id ON public.ad_unit_versions USING btree (item_type, item_id);


--
-- Name: index_ad_units_on_account_id_and_account_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ad_units_on_account_id_and_account_type ON public.ad_units_backup USING btree (account_id, account_type);


--
-- Name: index_ad_units_on_account_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ad_units_on_account_type ON public.ad_units USING btree (account_type);


--
-- Name: index_ad_units_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ad_units_on_app_id ON public.ad_units_backup USING btree (app_id);


--
-- Name: index_ad_units_on_campaign_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ad_units_on_campaign_id ON public.ad_units USING btree (campaign_id);


--
-- Name: index_ad_units_on_code; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ad_units_on_code ON public.ad_units USING btree (code);


--
-- Name: index_ad_units_on_package_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ad_units_on_package_name ON public.ad_units USING btree (package_name);


--
-- Name: index_admob_accounts_on_owner_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_admob_accounts_on_owner_id ON public.admob_accounts USING btree (owner_id);


--
-- Name: index_adms_on_complain_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_adms_on_complain_id ON public.adms USING btree (complain_id);


--
-- Name: index_advertisement_partners_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_advertisement_partners_on_user_id ON public.advertisement_partners USING btree (user_id);


--
-- Name: index_ae_settings_on_app_ids_array; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ae_settings_on_app_ids_array ON public.ae_settings USING gin (app_ids_array);


--
-- Name: index_app_fraud_logs_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_app_fraud_logs_on_app_id ON public.app_fraud_logs USING btree (app_id);


--
-- Name: index_app_installer_data_on_app_id_and_report_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_app_installer_data_on_app_id_and_report_date ON public.app_installer_data USING btree (app_id, report_date);


--
-- Name: index_applovin_accounts_on_owner_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_applovin_accounts_on_owner_id ON public.applovin_accounts USING btree (owner_id);


--
-- Name: index_apps_campaign_images_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_campaign_images_on_app_id ON public.apps_campaign_images USING btree (app_id);


--
-- Name: index_apps_campaign_images_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_campaign_images_on_campaign_image_id ON public.apps_campaign_images USING btree (campaign_image_id);


--
-- Name: index_apps_categories_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_categories_on_app_id ON public.apps_categories USING btree (app_id);


--
-- Name: index_apps_categories_on_app_id_and_category_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_apps_categories_on_app_id_and_category_id ON public.apps_categories USING btree (app_id, category_id);


--
-- Name: index_apps_categories_on_category_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_categories_on_category_id ON public.apps_categories USING btree (category_id);


--
-- Name: index_apps_on_app_key; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_app_key ON public.apps USING btree (app_key);


--
-- Name: index_apps_on_interstitial_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_interstitial_type ON public.apps USING btree (interstitial_type);


--
-- Name: index_apps_on_is_deleted; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_is_deleted ON public.apps USING btree (is_deleted);


--
-- Name: index_apps_on_lower_package_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_lower_package_name ON public.apps USING btree (lower((package_name)::text));


--
-- Name: index_apps_on_package_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_package_name ON public.apps USING btree (package_name);


--
-- Name: index_apps_on_platform_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_platform_id ON public.apps USING btree (platform_id);


--
-- Name: index_apps_on_state; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_state ON public.apps USING btree (state);


--
-- Name: index_apps_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_user_id ON public.apps USING btree (user_id);


--
-- Name: index_apps_on_video_views_reached; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_video_views_reached ON public.apps USING btree (video_views_reached);


--
-- Name: index_apps_on_views_reached; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_apps_on_views_reached ON public.apps USING btree (views_reached);


--
-- Name: index_bad_devices_on_android_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_bad_devices_on_android_id ON public.bad_devices USING btree (android_id);


--
-- Name: index_bidder_bidder_formats_on_bidder_format_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bidder_bidder_formats_on_bidder_format_id ON public.bidder_bidder_formats USING btree (bidder_format_id);


--
-- Name: index_bidder_bidder_formats_on_bidder_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bidder_bidder_formats_on_bidder_id ON public.bidder_bidder_formats USING btree (bidder_id);


--
-- Name: index_bidder_campaigns_on_bidder_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bidder_campaigns_on_bidder_id ON public.bidder_campaigns USING btree (bidder_id);


--
-- Name: index_bidder_campaigns_on_campaign_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bidder_campaigns_on_campaign_id ON public.bidder_campaigns USING btree (campaign_id);


--
-- Name: index_bidder_countries_on_bidder_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bidder_countries_on_bidder_id ON public.bidder_countries USING btree (bidder_id);


--
-- Name: index_bidder_countries_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bidder_countries_on_country_id ON public.bidder_countries USING btree (country_id);


--
-- Name: index_bidder_settings_on_bidder_id_and_setting_name; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_bidder_settings_on_bidder_id_and_setting_name ON public.bidder_settings USING btree (bidder_id, setting_name);


--
-- Name: index_bids_on_bidder_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bids_on_bidder_id ON public.bids USING btree (bidder_id);


--
-- Name: index_bids_on_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_bids_on_date ON public.bids USING btree (date);


--
-- Name: index_cached_files_on_account_id_and_account_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_cached_files_on_account_id_and_account_type ON public.cached_files USING btree (account_id, account_type);


--
-- Name: index_cached_files_on_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_cached_files_on_name ON public.cached_files USING btree (name);


--
-- Name: index_campaign_coordinates_on_spot; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_coordinates_on_spot ON public.campaign_coordinates USING gist (spot);


--
-- Name: index_campaign_image_coordinates_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_image_coordinates_on_country_id ON public.campaign_image_coordinates USING btree (country_id);


--
-- Name: index_campaign_image_coordinates_on_latitude_and_longitude; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_image_coordinates_on_latitude_and_longitude ON public.campaign_image_coordinates USING btree (latitude, longitude);


--
-- Name: index_campaign_image_devices_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_image_devices_on_campaign_image_id ON public.campaign_image_devices USING btree (campaign_image_id);


--
-- Name: index_campaign_image_devices_on_campaign_image_id_and_device_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_campaign_image_devices_on_campaign_image_id_and_device_id ON public.campaign_image_devices USING btree (campaign_image_id, device_id);


--
-- Name: index_campaign_image_devices_on_device_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_image_devices_on_device_id ON public.campaign_image_devices USING btree (device_id);


--
-- Name: index_campaign_images_categories_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_categories_on_campaign_image_id ON public.campaign_images_categories USING btree (campaign_image_id);


--
-- Name: index_campaign_images_categories_on_category_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_categories_on_category_id ON public.campaign_images_categories USING btree (category_id);


--
-- Name: index_campaign_images_countries_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_countries_on_campaign_image_id ON public.campaign_images_countries USING btree (campaign_image_id);


--
-- Name: index_campaign_images_countries_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_countries_on_country_id ON public.campaign_images_countries USING btree (country_id);


--
-- Name: index_campaign_images_max_mind_cities_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_max_mind_cities_on_campaign_image_id ON public.campaign_images_max_mind_cities USING btree (campaign_image_id);


--
-- Name: index_campaign_images_max_mind_cities_on_max_mind_city_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_max_mind_cities_on_max_mind_city_id ON public.campaign_images_max_mind_cities USING btree (max_mind_city_id);


--
-- Name: index_campaign_images_mobile_networks_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_mobile_networks_on_campaign_image_id ON public.campaign_images_mobile_networks USING btree (campaign_image_id);


--
-- Name: index_campaign_images_mobile_networks_on_mobile_network_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_mobile_networks_on_mobile_network_id ON public.campaign_images_mobile_networks USING btree (mobile_network_id);


--
-- Name: index_campaign_images_on_campaign_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_campaign_id ON public.campaign_images USING btree (campaign_id);


--
-- Name: index_campaign_images_on_connection_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_connection_type ON public.campaign_images USING btree (connection_type);


--
-- Name: index_campaign_images_on_creative_id_and_creative_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_creative_id_and_creative_type ON public.campaign_images USING btree (creative_id, creative_type);


--
-- Name: index_campaign_images_on_device_model_ids; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_device_model_ids ON public.campaign_images USING gin (device_model_ids);


--
-- Name: index_campaign_images_on_device_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_device_type ON public.campaign_images USING btree (device_type);


--
-- Name: index_campaign_images_on_frequency_capping_blocked; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_frequency_capping_blocked ON public.campaign_images USING btree (frequency_capping_blocked);


--
-- Name: index_campaign_images_on_gender; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_gender ON public.campaign_images USING btree (gender);


--
-- Name: index_campaign_images_on_has_android_sdk_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_has_android_sdk_targeting ON public.campaign_images USING btree (has_android_sdk_targeting);


--
-- Name: index_campaign_images_on_has_apps_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_has_apps_targeting ON public.campaign_images USING btree (has_apps_targeting);


--
-- Name: index_campaign_images_on_has_device_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_has_device_targeting ON public.campaign_images USING btree (has_device_targeting);


--
-- Name: index_campaign_images_on_has_ios_sdk_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_has_ios_sdk_targeting ON public.campaign_images USING btree (has_ios_sdk_targeting);


--
-- Name: index_campaign_images_on_has_mobile_network_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_has_mobile_network_targeting ON public.campaign_images USING btree (has_mobile_network_targeting);


--
-- Name: index_campaign_images_on_has_models_exclusion_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_has_models_exclusion_targeting ON public.campaign_images USING btree (has_models_exclusion_targeting);


--
-- Name: index_campaign_images_on_has_models_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_has_models_targeting ON public.campaign_images USING btree (has_models_targeting);


--
-- Name: index_campaign_images_on_is_rewarded_video_only; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_is_rewarded_video_only ON public.campaign_images USING btree (is_rewarded_video_only);


--
-- Name: index_campaign_images_on_max_age; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_max_age ON public.campaign_images USING btree (max_age);


--
-- Name: index_campaign_images_on_max_android_version; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_max_android_version ON public.campaign_images USING btree (max_android_version);


--
-- Name: index_campaign_images_on_max_ios_version; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_max_ios_version ON public.campaign_images USING btree (max_ios_version);


--
-- Name: index_campaign_images_on_min_age; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_min_age ON public.campaign_images USING btree (min_age);


--
-- Name: index_campaign_images_on_min_android_version; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_min_android_version ON public.campaign_images USING btree (min_android_version);


--
-- Name: index_campaign_images_on_min_ios_version; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_min_ios_version ON public.campaign_images USING btree (min_ios_version);


--
-- Name: index_campaign_images_on_negative_keywords; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_negative_keywords ON public.campaign_images USING gin (negative_keywords);


--
-- Name: index_campaign_images_on_priority_finish_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_priority_finish_at ON public.campaign_images USING btree (priority_finish_at);


--
-- Name: index_campaign_images_on_schedule; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_schedule ON public.campaign_images USING gin (schedule);


--
-- Name: index_campaign_images_on_system_locale_targeting; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_images_on_system_locale_targeting ON public.campaign_images USING btree (system_locale_targeting);


--
-- Name: index_campaign_types_on_api_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_types_on_api_name ON public.campaign_types USING btree (api_name);


--
-- Name: index_campaign_types_on_partner_type_and_partner_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_types_on_partner_type_and_partner_id ON public.campaign_types USING btree (partner_type, partner_id);


--
-- Name: index_campaign_types_on_show_at_frontend; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_types_on_show_at_frontend ON public.campaign_types USING btree (show_at_frontend);


--
-- Name: index_campaign_types_on_sys_package_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaign_types_on_sys_package_name ON public.campaign_types USING btree (sys_package_name);


--
-- Name: index_campaigns_creative_sets_on_campaign_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_creative_sets_on_campaign_id ON public.campaigns_creative_sets USING btree (campaign_id);


--
-- Name: index_campaigns_creative_sets_on_campaign_image_ids; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_creative_sets_on_campaign_image_ids ON public.campaigns_creative_sets USING gin (campaign_image_ids);


--
-- Name: index_campaigns_creative_sets_on_creative_set_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_creative_sets_on_creative_set_id ON public.campaigns_creative_sets USING btree (creative_set_id);


--
-- Name: index_campaigns_on_app_ids_array; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_app_ids_array ON public.campaigns USING gin (app_ids_array);


--
-- Name: index_campaigns_on_app_ids_include; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_app_ids_include ON public.campaigns USING btree (app_ids_include);


--
-- Name: index_campaigns_on_coppa_complain; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_coppa_complain ON public.campaigns USING btree (coppa_complain);


--
-- Name: index_campaigns_on_frequency_capping_period; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_frequency_capping_period ON public.campaigns USING btree (frequency_capping_period);


--
-- Name: index_campaigns_on_frequency_capping_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_frequency_capping_type ON public.campaigns USING btree (frequency_capping_type);


--
-- Name: index_campaigns_on_loss_limit_achieved; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_loss_limit_achieved ON public.campaigns USING btree (loss_limit_achieved);


--
-- Name: index_campaigns_on_package_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_package_name ON public.campaigns USING btree (package_name);


--
-- Name: index_campaigns_on_target_locales; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_campaigns_on_target_locales ON public.campaigns USING gin (target_locales);


--
-- Name: index_categories_contents_on_category_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_categories_contents_on_category_id ON public.categories_contents USING btree (category_id);


--
-- Name: index_categories_contents_on_content_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_categories_contents_on_content_id ON public.categories_contents USING btree (content_id);


--
-- Name: index_categories_on_store_name_and_platform_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_categories_on_store_name_and_platform_id ON public.categories USING btree (store_name, platform_id);


--
-- Name: index_categories_users_on_category_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_categories_users_on_category_id ON public.categories_users USING btree (category_id);


--
-- Name: index_categories_users_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_categories_users_on_user_id ON public.categories_users USING btree (user_id);


--
-- Name: index_chartboost_accounts_on_owner_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_chartboost_accounts_on_owner_id ON public.chartboost_accounts USING btree (owner_id);


--
-- Name: index_chartboost_campaigns_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_chartboost_campaigns_on_app_id ON public.chartboost_campaigns USING btree (app_id);


--
-- Name: index_chartboost_campaigns_on_campaign_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_chartboost_campaigns_on_campaign_id ON public.chartboost_campaigns USING btree (campaign_id);


--
-- Name: index_chartboost_campaigns_on_chartboost_account_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_chartboost_campaigns_on_chartboost_account_id ON public.chartboost_campaigns USING btree (chartboost_account_id);


--
-- Name: index_clicks_on_uuid; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_on_uuid ON public.clicks USING btree (uuid);


--
-- Name: index_clicks_y2018m08_on_ad_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m08_on_ad_id_and_created_at ON public.clicks_y2018m08 USING btree (ad_id, created_at);


--
-- Name: index_clicks_y2018m08_on_app_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m08_on_app_id_and_created_at ON public.clicks_y2018m08 USING btree (app_id, created_at);


--
-- Name: index_clicks_y2018m08_on_app_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m08_on_app_id_and_date ON public.clicks_y2018m08 USING btree (app_id, date);


--
-- Name: index_clicks_y2018m09_on_ad_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m09_on_ad_id_and_created_at ON public.clicks_y2018m09 USING btree (ad_id, created_at);


--
-- Name: index_clicks_y2018m09_on_ad_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m09_on_ad_id_and_date ON public.clicks_y2018m09 USING btree (ad_id, date);


--
-- Name: index_clicks_y2018m09_on_app_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m09_on_app_id_and_created_at ON public.clicks_y2018m09 USING btree (app_id, created_at);


--
-- Name: index_clicks_y2018m09_on_app_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m09_on_app_id_and_date ON public.clicks_y2018m09 USING btree (app_id, date);


--
-- Name: index_clicks_y2018m10_on_ad_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m10_on_ad_id_and_created_at ON public.clicks_y2018m10 USING btree (ad_id, created_at);


--
-- Name: index_clicks_y2018m10_on_ad_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m10_on_ad_id_and_date ON public.clicks_y2018m10 USING btree (ad_id, date);


--
-- Name: index_clicks_y2018m10_on_app_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m10_on_app_id_and_created_at ON public.clicks_y2018m10 USING btree (app_id, created_at);


--
-- Name: index_clicks_y2018m10_on_app_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m10_on_app_id_and_date ON public.clicks_y2018m10 USING btree (app_id, date);


--
-- Name: index_clicks_y2018m11_on_ad_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m11_on_ad_id_and_created_at ON public.clicks_y2018m11 USING btree (ad_id, created_at);


--
-- Name: index_clicks_y2018m11_on_ad_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m11_on_ad_id_and_date ON public.clicks_y2018m11 USING btree (ad_id, date);


--
-- Name: index_clicks_y2018m11_on_app_id_and_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m11_on_app_id_and_created_at ON public.clicks_y2018m11 USING btree (app_id, created_at);


--
-- Name: index_clicks_y2018m11_on_app_id_and_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_clicks_y2018m11_on_app_id_and_date ON public.clicks_y2018m11 USING btree (app_id, date);


--
-- Name: index_complains_on_campaign_image_id_and_reason; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_complains_on_campaign_image_id_and_reason ON public.complains USING btree (campaign_image_id, reason);


--
-- Name: index_contents_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_contents_on_app_id ON public.contents USING btree (app_id);


--
-- Name: index_countries_on_alternatives; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_countries_on_alternatives ON public.countries USING btree (alternatives);


--
-- Name: index_countries_on_code; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_countries_on_code ON public.countries USING btree (code);


--
-- Name: index_countries_on_continent_code; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_countries_on_continent_code ON public.countries USING btree (continent_code);


--
-- Name: index_creative_files_on_creative_set_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_creative_files_on_creative_set_id ON public.creative_files USING btree (creative_set_id);


--
-- Name: index_creative_sets_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_creative_sets_on_user_id ON public.creative_sets USING btree (user_id);


--
-- Name: index_dash_views_logs_on_date_from; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_dash_views_logs_on_date_from ON public.dash_views_logs USING btree (date_from);


--
-- Name: index_dash_views_logs_on_date_to; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_dash_views_logs_on_date_to ON public.dash_views_logs USING btree (date_to);


--
-- Name: index_dash_views_logs_on_filter_params; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_dash_views_logs_on_filter_params ON public.dash_views_logs USING btree (filter_params);


--
-- Name: index_dash_views_logs_on_group_params; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_dash_views_logs_on_group_params ON public.dash_views_logs USING btree (group_params);


--
-- Name: index_dash_views_logs_on_log_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_dash_views_logs_on_log_date ON public.dash_views_logs USING btree (log_date);


--
-- Name: index_dash_views_logs_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_dash_views_logs_on_user_id ON public.dash_views_logs USING btree (user_id);


--
-- Name: index_dashboard_predefined_filters_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_dashboard_predefined_filters_on_user_id ON public.dashboard_predefined_filters USING btree (user_id);


--
-- Name: index_deals_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_deals_on_user_id ON public.deals USING btree (user_id);


--
-- Name: index_device_manufacturers_on_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_device_manufacturers_on_name ON public.device_manufacturers USING btree (name);


--
-- Name: index_device_models_on_device_manufacturer_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_device_models_on_device_manufacturer_id ON public.device_models USING btree (device_manufacturer_id);


--
-- Name: index_device_models_on_device_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_device_models_on_device_type ON public.device_models USING btree (device_type);


--
-- Name: index_device_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_device_name ON public.device_models USING gin (device_name gin_trgm_ops);


--
-- Name: index_devices_on_android; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_devices_on_android ON public.devices USING btree (android);


--
-- Name: index_devices_on_model; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_devices_on_model ON public.devices USING btree (model);


--
-- Name: index_devices_on_sdk; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_devices_on_sdk ON public.devices USING btree (sdk);


--
-- Name: index_docs_on_version; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_docs_on_version ON public.docs USING btree (version);


--
-- Name: index_docs_sets_on_history_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_docs_sets_on_history_id ON public.docs_sets USING btree (history_id);


--
-- Name: index_domains_on_advertisement_partners_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_domains_on_advertisement_partners_id ON public.domains USING btree (advertisement_partners_id);


--
-- Name: index_domains_on_deleted; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_domains_on_deleted ON public.domains USING btree (deleted);


--
-- Name: index_domains_on_locale; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_domains_on_locale ON public.domains USING btree (locale);


--
-- Name: index_domains_on_status; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_domains_on_status ON public.domains USING btree (status);


--
-- Name: index_exceptional_targetings_on_app_id_and_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_exceptional_targetings_on_app_id_and_country_id ON public.exceptional_targetings USING btree (app_id, country_id);


--
-- Name: index_exceptional_targetings_on_campaign_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_exceptional_targetings_on_campaign_id ON public.exceptional_targetings USING btree (campaign_id);


--
-- Name: index_extension_logs_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_extension_logs_on_user_id ON public.extension_logs USING btree (user_id);


--
-- Name: index_fast_affiliate_stats_on_date_and_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_fast_affiliate_stats_on_date_and_user_id ON public.fast_affiliate_stats USING btree (date, user_id);


--
-- Name: index_fast_ecpm_capping_options_on_enabled; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_fast_ecpm_capping_options_on_enabled ON public.fast_ecpm_capping_options USING btree (enabled);


--
-- Name: index_fast_ecpm_stats_exclusions_on_deal_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_fast_ecpm_stats_exclusions_on_deal_id ON public.fast_ecpm_stats_exclusions USING btree (deal_id);


--
-- Name: index_fixed_admob_campaigns_sets_on_app_id_and_type_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_fixed_admob_campaigns_sets_on_app_id_and_type_id ON public.fixed_admob_campaigns_sets USING btree (app_id, type_id);


--
-- Name: index_fraud_click_series_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_fraud_click_series_on_app_id ON public.fraud_click_series USING btree (app_id);


--
-- Name: index_frequency_capping_stats_on_campaign_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_frequency_capping_stats_on_campaign_id ON public.frequency_capping_stats USING btree (campaign_id);


--
-- Name: index_global_settings_on_setting_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_global_settings_on_setting_name ON public.global_settings USING btree (setting_name);


--
-- Name: index_global_settings_sections_on_depth; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_global_settings_sections_on_depth ON public.global_settings_sections USING btree (depth);


--
-- Name: index_global_settings_sections_on_lft; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_global_settings_sections_on_lft ON public.global_settings_sections USING btree (lft);


--
-- Name: index_global_settings_sections_on_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_global_settings_sections_on_name ON public.global_settings_sections USING btree (name);


--
-- Name: index_global_settings_sections_on_parent_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_global_settings_sections_on_parent_id ON public.global_settings_sections USING btree (parent_id);


--
-- Name: index_global_settings_sections_on_rgt; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_global_settings_sections_on_rgt ON public.global_settings_sections USING btree (rgt);


--
-- Name: index_intercom_conversation_parts_on_conversation_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_intercom_conversation_parts_on_conversation_id ON public.intercom_conversation_parts USING btree (conversation_id);


--
-- Name: index_intercom_conversation_parts_on_created_at_intercom; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_intercom_conversation_parts_on_created_at_intercom ON public.intercom_conversation_parts USING btree (created_at_intercom);


--
-- Name: index_intercom_conversations_on_conversation_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_intercom_conversations_on_conversation_id ON public.intercom_conversations USING btree (conversation_id);


--
-- Name: index_intercom_conversations_on_created_at_intercom; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_intercom_conversations_on_created_at_intercom ON public.intercom_conversations USING btree (created_at_intercom);


--
-- Name: index_intercom_conversations_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_intercom_conversations_on_user_id ON public.intercom_conversations USING btree (user_id);


--
-- Name: index_intercom_network_recommendeds_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_intercom_network_recommendeds_on_app_id ON public.intercom_network_recommendeds USING btree (app_id);


--
-- Name: index_intercom_network_recommendeds_on_network; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_intercom_network_recommendeds_on_network ON public.intercom_network_recommendeds USING btree (network);


--
-- Name: index_invoice_requests_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_invoice_requests_on_user_id ON public.invoice_requests USING btree (user_id);


--
-- Name: index_ip_addresses_on_address; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_ip_addresses_on_address ON public.ip_addresses USING btree (address);


--
-- Name: index_ip_addresses_on_spamcop; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ip_addresses_on_spamcop ON public.ip_addresses USING btree (spamcop);


--
-- Name: index_ip_addresses_on_spamhaus; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ip_addresses_on_spamhaus ON public.ip_addresses USING btree (spamhaus);


--
-- Name: index_ip_addresses_on_stopforumspam; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ip_addresses_on_stopforumspam ON public.ip_addresses USING btree (stopforumspam);


--
-- Name: index_ip_addresses_on_vpn; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ip_addresses_on_vpn ON public.ip_addresses USING btree (vpn);


--
-- Name: index_ip_addresses_on_wpbl; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ip_addresses_on_wpbl ON public.ip_addresses USING btree (wpbl);


--
-- Name: index_magic_monies_on_type_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_magic_monies_on_type_id ON public.magic_monies USING btree (type_id);


--
-- Name: index_magic_monies_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_magic_monies_on_user_id ON public.magic_monies USING btree (user_id);


--
-- Name: index_managers_on_sf_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_managers_on_sf_id ON public.managers USING btree (sf_id);


--
-- Name: index_managers_on_team_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_managers_on_team_id ON public.managers USING btree (team_id);


--
-- Name: index_managers_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_managers_on_user_id ON public.managers USING btree (user_id);


--
-- Name: index_max_mind_cities_on_alternatives; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_max_mind_cities_on_alternatives ON public.max_mind_cities USING btree (alternatives);


--
-- Name: index_max_mind_cities_on_city_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_max_mind_cities_on_city_name ON public.max_mind_cities USING btree (city_name);


--
-- Name: index_max_mind_cities_on_country_iso_code; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_max_mind_cities_on_country_iso_code ON public.max_mind_cities USING btree (country_iso_code);


--
-- Name: index_max_mind_cities_on_country_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_max_mind_cities_on_country_name ON public.max_mind_cities USING btree (country_name);


--
-- Name: index_max_mind_cities_on_geoname_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_max_mind_cities_on_geoname_id ON public.max_mind_cities USING btree (geoname_id);


--
-- Name: index_mobile_networks_on_country_id_and_network; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_mobile_networks_on_country_id_and_network ON public.mobile_networks USING btree (country_id, network);


--
-- Name: index_mobile_networks_on_mnc_and_mcc; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_mobile_networks_on_mnc_and_mcc ON public.mobile_networks USING btree (mnc, mcc);


--
-- Name: index_mobile_networks_on_network; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_mobile_networks_on_network ON public.mobile_networks USING btree (network);


--
-- Name: index_monthly_book_keeping_reports_on_year_and_month; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_monthly_book_keeping_reports_on_year_and_month ON public.monthly_book_keeping_reports USING btree (year, month);


--
-- Name: index_mopub_accounts_on_owner_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_mopub_accounts_on_owner_id ON public.mopub_accounts USING btree (owner_id);


--
-- Name: index_ncs_app_restriction; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_ncs_app_restriction ON public.network_connection_statuses USING btree (app_id, network_connection_restriction_id);


--
-- Name: index_ncs_app_type_network; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_ncs_app_type_network ON public.network_connection_statuses USING btree (app_id, ad_type, network, network_connection_restriction_id);


--
-- Name: index_network_content_filters_on_network_name; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_network_content_filters_on_network_name ON public.network_content_filters USING btree (network_name);


--
-- Name: index_network_request_restrictions_on_app_ids_array; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_network_request_restrictions_on_app_ids_array ON public.network_request_restrictions USING gin (app_ids_array);


--
-- Name: index_network_request_restrictions_on_network_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_network_request_restrictions_on_network_id ON public.network_request_restrictions USING btree (network_id);


--
-- Name: index_on_coordinates_arrays; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_on_coordinates_arrays ON public.campaigns USING gin (app_ids_array, country_ids_array, max_mind_city_ids_array, coordinate_ids_array, target_locales) WHERE ((deleted IS FALSE) AND (active IS TRUE));


--
-- Name: index_organization_commission_presets_on_organization_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_organization_commission_presets_on_organization_id ON public.organization_commission_presets USING btree (organization_id);


--
-- Name: index_organization_roles_on_organization_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_organization_roles_on_organization_id ON public.organization_roles USING btree (organization_id);


--
-- Name: index_organization_roles_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_organization_roles_on_user_id ON public.organization_roles USING btree (user_id);


--
-- Name: index_organization_roles_on_user_id_and_organization_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_organization_roles_on_user_id_and_organization_id ON public.organization_roles USING btree (user_id, organization_id);


--
-- Name: index_page_categories_apps_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_page_categories_apps_on_app_id ON public.page_categories_apps USING btree (app_id);


--
-- Name: index_page_categories_apps_on_page_category_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_page_categories_apps_on_page_category_id ON public.page_categories_apps USING btree (page_category_id);


--
-- Name: index_payment_requests_on_admin_notified_about_creation; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payment_requests_on_admin_notified_about_creation ON public.payment_requests USING btree (admin_notified_about_creation);


--
-- Name: index_payment_requests_on_payment_system; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payment_requests_on_payment_system ON public.payment_requests USING btree (payment_system);


--
-- Name: index_payment_requests_on_regular; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payment_requests_on_regular ON public.payment_requests USING btree (regular);


--
-- Name: index_payment_requests_on_status; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payment_requests_on_status ON public.payment_requests USING btree (status);


--
-- Name: index_payment_requests_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payment_requests_on_user_id ON public.payment_requests USING btree (user_id);


--
-- Name: index_payment_requests_on_user_notified_about_creation; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payment_requests_on_user_notified_about_creation ON public.payment_requests USING btree (user_notified_about_creation);


--
-- Name: index_payment_requests_on_user_notified_about_processing; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payment_requests_on_user_notified_about_processing ON public.payment_requests USING btree (user_notified_about_processing);


--
-- Name: index_payouts_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payouts_on_app_id ON public.payouts USING btree (app_id);


--
-- Name: index_payouts_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payouts_on_campaign_image_id ON public.payouts USING btree (campaign_image_id);


--
-- Name: index_payouts_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_payouts_on_country_id ON public.payouts USING btree (country_id);


--
-- Name: index_price_floors_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_price_floors_on_app_id ON public.price_floors USING btree (app_id);


--
-- Name: index_price_floors_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_price_floors_on_country_id ON public.price_floors USING btree (country_id);


--
-- Name: index_qubole_sdk_data_on_report_date_and_app_id_and_ad_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_qubole_sdk_data_on_report_date_and_app_id_and_ad_type ON public.qubole_sdk_data USING btree (report_date, app_id, ad_type);


--
-- Name: index_qubole_sdk_data_on_sdk_version_and_report_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_qubole_sdk_data_on_sdk_version_and_report_date ON public.qubole_sdk_data USING btree (sdk_version, report_date);


--
-- Name: index_report_processing_logs_on_account_type_and_process_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_report_processing_logs_on_account_type_and_process_id ON public.report_processing_logs USING btree (account_type, process_id);


--
-- Name: index_report_processing_logs_on_report_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_report_processing_logs_on_report_date ON public.report_processing_logs USING btree (report_date);


--
-- Name: index_reports_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_reports_on_country_id ON public.reports USING btree (country_id);


--
-- Name: index_request_blacklists_on_bl_type_and_active; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_request_blacklists_on_bl_type_and_active ON public.request_blacklists USING btree (bl_type, active);


--
-- Name: index_request_blacklists_on_ends_on; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_request_blacklists_on_ends_on ON public.request_blacklists USING btree (ends_on);


--
-- Name: index_request_blacklists_on_regex; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_request_blacklists_on_regex ON public.request_blacklists USING btree (regex);


--
-- Name: index_roles_on_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_roles_on_name ON public.roles USING btree (name);


--
-- Name: index_sdk_releases_on_sdk_version_and_platform_id_and_framework; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_sdk_releases_on_sdk_version_and_platform_id_and_framework ON public.sdk_releases USING btree (sdk_version, platform_id, framework);


--
-- Name: index_section_categories_apps_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_section_categories_apps_on_app_id ON public.section_categories_apps USING btree (app_id);


--
-- Name: index_section_categories_apps_on_section_category_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_section_categories_apps_on_section_category_id ON public.section_categories_apps USING btree (section_category_id);


--
-- Name: index_segment_rule_countries_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_segment_rule_countries_on_country_id ON public.segment_rule_countries USING btree (country_id);


--
-- Name: index_segment_rule_countries_on_segment_rule_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_segment_rule_countries_on_segment_rule_id ON public.segment_rule_countries USING btree (segment_rule_id);


--
-- Name: index_segments_on_deleted; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_segments_on_deleted ON public.segments USING btree (deleted);


--
-- Name: index_share_tokens_on_email; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_share_tokens_on_email ON public.share_tokens USING btree (email);


--
-- Name: index_share_tokens_on_value; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_share_tokens_on_value ON public.share_tokens USING btree (value);


--
-- Name: index_simple_dashboard_monthly_on_year_and_month; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_simple_dashboard_monthly_on_year_and_month ON public.simple_dashboard_monthly USING btree (year, month);


--
-- Name: index_simple_dashboard_on_date; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_simple_dashboard_on_date ON public.simple_dashboard USING btree (date);


--
-- Name: index_simple_dashboard_quarterly_on_year_and_quarter; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_simple_dashboard_quarterly_on_year_and_quarter ON public.simple_dashboard_quarterly USING btree (year, quarter);


--
-- Name: index_simple_dashboard_weekly_on_year_and_week; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_simple_dashboard_weekly_on_year_and_week ON public.simple_dashboard_weekly USING btree (year, week);


--
-- Name: index_simple_dashboard_yearly_on_year; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_simple_dashboard_yearly_on_year ON public.simple_dashboard_yearly USING btree (year);


--
-- Name: index_statistics_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_app_id ON public.statistics USING btree (app_id);


--
-- Name: index_statistics_on_campaign_image_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_campaign_image_id ON public.statistics USING btree (campaign_image_id);


--
-- Name: index_statistics_on_campaign_url_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_campaign_url_id ON public.statistics USING btree (campaign_url_id);


--
-- Name: index_statistics_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_country_id ON public.statistics USING btree (country_id);


--
-- Name: index_statistics_on_created_at; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_created_at ON public.statistics USING btree (created_at);


--
-- Name: index_statistics_on_device_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_device_id ON public.statistics USING btree (device_id);


--
-- Name: index_statistics_on_ip; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_ip ON public.statistics USING btree (ip);


--
-- Name: index_statistics_on_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_type ON public.statistics USING btree (type);


--
-- Name: index_statistics_on_ua; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_ua ON public.statistics USING btree (ua);


--
-- Name: index_statistics_on_user_only; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_user_only ON public.statistics USING btree (user_only);


--
-- Name: index_statistics_on_white_sdk; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_white_sdk ON public.statistics USING btree (white_sdk);


--
-- Name: index_statistics_on_x_forwarded; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_statistics_on_x_forwarded ON public.statistics USING btree (x_forwarded);


--
-- Name: index_store_infos_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_store_infos_on_app_id ON public.store_infos USING btree (app_id);


--
-- Name: index_support_work_metriks_on_support_work_report_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_support_work_metriks_on_support_work_report_id ON public.support_work_metriks USING btree (support_work_report_id);


--
-- Name: index_support_work_values_on_date_and_support_work_metrik_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_support_work_values_on_date_and_support_work_metrik_id ON public.support_work_values USING btree (date, support_work_metrik_id);


--
-- Name: index_support_work_values_on_support_work_metrik_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_support_work_values_on_support_work_metrik_id ON public.support_work_values USING btree (support_work_metrik_id);


--
-- Name: index_survey_responses_on_survey_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_survey_responses_on_survey_id ON public.survey_responses USING btree (survey_id);


--
-- Name: index_survey_responses_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_survey_responses_on_user_id ON public.survey_responses USING btree (user_id);


--
-- Name: index_survey_responses_on_user_id_and_survey_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_survey_responses_on_user_id_and_survey_id ON public.survey_responses USING btree (user_id, survey_id);


--
-- Name: index_surveys_on_active; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_surveys_on_active ON public.surveys USING btree (active);


--
-- Name: index_traffic_limit_exclusions_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_traffic_limit_exclusions_on_app_id ON public.traffic_limit_exclusions USING btree (app_id);


--
-- Name: index_user_ach_requisites_on_country_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_ach_requisites_on_country_id ON public.user_ach_requisites USING btree (country_id);


--
-- Name: index_user_ach_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_ach_requisites_on_user_id ON public.user_ach_requisites USING btree (user_id);


--
-- Name: index_user_apps_commissions_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_apps_commissions_on_user_id ON public.user_apps_commissions USING btree (user_id);


--
-- Name: index_user_bitcoin_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_bitcoin_requisites_on_user_id ON public.user_bitcoin_requisites USING btree (user_id);


--
-- Name: index_user_daily_stats_on_date; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_daily_stats_on_date ON public.user_daily_stats USING btree (date);


--
-- Name: index_user_daily_stats_on_date_and_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_user_daily_stats_on_date_and_user_id ON public.user_daily_stats USING btree (date, user_id);


--
-- Name: index_user_daily_stats_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_daily_stats_on_user_id ON public.user_daily_stats USING btree (user_id);


--
-- Name: index_user_ethereum_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_ethereum_requisites_on_user_id ON public.user_ethereum_requisites USING btree (user_id);


--
-- Name: index_user_fraud_warning_logs_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_fraud_warning_logs_on_user_id ON public.user_fraud_warning_logs USING btree (user_id);


--
-- Name: index_user_google_wallet_requisites_on_active; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_google_wallet_requisites_on_active ON public.user_google_wallet_requisites USING btree (active);


--
-- Name: index_user_google_wallet_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_google_wallet_requisites_on_user_id ON public.user_google_wallet_requisites USING btree (user_id);


--
-- Name: index_user_intercom_events_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_intercom_events_on_user_id ON public.user_intercom_events USING btree (user_id);


--
-- Name: index_user_intercom_events_on_user_id_and_app_id_and_event_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_intercom_events_on_user_id_and_app_id_and_event_id ON public.user_intercom_events USING btree (user_id, app_id, event_id);


--
-- Name: index_user_login_logs_on_cookies; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_login_logs_on_cookies ON public.user_login_logs USING btree (cookies);


--
-- Name: index_user_login_logs_on_developer_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_login_logs_on_developer_id ON public.user_login_logs USING btree (developer_id);


--
-- Name: index_user_login_logs_on_ip; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_login_logs_on_ip ON public.user_login_logs USING btree (ip);


--
-- Name: index_user_login_logs_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_login_logs_on_user_id ON public.user_login_logs USING btree (user_id);


--
-- Name: index_user_payoneer_bank_transfer_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_payoneer_bank_transfer_requisites_on_user_id ON public.user_payoneer_bank_transfer_requisites USING btree (user_id);


--
-- Name: index_user_payoneer_paypal_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_payoneer_paypal_requisites_on_user_id ON public.user_payoneer_paypal_requisites USING btree (user_id);


--
-- Name: index_user_payoneer_prepaid_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_payoneer_prepaid_requisites_on_user_id ON public.user_payoneer_prepaid_requisites USING btree (user_id);


--
-- Name: index_user_paypal_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_paypal_requisites_on_user_id ON public.user_paypal_requisites USING btree (user_id);


--
-- Name: index_user_webmoney_requisites_on_active; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_webmoney_requisites_on_active ON public.user_webmoney_requisites USING btree (active);


--
-- Name: index_user_webmoney_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_webmoney_requisites_on_user_id ON public.user_webmoney_requisites USING btree (user_id);


--
-- Name: index_user_wire_requisites_on_active; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_wire_requisites_on_active ON public.user_wire_requisites USING btree (active);


--
-- Name: index_user_wire_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_wire_requisites_on_user_id ON public.user_wire_requisites USING btree (user_id);


--
-- Name: index_user_yandex_money_requisites_on_active; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_yandex_money_requisites_on_active ON public.user_yandex_money_requisites USING btree (active);


--
-- Name: index_user_yandex_money_requisites_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_user_yandex_money_requisites_on_user_id ON public.user_yandex_money_requisites USING btree (user_id);


--
-- Name: index_users_abilities_on_member_action; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_abilities_on_member_action ON public.users_abilities USING gin (member_action);


--
-- Name: index_users_abilities_on_resource_category; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_abilities_on_resource_category ON public.users_abilities USING btree (resource_category);


--
-- Name: index_users_abilities_on_resources_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_abilities_on_resources_name ON public.users_abilities USING btree (resources_name);


--
-- Name: index_users_abilities_on_role_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_abilities_on_role_id ON public.users_abilities USING btree (role_id);


--
-- Name: index_users_abilities_on_user_role; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_abilities_on_user_role ON public.users_abilities USING btree (user_role);


--
-- Name: index_users_abilities_on_user_role_and_resources_name; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_users_abilities_on_user_role_and_resources_name ON public.users_abilities USING btree (user_role, resources_name);


--
-- Name: index_users_admin_notifications_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_admin_notifications_on_user_id ON public.users_admin_notifications USING btree (user_id);


--
-- Name: index_users_on_closeio_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_on_closeio_id ON public.users USING btree (closeio_id);


--
-- Name: index_users_on_confirmation_token; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_users_on_confirmation_token ON public.users USING btree (confirmation_token);


--
-- Name: index_users_on_email; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_users_on_email ON public.users USING btree (email);


--
-- Name: index_users_on_lead_owner_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_on_lead_owner_id ON public.users USING btree (lead_owner_id);


--
-- Name: index_users_on_sf_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_on_sf_id ON public.users USING btree (sf_id);


--
-- Name: index_users_on_wl_domain_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_on_wl_domain_id ON public.users USING btree (wl_domain_id);


--
-- Name: index_users_roles_on_user_id_and_role_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_roles_on_user_id_and_role_id ON public.users_roles USING btree (user_id, role_id);


--
-- Name: index_users_shared_apps_on_account_share_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_users_shared_apps_on_account_share_id ON public.users_shared_apps USING btree (account_share_id);


--
-- Name: index_v2_placements_versions_on_item_type_and_item_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_v2_placements_versions_on_item_type_and_item_id ON public.v2_placements_versions USING btree (item_type, item_id);


--
-- Name: index_v2_segments_on_default_preset; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_v2_segments_on_default_preset ON public.v2_segments USING btree (default_preset);


--
-- Name: index_v2_segments_on_name; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_v2_segments_on_name ON public.v2_segments USING btree (name);


--
-- Name: index_v2_segments_on_user_id_and_preset; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_v2_segments_on_user_id_and_preset ON public.v2_segments USING btree (user_id, preset);


--
-- Name: index_v2_segments_versions_on_item_type_and_item_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_v2_segments_versions_on_item_type_and_item_id ON public.v2_segments_versions USING btree (item_type, item_id);


--
-- Name: index_versions_on_item_type_and_item_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_versions_on_item_type_and_item_id ON public.versions USING btree (item_type, item_id);


--
-- Name: index_vpn_ips_on_address_and_is_vpn; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_vpn_ips_on_address_and_is_vpn ON public.vpn_ips USING btree (address, is_vpn);


--
-- Name: index_waterfall_logs_on_country_id_and_ad_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_waterfall_logs_on_country_id_and_ad_type ON public.waterfall_logs USING btree (country_id, ad_type);


--
-- Name: index_waterfall_logs_on_waterfall_monitor_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_waterfall_logs_on_waterfall_monitor_id ON public.waterfall_logs USING btree (waterfall_monitor_id);


--
-- Name: index_waterfall_monitors_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_waterfall_monitors_on_app_id ON public.waterfall_monitors USING btree (app_id);


--
-- Name: index_waterfall_settings_on_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_waterfall_settings_on_app_id ON public.waterfall_settings USING btree (app_id);


--
-- Name: index_waterfall_settings_on_app_id_and_network_id; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX index_waterfall_settings_on_app_id_and_network_id ON public.waterfall_settings USING btree (app_id, network_id);


--
-- Name: index_waterfall_settings_on_network_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_waterfall_settings_on_network_id ON public.waterfall_settings USING btree (network_id);


--
-- Name: index_wl_domain_fees_on_fee_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_wl_domain_fees_on_fee_type ON public.wl_domain_fees USING btree (fee_type);


--
-- Name: index_wl_domain_fees_on_user_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_wl_domain_fees_on_user_id ON public.wl_domain_fees USING btree (user_id);


--
-- Name: index_wl_domain_fees_on_wl_domain_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_wl_domain_fees_on_wl_domain_id ON public.wl_domain_fees USING btree (wl_domain_id);


--
-- Name: index_wl_domains_on_deleted; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_wl_domains_on_deleted ON public.wl_domains USING btree (deleted);


--
-- Name: index_wl_domains_on_locale; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_wl_domains_on_locale ON public.wl_domains USING btree (locale);


--
-- Name: index_wl_domains_on_status; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX index_wl_domains_on_status ON public.wl_domains USING btree (status);


--
-- Name: intercom_network_recommend_uniq; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX intercom_network_recommend_uniq ON public.intercom_network_recommendeds USING btree (app_id, network);


--
-- Name: ios_sdk_mixed1; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX ios_sdk_mixed1 ON public.campaign_images USING btree (ios_sdk_version_min, ios_sdk_version_max);


--
-- Name: ios_sdk_mixed2; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX ios_sdk_mixed2 ON public.campaign_images USING btree (has_ios_sdk_targeting, ios_sdk_version_min, ios_sdk_version_max);


--
-- Name: member_roles_index; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX member_roles_index ON public.organization_roles USING btree (organization_id, role) WHERE (role = 0);


--
-- Name: net_conn_stat_ad_type; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX net_conn_stat_ad_type ON public.network_connection_statuses USING btree (ad_type);


--
-- Name: net_conn_stat_app_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX net_conn_stat_app_id ON public.network_connection_statuses USING btree (app_id);


--
-- Name: net_conn_stat_restriction_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX net_conn_stat_restriction_id ON public.network_connection_statuses USING btree (network_connection_restriction_id);


--
-- Name: new_fast_ecpm_stats_campaign_image_id_country_id_app_id_idx; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX new_fast_ecpm_stats_campaign_image_id_country_id_app_id_idx ON public.new_fast_ecpm_stats_old USING btree (campaign_image_id, country_id, app_id);


--
-- Name: new_fast_ecpm_stats_cleaned_uniq; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX new_fast_ecpm_stats_cleaned_uniq ON public.new_fast_ecpm_stats USING btree (app_id, country_id, campaign_image_id);


--
-- Name: pgcompact_index_104606; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX pgcompact_index_104606 ON public.apps USING btree (user_id);


--
-- Name: pgcompact_index_6081; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX pgcompact_index_6081 ON public.campaign_images USING btree (has_ios_sdk_targeting);


--
-- Name: pgcompact_index_9785; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX pgcompact_index_9785 ON public.campaign_images USING btree (priority_finish_at);


--
-- Name: publisher_monitor_message_unique; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX publisher_monitor_message_unique ON public.publisher_monitor_messages USING btree (network, ad_type);


--
-- Name: roles_index; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX roles_index ON public.organization_roles USING btree (user_id, organization_id, role);


--
-- Name: section_index; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX section_index ON public.global_settings USING btree (global_settings_section_id, setting_name);


--
-- Name: stats_android_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX stats_android_id ON public.statistics USING btree (android_id);


--
-- Name: stats_date_android_id; Type: INDEX; Schema: public; Owner: -
--

CREATE INDEX stats_date_android_id ON public.statistics USING btree (date, android_id);


--
-- Name: subdiv1_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX subdiv1_index ON public.subdivision_ones USING btree (country_id, iso_code);


--
-- Name: subdiv2_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX subdiv2_index ON public.subdivision_twos USING btree (country_id, subdivision1_id, iso_code);


--
-- Name: traffic_limit_blocks_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX traffic_limit_blocks_index ON public.traffic_limit_blocks USING btree (app_id, campaign_image_id, country_id, date);


--
-- Name: uniq_admob_version; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX uniq_admob_version ON public.admob_ad_unit_versions USING btree (app_id, account_id, ad_unit_id);


--
-- Name: uniq_bid_floor_setting; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX uniq_bid_floor_setting ON public.bid_floor_settings USING btree (account_type, ad_type, format, device_type);


--
-- Name: unique_account_connection; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_account_connection ON public.apps_network_accounts USING btree (app_id, method_name, ad_type);


--
-- Name: unique_ad_unit_for_account; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_ad_unit_for_account ON public.ad_units_backup USING btree (app_id, bid_floor, width, height, account_type, account_id, ad_type, format, is_landscape, device_type);


--
-- Name: unique_app_network_profile; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_app_network_profile ON public.app_network_profiles USING btree (network, app_id, account_id);


--
-- Name: unique_billing_role_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_billing_role_index ON public.organization_roles USING btree (organization_id, role) WHERE (role = 2);


--
-- Name: unique_daemon_job_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_daemon_job_index ON public.network_daemon_jobs USING btree (app_id, network, ad_type, phase_number);


--
-- Name: unique_data_migrations; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_data_migrations ON public.data_migrations USING btree (version);


--
-- Name: unique_index_apps_on_app_key; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_index_apps_on_app_key ON public.apps USING btree (app_key);


--
-- Name: unique_names_index_liverail_ad_units; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_names_index_liverail_ad_units ON public.liverail_ad_units USING btree (ad_type, width, height, bid_floor, liverail_account_id, is_rewarded);


--
-- Name: unique_owner_role_index; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_owner_role_index ON public.organization_roles USING btree (organization_id, role) WHERE (role = 1);


--
-- Name: unique_schema_migrations; Type: INDEX; Schema: public; Owner: -
--

CREATE UNIQUE INDEX unique_schema_migrations ON public.schema_migrations USING btree (version);


--
-- Name: billings billings_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER billings_insert BEFORE INSERT OR UPDATE ON public.billings FOR EACH ROW EXECUTE PROCEDURE billings_insert_trigger();


--
-- Name: clicks clicks_after_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER clicks_after_insert AFTER INSERT ON public.clicks FOR EACH ROW EXECUTE PROCEDURE clicks_delete_trigger();


--
-- Name: clicks clicks_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER clicks_insert BEFORE INSERT ON public.clicks FOR EACH ROW EXECUTE PROCEDURE clicks_insert_trigger();


--
-- Name: insight_stats insight_stats_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER insight_stats_insert BEFORE INSERT ON public.insight_stats FOR EACH ROW EXECUTE PROCEDURE insight_stats_insert_trigger();


--
-- Name: ip_addresses set_fraud_score_in_ip_addresses_trigg; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER set_fraud_score_in_ip_addresses_trigg BEFORE INSERT OR UPDATE ON public.ip_addresses FOR EACH ROW EXECUTE PROCEDURE set_fraud_score_in_ip_addresses_func();


--
-- Name: vpn_ips vpn_ips_after_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER vpn_ips_after_insert AFTER INSERT ON public.vpn_ips FOR EACH ROW EXECUTE PROCEDURE vpn_ips_delete_trigger();


--
-- Name: vpn_ips vpn_ips_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER vpn_ips_insert BEFORE INSERT ON public.vpn_ips FOR EACH ROW EXECUTE PROCEDURE vpn_ips_insert_trigger();


--
-- Name: organization_roles fk_rails_1edd21f138; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization_roles
    ADD CONSTRAINT fk_rails_1edd21f138 FOREIGN KEY (organization_id) REFERENCES organizations(id);


--
-- Name: organization_roles fk_rails_2e11ac264f; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization_roles
    ADD CONSTRAINT fk_rails_2e11ac264f FOREIGN KEY (user_id) REFERENCES users(id);


--
-- Name: invoice_requests fk_rails_49b01ecbb6; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY invoice_requests
    ADD CONSTRAINT fk_rails_49b01ecbb6 FOREIGN KEY (user_id) REFERENCES users(id);


--
-- Name: store_infos fk_rails_5298e9139e; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY store_infos
    ADD CONSTRAINT fk_rails_5298e9139e FOREIGN KEY (app_id) REFERENCES apps(id);


--
-- Name: organization_commission_presets fk_rails_961e28d057; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization_commission_presets
    ADD CONSTRAINT fk_rails_961e28d057 FOREIGN KEY (organization_id) REFERENCES organizations(id);


--
-- Name: users_abilities fk_rails_c324c54940; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY users_abilities
    ADD CONSTRAINT fk_rails_c324c54940 FOREIGN KEY (role_id) REFERENCES roles(id);


--
-- Name: waterfall_settings_root_flags fk_root_flag_setting; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_flags
    ADD CONSTRAINT fk_root_flag_setting FOREIGN KEY (setting_id) REFERENCES waterfall_settings_root_settings(id);


--
-- Name: waterfall_settings_root_flags fk_root_flag_type; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_root_flags
    ADD CONSTRAINT fk_root_flag_type FOREIGN KEY (flag_type_id) REFERENCES waterfall_settings_root_flag_types(id);


--
-- Name: waterfall_settings_unit_flags fk_unit_flag_setting; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_flags
    ADD CONSTRAINT fk_unit_flag_setting FOREIGN KEY (setting_id) REFERENCES waterfall_settings_unit_settings(id);


--
-- Name: waterfall_settings_unit_flags fk_unit_flag_type; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY waterfall_settings_unit_flags
    ADD CONSTRAINT fk_unit_flag_type FOREIGN KEY (flag_type_id) REFERENCES waterfall_settings_unit_flag_types(id);


--
-- Name: public; Type: ACL; Schema: -; Owner: -
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;
GRANT USAGE ON SCHEMA public TO support_bot;
GRANT USAGE ON SCHEMA public TO readonly_user;


--
-- Name: account_employees; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE account_employees FROM PUBLIC;
REVOKE ALL ON TABLE account_employees FROM appodeal;
GRANT ALL ON TABLE account_employees TO appodeal;
GRANT SELECT ON TABLE account_employees TO support_bot;
GRANT SELECT ON TABLE account_employees TO readonly_user;


--
-- Name: account_employees_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE account_employees_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE account_employees_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE account_employees_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE account_employees_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE account_employees_id_seq TO readonly_user;


--
-- Name: account_shares; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE account_shares FROM PUBLIC;
REVOKE ALL ON TABLE account_shares FROM appodeal;
GRANT ALL ON TABLE account_shares TO appodeal;
GRANT SELECT ON TABLE account_shares TO support_bot;
GRANT SELECT ON TABLE account_shares TO readonly_user;


--
-- Name: account_shares_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE account_shares_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE account_shares_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE account_shares_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE account_shares_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE account_shares_id_seq TO readonly_user;


--
-- Name: active_admin_comments; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE active_admin_comments FROM PUBLIC;
REVOKE ALL ON TABLE active_admin_comments FROM appodeal;
GRANT ALL ON TABLE active_admin_comments TO appodeal;
GRANT SELECT ON TABLE active_admin_comments TO support_bot;
GRANT SELECT ON TABLE active_admin_comments TO readonly_user;


--
-- Name: active_admin_comments_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE active_admin_comments_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE active_admin_comments_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE active_admin_comments_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE active_admin_comments_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE active_admin_comments_id_seq TO readonly_user;


--
-- Name: active_apps; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE active_apps FROM PUBLIC;
REVOKE ALL ON TABLE active_apps FROM appodeal;
GRANT ALL ON TABLE active_apps TO appodeal;
GRANT SELECT ON TABLE active_apps TO readonly_user;


--
-- Name: active_apps_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE active_apps_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE active_apps_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE active_apps_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE active_apps_id_seq TO readonly_user;


--
-- Name: active_users; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE active_users FROM PUBLIC;
REVOKE ALL ON TABLE active_users FROM appodeal;
GRANT ALL ON TABLE active_users TO appodeal;
GRANT SELECT ON TABLE active_users TO readonly_user;


--
-- Name: active_users_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE active_users_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE active_users_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE active_users_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE active_users_id_seq TO readonly_user;


--
-- Name: ad_request_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE ad_request_settings FROM PUBLIC;
REVOKE ALL ON TABLE ad_request_settings FROM appodeal;
GRANT ALL ON TABLE ad_request_settings TO appodeal;
GRANT SELECT ON TABLE ad_request_settings TO support_bot;
GRANT SELECT ON TABLE ad_request_settings TO readonly_user;


--
-- Name: ad_request_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE ad_request_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE ad_request_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE ad_request_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE ad_request_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE ad_request_settings_id_seq TO readonly_user;


--
-- Name: ad_unit_templates; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE ad_unit_templates FROM PUBLIC;
REVOKE ALL ON TABLE ad_unit_templates FROM appodeal;
GRANT ALL ON TABLE ad_unit_templates TO appodeal;
GRANT SELECT ON TABLE ad_unit_templates TO readonly_user;


--
-- Name: ad_unit_templates_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE ad_unit_templates_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE ad_unit_templates_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE ad_unit_templates_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE ad_unit_templates_id_seq TO readonly_user;


--
-- Name: ad_unit_versions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE ad_unit_versions FROM PUBLIC;
REVOKE ALL ON TABLE ad_unit_versions FROM appodeal;
GRANT ALL ON TABLE ad_unit_versions TO appodeal;
GRANT SELECT ON TABLE ad_unit_versions TO support_bot;
GRANT SELECT ON TABLE ad_unit_versions TO readonly_user;


--
-- Name: ad_unit_versions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE ad_unit_versions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE ad_unit_versions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE ad_unit_versions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE ad_unit_versions_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE ad_unit_versions_id_seq TO readonly_user;


--
-- Name: ad_units_backup; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE ad_units_backup FROM PUBLIC;
REVOKE ALL ON TABLE ad_units_backup FROM appodeal;
GRANT ALL ON TABLE ad_units_backup TO appodeal;
GRANT SELECT ON TABLE ad_units_backup TO support_bot;
GRANT SELECT ON TABLE ad_units_backup TO readonly_user;


--
-- Name: ad_units_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE ad_units_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE ad_units_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE ad_units_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE ad_units_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE ad_units_id_seq TO readonly_user;


--
-- Name: ad_units; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE ad_units FROM PUBLIC;
REVOKE ALL ON TABLE ad_units FROM appodeal;
GRANT ALL ON TABLE ad_units TO appodeal;
GRANT SELECT ON TABLE ad_units TO readonly_user;


--
-- Name: adcolony_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE adcolony_accounts FROM PUBLIC;
REVOKE ALL ON TABLE adcolony_accounts FROM appodeal;
GRANT ALL ON TABLE adcolony_accounts TO appodeal;
GRANT SELECT ON TABLE adcolony_accounts TO support_bot;
GRANT SELECT ON TABLE adcolony_accounts TO readonly_user;


--
-- Name: adcolony_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE adcolony_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE adcolony_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE adcolony_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE adcolony_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE adcolony_accounts_id_seq TO readonly_user;


--
-- Name: additional_user_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE additional_user_logs FROM PUBLIC;
REVOKE ALL ON TABLE additional_user_logs FROM appodeal;
GRANT ALL ON TABLE additional_user_logs TO appodeal;
GRANT SELECT ON TABLE additional_user_logs TO readonly_user;


--
-- Name: additional_user_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE additional_user_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE additional_user_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE additional_user_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE additional_user_logs_id_seq TO readonly_user;


--
-- Name: admob_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE admob_accounts FROM PUBLIC;
REVOKE ALL ON TABLE admob_accounts FROM appodeal;
GRANT ALL ON TABLE admob_accounts TO appodeal;
GRANT SELECT ON TABLE admob_accounts TO support_bot;
GRANT SELECT ON TABLE admob_accounts TO readonly_user;


--
-- Name: admob_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE admob_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE admob_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE admob_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE admob_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE admob_accounts_id_seq TO readonly_user;


--
-- Name: admob_ad_unit_versions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE admob_ad_unit_versions FROM PUBLIC;
REVOKE ALL ON TABLE admob_ad_unit_versions FROM appodeal;
GRANT ALL ON TABLE admob_ad_unit_versions TO appodeal;
GRANT SELECT ON TABLE admob_ad_unit_versions TO readonly_user;


--
-- Name: admob_ad_unit_versions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE admob_ad_unit_versions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE admob_ad_unit_versions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE admob_ad_unit_versions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE admob_ad_unit_versions_id_seq TO readonly_user;


--
-- Name: admob_app_bid_floors; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE admob_app_bid_floors FROM PUBLIC;
REVOKE ALL ON TABLE admob_app_bid_floors FROM appodeal;
GRANT ALL ON TABLE admob_app_bid_floors TO appodeal;
GRANT SELECT ON TABLE admob_app_bid_floors TO readonly_user;


--
-- Name: admob_app_bid_floors_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE admob_app_bid_floors_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE admob_app_bid_floors_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE admob_app_bid_floors_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE admob_app_bid_floors_id_seq TO readonly_user;


--
-- Name: admob_user_bid_floors; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE admob_user_bid_floors FROM PUBLIC;
REVOKE ALL ON TABLE admob_user_bid_floors FROM appodeal;
GRANT ALL ON TABLE admob_user_bid_floors TO appodeal;
GRANT SELECT ON TABLE admob_user_bid_floors TO readonly_user;


--
-- Name: admob_user_bid_floors_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE admob_user_bid_floors_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE admob_user_bid_floors_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE admob_user_bid_floors_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE admob_user_bid_floors_id_seq TO readonly_user;


--
-- Name: adms; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE adms FROM PUBLIC;
REVOKE ALL ON TABLE adms FROM appodeal;
GRANT ALL ON TABLE adms TO appodeal;
GRANT SELECT ON TABLE adms TO support_bot;
GRANT SELECT ON TABLE adms TO readonly_user;


--
-- Name: adms_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE adms_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE adms_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE adms_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE adms_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE adms_id_seq TO readonly_user;


--
-- Name: advertisement_partners; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE advertisement_partners FROM PUBLIC;
REVOKE ALL ON TABLE advertisement_partners FROM appodeal;
GRANT ALL ON TABLE advertisement_partners TO appodeal;
GRANT SELECT ON TABLE advertisement_partners TO support_bot;
GRANT SELECT ON TABLE advertisement_partners TO readonly_user;


--
-- Name: advertisement_partners_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE advertisement_partners_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE advertisement_partners_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE advertisement_partners_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE advertisement_partners_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE advertisement_partners_id_seq TO readonly_user;


--
-- Name: advertisement_partners_users; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE advertisement_partners_users FROM PUBLIC;
REVOKE ALL ON TABLE advertisement_partners_users FROM appodeal;
GRANT ALL ON TABLE advertisement_partners_users TO appodeal;
GRANT SELECT ON TABLE advertisement_partners_users TO support_bot;
GRANT SELECT ON TABLE advertisement_partners_users TO readonly_user;


--
-- Name: ae_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE ae_settings FROM PUBLIC;
REVOKE ALL ON TABLE ae_settings FROM appodeal;
GRANT ALL ON TABLE ae_settings TO appodeal;
GRANT SELECT ON TABLE ae_settings TO readonly_user;


--
-- Name: ae_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE ae_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE ae_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE ae_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE ae_settings_id_seq TO readonly_user;


--
-- Name: amazon_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE amazon_accounts FROM PUBLIC;
REVOKE ALL ON TABLE amazon_accounts FROM appodeal;
GRANT ALL ON TABLE amazon_accounts TO appodeal;
GRANT SELECT ON TABLE amazon_accounts TO support_bot;
GRANT SELECT ON TABLE amazon_accounts TO readonly_user;


--
-- Name: amazon_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE amazon_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE amazon_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE amazon_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE amazon_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE amazon_accounts_id_seq TO readonly_user;


--
-- Name: api_queues; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE api_queues FROM PUBLIC;
REVOKE ALL ON TABLE api_queues FROM appodeal;
GRANT ALL ON TABLE api_queues TO appodeal;
GRANT SELECT ON TABLE api_queues TO support_bot;
GRANT SELECT ON TABLE api_queues TO readonly_user;


--
-- Name: api_queues_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE api_queues_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE api_queues_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE api_queues_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE api_queues_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE api_queues_id_seq TO readonly_user;


--
-- Name: app_fraud_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE app_fraud_logs FROM PUBLIC;
REVOKE ALL ON TABLE app_fraud_logs FROM appodeal;
GRANT ALL ON TABLE app_fraud_logs TO appodeal;
GRANT SELECT ON TABLE app_fraud_logs TO support_bot;
GRANT SELECT ON TABLE app_fraud_logs TO readonly_user;


--
-- Name: app_fraud_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE app_fraud_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE app_fraud_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE app_fraud_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE app_fraud_logs_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE app_fraud_logs_id_seq TO readonly_user;


--
-- Name: app_frauds; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE app_frauds FROM PUBLIC;
REVOKE ALL ON TABLE app_frauds FROM appodeal;
GRANT ALL ON TABLE app_frauds TO appodeal;
GRANT SELECT ON TABLE app_frauds TO support_bot;
GRANT SELECT ON TABLE app_frauds TO readonly_user;


--
-- Name: app_frauds_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE app_frauds_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE app_frauds_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE app_frauds_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE app_frauds_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE app_frauds_id_seq TO readonly_user;


--
-- Name: app_installer_data; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE app_installer_data FROM PUBLIC;
REVOKE ALL ON TABLE app_installer_data FROM appodeal;
GRANT ALL ON TABLE app_installer_data TO appodeal;
GRANT SELECT ON TABLE app_installer_data TO readonly_user;


--
-- Name: app_installer_data_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE app_installer_data_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE app_installer_data_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE app_installer_data_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE app_installer_data_id_seq TO readonly_user;


--
-- Name: app_network_profiles; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE app_network_profiles FROM PUBLIC;
REVOKE ALL ON TABLE app_network_profiles FROM appodeal;
GRANT ALL ON TABLE app_network_profiles TO appodeal;
GRANT SELECT ON TABLE app_network_profiles TO readonly_user;


--
-- Name: app_network_profiles_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE app_network_profiles_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE app_network_profiles_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE app_network_profiles_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE app_network_profiles_id_seq TO readonly_user;


--
-- Name: app_revenue_shares; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE app_revenue_shares FROM PUBLIC;
REVOKE ALL ON TABLE app_revenue_shares FROM appodeal;
GRANT ALL ON TABLE app_revenue_shares TO appodeal;
GRANT SELECT ON TABLE app_revenue_shares TO readonly_user;


--
-- Name: app_revenue_shares_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE app_revenue_shares_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE app_revenue_shares_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE app_revenue_shares_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE app_revenue_shares_id_seq TO readonly_user;


--
-- Name: appflood_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE appflood_accounts FROM PUBLIC;
REVOKE ALL ON TABLE appflood_accounts FROM appodeal;
GRANT ALL ON TABLE appflood_accounts TO appodeal;
GRANT SELECT ON TABLE appflood_accounts TO support_bot;
GRANT SELECT ON TABLE appflood_accounts TO readonly_user;


--
-- Name: appflood_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE appflood_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE appflood_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE appflood_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE appflood_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE appflood_accounts_id_seq TO readonly_user;


--
-- Name: applovin_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE applovin_accounts FROM PUBLIC;
REVOKE ALL ON TABLE applovin_accounts FROM appodeal;
GRANT ALL ON TABLE applovin_accounts TO appodeal;
GRANT SELECT ON TABLE applovin_accounts TO support_bot;
GRANT SELECT ON TABLE applovin_accounts TO readonly_user;


--
-- Name: applovin_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE applovin_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE applovin_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE applovin_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE applovin_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE applovin_accounts_id_seq TO readonly_user;


--
-- Name: applovin_zones; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE applovin_zones FROM PUBLIC;
REVOKE ALL ON TABLE applovin_zones FROM appodeal;
GRANT ALL ON TABLE applovin_zones TO appodeal;
GRANT SELECT ON TABLE applovin_zones TO readonly_user;


--
-- Name: applovin_zones_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE applovin_zones_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE applovin_zones_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE applovin_zones_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE applovin_zones_id_seq TO readonly_user;


--
-- Name: appnexus_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE appnexus_accounts FROM PUBLIC;
REVOKE ALL ON TABLE appnexus_accounts FROM appodeal;
GRANT ALL ON TABLE appnexus_accounts TO appodeal;
GRANT SELECT ON TABLE appnexus_accounts TO support_bot;
GRANT SELECT ON TABLE appnexus_accounts TO readonly_user;


--
-- Name: appnexus_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE appnexus_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE appnexus_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE appnexus_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE appnexus_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE appnexus_accounts_id_seq TO readonly_user;


--
-- Name: apps; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE apps FROM PUBLIC;
REVOKE ALL ON TABLE apps FROM appodeal;
GRANT ALL ON TABLE apps TO appodeal;
GRANT SELECT ON TABLE apps TO support_bot;
GRANT SELECT ON TABLE apps TO readonly_user;


--
-- Name: apps_campaign_images; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE apps_campaign_images FROM PUBLIC;
REVOKE ALL ON TABLE apps_campaign_images FROM appodeal;
GRANT ALL ON TABLE apps_campaign_images TO appodeal;
GRANT SELECT ON TABLE apps_campaign_images TO support_bot;
GRANT SELECT ON TABLE apps_campaign_images TO readonly_user;


--
-- Name: apps_campaign_images_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE apps_campaign_images_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE apps_campaign_images_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE apps_campaign_images_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE apps_campaign_images_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE apps_campaign_images_id_seq TO readonly_user;


--
-- Name: apps_categories; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE apps_categories FROM PUBLIC;
REVOKE ALL ON TABLE apps_categories FROM appodeal;
GRANT ALL ON TABLE apps_categories TO appodeal;
GRANT SELECT ON TABLE apps_categories TO support_bot;
GRANT SELECT ON TABLE apps_categories TO readonly_user;


--
-- Name: apps_categories_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE apps_categories_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE apps_categories_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE apps_categories_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE apps_categories_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE apps_categories_id_seq TO readonly_user;


--
-- Name: apps_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE apps_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE apps_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE apps_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE apps_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE apps_id_seq TO readonly_user;


--
-- Name: apps_network_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE apps_network_accounts FROM PUBLIC;
REVOKE ALL ON TABLE apps_network_accounts FROM appodeal;
GRANT ALL ON TABLE apps_network_accounts TO appodeal;
GRANT SELECT ON TABLE apps_network_accounts TO support_bot;
GRANT SELECT ON TABLE apps_network_accounts TO readonly_user;


--
-- Name: apps_network_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE apps_network_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE apps_network_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE apps_network_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE apps_network_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE apps_network_accounts_id_seq TO readonly_user;


--
-- Name: apps_website_links; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE apps_website_links FROM PUBLIC;
REVOKE ALL ON TABLE apps_website_links FROM appodeal;
GRANT ALL ON TABLE apps_website_links TO appodeal;
GRANT SELECT ON TABLE apps_website_links TO readonly_user;


--
-- Name: apps_website_links_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE apps_website_links_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE apps_website_links_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE apps_website_links_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE apps_website_links_id_seq TO readonly_user;


--
-- Name: avocarrot_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE avocarrot_accounts FROM PUBLIC;
REVOKE ALL ON TABLE avocarrot_accounts FROM appodeal;
GRANT ALL ON TABLE avocarrot_accounts TO appodeal;
GRANT SELECT ON TABLE avocarrot_accounts TO support_bot;
GRANT SELECT ON TABLE avocarrot_accounts TO readonly_user;


--
-- Name: avocarrot_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE avocarrot_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE avocarrot_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE avocarrot_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE avocarrot_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE avocarrot_accounts_id_seq TO readonly_user;


--
-- Name: bad_devices; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bad_devices FROM PUBLIC;
REVOKE ALL ON TABLE bad_devices FROM appodeal;
GRANT ALL ON TABLE bad_devices TO appodeal;
GRANT SELECT ON TABLE bad_devices TO support_bot;
GRANT SELECT ON TABLE bad_devices TO readonly_user;


--
-- Name: bad_devices_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE bad_devices_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE bad_devices_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE bad_devices_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE bad_devices_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE bad_devices_id_seq TO readonly_user;


--
-- Name: bid_floor_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bid_floor_settings FROM PUBLIC;
REVOKE ALL ON TABLE bid_floor_settings FROM appodeal;
GRANT ALL ON TABLE bid_floor_settings TO appodeal;
GRANT SELECT ON TABLE bid_floor_settings TO support_bot;
GRANT SELECT ON TABLE bid_floor_settings TO readonly_user;


--
-- Name: bid_floor_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE bid_floor_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE bid_floor_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE bid_floor_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE bid_floor_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE bid_floor_settings_id_seq TO readonly_user;


--
-- Name: bidder_bidder_formats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bidder_bidder_formats FROM PUBLIC;
REVOKE ALL ON TABLE bidder_bidder_formats FROM appodeal;
GRANT ALL ON TABLE bidder_bidder_formats TO appodeal;
GRANT SELECT ON TABLE bidder_bidder_formats TO support_bot;
GRANT SELECT ON TABLE bidder_bidder_formats TO readonly_user;


--
-- Name: bidder_bidder_formats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE bidder_bidder_formats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE bidder_bidder_formats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE bidder_bidder_formats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE bidder_bidder_formats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE bidder_bidder_formats_id_seq TO readonly_user;


--
-- Name: bidder_campaigns; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bidder_campaigns FROM PUBLIC;
REVOKE ALL ON TABLE bidder_campaigns FROM appodeal;
GRANT ALL ON TABLE bidder_campaigns TO appodeal;
GRANT SELECT ON TABLE bidder_campaigns TO support_bot;
GRANT SELECT ON TABLE bidder_campaigns TO readonly_user;


--
-- Name: bidder_campaigns_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE bidder_campaigns_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE bidder_campaigns_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE bidder_campaigns_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE bidder_campaigns_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE bidder_campaigns_id_seq TO readonly_user;


--
-- Name: bidder_countries; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bidder_countries FROM PUBLIC;
REVOKE ALL ON TABLE bidder_countries FROM appodeal;
GRANT ALL ON TABLE bidder_countries TO appodeal;
GRANT SELECT ON TABLE bidder_countries TO support_bot;
GRANT SELECT ON TABLE bidder_countries TO readonly_user;


--
-- Name: bidder_countries_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE bidder_countries_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE bidder_countries_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE bidder_countries_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE bidder_countries_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE bidder_countries_id_seq TO readonly_user;


--
-- Name: bidder_formats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bidder_formats FROM PUBLIC;
REVOKE ALL ON TABLE bidder_formats FROM appodeal;
GRANT ALL ON TABLE bidder_formats TO appodeal;
GRANT SELECT ON TABLE bidder_formats TO support_bot;
GRANT SELECT ON TABLE bidder_formats TO readonly_user;


--
-- Name: bidder_formats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE bidder_formats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE bidder_formats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE bidder_formats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE bidder_formats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE bidder_formats_id_seq TO readonly_user;


--
-- Name: bidder_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bidder_settings FROM PUBLIC;
REVOKE ALL ON TABLE bidder_settings FROM appodeal;
GRANT ALL ON TABLE bidder_settings TO appodeal;
GRANT SELECT ON TABLE bidder_settings TO support_bot;
GRANT SELECT ON TABLE bidder_settings TO readonly_user;


--
-- Name: bidder_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE bidder_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE bidder_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE bidder_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE bidder_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE bidder_settings_id_seq TO readonly_user;


--
-- Name: bids; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE bids FROM PUBLIC;
REVOKE ALL ON TABLE bids FROM appodeal;
GRANT ALL ON TABLE bids TO appodeal;
GRANT SELECT ON TABLE bids TO support_bot;
GRANT SELECT ON TABLE bids TO readonly_user;


--
-- Name: billings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings FROM PUBLIC;
REVOKE ALL ON TABLE billings FROM appodeal;
GRANT ALL ON TABLE billings TO appodeal;
GRANT SELECT ON TABLE billings TO readonly_user;


--
-- Name: billings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE billings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE billings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE billings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE billings_id_seq TO readonly_user;


--
-- Name: billings_y2016m10; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2016m10 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2016m10 FROM appodeal;
GRANT ALL ON TABLE billings_y2016m10 TO appodeal;
GRANT SELECT ON TABLE billings_y2016m10 TO readonly_user;


--
-- Name: billings_y2016m11; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2016m11 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2016m11 FROM appodeal;
GRANT ALL ON TABLE billings_y2016m11 TO appodeal;
GRANT SELECT ON TABLE billings_y2016m11 TO readonly_user;


--
-- Name: billings_y2016m12; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2016m12 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2016m12 FROM appodeal;
GRANT ALL ON TABLE billings_y2016m12 TO appodeal;
GRANT SELECT ON TABLE billings_y2016m12 TO readonly_user;


--
-- Name: billings_y2017m01; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m01 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m01 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m01 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m01 TO readonly_user;


--
-- Name: billings_y2017m02; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m02 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m02 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m02 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m02 TO readonly_user;


--
-- Name: billings_y2017m03; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m03 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m03 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m03 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m03 TO readonly_user;


--
-- Name: billings_y2017m04; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m04 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m04 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m04 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m04 TO readonly_user;


--
-- Name: billings_y2017m05; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m05 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m05 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m05 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m05 TO readonly_user;


--
-- Name: billings_y2017m06; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m06 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m06 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m06 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m06 TO readonly_user;


--
-- Name: billings_y2017m07; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m07 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m07 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m07 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m07 TO readonly_user;


--
-- Name: billings_y2017m08; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m08 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m08 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m08 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m08 TO readonly_user;


--
-- Name: billings_y2017m09; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m09 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m09 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m09 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m09 TO readonly_user;


--
-- Name: billings_y2017m10; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m10 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m10 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m10 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m10 TO readonly_user;


--
-- Name: billings_y2017m11; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m11 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m11 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m11 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m11 TO readonly_user;


--
-- Name: billings_y2017m12; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2017m12 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2017m12 FROM appodeal;
GRANT ALL ON TABLE billings_y2017m12 TO appodeal;
GRANT SELECT ON TABLE billings_y2017m12 TO readonly_user;


--
-- Name: billings_y2018m01; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m01 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m01 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m01 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m01 TO readonly_user;


--
-- Name: billings_y2018m02; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m02 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m02 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m02 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m02 TO readonly_user;


--
-- Name: billings_y2018m03; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m03 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m03 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m03 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m03 TO readonly_user;


--
-- Name: billings_y2018m04; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m04 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m04 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m04 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m04 TO readonly_user;


--
-- Name: billings_y2018m05; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m05 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m05 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m05 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m05 TO readonly_user;


--
-- Name: billings_y2018m06; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m06 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m06 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m06 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m06 TO readonly_user;


--
-- Name: billings_y2018m07; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m07 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m07 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m07 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m07 TO readonly_user;


--
-- Name: billings_y2018m08; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m08 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m08 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m08 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m08 TO readonly_user;


--
-- Name: billings_y2018m09; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m09 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m09 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m09 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m09 TO readonly_user;


--
-- Name: billings_y2018m10; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE billings_y2018m10 FROM PUBLIC;
REVOKE ALL ON TABLE billings_y2018m10 FROM appodeal;
GRANT ALL ON TABLE billings_y2018m10 TO appodeal;
GRANT SELECT ON TABLE billings_y2018m10 TO readonly_user;


--
-- Name: block_devices; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE block_devices FROM PUBLIC;
REVOKE ALL ON TABLE block_devices FROM appodeal;
GRANT ALL ON TABLE block_devices TO appodeal;
GRANT SELECT ON TABLE block_devices TO readonly_user;


--
-- Name: block_devices_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE block_devices_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE block_devices_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE block_devices_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE block_devices_id_seq TO readonly_user;


--
-- Name: book_keeping_reports; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE book_keeping_reports FROM PUBLIC;
REVOKE ALL ON TABLE book_keeping_reports FROM appodeal;
GRANT ALL ON TABLE book_keeping_reports TO appodeal;
GRANT SELECT ON TABLE book_keeping_reports TO support_bot;
GRANT SELECT ON TABLE book_keeping_reports TO readonly_user;


--
-- Name: book_keeping_reports_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE book_keeping_reports_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE book_keeping_reports_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE book_keeping_reports_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE book_keeping_reports_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE book_keeping_reports_id_seq TO readonly_user;


--
-- Name: cached_files; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE cached_files FROM PUBLIC;
REVOKE ALL ON TABLE cached_files FROM appodeal;
GRANT ALL ON TABLE cached_files TO appodeal;
GRANT SELECT ON TABLE cached_files TO support_bot;
GRANT SELECT ON TABLE cached_files TO readonly_user;


--
-- Name: cached_files_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE cached_files_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE cached_files_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE cached_files_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE cached_files_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE cached_files_id_seq TO readonly_user;


--
-- Name: campaign_aggregated_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_aggregated_stats FROM PUBLIC;
REVOKE ALL ON TABLE campaign_aggregated_stats FROM appodeal;
GRANT ALL ON TABLE campaign_aggregated_stats TO appodeal;
GRANT SELECT ON TABLE campaign_aggregated_stats TO readonly_user;


--
-- Name: campaign_aggregated_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_aggregated_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_aggregated_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_aggregated_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_aggregated_stats_id_seq TO readonly_user;


--
-- Name: campaign_commissions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_commissions FROM PUBLIC;
REVOKE ALL ON TABLE campaign_commissions FROM appodeal;
GRANT ALL ON TABLE campaign_commissions TO appodeal;
GRANT SELECT ON TABLE campaign_commissions TO support_bot;
GRANT SELECT ON TABLE campaign_commissions TO readonly_user;


--
-- Name: campaign_commissions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_commissions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_commissions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_commissions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_commissions_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_commissions_id_seq TO readonly_user;


--
-- Name: campaign_coordinates; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_coordinates FROM PUBLIC;
REVOKE ALL ON TABLE campaign_coordinates FROM appodeal;
GRANT ALL ON TABLE campaign_coordinates TO appodeal;
GRANT SELECT ON TABLE campaign_coordinates TO readonly_user;


--
-- Name: campaign_coordinates_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_coordinates_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_coordinates_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_coordinates_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_coordinates_id_seq TO readonly_user;


--
-- Name: campaign_image_coordinates; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_image_coordinates FROM PUBLIC;
REVOKE ALL ON TABLE campaign_image_coordinates FROM appodeal;
GRANT ALL ON TABLE campaign_image_coordinates TO appodeal;
GRANT SELECT ON TABLE campaign_image_coordinates TO support_bot;
GRANT SELECT ON TABLE campaign_image_coordinates TO readonly_user;


--
-- Name: campaign_image_coordinates_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_image_coordinates_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_image_coordinates_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_image_coordinates_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_image_coordinates_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_image_coordinates_id_seq TO readonly_user;


--
-- Name: campaign_image_coordinates_images; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_image_coordinates_images FROM PUBLIC;
REVOKE ALL ON TABLE campaign_image_coordinates_images FROM appodeal;
GRANT ALL ON TABLE campaign_image_coordinates_images TO appodeal;
GRANT SELECT ON TABLE campaign_image_coordinates_images TO readonly_user;


--
-- Name: campaign_image_coordinates_images_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_image_coordinates_images_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_image_coordinates_images_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_image_coordinates_images_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_image_coordinates_images_id_seq TO readonly_user;


--
-- Name: campaign_image_devices; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_image_devices FROM PUBLIC;
REVOKE ALL ON TABLE campaign_image_devices FROM appodeal;
GRANT ALL ON TABLE campaign_image_devices TO appodeal;
GRANT SELECT ON TABLE campaign_image_devices TO support_bot;
GRANT SELECT ON TABLE campaign_image_devices TO readonly_user;


--
-- Name: campaign_image_devices_csvs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_image_devices_csvs FROM PUBLIC;
REVOKE ALL ON TABLE campaign_image_devices_csvs FROM appodeal;
GRANT ALL ON TABLE campaign_image_devices_csvs TO appodeal;
GRANT SELECT ON TABLE campaign_image_devices_csvs TO support_bot;
GRANT SELECT ON TABLE campaign_image_devices_csvs TO readonly_user;


--
-- Name: campaign_image_devices_csvs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_image_devices_csvs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_image_devices_csvs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_image_devices_csvs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_image_devices_csvs_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_image_devices_csvs_id_seq TO readonly_user;


--
-- Name: campaign_image_devices_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_image_devices_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_image_devices_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_image_devices_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_image_devices_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_image_devices_id_seq TO readonly_user;


--
-- Name: campaign_images; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_images FROM PUBLIC;
REVOKE ALL ON TABLE campaign_images FROM appodeal;
GRANT ALL ON TABLE campaign_images TO appodeal;
GRANT SELECT ON TABLE campaign_images TO support_bot;
GRANT SELECT ON TABLE campaign_images TO readonly_user;


--
-- Name: campaign_images_categories; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_images_categories FROM PUBLIC;
REVOKE ALL ON TABLE campaign_images_categories FROM appodeal;
GRANT ALL ON TABLE campaign_images_categories TO appodeal;
GRANT SELECT ON TABLE campaign_images_categories TO support_bot;
GRANT SELECT ON TABLE campaign_images_categories TO readonly_user;


--
-- Name: campaign_images_categories_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_images_categories_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_images_categories_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_images_categories_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_images_categories_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_images_categories_id_seq TO readonly_user;


--
-- Name: campaign_images_countries; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_images_countries FROM PUBLIC;
REVOKE ALL ON TABLE campaign_images_countries FROM appodeal;
GRANT ALL ON TABLE campaign_images_countries TO appodeal;
GRANT SELECT ON TABLE campaign_images_countries TO support_bot;
GRANT SELECT ON TABLE campaign_images_countries TO readonly_user;


--
-- Name: campaign_images_countries_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_images_countries_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_images_countries_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_images_countries_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_images_countries_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_images_countries_id_seq TO readonly_user;


--
-- Name: campaign_images_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_images_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_images_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_images_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_images_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_images_id_seq TO readonly_user;


--
-- Name: campaign_images_max_mind_cities; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_images_max_mind_cities FROM PUBLIC;
REVOKE ALL ON TABLE campaign_images_max_mind_cities FROM appodeal;
GRANT ALL ON TABLE campaign_images_max_mind_cities TO appodeal;
GRANT SELECT ON TABLE campaign_images_max_mind_cities TO readonly_user;


--
-- Name: campaign_images_max_mind_cities_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_images_max_mind_cities_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_images_max_mind_cities_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_images_max_mind_cities_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_images_max_mind_cities_id_seq TO readonly_user;


--
-- Name: campaign_images_mobile_networks; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_images_mobile_networks FROM PUBLIC;
REVOKE ALL ON TABLE campaign_images_mobile_networks FROM appodeal;
GRANT ALL ON TABLE campaign_images_mobile_networks TO appodeal;
GRANT SELECT ON TABLE campaign_images_mobile_networks TO support_bot;
GRANT SELECT ON TABLE campaign_images_mobile_networks TO readonly_user;


--
-- Name: campaign_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_settings FROM PUBLIC;
REVOKE ALL ON TABLE campaign_settings FROM appodeal;
GRANT ALL ON TABLE campaign_settings TO appodeal;
GRANT SELECT ON TABLE campaign_settings TO readonly_user;


--
-- Name: campaign_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_settings_id_seq TO readonly_user;


--
-- Name: campaign_types; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_types FROM PUBLIC;
REVOKE ALL ON TABLE campaign_types FROM appodeal;
GRANT ALL ON TABLE campaign_types TO appodeal;
GRANT SELECT ON TABLE campaign_types TO support_bot;
GRANT SELECT ON TABLE campaign_types TO readonly_user;


--
-- Name: campaign_types_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_types_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_types_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_types_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_types_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaign_types_id_seq TO readonly_user;


--
-- Name: campaign_versions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaign_versions FROM PUBLIC;
REVOKE ALL ON TABLE campaign_versions FROM appodeal;
GRANT ALL ON TABLE campaign_versions TO appodeal;
GRANT SELECT ON TABLE campaign_versions TO readonly_user;


--
-- Name: campaign_versions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaign_versions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaign_versions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaign_versions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaign_versions_id_seq TO readonly_user;


--
-- Name: campaigns; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaigns FROM PUBLIC;
REVOKE ALL ON TABLE campaigns FROM appodeal;
GRANT ALL ON TABLE campaigns TO appodeal;
GRANT SELECT ON TABLE campaigns TO support_bot;
GRANT SELECT ON TABLE campaigns TO readonly_user;


--
-- Name: campaigns_creative_sets; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE campaigns_creative_sets FROM PUBLIC;
REVOKE ALL ON TABLE campaigns_creative_sets FROM appodeal;
GRANT ALL ON TABLE campaigns_creative_sets TO appodeal;
GRANT SELECT ON TABLE campaigns_creative_sets TO readonly_user;


--
-- Name: campaigns_creative_sets_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaigns_creative_sets_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaigns_creative_sets_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaigns_creative_sets_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaigns_creative_sets_id_seq TO readonly_user;


--
-- Name: campaigns_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE campaigns_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE campaigns_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE campaigns_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE campaigns_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE campaigns_id_seq TO readonly_user;


--
-- Name: categories; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE categories FROM PUBLIC;
REVOKE ALL ON TABLE categories FROM appodeal;
GRANT ALL ON TABLE categories TO appodeal;
GRANT SELECT ON TABLE categories TO support_bot;
GRANT SELECT ON TABLE categories TO readonly_user;


--
-- Name: categories_contents; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE categories_contents FROM PUBLIC;
REVOKE ALL ON TABLE categories_contents FROM appodeal;
GRANT ALL ON TABLE categories_contents TO appodeal;
GRANT SELECT ON TABLE categories_contents TO readonly_user;


--
-- Name: categories_contents_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE categories_contents_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE categories_contents_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE categories_contents_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE categories_contents_id_seq TO readonly_user;


--
-- Name: categories_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE categories_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE categories_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE categories_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE categories_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE categories_id_seq TO readonly_user;


--
-- Name: categories_users; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE categories_users FROM PUBLIC;
REVOKE ALL ON TABLE categories_users FROM appodeal;
GRANT ALL ON TABLE categories_users TO appodeal;
GRANT SELECT ON TABLE categories_users TO readonly_user;


--
-- Name: categories_users_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE categories_users_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE categories_users_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE categories_users_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE categories_users_id_seq TO readonly_user;


--
-- Name: changelogs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE changelogs FROM PUBLIC;
REVOKE ALL ON TABLE changelogs FROM appodeal;
GRANT ALL ON TABLE changelogs TO appodeal;
GRANT SELECT ON TABLE changelogs TO readonly_user;


--
-- Name: changelogs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE changelogs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE changelogs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE changelogs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE changelogs_id_seq TO readonly_user;


--
-- Name: chartboost_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE chartboost_accounts FROM PUBLIC;
REVOKE ALL ON TABLE chartboost_accounts FROM appodeal;
GRANT ALL ON TABLE chartboost_accounts TO appodeal;
GRANT SELECT ON TABLE chartboost_accounts TO support_bot;
GRANT SELECT ON TABLE chartboost_accounts TO readonly_user;


--
-- Name: chartboost_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE chartboost_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE chartboost_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE chartboost_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE chartboost_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE chartboost_accounts_id_seq TO readonly_user;


--
-- Name: chartboost_campaigns; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE chartboost_campaigns FROM PUBLIC;
REVOKE ALL ON TABLE chartboost_campaigns FROM appodeal;
GRANT ALL ON TABLE chartboost_campaigns TO appodeal;
GRANT SELECT ON TABLE chartboost_campaigns TO support_bot;
GRANT SELECT ON TABLE chartboost_campaigns TO readonly_user;


--
-- Name: chartboost_campaigns_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE chartboost_campaigns_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE chartboost_campaigns_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE chartboost_campaigns_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE chartboost_campaigns_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE chartboost_campaigns_id_seq TO readonly_user;


--
-- Name: cities; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE cities FROM PUBLIC;
REVOKE ALL ON TABLE cities FROM appodeal;
GRANT ALL ON TABLE cities TO appodeal;
GRANT SELECT ON TABLE cities TO support_bot;
GRANT SELECT ON TABLE cities TO readonly_user;


--
-- Name: cities_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE cities_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE cities_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE cities_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE cities_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE cities_id_seq TO readonly_user;


--
-- Name: clicks; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE clicks FROM PUBLIC;
REVOKE ALL ON TABLE clicks FROM appodeal;
GRANT ALL ON TABLE clicks TO appodeal;
GRANT SELECT ON TABLE clicks TO support_bot;
GRANT SELECT ON TABLE clicks TO readonly_user;


--
-- Name: clicks_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE clicks_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE clicks_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE clicks_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE clicks_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE clicks_id_seq TO readonly_user;


--
-- Name: clicks_master; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE clicks_master FROM PUBLIC;
REVOKE ALL ON TABLE clicks_master FROM appodeal;
GRANT ALL ON TABLE clicks_master TO appodeal;
GRANT SELECT ON TABLE clicks_master TO support_bot;
GRANT SELECT ON TABLE clicks_master TO readonly_user;


--
-- Name: clicks_y2018m08; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE clicks_y2018m08 FROM PUBLIC;
REVOKE ALL ON TABLE clicks_y2018m08 FROM appodeal;
GRANT ALL ON TABLE clicks_y2018m08 TO appodeal;
GRANT SELECT ON TABLE clicks_y2018m08 TO readonly_user;


--
-- Name: clicks_y2018m09; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE clicks_y2018m09 FROM PUBLIC;
REVOKE ALL ON TABLE clicks_y2018m09 FROM appodeal;
GRANT ALL ON TABLE clicks_y2018m09 TO appodeal;
GRANT SELECT ON TABLE clicks_y2018m09 TO readonly_user;


--
-- Name: clicks_y2018m10; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE clicks_y2018m10 FROM PUBLIC;
REVOKE ALL ON TABLE clicks_y2018m10 FROM appodeal;
GRANT ALL ON TABLE clicks_y2018m10 TO appodeal;
GRANT SELECT ON TABLE clicks_y2018m10 TO readonly_user;


--
-- Name: clicks_y2018m11; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE clicks_y2018m11 FROM PUBLIC;
REVOKE ALL ON TABLE clicks_y2018m11 FROM appodeal;
GRANT ALL ON TABLE clicks_y2018m11 TO appodeal;
GRANT SELECT ON TABLE clicks_y2018m11 TO readonly_user;


--
-- Name: client_histories; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE client_histories FROM PUBLIC;
REVOKE ALL ON TABLE client_histories FROM appodeal;
GRANT ALL ON TABLE client_histories TO appodeal;
GRANT SELECT ON TABLE client_histories TO support_bot;
GRANT SELECT ON TABLE client_histories TO readonly_user;


--
-- Name: client_histories_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE client_histories_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE client_histories_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE client_histories_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE client_histories_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE client_histories_id_seq TO readonly_user;


--
-- Name: client_history_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE client_history_stats FROM PUBLIC;
REVOKE ALL ON TABLE client_history_stats FROM appodeal;
GRANT ALL ON TABLE client_history_stats TO appodeal;
GRANT SELECT ON TABLE client_history_stats TO support_bot;
GRANT SELECT ON TABLE client_history_stats TO readonly_user;


--
-- Name: client_history_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE client_history_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE client_history_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE client_history_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE client_history_stats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE client_history_stats_id_seq TO readonly_user;


--
-- Name: compensations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE compensations FROM PUBLIC;
REVOKE ALL ON TABLE compensations FROM appodeal;
GRANT ALL ON TABLE compensations TO appodeal;
GRANT SELECT ON TABLE compensations TO readonly_user;


--
-- Name: compensations_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE compensations_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE compensations_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE compensations_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE compensations_id_seq TO readonly_user;


--
-- Name: complains; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE complains FROM PUBLIC;
REVOKE ALL ON TABLE complains FROM appodeal;
GRANT ALL ON TABLE complains TO appodeal;
GRANT SELECT ON TABLE complains TO support_bot;
GRANT SELECT ON TABLE complains TO readonly_user;


--
-- Name: complains_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE complains_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE complains_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE complains_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE complains_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE complains_id_seq TO readonly_user;


--
-- Name: contents; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE contents FROM PUBLIC;
REVOKE ALL ON TABLE contents FROM appodeal;
GRANT ALL ON TABLE contents TO appodeal;
GRANT SELECT ON TABLE contents TO readonly_user;


--
-- Name: contents_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE contents_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE contents_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE contents_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE contents_id_seq TO readonly_user;


--
-- Name: countries; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE countries FROM PUBLIC;
REVOKE ALL ON TABLE countries FROM appodeal;
GRANT ALL ON TABLE countries TO appodeal;
GRANT SELECT ON TABLE countries TO support_bot;
GRANT SELECT ON TABLE countries TO readonly_user;


--
-- Name: countries_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE countries_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE countries_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE countries_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE countries_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE countries_id_seq TO readonly_user;


--
-- Name: creative_files; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE creative_files FROM PUBLIC;
REVOKE ALL ON TABLE creative_files FROM appodeal;
GRANT ALL ON TABLE creative_files TO appodeal;
GRANT SELECT ON TABLE creative_files TO readonly_user;


--
-- Name: creative_files_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE creative_files_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE creative_files_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE creative_files_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE creative_files_id_seq TO readonly_user;


--
-- Name: creative_items; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE creative_items FROM PUBLIC;
REVOKE ALL ON TABLE creative_items FROM appodeal;
GRANT ALL ON TABLE creative_items TO appodeal;
GRANT SELECT ON TABLE creative_items TO readonly_user;


--
-- Name: creative_items_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE creative_items_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE creative_items_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE creative_items_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE creative_items_id_seq TO readonly_user;


--
-- Name: creative_sets; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE creative_sets FROM PUBLIC;
REVOKE ALL ON TABLE creative_sets FROM appodeal;
GRANT ALL ON TABLE creative_sets TO appodeal;
GRANT SELECT ON TABLE creative_sets TO readonly_user;


--
-- Name: creative_sets_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE creative_sets_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE creative_sets_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE creative_sets_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE creative_sets_id_seq TO readonly_user;


--
-- Name: custom_app_bid_floors; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE custom_app_bid_floors FROM PUBLIC;
REVOKE ALL ON TABLE custom_app_bid_floors FROM appodeal;
GRANT ALL ON TABLE custom_app_bid_floors TO appodeal;
GRANT SELECT ON TABLE custom_app_bid_floors TO readonly_user;


--
-- Name: custom_app_bid_floors_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE custom_app_bid_floors_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE custom_app_bid_floors_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE custom_app_bid_floors_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE custom_app_bid_floors_id_seq TO readonly_user;


--
-- Name: custom_user_commissions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE custom_user_commissions FROM PUBLIC;
REVOKE ALL ON TABLE custom_user_commissions FROM appodeal;
GRANT ALL ON TABLE custom_user_commissions TO appodeal;
GRANT SELECT ON TABLE custom_user_commissions TO readonly_user;


--
-- Name: custom_user_commissions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE custom_user_commissions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE custom_user_commissions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE custom_user_commissions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE custom_user_commissions_id_seq TO readonly_user;


--
-- Name: dash_views_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE dash_views_logs FROM PUBLIC;
REVOKE ALL ON TABLE dash_views_logs FROM appodeal;
GRANT ALL ON TABLE dash_views_logs TO appodeal;
GRANT SELECT ON TABLE dash_views_logs TO support_bot;
GRANT SELECT ON TABLE dash_views_logs TO readonly_user;


--
-- Name: dash_views_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE dash_views_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE dash_views_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE dash_views_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE dash_views_logs_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE dash_views_logs_id_seq TO readonly_user;


--
-- Name: dashboard_predefined_filters; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE dashboard_predefined_filters FROM PUBLIC;
REVOKE ALL ON TABLE dashboard_predefined_filters FROM appodeal;
GRANT ALL ON TABLE dashboard_predefined_filters TO appodeal;
GRANT SELECT ON TABLE dashboard_predefined_filters TO support_bot;
GRANT SELECT ON TABLE dashboard_predefined_filters TO readonly_user;


--
-- Name: dashboard_predefined_filters_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE dashboard_predefined_filters_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE dashboard_predefined_filters_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE dashboard_predefined_filters_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE dashboard_predefined_filters_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE dashboard_predefined_filters_id_seq TO readonly_user;


--
-- Name: data_migrations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE data_migrations FROM PUBLIC;
REVOKE ALL ON TABLE data_migrations FROM appodeal;
GRANT ALL ON TABLE data_migrations TO appodeal;
GRANT SELECT ON TABLE data_migrations TO support_bot;
GRANT SELECT ON TABLE data_migrations TO readonly_user;


--
-- Name: deals; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE deals FROM PUBLIC;
REVOKE ALL ON TABLE deals FROM appodeal;
GRANT ALL ON TABLE deals TO appodeal;
GRANT SELECT ON TABLE deals TO readonly_user;


--
-- Name: deals_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE deals_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE deals_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE deals_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE deals_id_seq TO readonly_user;


--
-- Name: device_manufacturers; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE device_manufacturers FROM PUBLIC;
REVOKE ALL ON TABLE device_manufacturers FROM appodeal;
GRANT ALL ON TABLE device_manufacturers TO appodeal;
GRANT SELECT ON TABLE device_manufacturers TO support_bot;
GRANT SELECT ON TABLE device_manufacturers TO readonly_user;


--
-- Name: device_manufacturers_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE device_manufacturers_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE device_manufacturers_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE device_manufacturers_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE device_manufacturers_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE device_manufacturers_id_seq TO readonly_user;


--
-- Name: device_models; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE device_models FROM PUBLIC;
REVOKE ALL ON TABLE device_models FROM appodeal;
GRANT ALL ON TABLE device_models TO appodeal;
GRANT SELECT ON TABLE device_models TO support_bot;
GRANT SELECT ON TABLE device_models TO readonly_user;


--
-- Name: device_models_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE device_models_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE device_models_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE device_models_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE device_models_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE device_models_id_seq TO readonly_user;


--
-- Name: devices; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE devices FROM PUBLIC;
REVOKE ALL ON TABLE devices FROM appodeal;
GRANT ALL ON TABLE devices TO appodeal;
GRANT SELECT ON TABLE devices TO support_bot;
GRANT SELECT ON TABLE devices TO readonly_user;


--
-- Name: devices_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE devices_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE devices_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE devices_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE devices_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE devices_id_seq TO readonly_user;


--
-- Name: disabled_networks; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE disabled_networks FROM PUBLIC;
REVOKE ALL ON TABLE disabled_networks FROM appodeal;
GRANT ALL ON TABLE disabled_networks TO appodeal;
GRANT SELECT ON TABLE disabled_networks TO support_bot;
GRANT SELECT ON TABLE disabled_networks TO readonly_user;


--
-- Name: disabled_networks_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE disabled_networks_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE disabled_networks_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE disabled_networks_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE disabled_networks_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE disabled_networks_id_seq TO readonly_user;


--
-- Name: docs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE docs FROM PUBLIC;
REVOKE ALL ON TABLE docs FROM appodeal;
GRANT ALL ON TABLE docs TO appodeal;
GRANT SELECT ON TABLE docs TO support_bot;
GRANT SELECT ON TABLE docs TO readonly_user;


--
-- Name: docs_histories; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE docs_histories FROM PUBLIC;
REVOKE ALL ON TABLE docs_histories FROM appodeal;
GRANT ALL ON TABLE docs_histories TO appodeal;
GRANT SELECT ON TABLE docs_histories TO readonly_user;


--
-- Name: docs_histories_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE docs_histories_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE docs_histories_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE docs_histories_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE docs_histories_id_seq TO readonly_user;


--
-- Name: docs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE docs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE docs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE docs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE docs_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE docs_id_seq TO readonly_user;


--
-- Name: docs_sections; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE docs_sections FROM PUBLIC;
REVOKE ALL ON TABLE docs_sections FROM appodeal;
GRANT ALL ON TABLE docs_sections TO appodeal;
GRANT SELECT ON TABLE docs_sections TO support_bot;
GRANT SELECT ON TABLE docs_sections TO readonly_user;


--
-- Name: docs_sections_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE docs_sections_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE docs_sections_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE docs_sections_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE docs_sections_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE docs_sections_id_seq TO readonly_user;


--
-- Name: docs_sets; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE docs_sets FROM PUBLIC;
REVOKE ALL ON TABLE docs_sets FROM appodeal;
GRANT ALL ON TABLE docs_sets TO appodeal;
GRANT SELECT ON TABLE docs_sets TO support_bot;
GRANT SELECT ON TABLE docs_sets TO readonly_user;


--
-- Name: docs_sets_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE docs_sets_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE docs_sets_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE docs_sets_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE docs_sets_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE docs_sets_id_seq TO readonly_user;


--
-- Name: domain_font_files; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE domain_font_files FROM PUBLIC;
REVOKE ALL ON TABLE domain_font_files FROM appodeal;
GRANT ALL ON TABLE domain_font_files TO appodeal;
GRANT SELECT ON TABLE domain_font_files TO readonly_user;


--
-- Name: domain_font_files_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE domain_font_files_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE domain_font_files_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE domain_font_files_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE domain_font_files_id_seq TO readonly_user;


--
-- Name: domain_fonts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE domain_fonts FROM PUBLIC;
REVOKE ALL ON TABLE domain_fonts FROM appodeal;
GRANT ALL ON TABLE domain_fonts TO appodeal;
GRANT SELECT ON TABLE domain_fonts TO readonly_user;


--
-- Name: domain_fonts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE domain_fonts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE domain_fonts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE domain_fonts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE domain_fonts_id_seq TO readonly_user;


--
-- Name: domains; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE domains FROM PUBLIC;
REVOKE ALL ON TABLE domains FROM appodeal;
GRANT ALL ON TABLE domains TO appodeal;
GRANT SELECT ON TABLE domains TO readonly_user;


--
-- Name: domains_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE domains_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE domains_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE domains_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE domains_id_seq TO readonly_user;


--
-- Name: email_smtp_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE email_smtp_settings FROM PUBLIC;
REVOKE ALL ON TABLE email_smtp_settings FROM appodeal;
GRANT ALL ON TABLE email_smtp_settings TO appodeal;
GRANT SELECT ON TABLE email_smtp_settings TO readonly_user;


--
-- Name: email_smtp_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE email_smtp_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE email_smtp_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE email_smtp_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE email_smtp_settings_id_seq TO readonly_user;


--
-- Name: email_templates; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE email_templates FROM PUBLIC;
REVOKE ALL ON TABLE email_templates FROM appodeal;
GRANT ALL ON TABLE email_templates TO appodeal;
GRANT SELECT ON TABLE email_templates TO readonly_user;


--
-- Name: email_templates_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE email_templates_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE email_templates_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE email_templates_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE email_templates_id_seq TO readonly_user;


--
-- Name: exceptional_targetings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE exceptional_targetings FROM PUBLIC;
REVOKE ALL ON TABLE exceptional_targetings FROM appodeal;
GRANT ALL ON TABLE exceptional_targetings TO appodeal;
GRANT SELECT ON TABLE exceptional_targetings TO readonly_user;


--
-- Name: exceptional_targetings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE exceptional_targetings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE exceptional_targetings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE exceptional_targetings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE exceptional_targetings_id_seq TO readonly_user;


--
-- Name: exchange_coefficients; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE exchange_coefficients FROM PUBLIC;
REVOKE ALL ON TABLE exchange_coefficients FROM appodeal;
GRANT ALL ON TABLE exchange_coefficients TO appodeal;
GRANT SELECT ON TABLE exchange_coefficients TO support_bot;
GRANT SELECT ON TABLE exchange_coefficients TO readonly_user;


--
-- Name: exchange_coefficients_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE exchange_coefficients_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE exchange_coefficients_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE exchange_coefficients_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE exchange_coefficients_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE exchange_coefficients_id_seq TO readonly_user;


--
-- Name: exchange_rates; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE exchange_rates FROM PUBLIC;
REVOKE ALL ON TABLE exchange_rates FROM appodeal;
GRANT ALL ON TABLE exchange_rates TO appodeal;
GRANT SELECT ON TABLE exchange_rates TO support_bot;
GRANT SELECT ON TABLE exchange_rates TO readonly_user;


--
-- Name: exchange_rates_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE exchange_rates_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE exchange_rates_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE exchange_rates_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE exchange_rates_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE exchange_rates_id_seq TO readonly_user;


--
-- Name: extension_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE extension_logs FROM PUBLIC;
REVOKE ALL ON TABLE extension_logs FROM appodeal;
GRANT ALL ON TABLE extension_logs TO appodeal;
GRANT SELECT ON TABLE extension_logs TO support_bot;
GRANT SELECT ON TABLE extension_logs TO readonly_user;


--
-- Name: extension_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE extension_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE extension_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE extension_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE extension_logs_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE extension_logs_id_seq TO readonly_user;


--
-- Name: facebook_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE facebook_accounts FROM PUBLIC;
REVOKE ALL ON TABLE facebook_accounts FROM appodeal;
GRANT ALL ON TABLE facebook_accounts TO appodeal;
GRANT SELECT ON TABLE facebook_accounts TO support_bot;
GRANT SELECT ON TABLE facebook_accounts TO readonly_user;


--
-- Name: facebook_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE facebook_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE facebook_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE facebook_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE facebook_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE facebook_accounts_id_seq TO readonly_user;


--
-- Name: fast_affiliate_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_affiliate_stats FROM PUBLIC;
REVOKE ALL ON TABLE fast_affiliate_stats FROM appodeal;
GRANT ALL ON TABLE fast_affiliate_stats TO appodeal;
GRANT SELECT ON TABLE fast_affiliate_stats TO support_bot;
GRANT SELECT ON TABLE fast_affiliate_stats TO readonly_user;


--
-- Name: fast_affiliate_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fast_affiliate_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fast_affiliate_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fast_affiliate_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fast_affiliate_stats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE fast_affiliate_stats_id_seq TO readonly_user;


--
-- Name: fast_ecpm_capping_options; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_capping_options FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_capping_options FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_capping_options TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_capping_options TO readonly_user;


--
-- Name: fast_ecpm_capping_options_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fast_ecpm_capping_options_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fast_ecpm_capping_options_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fast_ecpm_capping_options_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fast_ecpm_capping_options_id_seq TO readonly_user;


--
-- Name: fast_ecpm_comparison_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_comparison_stats FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_comparison_stats FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_comparison_stats TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_comparison_stats TO readonly_user;


--
-- Name: fast_ecpm_comparison_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fast_ecpm_comparison_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fast_ecpm_comparison_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fast_ecpm_comparison_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fast_ecpm_comparison_stats_id_seq TO readonly_user;


--
-- Name: fast_ecpm_comparison_stats_loggers; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_comparison_stats_loggers FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_comparison_stats_loggers FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_comparison_stats_loggers TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_comparison_stats_loggers TO readonly_user;


--
-- Name: fast_ecpm_comparison_stats_loggers_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fast_ecpm_comparison_stats_loggers_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fast_ecpm_comparison_stats_loggers_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fast_ecpm_comparison_stats_loggers_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fast_ecpm_comparison_stats_loggers_id_seq TO readonly_user;


--
-- Name: fast_ecpm_new_options; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_new_options FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_new_options FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_new_options TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_new_options TO readonly_user;


--
-- Name: fast_ecpm_new_options_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fast_ecpm_new_options_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fast_ecpm_new_options_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fast_ecpm_new_options_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fast_ecpm_new_options_id_seq TO readonly_user;


--
-- Name: fast_ecpm_stats_old; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_stats_old FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_stats_old FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_stats_old TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_stats_old TO support_bot;
GRANT SELECT ON TABLE fast_ecpm_stats_old TO readonly_user;


--
-- Name: fast_ecpm_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fast_ecpm_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fast_ecpm_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fast_ecpm_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fast_ecpm_stats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE fast_ecpm_stats_id_seq TO readonly_user;


--
-- Name: fast_ecpm_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_stats FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_stats FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_stats TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_stats TO readonly_user;


--
-- Name: fast_ecpm_stats_exclusions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_stats_exclusions FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_stats_exclusions FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_stats_exclusions TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_stats_exclusions TO support_bot;
GRANT SELECT ON TABLE fast_ecpm_stats_exclusions TO readonly_user;


--
-- Name: fast_ecpm_stats_exclusions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fast_ecpm_stats_exclusions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fast_ecpm_stats_exclusions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fast_ecpm_stats_exclusions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fast_ecpm_stats_exclusions_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE fast_ecpm_stats_exclusions_id_seq TO readonly_user;


--
-- Name: fast_ecpm_stats_tmp; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fast_ecpm_stats_tmp FROM PUBLIC;
REVOKE ALL ON TABLE fast_ecpm_stats_tmp FROM appodeal;
GRANT ALL ON TABLE fast_ecpm_stats_tmp TO appodeal;
GRANT SELECT ON TABLE fast_ecpm_stats_tmp TO readonly_user;


--
-- Name: finalized_time_for_reports; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE finalized_time_for_reports FROM PUBLIC;
REVOKE ALL ON TABLE finalized_time_for_reports FROM appodeal;
GRANT ALL ON TABLE finalized_time_for_reports TO appodeal;
GRANT SELECT ON TABLE finalized_time_for_reports TO readonly_user;


--
-- Name: finalized_time_for_reports_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE finalized_time_for_reports_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE finalized_time_for_reports_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE finalized_time_for_reports_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE finalized_time_for_reports_id_seq TO readonly_user;


--
-- Name: fixed_admob_campaigns_sets; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fixed_admob_campaigns_sets FROM PUBLIC;
REVOKE ALL ON TABLE fixed_admob_campaigns_sets FROM appodeal;
GRANT ALL ON TABLE fixed_admob_campaigns_sets TO appodeal;
GRANT SELECT ON TABLE fixed_admob_campaigns_sets TO readonly_user;


--
-- Name: fixed_admob_campaigns_sets_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fixed_admob_campaigns_sets_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fixed_admob_campaigns_sets_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fixed_admob_campaigns_sets_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fixed_admob_campaigns_sets_id_seq TO readonly_user;


--
-- Name: flurry_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE flurry_accounts FROM PUBLIC;
REVOKE ALL ON TABLE flurry_accounts FROM appodeal;
GRANT ALL ON TABLE flurry_accounts TO appodeal;
GRANT SELECT ON TABLE flurry_accounts TO support_bot;
GRANT SELECT ON TABLE flurry_accounts TO readonly_user;


--
-- Name: flurry_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE flurry_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE flurry_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE flurry_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE flurry_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE flurry_accounts_id_seq TO readonly_user;


--
-- Name: fraud_click_series; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fraud_click_series FROM PUBLIC;
REVOKE ALL ON TABLE fraud_click_series FROM appodeal;
GRANT ALL ON TABLE fraud_click_series TO appodeal;
GRANT SELECT ON TABLE fraud_click_series TO support_bot;
GRANT SELECT ON TABLE fraud_click_series TO readonly_user;


--
-- Name: fraud_click_series_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fraud_click_series_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fraud_click_series_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fraud_click_series_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fraud_click_series_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE fraud_click_series_id_seq TO readonly_user;


--
-- Name: fraud_detections; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fraud_detections FROM PUBLIC;
REVOKE ALL ON TABLE fraud_detections FROM appodeal;
GRANT ALL ON TABLE fraud_detections TO appodeal;
GRANT SELECT ON TABLE fraud_detections TO support_bot;
GRANT SELECT ON TABLE fraud_detections TO readonly_user;


--
-- Name: fraud_detections_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fraud_detections_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fraud_detections_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fraud_detections_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fraud_detections_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE fraud_detections_id_seq TO readonly_user;


--
-- Name: fraud_user_emails; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fraud_user_emails FROM PUBLIC;
REVOKE ALL ON TABLE fraud_user_emails FROM appodeal;
GRANT ALL ON TABLE fraud_user_emails TO appodeal;
GRANT SELECT ON TABLE fraud_user_emails TO readonly_user;


--
-- Name: fraud_user_emails_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fraud_user_emails_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fraud_user_emails_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fraud_user_emails_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fraud_user_emails_id_seq TO readonly_user;


--
-- Name: fraud_user_keys; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE fraud_user_keys FROM PUBLIC;
REVOKE ALL ON TABLE fraud_user_keys FROM appodeal;
GRANT ALL ON TABLE fraud_user_keys TO appodeal;
GRANT SELECT ON TABLE fraud_user_keys TO readonly_user;


--
-- Name: fraud_user_keys_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE fraud_user_keys_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE fraud_user_keys_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE fraud_user_keys_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE fraud_user_keys_id_seq TO readonly_user;


--
-- Name: frequency_capping_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE frequency_capping_stats FROM PUBLIC;
REVOKE ALL ON TABLE frequency_capping_stats FROM appodeal;
GRANT ALL ON TABLE frequency_capping_stats TO appodeal;
GRANT SELECT ON TABLE frequency_capping_stats TO readonly_user;


--
-- Name: frequency_capping_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE frequency_capping_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE frequency_capping_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE frequency_capping_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE frequency_capping_stats_id_seq TO readonly_user;


--
-- Name: global_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE global_settings FROM PUBLIC;
REVOKE ALL ON TABLE global_settings FROM appodeal;
GRANT ALL ON TABLE global_settings TO appodeal;
GRANT SELECT ON TABLE global_settings TO support_bot;
GRANT SELECT ON TABLE global_settings TO readonly_user;


--
-- Name: global_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE global_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE global_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE global_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE global_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE global_settings_id_seq TO readonly_user;


--
-- Name: global_settings_sections; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE global_settings_sections FROM PUBLIC;
REVOKE ALL ON TABLE global_settings_sections FROM appodeal;
GRANT ALL ON TABLE global_settings_sections TO appodeal;
GRANT SELECT ON TABLE global_settings_sections TO readonly_user;


--
-- Name: global_settings_sections_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE global_settings_sections_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE global_settings_sections_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE global_settings_sections_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE global_settings_sections_id_seq TO readonly_user;


--
-- Name: globals; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE globals FROM PUBLIC;
REVOKE ALL ON TABLE globals FROM appodeal;
GRANT ALL ON TABLE globals TO appodeal;
GRANT SELECT ON TABLE globals TO support_bot;
GRANT SELECT ON TABLE globals TO readonly_user;


--
-- Name: globals_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE globals_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE globals_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE globals_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE globals_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE globals_id_seq TO readonly_user;


--
-- Name: heyzap_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE heyzap_accounts FROM PUBLIC;
REVOKE ALL ON TABLE heyzap_accounts FROM appodeal;
GRANT ALL ON TABLE heyzap_accounts TO appodeal;
GRANT SELECT ON TABLE heyzap_accounts TO support_bot;
GRANT SELECT ON TABLE heyzap_accounts TO readonly_user;


--
-- Name: heyzap_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE heyzap_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE heyzap_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE heyzap_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE heyzap_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE heyzap_accounts_id_seq TO readonly_user;


--
-- Name: inmobi_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE inmobi_accounts FROM PUBLIC;
REVOKE ALL ON TABLE inmobi_accounts FROM appodeal;
GRANT ALL ON TABLE inmobi_accounts TO appodeal;
GRANT SELECT ON TABLE inmobi_accounts TO support_bot;
GRANT SELECT ON TABLE inmobi_accounts TO readonly_user;


--
-- Name: inmobi_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE inmobi_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE inmobi_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE inmobi_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE inmobi_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE inmobi_accounts_id_seq TO readonly_user;


--
-- Name: inmobi_ad_units; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE inmobi_ad_units FROM PUBLIC;
REVOKE ALL ON TABLE inmobi_ad_units FROM appodeal;
GRANT ALL ON TABLE inmobi_ad_units TO appodeal;
GRANT SELECT ON TABLE inmobi_ad_units TO support_bot;
GRANT SELECT ON TABLE inmobi_ad_units TO readonly_user;


--
-- Name: inmobi_ad_units_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE inmobi_ad_units_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE inmobi_ad_units_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE inmobi_ad_units_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE inmobi_ad_units_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE inmobi_ad_units_id_seq TO readonly_user;


--
-- Name: inner_active_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE inner_active_accounts FROM PUBLIC;
REVOKE ALL ON TABLE inner_active_accounts FROM appodeal;
GRANT ALL ON TABLE inner_active_accounts TO appodeal;
GRANT SELECT ON TABLE inner_active_accounts TO support_bot;
GRANT SELECT ON TABLE inner_active_accounts TO readonly_user;


--
-- Name: inner_active_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE inner_active_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE inner_active_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE inner_active_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE inner_active_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE inner_active_accounts_id_seq TO readonly_user;


--
-- Name: insight_stat_histories; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE insight_stat_histories FROM PUBLIC;
REVOKE ALL ON TABLE insight_stat_histories FROM appodeal;
GRANT ALL ON TABLE insight_stat_histories TO appodeal;
GRANT SELECT ON TABLE insight_stat_histories TO support_bot;
GRANT SELECT ON TABLE insight_stat_histories TO readonly_user;


--
-- Name: insight_stat_histories_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE insight_stat_histories_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE insight_stat_histories_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE insight_stat_histories_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE insight_stat_histories_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE insight_stat_histories_id_seq TO readonly_user;


--
-- Name: insight_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE insight_stats FROM PUBLIC;
REVOKE ALL ON TABLE insight_stats FROM appodeal;
GRANT ALL ON TABLE insight_stats TO appodeal;
GRANT SELECT ON TABLE insight_stats TO support_bot;
GRANT SELECT ON TABLE insight_stats TO readonly_user;


--
-- Name: insight_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE insight_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE insight_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE insight_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE insight_stats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE insight_stats_id_seq TO readonly_user;


--
-- Name: insight_stats_y2018m03; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE insight_stats_y2018m03 FROM PUBLIC;
REVOKE ALL ON TABLE insight_stats_y2018m03 FROM appodeal;
GRANT ALL ON TABLE insight_stats_y2018m03 TO appodeal;
GRANT SELECT ON TABLE insight_stats_y2018m03 TO readonly_user;


--
-- Name: insight_stats_y2018m04; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE insight_stats_y2018m04 FROM PUBLIC;
REVOKE ALL ON TABLE insight_stats_y2018m04 FROM appodeal;
GRANT ALL ON TABLE insight_stats_y2018m04 TO appodeal;
GRANT SELECT ON TABLE insight_stats_y2018m04 TO readonly_user;


--
-- Name: insight_stats_y2018m05; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE insight_stats_y2018m05 FROM PUBLIC;
REVOKE ALL ON TABLE insight_stats_y2018m05 FROM appodeal;
GRANT ALL ON TABLE insight_stats_y2018m05 TO appodeal;
GRANT SELECT ON TABLE insight_stats_y2018m05 TO readonly_user;


--
-- Name: integration_frameworks; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE integration_frameworks FROM PUBLIC;
REVOKE ALL ON TABLE integration_frameworks FROM appodeal;
GRANT ALL ON TABLE integration_frameworks TO appodeal;
GRANT SELECT ON TABLE integration_frameworks TO support_bot;
GRANT SELECT ON TABLE integration_frameworks TO readonly_user;


--
-- Name: integration_frameworks_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE integration_frameworks_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE integration_frameworks_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE integration_frameworks_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE integration_frameworks_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE integration_frameworks_id_seq TO readonly_user;


--
-- Name: intercom_conversation_parts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE intercom_conversation_parts FROM PUBLIC;
REVOKE ALL ON TABLE intercom_conversation_parts FROM appodeal;
GRANT ALL ON TABLE intercom_conversation_parts TO appodeal;
GRANT SELECT ON TABLE intercom_conversation_parts TO support_bot;
GRANT SELECT ON TABLE intercom_conversation_parts TO readonly_user;


--
-- Name: intercom_conversation_parts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE intercom_conversation_parts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE intercom_conversation_parts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE intercom_conversation_parts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE intercom_conversation_parts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE intercom_conversation_parts_id_seq TO readonly_user;


--
-- Name: intercom_conversations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE intercom_conversations FROM PUBLIC;
REVOKE ALL ON TABLE intercom_conversations FROM appodeal;
GRANT ALL ON TABLE intercom_conversations TO appodeal;
GRANT SELECT ON TABLE intercom_conversations TO support_bot;
GRANT SELECT ON TABLE intercom_conversations TO readonly_user;


--
-- Name: intercom_conversations_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE intercom_conversations_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE intercom_conversations_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE intercom_conversations_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE intercom_conversations_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE intercom_conversations_id_seq TO readonly_user;


--
-- Name: intercom_network_recommendeds; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE intercom_network_recommendeds FROM PUBLIC;
REVOKE ALL ON TABLE intercom_network_recommendeds FROM appodeal;
GRANT ALL ON TABLE intercom_network_recommendeds TO appodeal;
GRANT SELECT ON TABLE intercom_network_recommendeds TO readonly_user;


--
-- Name: intercom_network_recommendeds_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE intercom_network_recommendeds_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE intercom_network_recommendeds_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE intercom_network_recommendeds_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE intercom_network_recommendeds_id_seq TO readonly_user;


--
-- Name: invoice_requests; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE invoice_requests FROM PUBLIC;
REVOKE ALL ON TABLE invoice_requests FROM appodeal;
GRANT ALL ON TABLE invoice_requests TO appodeal;
GRANT SELECT ON TABLE invoice_requests TO readonly_user;


--
-- Name: invoice_requests_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE invoice_requests_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE invoice_requests_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE invoice_requests_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE invoice_requests_id_seq TO readonly_user;


--
-- Name: ip_addresses; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE ip_addresses FROM PUBLIC;
REVOKE ALL ON TABLE ip_addresses FROM appodeal;
GRANT ALL ON TABLE ip_addresses TO appodeal;
GRANT SELECT ON TABLE ip_addresses TO support_bot;
GRANT SELECT ON TABLE ip_addresses TO readonly_user;


--
-- Name: ip_addresses_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE ip_addresses_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE ip_addresses_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE ip_addresses_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE ip_addresses_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE ip_addresses_id_seq TO readonly_user;


--
-- Name: liverail_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE liverail_accounts FROM PUBLIC;
REVOKE ALL ON TABLE liverail_accounts FROM appodeal;
GRANT ALL ON TABLE liverail_accounts TO appodeal;
GRANT SELECT ON TABLE liverail_accounts TO support_bot;
GRANT SELECT ON TABLE liverail_accounts TO readonly_user;


--
-- Name: liverail_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE liverail_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE liverail_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE liverail_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE liverail_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE liverail_accounts_id_seq TO readonly_user;


--
-- Name: liverail_ad_units; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE liverail_ad_units FROM PUBLIC;
REVOKE ALL ON TABLE liverail_ad_units FROM appodeal;
GRANT ALL ON TABLE liverail_ad_units TO appodeal;
GRANT SELECT ON TABLE liverail_ad_units TO support_bot;
GRANT SELECT ON TABLE liverail_ad_units TO readonly_user;


--
-- Name: liverail_ad_units_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE liverail_ad_units_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE liverail_ad_units_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE liverail_ad_units_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE liverail_ad_units_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE liverail_ad_units_id_seq TO readonly_user;


--
-- Name: long_term_account_revenues; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE long_term_account_revenues FROM PUBLIC;
REVOKE ALL ON TABLE long_term_account_revenues FROM appodeal;
GRANT ALL ON TABLE long_term_account_revenues TO appodeal;
GRANT SELECT ON TABLE long_term_account_revenues TO support_bot;
GRANT SELECT ON TABLE long_term_account_revenues TO readonly_user;


--
-- Name: long_term_account_revenues_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE long_term_account_revenues_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE long_term_account_revenues_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE long_term_account_revenues_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE long_term_account_revenues_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE long_term_account_revenues_id_seq TO readonly_user;


--
-- Name: magic_monies; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE magic_monies FROM PUBLIC;
REVOKE ALL ON TABLE magic_monies FROM appodeal;
GRANT ALL ON TABLE magic_monies TO appodeal;
GRANT SELECT ON TABLE magic_monies TO support_bot;
GRANT SELECT ON TABLE magic_monies TO readonly_user;


--
-- Name: magic_monies_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE magic_monies_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE magic_monies_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE magic_monies_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE magic_monies_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE magic_monies_id_seq TO readonly_user;


--
-- Name: managers; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE managers FROM PUBLIC;
REVOKE ALL ON TABLE managers FROM appodeal;
GRANT ALL ON TABLE managers TO appodeal;
GRANT SELECT ON TABLE managers TO support_bot;
GRANT SELECT ON TABLE managers TO readonly_user;


--
-- Name: managers_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE managers_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE managers_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE managers_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE managers_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE managers_id_seq TO readonly_user;


--
-- Name: max_mind_cities; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE max_mind_cities FROM PUBLIC;
REVOKE ALL ON TABLE max_mind_cities FROM appodeal;
GRANT ALL ON TABLE max_mind_cities TO appodeal;
GRANT SELECT ON TABLE max_mind_cities TO readonly_user;


--
-- Name: max_mind_cities_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE max_mind_cities_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE max_mind_cities_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE max_mind_cities_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE max_mind_cities_id_seq TO readonly_user;


--
-- Name: mmedia_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE mmedia_accounts FROM PUBLIC;
REVOKE ALL ON TABLE mmedia_accounts FROM appodeal;
GRANT ALL ON TABLE mmedia_accounts TO appodeal;
GRANT SELECT ON TABLE mmedia_accounts TO support_bot;
GRANT SELECT ON TABLE mmedia_accounts TO readonly_user;


--
-- Name: mmedia_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE mmedia_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE mmedia_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE mmedia_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE mmedia_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE mmedia_accounts_id_seq TO readonly_user;


--
-- Name: mobile_networks; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE mobile_networks FROM PUBLIC;
REVOKE ALL ON TABLE mobile_networks FROM appodeal;
GRANT ALL ON TABLE mobile_networks TO appodeal;
GRANT SELECT ON TABLE mobile_networks TO support_bot;
GRANT SELECT ON TABLE mobile_networks TO readonly_user;


--
-- Name: mobile_networks_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE mobile_networks_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE mobile_networks_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE mobile_networks_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE mobile_networks_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE mobile_networks_id_seq TO readonly_user;


--
-- Name: mobilecore_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE mobilecore_accounts FROM PUBLIC;
REVOKE ALL ON TABLE mobilecore_accounts FROM appodeal;
GRANT ALL ON TABLE mobilecore_accounts TO appodeal;
GRANT SELECT ON TABLE mobilecore_accounts TO support_bot;
GRANT SELECT ON TABLE mobilecore_accounts TO readonly_user;


--
-- Name: mobilecore_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE mobilecore_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE mobilecore_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE mobilecore_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE mobilecore_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE mobilecore_accounts_id_seq TO readonly_user;


--
-- Name: monitor_events; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE monitor_events FROM PUBLIC;
REVOKE ALL ON TABLE monitor_events FROM appodeal;
GRANT ALL ON TABLE monitor_events TO appodeal;
GRANT SELECT ON TABLE monitor_events TO readonly_user;


--
-- Name: monitor_events_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE monitor_events_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE monitor_events_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE monitor_events_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE monitor_events_id_seq TO readonly_user;


--
-- Name: monthly_book_keeping_reports; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE monthly_book_keeping_reports FROM PUBLIC;
REVOKE ALL ON TABLE monthly_book_keeping_reports FROM appodeal;
GRANT ALL ON TABLE monthly_book_keeping_reports TO appodeal;
GRANT SELECT ON TABLE monthly_book_keeping_reports TO readonly_user;


--
-- Name: monthly_book_keeping_reports_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE monthly_book_keeping_reports_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE monthly_book_keeping_reports_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE monthly_book_keeping_reports_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE monthly_book_keeping_reports_id_seq TO readonly_user;


--
-- Name: mopub_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE mopub_accounts FROM PUBLIC;
REVOKE ALL ON TABLE mopub_accounts FROM appodeal;
GRANT ALL ON TABLE mopub_accounts TO appodeal;
GRANT SELECT ON TABLE mopub_accounts TO support_bot;
GRANT SELECT ON TABLE mopub_accounts TO readonly_user;


--
-- Name: mopub_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE mopub_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE mopub_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE mopub_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE mopub_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE mopub_accounts_id_seq TO readonly_user;


--
-- Name: network_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_accounts FROM PUBLIC;
REVOKE ALL ON TABLE network_accounts FROM appodeal;
GRANT ALL ON TABLE network_accounts TO appodeal;
GRANT SELECT ON TABLE network_accounts TO support_bot;
GRANT SELECT ON TABLE network_accounts TO readonly_user;


--
-- Name: network_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE network_accounts_id_seq TO readonly_user;


--
-- Name: network_active_types; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_active_types FROM PUBLIC;
REVOKE ALL ON TABLE network_active_types FROM appodeal;
GRANT ALL ON TABLE network_active_types TO appodeal;
GRANT SELECT ON TABLE network_active_types TO support_bot;
GRANT SELECT ON TABLE network_active_types TO readonly_user;


--
-- Name: network_active_types_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_active_types_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_active_types_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_active_types_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_active_types_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE network_active_types_id_seq TO readonly_user;


--
-- Name: network_connection_restrictions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_connection_restrictions FROM PUBLIC;
REVOKE ALL ON TABLE network_connection_restrictions FROM appodeal;
GRANT ALL ON TABLE network_connection_restrictions TO appodeal;
GRANT SELECT ON TABLE network_connection_restrictions TO support_bot;
GRANT SELECT ON TABLE network_connection_restrictions TO readonly_user;


--
-- Name: network_connection_restrictions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_connection_restrictions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_connection_restrictions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_connection_restrictions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_connection_restrictions_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE network_connection_restrictions_id_seq TO readonly_user;


--
-- Name: network_connection_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_connection_settings FROM PUBLIC;
REVOKE ALL ON TABLE network_connection_settings FROM appodeal;
GRANT ALL ON TABLE network_connection_settings TO appodeal;
GRANT SELECT ON TABLE network_connection_settings TO support_bot;
GRANT SELECT ON TABLE network_connection_settings TO readonly_user;


--
-- Name: network_connection_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_connection_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_connection_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_connection_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_connection_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE network_connection_settings_id_seq TO readonly_user;


--
-- Name: network_connection_statuses; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_connection_statuses FROM PUBLIC;
REVOKE ALL ON TABLE network_connection_statuses FROM appodeal;
GRANT ALL ON TABLE network_connection_statuses TO appodeal;
GRANT SELECT ON TABLE network_connection_statuses TO support_bot;
GRANT SELECT ON TABLE network_connection_statuses TO readonly_user;


--
-- Name: network_connection_statuses_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_connection_statuses_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_connection_statuses_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_connection_statuses_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_connection_statuses_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE network_connection_statuses_id_seq TO readonly_user;


--
-- Name: network_content_filters; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_content_filters FROM PUBLIC;
REVOKE ALL ON TABLE network_content_filters FROM appodeal;
GRANT ALL ON TABLE network_content_filters TO appodeal;
GRANT SELECT ON TABLE network_content_filters TO readonly_user;


--
-- Name: network_content_filters_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_content_filters_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_content_filters_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_content_filters_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_content_filters_id_seq TO readonly_user;


--
-- Name: network_daemon_jobs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_daemon_jobs FROM PUBLIC;
REVOKE ALL ON TABLE network_daemon_jobs FROM appodeal;
GRANT ALL ON TABLE network_daemon_jobs TO appodeal;
GRANT SELECT ON TABLE network_daemon_jobs TO readonly_user;


--
-- Name: network_daemon_jobs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_daemon_jobs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_daemon_jobs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_daemon_jobs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_daemon_jobs_id_seq TO readonly_user;


--
-- Name: network_mailing_app_statuses; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_mailing_app_statuses FROM PUBLIC;
REVOKE ALL ON TABLE network_mailing_app_statuses FROM appodeal;
GRANT ALL ON TABLE network_mailing_app_statuses TO appodeal;
GRANT SELECT ON TABLE network_mailing_app_statuses TO readonly_user;


--
-- Name: network_mailing_app_statuses_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_mailing_app_statuses_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_mailing_app_statuses_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_mailing_app_statuses_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_mailing_app_statuses_id_seq TO readonly_user;


--
-- Name: network_mailing_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_mailing_settings FROM PUBLIC;
REVOKE ALL ON TABLE network_mailing_settings FROM appodeal;
GRANT ALL ON TABLE network_mailing_settings TO appodeal;
GRANT SELECT ON TABLE network_mailing_settings TO readonly_user;


--
-- Name: network_mailing_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_mailing_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_mailing_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_mailing_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_mailing_settings_id_seq TO readonly_user;


--
-- Name: network_migrations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_migrations FROM PUBLIC;
REVOKE ALL ON TABLE network_migrations FROM appodeal;
GRANT ALL ON TABLE network_migrations TO appodeal;
GRANT SELECT ON TABLE network_migrations TO support_bot;
GRANT SELECT ON TABLE network_migrations TO readonly_user;


--
-- Name: network_migrations_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_migrations_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_migrations_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_migrations_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_migrations_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE network_migrations_id_seq TO readonly_user;


--
-- Name: network_reporting_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_reporting_settings FROM PUBLIC;
REVOKE ALL ON TABLE network_reporting_settings FROM appodeal;
GRANT ALL ON TABLE network_reporting_settings TO appodeal;
GRANT SELECT ON TABLE network_reporting_settings TO readonly_user;


--
-- Name: network_reporting_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_reporting_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_reporting_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_reporting_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_reporting_settings_id_seq TO readonly_user;


--
-- Name: network_request_restrictions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE network_request_restrictions FROM PUBLIC;
REVOKE ALL ON TABLE network_request_restrictions FROM appodeal;
GRANT ALL ON TABLE network_request_restrictions TO appodeal;
GRANT SELECT ON TABLE network_request_restrictions TO readonly_user;


--
-- Name: network_request_restrictions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE network_request_restrictions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE network_request_restrictions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE network_request_restrictions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE network_request_restrictions_id_seq TO readonly_user;


--
-- Name: new_fast_ecpm_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE new_fast_ecpm_stats FROM PUBLIC;
REVOKE ALL ON TABLE new_fast_ecpm_stats FROM appodeal;
GRANT ALL ON TABLE new_fast_ecpm_stats TO appodeal;
GRANT SELECT ON TABLE new_fast_ecpm_stats TO readonly_user;


--
-- Name: new_fast_ecpm_stats_old; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE new_fast_ecpm_stats_old FROM PUBLIC;
REVOKE ALL ON TABLE new_fast_ecpm_stats_old FROM appodeal;
GRANT ALL ON TABLE new_fast_ecpm_stats_old TO appodeal;
GRANT SELECT ON TABLE new_fast_ecpm_stats_old TO readonly_user;


--
-- Name: nexage_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE nexage_accounts FROM PUBLIC;
REVOKE ALL ON TABLE nexage_accounts FROM appodeal;
GRANT ALL ON TABLE nexage_accounts TO appodeal;
GRANT SELECT ON TABLE nexage_accounts TO support_bot;
GRANT SELECT ON TABLE nexage_accounts TO readonly_user;


--
-- Name: nexage_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE nexage_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE nexage_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE nexage_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE nexage_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE nexage_accounts_id_seq TO readonly_user;


--
-- Name: openx_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE openx_accounts FROM PUBLIC;
REVOKE ALL ON TABLE openx_accounts FROM appodeal;
GRANT ALL ON TABLE openx_accounts TO appodeal;
GRANT SELECT ON TABLE openx_accounts TO support_bot;
GRANT SELECT ON TABLE openx_accounts TO readonly_user;


--
-- Name: openx_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE openx_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE openx_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE openx_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE openx_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE openx_accounts_id_seq TO readonly_user;


--
-- Name: organization_commission_presets; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE organization_commission_presets FROM PUBLIC;
REVOKE ALL ON TABLE organization_commission_presets FROM appodeal;
GRANT ALL ON TABLE organization_commission_presets TO appodeal;
GRANT SELECT ON TABLE organization_commission_presets TO readonly_user;


--
-- Name: organization_commission_presets_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE organization_commission_presets_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE organization_commission_presets_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE organization_commission_presets_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE organization_commission_presets_id_seq TO readonly_user;


--
-- Name: organization_roles; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE organization_roles FROM PUBLIC;
REVOKE ALL ON TABLE organization_roles FROM appodeal;
GRANT ALL ON TABLE organization_roles TO appodeal;
GRANT SELECT ON TABLE organization_roles TO readonly_user;


--
-- Name: organization_roles_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE organization_roles_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE organization_roles_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE organization_roles_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE organization_roles_id_seq TO readonly_user;


--
-- Name: organizations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE organizations FROM PUBLIC;
REVOKE ALL ON TABLE organizations FROM appodeal;
GRANT ALL ON TABLE organizations TO appodeal;
GRANT SELECT ON TABLE organizations TO readonly_user;


--
-- Name: organizations_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE organizations_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE organizations_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE organizations_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE organizations_id_seq TO readonly_user;


--
-- Name: page_categories_apps; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE page_categories_apps FROM PUBLIC;
REVOKE ALL ON TABLE page_categories_apps FROM appodeal;
GRANT ALL ON TABLE page_categories_apps TO appodeal;
GRANT SELECT ON TABLE page_categories_apps TO readonly_user;


--
-- Name: page_categories_apps_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE page_categories_apps_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE page_categories_apps_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE page_categories_apps_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE page_categories_apps_id_seq TO readonly_user;


--
-- Name: partner_app_commissions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE partner_app_commissions FROM PUBLIC;
REVOKE ALL ON TABLE partner_app_commissions FROM appodeal;
GRANT ALL ON TABLE partner_app_commissions TO appodeal;
GRANT SELECT ON TABLE partner_app_commissions TO readonly_user;


--
-- Name: partner_app_commissions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE partner_app_commissions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE partner_app_commissions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE partner_app_commissions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE partner_app_commissions_id_seq TO readonly_user;


--
-- Name: payment_requests; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE payment_requests FROM PUBLIC;
REVOKE ALL ON TABLE payment_requests FROM appodeal;
GRANT ALL ON TABLE payment_requests TO appodeal;
GRANT SELECT ON TABLE payment_requests TO support_bot;
GRANT SELECT ON TABLE payment_requests TO readonly_user;


--
-- Name: payment_requests_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE payment_requests_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE payment_requests_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE payment_requests_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE payment_requests_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE payment_requests_id_seq TO readonly_user;


--
-- Name: payouts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE payouts FROM PUBLIC;
REVOKE ALL ON TABLE payouts FROM appodeal;
GRANT ALL ON TABLE payouts TO appodeal;
GRANT SELECT ON TABLE payouts TO support_bot;
GRANT SELECT ON TABLE payouts TO readonly_user;


--
-- Name: payouts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE payouts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE payouts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE payouts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE payouts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE payouts_id_seq TO readonly_user;


--
-- Name: platforms; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE platforms FROM PUBLIC;
REVOKE ALL ON TABLE platforms FROM appodeal;
GRANT ALL ON TABLE platforms TO appodeal;
GRANT SELECT ON TABLE platforms TO support_bot;
GRANT SELECT ON TABLE platforms TO readonly_user;


--
-- Name: platforms_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE platforms_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE platforms_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE platforms_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE platforms_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE platforms_id_seq TO readonly_user;


--
-- Name: premium_user_commissions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE premium_user_commissions FROM PUBLIC;
REVOKE ALL ON TABLE premium_user_commissions FROM appodeal;
GRANT ALL ON TABLE premium_user_commissions TO appodeal;
GRANT SELECT ON TABLE premium_user_commissions TO readonly_user;


--
-- Name: premium_user_commissions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE premium_user_commissions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE premium_user_commissions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE premium_user_commissions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE premium_user_commissions_id_seq TO readonly_user;


--
-- Name: price_floors; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE price_floors FROM PUBLIC;
REVOKE ALL ON TABLE price_floors FROM appodeal;
GRANT ALL ON TABLE price_floors TO appodeal;
GRANT SELECT ON TABLE price_floors TO support_bot;
GRANT SELECT ON TABLE price_floors TO readonly_user;


--
-- Name: price_floors_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE price_floors_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE price_floors_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE price_floors_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE price_floors_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE price_floors_id_seq TO readonly_user;


--
-- Name: publisher_monitor_messages; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE publisher_monitor_messages FROM PUBLIC;
REVOKE ALL ON TABLE publisher_monitor_messages FROM appodeal;
GRANT ALL ON TABLE publisher_monitor_messages TO appodeal;
GRANT SELECT ON TABLE publisher_monitor_messages TO readonly_user;


--
-- Name: publisher_monitor_messages_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE publisher_monitor_messages_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE publisher_monitor_messages_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE publisher_monitor_messages_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE publisher_monitor_messages_id_seq TO readonly_user;


--
-- Name: pubnative_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE pubnative_accounts FROM PUBLIC;
REVOKE ALL ON TABLE pubnative_accounts FROM appodeal;
GRANT ALL ON TABLE pubnative_accounts TO appodeal;
GRANT SELECT ON TABLE pubnative_accounts TO support_bot;
GRANT SELECT ON TABLE pubnative_accounts TO readonly_user;


--
-- Name: pubnative_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE pubnative_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE pubnative_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE pubnative_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE pubnative_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE pubnative_accounts_id_seq TO readonly_user;


--
-- Name: qubole_sdk_data; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE qubole_sdk_data FROM PUBLIC;
REVOKE ALL ON TABLE qubole_sdk_data FROM appodeal;
GRANT ALL ON TABLE qubole_sdk_data TO appodeal;
GRANT SELECT ON TABLE qubole_sdk_data TO support_bot;
GRANT SELECT ON TABLE qubole_sdk_data TO readonly_user;


--
-- Name: qubole_sdk_data_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE qubole_sdk_data_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE qubole_sdk_data_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE qubole_sdk_data_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE qubole_sdk_data_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE qubole_sdk_data_id_seq TO readonly_user;


--
-- Name: report_processing_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE report_processing_logs FROM PUBLIC;
REVOKE ALL ON TABLE report_processing_logs FROM appodeal;
GRANT ALL ON TABLE report_processing_logs TO appodeal;
GRANT SELECT ON TABLE report_processing_logs TO readonly_user;


--
-- Name: report_processing_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE report_processing_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE report_processing_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE report_processing_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE report_processing_logs_id_seq TO readonly_user;


--
-- Name: reports; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE reports FROM PUBLIC;
REVOKE ALL ON TABLE reports FROM appodeal;
GRANT ALL ON TABLE reports TO appodeal;
GRANT SELECT ON TABLE reports TO support_bot;
GRANT SELECT ON TABLE reports TO readonly_user;


--
-- Name: reports_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE reports_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE reports_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE reports_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE reports_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE reports_id_seq TO readonly_user;


--
-- Name: request_blacklists; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE request_blacklists FROM PUBLIC;
REVOKE ALL ON TABLE request_blacklists FROM appodeal;
GRANT ALL ON TABLE request_blacklists TO appodeal;
GRANT SELECT ON TABLE request_blacklists TO support_bot;
GRANT SELECT ON TABLE request_blacklists TO readonly_user;


--
-- Name: request_blacklists_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE request_blacklists_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE request_blacklists_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE request_blacklists_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE request_blacklists_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE request_blacklists_id_seq TO readonly_user;


--
-- Name: revenue_analysis_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE revenue_analysis_settings FROM PUBLIC;
REVOKE ALL ON TABLE revenue_analysis_settings FROM appodeal;
GRANT ALL ON TABLE revenue_analysis_settings TO appodeal;
GRANT SELECT ON TABLE revenue_analysis_settings TO support_bot;
GRANT SELECT ON TABLE revenue_analysis_settings TO readonly_user;


--
-- Name: revenue_analysis_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE revenue_analysis_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE revenue_analysis_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE revenue_analysis_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE revenue_analysis_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE revenue_analysis_settings_id_seq TO readonly_user;


--
-- Name: revenue_change_limit_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE revenue_change_limit_settings FROM PUBLIC;
REVOKE ALL ON TABLE revenue_change_limit_settings FROM appodeal;
GRANT ALL ON TABLE revenue_change_limit_settings TO appodeal;
GRANT SELECT ON TABLE revenue_change_limit_settings TO readonly_user;


--
-- Name: revenue_change_limit_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE revenue_change_limit_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE revenue_change_limit_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE revenue_change_limit_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE revenue_change_limit_settings_id_seq TO readonly_user;


--
-- Name: revenue_comparison_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE revenue_comparison_stats FROM PUBLIC;
REVOKE ALL ON TABLE revenue_comparison_stats FROM appodeal;
GRANT ALL ON TABLE revenue_comparison_stats TO appodeal;
GRANT SELECT ON TABLE revenue_comparison_stats TO support_bot;
GRANT SELECT ON TABLE revenue_comparison_stats TO readonly_user;


--
-- Name: revenue_comparison_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE revenue_comparison_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE revenue_comparison_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE revenue_comparison_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE revenue_comparison_stats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE revenue_comparison_stats_id_seq TO readonly_user;


--
-- Name: roles; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE roles FROM PUBLIC;
REVOKE ALL ON TABLE roles FROM appodeal;
GRANT ALL ON TABLE roles TO appodeal;
GRANT SELECT ON TABLE roles TO readonly_user;


--
-- Name: roles_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE roles_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE roles_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE roles_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE roles_id_seq TO readonly_user;


--
-- Name: rubicon_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE rubicon_accounts FROM PUBLIC;
REVOKE ALL ON TABLE rubicon_accounts FROM appodeal;
GRANT ALL ON TABLE rubicon_accounts TO appodeal;
GRANT SELECT ON TABLE rubicon_accounts TO support_bot;
GRANT SELECT ON TABLE rubicon_accounts TO readonly_user;


--
-- Name: rubicon_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE rubicon_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE rubicon_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE rubicon_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE rubicon_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE rubicon_accounts_id_seq TO readonly_user;


--
-- Name: schema_migrations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE schema_migrations FROM PUBLIC;
REVOKE ALL ON TABLE schema_migrations FROM appodeal;
GRANT ALL ON TABLE schema_migrations TO appodeal;
GRANT SELECT ON TABLE schema_migrations TO support_bot;
GRANT SELECT ON TABLE schema_migrations TO readonly_user;


--
-- Name: sdk_mapping_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE sdk_mapping_settings FROM PUBLIC;
REVOKE ALL ON TABLE sdk_mapping_settings FROM appodeal;
GRANT ALL ON TABLE sdk_mapping_settings TO appodeal;
GRANT SELECT ON TABLE sdk_mapping_settings TO support_bot;
GRANT SELECT ON TABLE sdk_mapping_settings TO readonly_user;


--
-- Name: sdk_mapping_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE sdk_mapping_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE sdk_mapping_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE sdk_mapping_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE sdk_mapping_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE sdk_mapping_settings_id_seq TO readonly_user;


--
-- Name: sdk_releases; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE sdk_releases FROM PUBLIC;
REVOKE ALL ON TABLE sdk_releases FROM appodeal;
GRANT ALL ON TABLE sdk_releases TO appodeal;
GRANT SELECT ON TABLE sdk_releases TO support_bot;
GRANT SELECT ON TABLE sdk_releases TO readonly_user;


--
-- Name: sdk_releases_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE sdk_releases_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE sdk_releases_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE sdk_releases_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE sdk_releases_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE sdk_releases_id_seq TO readonly_user;


--
-- Name: section_categories_apps; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE section_categories_apps FROM PUBLIC;
REVOKE ALL ON TABLE section_categories_apps FROM appodeal;
GRANT ALL ON TABLE section_categories_apps TO appodeal;
GRANT SELECT ON TABLE section_categories_apps TO readonly_user;


--
-- Name: section_categories_apps_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE section_categories_apps_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE section_categories_apps_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE section_categories_apps_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE section_categories_apps_id_seq TO readonly_user;


--
-- Name: segment_placements; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE segment_placements FROM PUBLIC;
REVOKE ALL ON TABLE segment_placements FROM appodeal;
GRANT ALL ON TABLE segment_placements TO appodeal;
GRANT SELECT ON TABLE segment_placements TO support_bot;
GRANT SELECT ON TABLE segment_placements TO readonly_user;


--
-- Name: segment_placements_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE segment_placements_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE segment_placements_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE segment_placements_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE segment_placements_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE segment_placements_id_seq TO readonly_user;


--
-- Name: segment_rule_countries; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE segment_rule_countries FROM PUBLIC;
REVOKE ALL ON TABLE segment_rule_countries FROM appodeal;
GRANT ALL ON TABLE segment_rule_countries TO appodeal;
GRANT SELECT ON TABLE segment_rule_countries TO support_bot;
GRANT SELECT ON TABLE segment_rule_countries TO readonly_user;


--
-- Name: segment_rule_countries_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE segment_rule_countries_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE segment_rule_countries_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE segment_rule_countries_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE segment_rule_countries_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE segment_rule_countries_id_seq TO readonly_user;


--
-- Name: segment_rules; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE segment_rules FROM PUBLIC;
REVOKE ALL ON TABLE segment_rules FROM appodeal;
GRANT ALL ON TABLE segment_rules TO appodeal;
GRANT SELECT ON TABLE segment_rules TO support_bot;
GRANT SELECT ON TABLE segment_rules TO readonly_user;


--
-- Name: segment_rules_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE segment_rules_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE segment_rules_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE segment_rules_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE segment_rules_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE segment_rules_id_seq TO readonly_user;


--
-- Name: segment_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE segment_settings FROM PUBLIC;
REVOKE ALL ON TABLE segment_settings FROM appodeal;
GRANT ALL ON TABLE segment_settings TO appodeal;
GRANT SELECT ON TABLE segment_settings TO support_bot;
GRANT SELECT ON TABLE segment_settings TO readonly_user;


--
-- Name: segment_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE segment_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE segment_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE segment_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE segment_settings_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE segment_settings_id_seq TO readonly_user;


--
-- Name: segments; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE segments FROM PUBLIC;
REVOKE ALL ON TABLE segments FROM appodeal;
GRANT ALL ON TABLE segments TO appodeal;
GRANT SELECT ON TABLE segments TO support_bot;
GRANT SELECT ON TABLE segments TO readonly_user;


--
-- Name: segments_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE segments_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE segments_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE segments_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE segments_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE segments_id_seq TO readonly_user;


--
-- Name: share_tokens; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE share_tokens FROM PUBLIC;
REVOKE ALL ON TABLE share_tokens FROM appodeal;
GRANT ALL ON TABLE share_tokens TO appodeal;
GRANT SELECT ON TABLE share_tokens TO readonly_user;


--
-- Name: share_tokens_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE share_tokens_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE share_tokens_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE share_tokens_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE share_tokens_id_seq TO readonly_user;


--
-- Name: simple_dashboard; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE simple_dashboard FROM PUBLIC;
REVOKE ALL ON TABLE simple_dashboard FROM appodeal;
GRANT ALL ON TABLE simple_dashboard TO appodeal;
GRANT SELECT ON TABLE simple_dashboard TO readonly_user;


--
-- Name: simple_dashboard_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE simple_dashboard_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE simple_dashboard_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE simple_dashboard_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE simple_dashboard_id_seq TO readonly_user;


--
-- Name: simple_dashboard_monthly; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE simple_dashboard_monthly FROM PUBLIC;
REVOKE ALL ON TABLE simple_dashboard_monthly FROM appodeal;
GRANT ALL ON TABLE simple_dashboard_monthly TO appodeal;
GRANT SELECT ON TABLE simple_dashboard_monthly TO readonly_user;


--
-- Name: simple_dashboard_monthly_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE simple_dashboard_monthly_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE simple_dashboard_monthly_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE simple_dashboard_monthly_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE simple_dashboard_monthly_id_seq TO readonly_user;


--
-- Name: simple_dashboard_quarterly; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE simple_dashboard_quarterly FROM PUBLIC;
REVOKE ALL ON TABLE simple_dashboard_quarterly FROM appodeal;
GRANT ALL ON TABLE simple_dashboard_quarterly TO appodeal;
GRANT SELECT ON TABLE simple_dashboard_quarterly TO readonly_user;


--
-- Name: simple_dashboard_quarterly_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE simple_dashboard_quarterly_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE simple_dashboard_quarterly_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE simple_dashboard_quarterly_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE simple_dashboard_quarterly_id_seq TO readonly_user;


--
-- Name: simple_dashboard_weekly; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE simple_dashboard_weekly FROM PUBLIC;
REVOKE ALL ON TABLE simple_dashboard_weekly FROM appodeal;
GRANT ALL ON TABLE simple_dashboard_weekly TO appodeal;
GRANT SELECT ON TABLE simple_dashboard_weekly TO readonly_user;


--
-- Name: simple_dashboard_weekly_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE simple_dashboard_weekly_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE simple_dashboard_weekly_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE simple_dashboard_weekly_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE simple_dashboard_weekly_id_seq TO readonly_user;


--
-- Name: simple_dashboard_yearly; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE simple_dashboard_yearly FROM PUBLIC;
REVOKE ALL ON TABLE simple_dashboard_yearly FROM appodeal;
GRANT ALL ON TABLE simple_dashboard_yearly TO appodeal;
GRANT SELECT ON TABLE simple_dashboard_yearly TO readonly_user;


--
-- Name: simple_dashboard_yearly_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE simple_dashboard_yearly_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE simple_dashboard_yearly_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE simple_dashboard_yearly_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE simple_dashboard_yearly_id_seq TO readonly_user;


--
-- Name: smaato_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE smaato_accounts FROM PUBLIC;
REVOKE ALL ON TABLE smaato_accounts FROM appodeal;
GRANT ALL ON TABLE smaato_accounts TO appodeal;
GRANT SELECT ON TABLE smaato_accounts TO support_bot;
GRANT SELECT ON TABLE smaato_accounts TO readonly_user;


--
-- Name: smaato_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE smaato_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE smaato_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE smaato_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE smaato_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE smaato_accounts_id_seq TO readonly_user;


--
-- Name: spotx_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE spotx_accounts FROM PUBLIC;
REVOKE ALL ON TABLE spotx_accounts FROM appodeal;
GRANT ALL ON TABLE spotx_accounts TO appodeal;
GRANT SELECT ON TABLE spotx_accounts TO support_bot;
GRANT SELECT ON TABLE spotx_accounts TO readonly_user;


--
-- Name: spotx_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE spotx_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE spotx_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE spotx_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE spotx_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE spotx_accounts_id_seq TO readonly_user;


--
-- Name: startapp_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE startapp_accounts FROM PUBLIC;
REVOKE ALL ON TABLE startapp_accounts FROM appodeal;
GRANT ALL ON TABLE startapp_accounts TO appodeal;
GRANT SELECT ON TABLE startapp_accounts TO support_bot;
GRANT SELECT ON TABLE startapp_accounts TO readonly_user;


--
-- Name: startapp_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE startapp_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE startapp_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE startapp_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE startapp_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE startapp_accounts_id_seq TO readonly_user;


--
-- Name: statistic_reports; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE statistic_reports FROM PUBLIC;
REVOKE ALL ON TABLE statistic_reports FROM appodeal;
GRANT ALL ON TABLE statistic_reports TO appodeal;
GRANT SELECT ON TABLE statistic_reports TO support_bot;
GRANT SELECT ON TABLE statistic_reports TO readonly_user;


--
-- Name: statistic_reports_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE statistic_reports_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE statistic_reports_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE statistic_reports_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE statistic_reports_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE statistic_reports_id_seq TO readonly_user;


--
-- Name: statistics; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE statistics FROM PUBLIC;
REVOKE ALL ON TABLE statistics FROM appodeal;
GRANT ALL ON TABLE statistics TO appodeal;
GRANT SELECT ON TABLE statistics TO support_bot;
GRANT SELECT ON TABLE statistics TO readonly_user;


--
-- Name: statistics_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE statistics_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE statistics_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE statistics_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE statistics_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE statistics_id_seq TO readonly_user;


--
-- Name: store_infos; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE store_infos FROM PUBLIC;
REVOKE ALL ON TABLE store_infos FROM appodeal;
GRANT ALL ON TABLE store_infos TO appodeal;
GRANT SELECT ON TABLE store_infos TO readonly_user;


--
-- Name: store_infos_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE store_infos_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE store_infos_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE store_infos_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE store_infos_id_seq TO readonly_user;


--
-- Name: subdivision_ones; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE subdivision_ones FROM PUBLIC;
REVOKE ALL ON TABLE subdivision_ones FROM appodeal;
GRANT ALL ON TABLE subdivision_ones TO appodeal;
GRANT SELECT ON TABLE subdivision_ones TO support_bot;
GRANT SELECT ON TABLE subdivision_ones TO readonly_user;


--
-- Name: subdivision_ones_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE subdivision_ones_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE subdivision_ones_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE subdivision_ones_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE subdivision_ones_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE subdivision_ones_id_seq TO readonly_user;


--
-- Name: subdivision_twos; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE subdivision_twos FROM PUBLIC;
REVOKE ALL ON TABLE subdivision_twos FROM appodeal;
GRANT ALL ON TABLE subdivision_twos TO appodeal;
GRANT SELECT ON TABLE subdivision_twos TO support_bot;
GRANT SELECT ON TABLE subdivision_twos TO readonly_user;


--
-- Name: subdivision_twos_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE subdivision_twos_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE subdivision_twos_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE subdivision_twos_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE subdivision_twos_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE subdivision_twos_id_seq TO readonly_user;


--
-- Name: supersonic_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE supersonic_accounts FROM PUBLIC;
REVOKE ALL ON TABLE supersonic_accounts FROM appodeal;
GRANT ALL ON TABLE supersonic_accounts TO appodeal;
GRANT SELECT ON TABLE supersonic_accounts TO support_bot;
GRANT SELECT ON TABLE supersonic_accounts TO readonly_user;


--
-- Name: supersonic_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE supersonic_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE supersonic_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE supersonic_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE supersonic_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE supersonic_accounts_id_seq TO readonly_user;


--
-- Name: support_work_metriks; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE support_work_metriks FROM PUBLIC;
REVOKE ALL ON TABLE support_work_metriks FROM appodeal;
GRANT ALL ON TABLE support_work_metriks TO appodeal;
GRANT SELECT ON TABLE support_work_metriks TO readonly_user;


--
-- Name: support_work_metriks_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE support_work_metriks_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE support_work_metriks_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE support_work_metriks_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE support_work_metriks_id_seq TO readonly_user;


--
-- Name: support_work_reports; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE support_work_reports FROM PUBLIC;
REVOKE ALL ON TABLE support_work_reports FROM appodeal;
GRANT ALL ON TABLE support_work_reports TO appodeal;
GRANT SELECT ON TABLE support_work_reports TO readonly_user;


--
-- Name: support_work_reports_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE support_work_reports_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE support_work_reports_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE support_work_reports_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE support_work_reports_id_seq TO readonly_user;


--
-- Name: support_work_values; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE support_work_values FROM PUBLIC;
REVOKE ALL ON TABLE support_work_values FROM appodeal;
GRANT ALL ON TABLE support_work_values TO appodeal;
GRANT SELECT ON TABLE support_work_values TO readonly_user;


--
-- Name: survey_responses; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE survey_responses FROM PUBLIC;
REVOKE ALL ON TABLE survey_responses FROM appodeal;
GRANT ALL ON TABLE survey_responses TO appodeal;
GRANT SELECT ON TABLE survey_responses TO readonly_user;


--
-- Name: survey_responses_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE survey_responses_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE survey_responses_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE survey_responses_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE survey_responses_id_seq TO readonly_user;


--
-- Name: surveys; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE surveys FROM PUBLIC;
REVOKE ALL ON TABLE surveys FROM appodeal;
GRANT ALL ON TABLE surveys TO appodeal;
GRANT SELECT ON TABLE surveys TO readonly_user;


--
-- Name: surveys_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE surveys_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE surveys_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE surveys_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE surveys_id_seq TO readonly_user;


--
-- Name: tapsense_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE tapsense_accounts FROM PUBLIC;
REVOKE ALL ON TABLE tapsense_accounts FROM appodeal;
GRANT ALL ON TABLE tapsense_accounts TO appodeal;
GRANT SELECT ON TABLE tapsense_accounts TO support_bot;
GRANT SELECT ON TABLE tapsense_accounts TO readonly_user;


--
-- Name: tapsense_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE tapsense_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE tapsense_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE tapsense_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE tapsense_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE tapsense_accounts_id_seq TO readonly_user;


--
-- Name: target_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE target_accounts FROM PUBLIC;
REVOKE ALL ON TABLE target_accounts FROM appodeal;
GRANT ALL ON TABLE target_accounts TO appodeal;
GRANT SELECT ON TABLE target_accounts TO support_bot;
GRANT SELECT ON TABLE target_accounts TO readonly_user;


--
-- Name: target_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE target_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE target_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE target_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE target_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE target_accounts_id_seq TO readonly_user;


--
-- Name: teams; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE teams FROM PUBLIC;
REVOKE ALL ON TABLE teams FROM appodeal;
GRANT ALL ON TABLE teams TO appodeal;
GRANT SELECT ON TABLE teams TO support_bot;
GRANT SELECT ON TABLE teams TO readonly_user;


--
-- Name: teams_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE teams_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE teams_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE teams_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE teams_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE teams_id_seq TO readonly_user;


--
-- Name: temp_apps_network_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE temp_apps_network_accounts FROM PUBLIC;
REVOKE ALL ON TABLE temp_apps_network_accounts FROM appodeal;
GRANT ALL ON TABLE temp_apps_network_accounts TO appodeal;
GRANT SELECT ON TABLE temp_apps_network_accounts TO support_bot;
GRANT SELECT ON TABLE temp_apps_network_accounts TO readonly_user;


--
-- Name: temp_apps_network_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE temp_apps_network_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE temp_apps_network_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE temp_apps_network_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE temp_apps_network_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE temp_apps_network_accounts_id_seq TO readonly_user;


--
-- Name: test_device_ids; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE test_device_ids FROM PUBLIC;
REVOKE ALL ON TABLE test_device_ids FROM appodeal;
GRANT ALL ON TABLE test_device_ids TO appodeal;
GRANT SELECT ON TABLE test_device_ids TO support_bot;
GRANT SELECT ON TABLE test_device_ids TO readonly_user;


--
-- Name: test_device_ids_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE test_device_ids_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE test_device_ids_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE test_device_ids_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE test_device_ids_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE test_device_ids_id_seq TO readonly_user;


--
-- Name: traffic_limit_blocks; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE traffic_limit_blocks FROM PUBLIC;
REVOKE ALL ON TABLE traffic_limit_blocks FROM appodeal;
GRANT ALL ON TABLE traffic_limit_blocks TO appodeal;
GRANT SELECT ON TABLE traffic_limit_blocks TO readonly_user;


--
-- Name: traffic_limit_blocks_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE traffic_limit_blocks_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE traffic_limit_blocks_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE traffic_limit_blocks_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE traffic_limit_blocks_id_seq TO readonly_user;


--
-- Name: traffic_limit_exclusions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE traffic_limit_exclusions FROM PUBLIC;
REVOKE ALL ON TABLE traffic_limit_exclusions FROM appodeal;
GRANT ALL ON TABLE traffic_limit_exclusions TO appodeal;
GRANT SELECT ON TABLE traffic_limit_exclusions TO readonly_user;


--
-- Name: traffic_limit_exclusions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE traffic_limit_exclusions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE traffic_limit_exclusions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE traffic_limit_exclusions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE traffic_limit_exclusions_id_seq TO readonly_user;


--
-- Name: unity_ads_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE unity_ads_accounts FROM PUBLIC;
REVOKE ALL ON TABLE unity_ads_accounts FROM appodeal;
GRANT ALL ON TABLE unity_ads_accounts TO appodeal;
GRANT SELECT ON TABLE unity_ads_accounts TO support_bot;
GRANT SELECT ON TABLE unity_ads_accounts TO readonly_user;


--
-- Name: unity_ads_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE unity_ads_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE unity_ads_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE unity_ads_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE unity_ads_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE unity_ads_accounts_id_seq TO readonly_user;


--
-- Name: user_ach_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_ach_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_ach_requisites FROM appodeal;
GRANT ALL ON TABLE user_ach_requisites TO appodeal;
GRANT SELECT ON TABLE user_ach_requisites TO support_bot;
GRANT SELECT ON TABLE user_ach_requisites TO readonly_user;


--
-- Name: user_ach_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_ach_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_ach_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_ach_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_ach_requisites_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_ach_requisites_id_seq TO readonly_user;


--
-- Name: user_apps_commissions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_apps_commissions FROM PUBLIC;
REVOKE ALL ON TABLE user_apps_commissions FROM appodeal;
GRANT ALL ON TABLE user_apps_commissions TO appodeal;
GRANT SELECT ON TABLE user_apps_commissions TO readonly_user;


--
-- Name: user_apps_commissions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_apps_commissions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_apps_commissions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_apps_commissions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_apps_commissions_id_seq TO readonly_user;


--
-- Name: user_bitcoin_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_bitcoin_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_bitcoin_requisites FROM appodeal;
GRANT ALL ON TABLE user_bitcoin_requisites TO appodeal;
GRANT SELECT ON TABLE user_bitcoin_requisites TO support_bot;
GRANT SELECT ON TABLE user_bitcoin_requisites TO readonly_user;


--
-- Name: user_bitcoin_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_bitcoin_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_bitcoin_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_bitcoin_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_bitcoin_requisites_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_bitcoin_requisites_id_seq TO readonly_user;


--
-- Name: user_confirmations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_confirmations FROM PUBLIC;
REVOKE ALL ON TABLE user_confirmations FROM appodeal;
GRANT ALL ON TABLE user_confirmations TO appodeal;
GRANT SELECT ON TABLE user_confirmations TO readonly_user;


--
-- Name: user_confirmations_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_confirmations_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_confirmations_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_confirmations_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_confirmations_id_seq TO readonly_user;


--
-- Name: user_daily_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_daily_stats FROM PUBLIC;
REVOKE ALL ON TABLE user_daily_stats FROM appodeal;
GRANT ALL ON TABLE user_daily_stats TO appodeal;
GRANT SELECT ON TABLE user_daily_stats TO support_bot;
GRANT SELECT ON TABLE user_daily_stats TO readonly_user;


--
-- Name: user_daily_stats_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_daily_stats_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_daily_stats_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_daily_stats_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_daily_stats_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_daily_stats_id_seq TO readonly_user;


--
-- Name: user_ethereum_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_ethereum_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_ethereum_requisites FROM appodeal;
GRANT ALL ON TABLE user_ethereum_requisites TO appodeal;
GRANT SELECT ON TABLE user_ethereum_requisites TO readonly_user;


--
-- Name: user_ethereum_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_ethereum_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_ethereum_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_ethereum_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_ethereum_requisites_id_seq TO readonly_user;


--
-- Name: user_fraud_warning_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_fraud_warning_logs FROM PUBLIC;
REVOKE ALL ON TABLE user_fraud_warning_logs FROM appodeal;
GRANT ALL ON TABLE user_fraud_warning_logs TO appodeal;
GRANT SELECT ON TABLE user_fraud_warning_logs TO readonly_user;


--
-- Name: user_fraud_warning_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_fraud_warning_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_fraud_warning_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_fraud_warning_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_fraud_warning_logs_id_seq TO readonly_user;


--
-- Name: user_google_wallet_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_google_wallet_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_google_wallet_requisites FROM appodeal;
GRANT ALL ON TABLE user_google_wallet_requisites TO appodeal;
GRANT SELECT ON TABLE user_google_wallet_requisites TO support_bot;
GRANT SELECT ON TABLE user_google_wallet_requisites TO readonly_user;


--
-- Name: user_google_wallet_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_google_wallet_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_google_wallet_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_google_wallet_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_google_wallet_requisites_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_google_wallet_requisites_id_seq TO readonly_user;


--
-- Name: user_intercom_events; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_intercom_events FROM PUBLIC;
REVOKE ALL ON TABLE user_intercom_events FROM appodeal;
GRANT ALL ON TABLE user_intercom_events TO appodeal;
GRANT SELECT ON TABLE user_intercom_events TO support_bot;
GRANT SELECT ON TABLE user_intercom_events TO readonly_user;


--
-- Name: user_intercom_events_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_intercom_events_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_intercom_events_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_intercom_events_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_intercom_events_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_intercom_events_id_seq TO readonly_user;


--
-- Name: user_login_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_login_logs FROM PUBLIC;
REVOKE ALL ON TABLE user_login_logs FROM appodeal;
GRANT ALL ON TABLE user_login_logs TO appodeal;
GRANT SELECT ON TABLE user_login_logs TO support_bot;
GRANT SELECT ON TABLE user_login_logs TO readonly_user;


--
-- Name: user_login_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_login_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_login_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_login_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_login_logs_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_login_logs_id_seq TO readonly_user;


--
-- Name: user_notifications; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_notifications FROM PUBLIC;
REVOKE ALL ON TABLE user_notifications FROM appodeal;
GRANT ALL ON TABLE user_notifications TO appodeal;
GRANT SELECT ON TABLE user_notifications TO support_bot;
GRANT SELECT ON TABLE user_notifications TO readonly_user;


--
-- Name: user_notifications_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_notifications_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_notifications_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_notifications_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_notifications_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_notifications_id_seq TO readonly_user;


--
-- Name: user_payment_details_change_confirms; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_payment_details_change_confirms FROM PUBLIC;
REVOKE ALL ON TABLE user_payment_details_change_confirms FROM appodeal;
GRANT ALL ON TABLE user_payment_details_change_confirms TO appodeal;
GRANT SELECT ON TABLE user_payment_details_change_confirms TO support_bot;
GRANT SELECT ON TABLE user_payment_details_change_confirms TO readonly_user;


--
-- Name: user_payment_details_change_confirms_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_payment_details_change_confirms_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_payment_details_change_confirms_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_payment_details_change_confirms_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_payment_details_change_confirms_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_payment_details_change_confirms_id_seq TO readonly_user;


--
-- Name: user_payoneer_bank_transfer_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_payoneer_bank_transfer_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_payoneer_bank_transfer_requisites FROM appodeal;
GRANT ALL ON TABLE user_payoneer_bank_transfer_requisites TO appodeal;
GRANT SELECT ON TABLE user_payoneer_bank_transfer_requisites TO readonly_user;


--
-- Name: user_payoneer_bank_transfer_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_payoneer_bank_transfer_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_payoneer_bank_transfer_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_payoneer_bank_transfer_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_payoneer_bank_transfer_requisites_id_seq TO readonly_user;


--
-- Name: user_payoneer_paypal_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_payoneer_paypal_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_payoneer_paypal_requisites FROM appodeal;
GRANT ALL ON TABLE user_payoneer_paypal_requisites TO appodeal;
GRANT SELECT ON TABLE user_payoneer_paypal_requisites TO readonly_user;


--
-- Name: user_payoneer_paypal_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_payoneer_paypal_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_payoneer_paypal_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_payoneer_paypal_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_payoneer_paypal_requisites_id_seq TO readonly_user;


--
-- Name: user_payoneer_prepaid_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_payoneer_prepaid_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_payoneer_prepaid_requisites FROM appodeal;
GRANT ALL ON TABLE user_payoneer_prepaid_requisites TO appodeal;
GRANT SELECT ON TABLE user_payoneer_prepaid_requisites TO readonly_user;


--
-- Name: user_payoneer_prepaid_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_payoneer_prepaid_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_payoneer_prepaid_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_payoneer_prepaid_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_payoneer_prepaid_requisites_id_seq TO readonly_user;


--
-- Name: user_paypal_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_paypal_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_paypal_requisites FROM appodeal;
GRANT ALL ON TABLE user_paypal_requisites TO appodeal;
GRANT SELECT ON TABLE user_paypal_requisites TO support_bot;
GRANT SELECT ON TABLE user_paypal_requisites TO readonly_user;


--
-- Name: user_paypal_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_paypal_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_paypal_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_paypal_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_paypal_requisites_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_paypal_requisites_id_seq TO readonly_user;


--
-- Name: user_webmoney_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_webmoney_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_webmoney_requisites FROM appodeal;
GRANT ALL ON TABLE user_webmoney_requisites TO appodeal;
GRANT SELECT ON TABLE user_webmoney_requisites TO support_bot;
GRANT SELECT ON TABLE user_webmoney_requisites TO readonly_user;


--
-- Name: user_webmoney_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_webmoney_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_webmoney_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_webmoney_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_webmoney_requisites_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_webmoney_requisites_id_seq TO readonly_user;


--
-- Name: user_wire_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_wire_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_wire_requisites FROM appodeal;
GRANT ALL ON TABLE user_wire_requisites TO appodeal;
GRANT SELECT ON TABLE user_wire_requisites TO support_bot;
GRANT SELECT ON TABLE user_wire_requisites TO readonly_user;


--
-- Name: user_wire_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_wire_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_wire_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_wire_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_wire_requisites_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_wire_requisites_id_seq TO readonly_user;


--
-- Name: user_yandex_money_requisites; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE user_yandex_money_requisites FROM PUBLIC;
REVOKE ALL ON TABLE user_yandex_money_requisites FROM appodeal;
GRANT ALL ON TABLE user_yandex_money_requisites TO appodeal;
GRANT SELECT ON TABLE user_yandex_money_requisites TO support_bot;
GRANT SELECT ON TABLE user_yandex_money_requisites TO readonly_user;


--
-- Name: user_yandex_money_requisites_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE user_yandex_money_requisites_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE user_yandex_money_requisites_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE user_yandex_money_requisites_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE user_yandex_money_requisites_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE user_yandex_money_requisites_id_seq TO readonly_user;


--
-- Name: users; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users FROM PUBLIC;
REVOKE ALL ON TABLE users FROM appodeal;
GRANT ALL ON TABLE users TO appodeal;
GRANT SELECT ON TABLE users TO support_bot;
GRANT SELECT ON TABLE users TO readonly_user;


--
-- Name: users_abilities; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users_abilities FROM PUBLIC;
REVOKE ALL ON TABLE users_abilities FROM appodeal;
GRANT ALL ON TABLE users_abilities TO appodeal;
GRANT SELECT ON TABLE users_abilities TO readonly_user;


--
-- Name: users_abilities_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE users_abilities_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE users_abilities_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE users_abilities_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE users_abilities_id_seq TO readonly_user;


--
-- Name: users_admin_notifications; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users_admin_notifications FROM PUBLIC;
REVOKE ALL ON TABLE users_admin_notifications FROM appodeal;
GRANT ALL ON TABLE users_admin_notifications TO appodeal;
GRANT SELECT ON TABLE users_admin_notifications TO readonly_user;


--
-- Name: users_admin_notifications_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE users_admin_notifications_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE users_admin_notifications_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE users_admin_notifications_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE users_admin_notifications_id_seq TO readonly_user;


--
-- Name: users_dev_apps; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users_dev_apps FROM PUBLIC;
REVOKE ALL ON TABLE users_dev_apps FROM appodeal;
GRANT ALL ON TABLE users_dev_apps TO appodeal;
GRANT SELECT ON TABLE users_dev_apps TO support_bot;
GRANT SELECT ON TABLE users_dev_apps TO readonly_user;


--
-- Name: users_dev_apps_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE users_dev_apps_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE users_dev_apps_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE users_dev_apps_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE users_dev_apps_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE users_dev_apps_id_seq TO readonly_user;


--
-- Name: users_fake_apps; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users_fake_apps FROM PUBLIC;
REVOKE ALL ON TABLE users_fake_apps FROM appodeal;
GRANT ALL ON TABLE users_fake_apps TO appodeal;
GRANT SELECT ON TABLE users_fake_apps TO support_bot;
GRANT SELECT ON TABLE users_fake_apps TO readonly_user;


--
-- Name: users_fake_apps_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE users_fake_apps_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE users_fake_apps_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE users_fake_apps_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE users_fake_apps_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE users_fake_apps_id_seq TO readonly_user;


--
-- Name: users_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE users_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE users_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE users_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE users_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE users_id_seq TO readonly_user;


--
-- Name: users_managed_campaigns; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users_managed_campaigns FROM PUBLIC;
REVOKE ALL ON TABLE users_managed_campaigns FROM appodeal;
GRANT ALL ON TABLE users_managed_campaigns TO appodeal;
GRANT SELECT ON TABLE users_managed_campaigns TO support_bot;
GRANT SELECT ON TABLE users_managed_campaigns TO readonly_user;


--
-- Name: users_managed_campaigns_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE users_managed_campaigns_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE users_managed_campaigns_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE users_managed_campaigns_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE users_managed_campaigns_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE users_managed_campaigns_id_seq TO readonly_user;


--
-- Name: users_roles; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users_roles FROM PUBLIC;
REVOKE ALL ON TABLE users_roles FROM appodeal;
GRANT ALL ON TABLE users_roles TO appodeal;
GRANT SELECT ON TABLE users_roles TO readonly_user;


--
-- Name: users_shared_apps; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE users_shared_apps FROM PUBLIC;
REVOKE ALL ON TABLE users_shared_apps FROM appodeal;
GRANT ALL ON TABLE users_shared_apps TO appodeal;
GRANT SELECT ON TABLE users_shared_apps TO support_bot;
GRANT SELECT ON TABLE users_shared_apps TO readonly_user;


--
-- Name: users_shared_apps_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE users_shared_apps_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE users_shared_apps_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE users_shared_apps_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE users_shared_apps_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE users_shared_apps_id_seq TO readonly_user;


--
-- Name: v2_placements; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE v2_placements FROM PUBLIC;
REVOKE ALL ON TABLE v2_placements FROM appodeal;
GRANT ALL ON TABLE v2_placements TO appodeal;
GRANT SELECT ON TABLE v2_placements TO readonly_user;


--
-- Name: v2_placements_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE v2_placements_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE v2_placements_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE v2_placements_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE v2_placements_id_seq TO readonly_user;


--
-- Name: v2_placements_versions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE v2_placements_versions FROM PUBLIC;
REVOKE ALL ON TABLE v2_placements_versions FROM appodeal;
GRANT ALL ON TABLE v2_placements_versions TO appodeal;
GRANT SELECT ON TABLE v2_placements_versions TO readonly_user;


--
-- Name: v2_placements_versions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE v2_placements_versions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE v2_placements_versions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE v2_placements_versions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE v2_placements_versions_id_seq TO readonly_user;


--
-- Name: v2_segments; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE v2_segments FROM PUBLIC;
REVOKE ALL ON TABLE v2_segments FROM appodeal;
GRANT ALL ON TABLE v2_segments TO appodeal;
GRANT SELECT ON TABLE v2_segments TO readonly_user;


--
-- Name: v2_segments_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE v2_segments_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE v2_segments_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE v2_segments_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE v2_segments_id_seq TO readonly_user;


--
-- Name: v2_segments_versions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE v2_segments_versions FROM PUBLIC;
REVOKE ALL ON TABLE v2_segments_versions FROM appodeal;
GRANT ALL ON TABLE v2_segments_versions TO appodeal;
GRANT SELECT ON TABLE v2_segments_versions TO readonly_user;


--
-- Name: v2_segments_versions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE v2_segments_versions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE v2_segments_versions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE v2_segments_versions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE v2_segments_versions_id_seq TO readonly_user;


--
-- Name: vacuum_stats; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vacuum_stats FROM PUBLIC;
REVOKE ALL ON TABLE vacuum_stats FROM postgres;
GRANT ALL ON TABLE vacuum_stats TO postgres;
GRANT SELECT ON TABLE vacuum_stats TO readonly_user;


--
-- Name: versions; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE versions FROM PUBLIC;
REVOKE ALL ON TABLE versions FROM appodeal;
GRANT ALL ON TABLE versions TO appodeal;
GRANT SELECT ON TABLE versions TO support_bot;
GRANT SELECT ON TABLE versions TO readonly_user;


--
-- Name: versions_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE versions_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE versions_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE versions_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE versions_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE versions_id_seq TO readonly_user;


--
-- Name: vpn_ips; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vpn_ips FROM PUBLIC;
REVOKE ALL ON TABLE vpn_ips FROM appodeal;
GRANT ALL ON TABLE vpn_ips TO appodeal;
GRANT SELECT ON TABLE vpn_ips TO readonly_user;


--
-- Name: vpn_ips_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE vpn_ips_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE vpn_ips_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE vpn_ips_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE vpn_ips_id_seq TO readonly_user;


--
-- Name: vpn_ips_y2018m08; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vpn_ips_y2018m08 FROM PUBLIC;
REVOKE ALL ON TABLE vpn_ips_y2018m08 FROM appodeal;
GRANT ALL ON TABLE vpn_ips_y2018m08 TO appodeal;
GRANT SELECT ON TABLE vpn_ips_y2018m08 TO readonly_user;


--
-- Name: vpn_ips_y2018m09; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vpn_ips_y2018m09 FROM PUBLIC;
REVOKE ALL ON TABLE vpn_ips_y2018m09 FROM appodeal;
GRANT ALL ON TABLE vpn_ips_y2018m09 TO appodeal;
GRANT SELECT ON TABLE vpn_ips_y2018m09 TO readonly_user;


--
-- Name: vpn_ips_y2018m10; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vpn_ips_y2018m10 FROM PUBLIC;
REVOKE ALL ON TABLE vpn_ips_y2018m10 FROM appodeal;
GRANT ALL ON TABLE vpn_ips_y2018m10 TO appodeal;
GRANT SELECT ON TABLE vpn_ips_y2018m10 TO readonly_user;


--
-- Name: vpn_ips_y2018m11; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vpn_ips_y2018m11 FROM PUBLIC;
REVOKE ALL ON TABLE vpn_ips_y2018m11 FROM appodeal;
GRANT ALL ON TABLE vpn_ips_y2018m11 TO appodeal;
GRANT SELECT ON TABLE vpn_ips_y2018m11 TO readonly_user;


--
-- Name: vungle_acceptations; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vungle_acceptations FROM PUBLIC;
REVOKE ALL ON TABLE vungle_acceptations FROM appodeal;
GRANT ALL ON TABLE vungle_acceptations TO appodeal;
GRANT SELECT ON TABLE vungle_acceptations TO support_bot;
GRANT SELECT ON TABLE vungle_acceptations TO readonly_user;


--
-- Name: vungle_acceptations_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE vungle_acceptations_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE vungle_acceptations_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE vungle_acceptations_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE vungle_acceptations_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE vungle_acceptations_id_seq TO readonly_user;


--
-- Name: vungle_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vungle_accounts FROM PUBLIC;
REVOKE ALL ON TABLE vungle_accounts FROM appodeal;
GRANT ALL ON TABLE vungle_accounts TO appodeal;
GRANT SELECT ON TABLE vungle_accounts TO support_bot;
GRANT SELECT ON TABLE vungle_accounts TO readonly_user;


--
-- Name: vungle_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE vungle_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE vungle_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE vungle_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE vungle_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE vungle_accounts_id_seq TO readonly_user;


--
-- Name: vungle_agreements; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE vungle_agreements FROM PUBLIC;
REVOKE ALL ON TABLE vungle_agreements FROM appodeal;
GRANT ALL ON TABLE vungle_agreements TO appodeal;
GRANT SELECT ON TABLE vungle_agreements TO support_bot;
GRANT SELECT ON TABLE vungle_agreements TO readonly_user;


--
-- Name: vungle_agreements_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE vungle_agreements_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE vungle_agreements_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE vungle_agreements_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE vungle_agreements_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE vungle_agreements_id_seq TO readonly_user;


--
-- Name: waterfall_logs; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_logs FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_logs FROM appodeal;
GRANT ALL ON TABLE waterfall_logs TO appodeal;
GRANT SELECT ON TABLE waterfall_logs TO readonly_user;


--
-- Name: waterfall_logs_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_logs_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_logs_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_logs_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_logs_id_seq TO readonly_user;


--
-- Name: waterfall_monitors; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_monitors FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_monitors FROM appodeal;
GRANT ALL ON TABLE waterfall_monitors TO appodeal;
GRANT SELECT ON TABLE waterfall_monitors TO readonly_user;


--
-- Name: waterfall_monitors_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_monitors_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_monitors_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_monitors_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_monitors_id_seq TO readonly_user;


--
-- Name: waterfall_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings FROM appodeal;
GRANT ALL ON TABLE waterfall_settings TO appodeal;
GRANT SELECT ON TABLE waterfall_settings TO readonly_user;


--
-- Name: waterfall_settings_categories; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings_categories FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings_categories FROM appodeal;
GRANT ALL ON TABLE waterfall_settings_categories TO appodeal;
GRANT SELECT ON TABLE waterfall_settings_categories TO readonly_user;


--
-- Name: waterfall_settings_categories_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_categories_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_categories_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_categories_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_categories_id_seq TO readonly_user;


--
-- Name: waterfall_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_id_seq TO readonly_user;


--
-- Name: waterfall_settings_root_flag_types; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings_root_flag_types FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings_root_flag_types FROM appodeal;
GRANT ALL ON TABLE waterfall_settings_root_flag_types TO appodeal;
GRANT SELECT ON TABLE waterfall_settings_root_flag_types TO readonly_user;


--
-- Name: waterfall_settings_root_flag_types_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_root_flag_types_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_root_flag_types_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_root_flag_types_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_root_flag_types_id_seq TO readonly_user;


--
-- Name: waterfall_settings_root_flags; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings_root_flags FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings_root_flags FROM appodeal;
GRANT ALL ON TABLE waterfall_settings_root_flags TO appodeal;
GRANT SELECT ON TABLE waterfall_settings_root_flags TO readonly_user;


--
-- Name: waterfall_settings_root_flags_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_root_flags_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_root_flags_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_root_flags_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_root_flags_id_seq TO readonly_user;


--
-- Name: waterfall_settings_root_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings_root_settings FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings_root_settings FROM appodeal;
GRANT ALL ON TABLE waterfall_settings_root_settings TO appodeal;
GRANT SELECT ON TABLE waterfall_settings_root_settings TO readonly_user;


--
-- Name: waterfall_settings_root_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_root_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_root_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_root_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_root_settings_id_seq TO readonly_user;


--
-- Name: waterfall_settings_unit_flag_types; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings_unit_flag_types FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings_unit_flag_types FROM appodeal;
GRANT ALL ON TABLE waterfall_settings_unit_flag_types TO appodeal;
GRANT SELECT ON TABLE waterfall_settings_unit_flag_types TO readonly_user;


--
-- Name: waterfall_settings_unit_flag_types_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_unit_flag_types_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_unit_flag_types_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_unit_flag_types_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_unit_flag_types_id_seq TO readonly_user;


--
-- Name: waterfall_settings_unit_flags; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings_unit_flags FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings_unit_flags FROM appodeal;
GRANT ALL ON TABLE waterfall_settings_unit_flags TO appodeal;
GRANT SELECT ON TABLE waterfall_settings_unit_flags TO readonly_user;


--
-- Name: waterfall_settings_unit_flags_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_unit_flags_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_unit_flags_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_unit_flags_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_unit_flags_id_seq TO readonly_user;


--
-- Name: waterfall_settings_unit_settings; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE waterfall_settings_unit_settings FROM PUBLIC;
REVOKE ALL ON TABLE waterfall_settings_unit_settings FROM appodeal;
GRANT ALL ON TABLE waterfall_settings_unit_settings TO appodeal;
GRANT SELECT ON TABLE waterfall_settings_unit_settings TO readonly_user;


--
-- Name: waterfall_settings_unit_settings_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE waterfall_settings_unit_settings_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE waterfall_settings_unit_settings_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE waterfall_settings_unit_settings_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE waterfall_settings_unit_settings_id_seq TO readonly_user;


--
-- Name: wl_domain_fees; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE wl_domain_fees FROM PUBLIC;
REVOKE ALL ON TABLE wl_domain_fees FROM appodeal;
GRANT ALL ON TABLE wl_domain_fees TO appodeal;
GRANT SELECT ON TABLE wl_domain_fees TO readonly_user;


--
-- Name: wl_domain_fees_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE wl_domain_fees_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE wl_domain_fees_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE wl_domain_fees_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE wl_domain_fees_id_seq TO readonly_user;


--
-- Name: wl_domain_font_files; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE wl_domain_font_files FROM PUBLIC;
REVOKE ALL ON TABLE wl_domain_font_files FROM appodeal;
GRANT ALL ON TABLE wl_domain_font_files TO appodeal;
GRANT SELECT ON TABLE wl_domain_font_files TO readonly_user;


--
-- Name: wl_domain_font_files_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE wl_domain_font_files_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE wl_domain_font_files_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE wl_domain_font_files_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE wl_domain_font_files_id_seq TO readonly_user;


--
-- Name: wl_domain_fonts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE wl_domain_fonts FROM PUBLIC;
REVOKE ALL ON TABLE wl_domain_fonts FROM appodeal;
GRANT ALL ON TABLE wl_domain_fonts TO appodeal;
GRANT SELECT ON TABLE wl_domain_fonts TO readonly_user;


--
-- Name: wl_domain_fonts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE wl_domain_fonts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE wl_domain_fonts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE wl_domain_fonts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE wl_domain_fonts_id_seq TO readonly_user;


--
-- Name: wl_domains; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE wl_domains FROM PUBLIC;
REVOKE ALL ON TABLE wl_domains FROM appodeal;
GRANT ALL ON TABLE wl_domains TO appodeal;
GRANT SELECT ON TABLE wl_domains TO readonly_user;


--
-- Name: wl_domains_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE wl_domains_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE wl_domains_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE wl_domains_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE wl_domains_id_seq TO readonly_user;


--
-- Name: yandex_accounts; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON TABLE yandex_accounts FROM PUBLIC;
REVOKE ALL ON TABLE yandex_accounts FROM appodeal;
GRANT ALL ON TABLE yandex_accounts TO appodeal;
GRANT SELECT ON TABLE yandex_accounts TO support_bot;
GRANT SELECT ON TABLE yandex_accounts TO readonly_user;


--
-- Name: yandex_accounts_id_seq; Type: ACL; Schema: public; Owner: -
--

REVOKE ALL ON SEQUENCE yandex_accounts_id_seq FROM PUBLIC;
REVOKE ALL ON SEQUENCE yandex_accounts_id_seq FROM appodeal;
GRANT ALL ON SEQUENCE yandex_accounts_id_seq TO appodeal;
GRANT SELECT ON SEQUENCE yandex_accounts_id_seq TO support_bot;
GRANT SELECT ON SEQUENCE yandex_accounts_id_seq TO readonly_user;


--
-- PostgreSQL database dump complete
--

